unit mp_rc16;

{Bit tables for Pi, ln(2), ln(10), exp(1)}

interface

{$i STD.INC}

(*************************************************************************

 DESCRIPTION   :  bit tables for Pi, ln(2), ln(10), exp(1)

 REQUIREMENTS  :  BP7, D1-D7/D9-D10/D12/D17-D18, FPC, VP

 REMARK        :  ln(10), exp(1) only if MPC_E1Ln10Tab is defined
                  last bits of short tables are NOT rounded, may be off by 1

 Version  Date      Author      Modification
 -------  --------  -------     ------------------------------------------
 0.00.10  25.01.08  W.Ehrhardt  Initial version: BP7

 1.16.00  12.06.10  we          Table for e = exp(1)
 1.16.01  13.06.10  we          Check table sizes, MAX_TCBITS moved to mp_types
 1.16.02  13.06.10  we          Uses MPC_FPrec30K for smaller tables

 1.20.00  15.01.12  we          BIT16 only, assumes MPC_FPrec30K
 1.20.01  14.02.12  we          ln(2) always available; new name MPC_E1Ln10Tab
**************************************************************************)


(*-------------------------------------------------------------------------
 (C) Copyright 2008-2012 Wolfgang Ehrhardt

 This software is provided 'as-is', without any express or implied warranty.
 In no event will the authors be held liable for any damages arising from
 the use of this software.

 Permission is granted to anyone to use this software for any purpose,
 including commercial applications, and to alter it and redistribute it
 freely, subject to the following restrictions:

 1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software in
    a product, an acknowledgment in the product documentation would be
    appreciated but is not required.

 2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

 3. This notice may not be removed or altered from any source distribution.
----------------------------------------------------------------------------*)

{$i mp_conf.inc}

function AddrPiBytes: pointer;
  {-Pointer to bits of Pi, big endian format}

function AddrLn2Bytes: pointer;
  {-Pointer to bits of ln(2), big endian format}

{$ifdef MPC_E1Ln10Tab}
function AddrEBytes: pointer;
  {-Pointer to bits of exp(1), big endian format}

function AddrLn10Bytes: pointer;
  {-Pointer to bits of ln(10), big endian format}
{$endif}


implementation


{$ifdef debug}
uses
  mp_types;
{$endif}

{---------------------------------------------------------------------------}
function AddrPiBytes: pointer;
  {-Pointer to bits of Pi, big endian format}
var
  PiBytes: pointer;
{$ifdef debug}
  PSkip: pointer;
{$endif}
begin
  asm
    jmp @@skip
    @@table:
     {calculated with t_cincpi}
      db $c9,$0f,$da,$a2,$21,$68,$c2,$34,$c4,$c6,$62,$8b,$80,$dc,$1c,$d1,$29,$02,$4e,$08,$8a,$67,$cc,$74,$02
      db $0b,$be,$a6,$3b,$13,$9b,$22,$51,$4a,$08,$79,$8e,$34,$04,$dd,$ef,$95,$19,$b3,$cd,$3a,$43,$1b,$30,$2b
      db $0a,$6d,$f2,$5f,$14,$37,$4f,$e1,$35,$6d,$6d,$51,$c2,$45,$e4,$85,$b5,$76,$62,$5e,$7e,$c6,$f4,$4c,$42
      db $e9,$a6,$37,$ed,$6b,$0b,$ff,$5c,$b6,$f4,$06,$b7,$ed,$ee,$38,$6b,$fb,$5a,$89,$9f,$a5,$ae,$9f,$24,$11
      db $7c,$4b,$1f,$e6,$49,$28,$66,$51,$ec,$e4,$5b,$3d,$c2,$00,$7c,$b8,$a1,$63,$bf,$05,$98,$da,$48,$36,$1c
      db $55,$d3,$9a,$69,$16,$3f,$a8,$fd,$24,$cf,$5f,$83,$65,$5d,$23,$dc,$a3,$ad,$96,$1c,$62,$f3,$56,$20,$85
      db $52,$bb,$9e,$d5,$29,$07,$70,$96,$96,$6d,$67,$0c,$35,$4e,$4a,$bc,$98,$04,$f1,$74,$6c,$08,$ca,$18,$21
      db $7c,$32,$90,$5e,$46,$2e,$36,$ce,$3b,$e3,$9e,$77,$2c,$18,$0e,$86,$03,$9b,$27,$83,$a2,$ec,$07,$a2,$8f
      db $b5,$c5,$5d,$f0,$6f,$4c,$52,$c9,$de,$2b,$cb,$f6,$95,$58,$17,$18,$39,$95,$49,$7c,$ea,$95,$6a,$e5,$15
      db $d2,$26,$18,$98,$fa,$05,$10,$15,$72,$8e,$5a,$8a,$aa,$c4,$2d,$ad,$33,$17,$0d,$04,$50,$7a,$33,$a8,$55
      db $21,$ab,$df,$1c,$ba,$64,$ec,$fb,$85,$04,$58,$db,$ef,$0a,$8a,$ea,$71,$57,$5d,$06,$0c,$7d,$b3,$97,$0f
      db $85,$a6,$e1,$e4,$c7,$ab,$f5,$ae,$8c,$db,$09,$33,$d7,$1e,$8c,$94,$e0,$4a,$25,$61,$9d,$ce,$e3,$d2,$26
      db $1a,$d2,$ee,$6b,$f1,$2f,$fa,$06,$d9,$8a,$08,$64,$d8,$76,$02,$73,$3e,$c8,$6a,$64,$52,$1f,$2b,$18,$17
      db $7b,$20,$0c,$bb,$e1,$17,$57,$7a,$61,$5d,$6c,$77,$09,$88,$c0,$ba,$d9,$46,$e2,$08,$e2,$4f,$a0,$74,$e5
      db $ab,$31,$43,$db,$5b,$fc,$e0,$fd,$10,$8e,$4b,$82,$d1,$20,$a9,$21,$08,$01,$1a,$72,$3c,$12,$a7,$87,$e6
      db $d7,$88,$71,$9a,$10,$bd,$ba,$5b,$26,$99,$c3,$27,$18,$6a,$f4,$e2,$3c,$1a,$94,$68,$34,$b6,$15,$0b,$da
      db $25,$83,$e9,$ca,$2a,$d4,$4c,$e8,$db,$bb,$c2,$db,$04,$de,$8e,$f9,$2e,$8e,$fc,$14,$1f,$be,$ca,$a6,$28
      db $7c,$59,$47,$4e,$6b,$c0,$5d,$99,$b2,$96,$4f,$a0,$90,$c3,$a2,$23,$3b,$a1,$86,$51,$5b,$e7,$ed,$1f,$61
      db $29,$70,$ce,$e2,$d7,$af,$b8,$1b,$dd,$76,$21,$70,$48,$1c,$d0,$06,$91,$27,$d5,$b0,$5a,$a9,$93,$b4,$ea
      db $98,$8d,$8f,$dd,$c1,$86,$ff,$b7,$dc,$90,$a6,$c0,$8f,$4d,$f4,$35,$c9,$34,$02,$84,$92,$36,$c3,$fa,$b4
      db $d2,$7c,$70,$26,$c1,$d4,$dc,$b2,$60,$26,$46,$de,$c9,$75,$1e,$76,$3d,$ba,$37,$bd,$f8,$ff,$94,$06,$ad
      db $9e,$53,$0e,$e5,$db,$38,$2f,$41,$30,$01,$ae,$b0,$6a,$53,$ed,$90,$27,$d8,$31,$17,$97,$27,$b0,$86,$5a
      db $89,$18,$da,$3e,$db,$eb,$cf,$9b,$14,$ed,$44,$ce,$6c,$ba,$ce,$d4,$bb,$1b,$db,$7f,$14,$47,$e6,$cc,$25
      db $4b,$33,$20,$51,$51,$2b,$d7,$af,$42,$6f,$b8,$f4,$01,$37,$8c,$d2,$bf,$59,$83,$ca,$01,$c6,$4b,$92,$ec
      db $f0,$32,$ea,$15,$d1,$72,$1d,$03,$f4,$82,$d7,$ce,$6e,$74,$fe,$f6,$d5,$5e,$70,$2f,$46,$98,$0c,$82,$b5
      db $a8,$40,$31,$90,$0b,$1c,$9e,$59,$e7,$c9,$7f,$be,$c7,$e8,$f3,$23,$a9,$7a,$7e,$36,$cc,$88,$be,$0f,$1d
      db $45,$b7,$ff,$58,$5a,$c5,$4b,$d4,$07,$b2,$2b,$41,$54,$aa,$cc,$8f,$6d,$7e,$bf,$48,$e1,$d8,$14,$cc,$5e
      db $d2,$0f,$80,$37,$e0,$a7,$97,$15,$ee,$f2,$9b,$e3,$28,$06,$a1,$d5,$8b,$b7,$c5,$da,$76,$f5,$50,$aa,$3d
      db $8a,$1f,$bf,$f0,$eb,$19,$cc,$b1,$a3,$13,$d5,$5c,$da,$56,$c9,$ec,$2e,$f2,$96,$32,$38,$7f,$e8,$d7,$6e
      db $3c,$04,$68,$04,$3e,$8f,$66,$3f,$48,$60,$ee,$12,$bf,$2d,$5b,$0b,$74,$74,$d6,$e6,$94,$f9,$1e,$6d,$be
      db $11,$59,$74,$a3,$92,$6f,$12,$fe,$e5,$e4,$38,$77,$7c,$b6,$a9,$32,$df,$8c,$d8,$be,$c4,$d0,$73,$b9,$31
      db $ba,$3b,$c8,$32,$b6,$8d,$9d,$d3,$00,$74,$1f,$a7,$bf,$8a,$fc,$47,$ed,$25,$76,$f6,$93,$6b,$a4,$24,$66
      db $3a,$ab,$63,$9c,$5a,$e4,$f5,$68,$34,$23,$b4,$74,$2b,$f1,$c9,$78,$23,$8f,$16,$cb,$e3,$9d,$65,$2d,$e3
      db $fd,$b8,$be,$fc,$84,$8a,$d9,$22,$22,$2e,$04,$a4,$03,$7c,$07,$13,$eb,$57,$a8,$1a,$23,$f0,$c7,$34,$73
      db $fc,$64,$6c,$ea,$30,$6b,$4b,$cb,$c8,$86,$2f,$83,$85,$dd,$fa,$9d,$4b,$7f,$a2,$c0,$87,$e8,$79,$68,$33
      db $03,$ed,$5b,$dd,$3a,$06,$2b,$3c,$f5,$b3,$a2,$78,$a6,$6d,$2a,$13,$f8,$3f,$44,$f8,$2d,$df,$31,$0e,$e0
      db $74,$ab,$6a,$36,$45,$97,$e8,$99,$a0,$25,$5d,$c1,$64,$f3,$1c,$c5,$08,$46,$85,$1d,$f9,$ab,$48,$19,$5d
      db $ed,$7e,$a1,$b1,$d5,$10,$bd,$7e,$e7,$4d,$73,$fa,$f3,$6b,$c3,$1e,$cf,$a2,$68,$35,$90,$46,$f4,$eb,$87
      db $9f,$92,$40,$09,$43,$8b,$48,$1c,$6c,$d7,$88,$9a,$00,$2e,$d5,$ee,$38,$2b,$c9,$19,$0d,$a6,$fc,$02,$6e
      db $47,$95,$58,$e4,$47,$56,$77,$e9,$aa,$9e,$30,$50,$e2,$76,$56,$94,$df,$c8,$1f,$56,$e8,$80,$b9,$6e,$71
      db $60,$c9,$80,$dd,$98,$a5,$73,$ea,$44,$72,$06,$5a,$13,$9c,$d2,$90,$6c,$d1,$cb,$72,$9e,$c5,$2a,$52,$86
      db $d4,$40,$14,$a6,$94,$ca,$45,$75,$83,$d5,$cf,$ef,$26,$f1,$b9,$0a,$d8,$29,$1d,$a0,$79,$9d,$00,$02,$2e
      db $9b,$ed,$55,$c6,$fa,$47,$fc,$a5,$bb,$1a,$ca,$83,$76,$45,$6d,$98,$d9,$48,$79,$ee,$7e,$6d,$bf,$cd,$01
      db $4b,$b1,$61,$55,$99,$14,$ec,$0b,$57,$6a,$67,$e3,$e8,$42,$2e,$91,$e6,$5b,$a1,$41,$da,$92,$de,$9c,$3a
      db $6d,$6c,$ca,$51,$36,$dd,$42,$4b,$b1,$06,$49,$88,$eb,$5b,$a9,$ac,$12,$69,$f7,$df,$67,$3b,$98,$2e,$23
      db $fb,$6c,$99,$bb,$2a,$a3,$1c,$5a,$66,$85,$ff,$d5,$99,$14,$9b,$30,$ac,$67,$b8,$46,$4d,$80,$a9,$5d,$42
      db $53,$0a,$68,$16,$44,$d0,$39,$06,$0e,$8f,$8f,$d5,$26,$26,$96,$d0,$a7,$59,$5a,$e3,$f9,$35,$a6,$7d,$cf
      db $f5,$a8,$74,$a7,$01,$fb,$fa,$0c,$3d,$53,$4b,$4e,$39,$bc,$09,$57,$70,$53,$37,$48,$21,$a1,$1c,$3a,$c9
      db $98,$e0,$ba,$71,$80,$87,$b3,$17,$82,$5a,$1a,$cf,$cf,$ae,$bb,$f2,$4f,$25,$c6,$05,$1a,$da,$9c,$28,$5a
      db $1f,$cd,$61,$14,$a8,$38,$a1,$ad,$e7,$14,$c1,$6a,$94,$01,$cd,$cf,$81,$e1,$07,$1f,$f7,$ab,$97,$23,$9f
      db $51,$3b,$15,$c5,$bc,$ae,$2c,$0e,$b6,$8d,$fc,$14,$03,$03,$7c,$07,$07,$c1,$00,$80,$2c,$ff,$eb,$83,$3d
      db $46,$8f,$2d,$5d,$2c,$89,$60,$de,$96,$37,$02,$48,$6f,$74,$64,$44,$fe,$5f,$2a,$4b,$fd,$a5,$0c,$91,$dc
      db $c8,$bd,$51,$c0,$4e,$b9,$79,$60,$4d,$f0,$b6,$b7,$32,$2d,$5d,$8d,$26,$bc,$f7,$69,$ea,$51,$18,$51,$83
      db $f4,$00,$c3,$bb,$32,$31,$cf,$a9,$1d,$47,$90,$78,$8e,$33,$66,$4e,$fa,$83,$8b,$cc,$a0,$2e,$e8,$46,$0f
      db $ac,$cc,$53,$95,$22,$ce,$13,$db,$6e,$42,$1b,$d0,$83,$40,$fd,$82,$81,$2f,$cb,$2e,$04,$a4,$09,$25,$df
      db $1e,$55,$9e,$6c,$1c,$af,$2b,$e2,$6b,$f7,$a6,$9d,$c7,$f6,$64,$c2,$04,$2c,$e2,$eb,$84,$b7,$33,$cf,$cb
      db $95,$44,$9c,$87,$cb,$9a,$dc,$49,$14,$06,$b7,$79,$a7,$e1,$33,$61,$de,$96,$11,$c6,$1d,$02,$36,$85,$ef
      db $27,$e6,$af,$3a,$52,$df,$63,$3b,$1e,$bb,$0e,$b6,$e1,$47,$7e,$98,$c2,$50,$d9,$b1,$19,$30,$f4,$bb,$c7
      db $06,$11,$cc,$85,$76,$42,$37,$50,$ce,$cd,$c9,$30,$ae,$85,$84,$a8,$53,$50,$ca,$99,$71,$14,$54,$25,$00
      db $00,$84,$ce,$b9,$37,$5c,$77,$fe,$27,$84,$0c,$53,$95,$60,$6b,$1d,$f5,$97,$c4,$46,$66,$c1,$0d,$55,$bc
      db $75,$e8,$f1,$da,$cf,$04,$46,$0e,$d6,$49,$29,$42,$7c,$a3,$f9,$bb,$65,$fc,$7e,$fe,$a7,$ae,$af,$cb,$07
      db $85,$4f,$1b,$a1,$b8,$d1,$5c,$3a,$ba,$5b,$ec,$61,$83,$97,$82,$96,$8f,$8a,$ac,$dd,$c7,$f9,$c7,$13,$8f
      db $41,$be,$8a,$59,$77,$2e,$66,$79,$c7,$43,$e0,$0f,$a2,$75,$94,$99,$b2,$09,$4b,$93,$32,$5e,$27,$04,$2c
      db $da,$b1,$85,$43,$ae,$a5,$38,$ba,$9e,$29,$7f,$0f,$14,$c7,$82,$8b,$7d,$3c,$bd,$d3,$a9,$cd,$87,$4a,$cf
      db $46,$4e,$49,$83,$c6,$70,$9e,$58,$14,$88,$e9,$c2,$3d,$c4,$c4,$ad,$ba,$eb,$7f,$9b,$ba,$b0,$c7,$d9,$b8
      db $ef,$11,$65,$69,$9e,$f2,$20,$ec,$5f,$cd,$f4,$40,$63,$3f,$ca,$30,$cc,$b7,$7b,$ef,$9b,$16,$a9,$59,$56
      db $08,$61,$5a,$2a,$e6,$00,$bb,$b3,$a9,$43,$f6,$cb,$e5,$4e,$ca,$bb,$df,$6b,$56,$db,$8b,$e1,$05,$48,$6d
      db $8a,$0a,$41,$d8,$5c,$3b,$37,$51,$dd,$58,$67,$c5,$44,$04,$f3,$2a,$0c,$3a,$d8,$6f,$65,$80,$cd,$3f,$87
      db $aa,$80,$d8,$f3,$ed,$5c,$d7,$24,$13,$1c,$28,$8e,$75,$67,$a7,$82,$f2,$ea,$b7,$85,$3b,$b3,$21,$af,$18
      db $18,$8b,$29,$e7,$2a,$d7,$2a,$ec,$bc,$e1,$1b,$99,$22,$c7,$ab,$c6,$6f,$7c,$32,$a8,$08,$da,$6e,$59,$56
      db $ae,$d4,$10,$1a,$16,$8c,$8f,$0a,$ad,$2c,$cc,$67,$ba,$75,$70,$08,$6e,$3d,$e6,$d5,$02,$c6,$61,$d7,$e8
      db $26,$65,$7d,$e6,$5f,$98,$8f,$5f,$6a,$3e,$0d,$e2,$26,$a5,$f8,$cb,$5d,$c4,$7b,$64,$d7,$c5,$9a,$04,$a0
      db $43,$8d,$62,$0a,$71,$f9,$87,$f5,$a5,$b7,$b7,$e8,$5e,$16,$2e,$a6,$55,$fd,$61,$29,$46,$c8,$9c,$98,$e6
      db $e0,$f0,$ff,$c6,$b0,$91,$a5,$b3,$6c,$c2,$ba,$d4,$cb,$8c,$15,$23,$f6,$52,$39,$1b,$6f,$0c,$4a,$16,$3a
      db $fc,$bb,$cd,$31,$bf,$fa,$bf,$8a,$3b,$58,$7b,$9f,$0f,$1c,$d7,$52,$85,$36,$7a,$19,$2d,$f8,$d0,$84,$17
      db $45,$08,$0f,$84,$f8,$11,$7b,$b8,$ad,$a8,$ea,$aa,$fa,$b6,$db,$13,$c5,$7e,$b2,$d3,$f4,$31,$d0,$bd,$10
      db $bb,$da,$ae,$ed,$59,$53,$ce,$c7,$50,$73,$48,$41,$76,$07,$9e,$67,$a1,$a1,$53,$71,$f9,$12,$d1,$da,$8f
      db $60,$58,$94,$33,$d8,$a8,$7c,$96,$e3,$49,$91,$bf,$22,$20,$e8,$30,$71,$ed,$a8,$df,$c5,$49,$30,$da,$72
      db $dd,$24,$91,$e1,$22,$82,$d5,$a4,$ac,$a1,$42,$56,$ef,$c0,$2b,$46,$52,$27,$45,$18,$ac,$5d,$08,$e0,$83
      db $80,$16,$10,$a3,$4a,$83,$15,$7d,$7a,$87,$6b,$7d,$0f,$88,$cf,$dc,$18,$4c,$dc,$bc,$24,$a3,$64,$df,$90
      db $75,$97,$fb,$3c,$5b,$08,$8e,$f6,$df,$37,$8d,$d6,$72,$fb,$9d,$18,$10,$21,$7c,$a9,$f3,$9d,$cc,$9b,$a9
      db $81,$e0,$21,$06,$7e,$14,$27,$ba,$3b,$f6,$15,$58,$76,$65,$cd,$6a,$5a,$69,$ea,$b1,$43,$01,$b0,$96,$81
      db $2a,$ff,$20,$02,$f2,$b7,$27,$91,$1e,$8b,$b0,$d1,$4d,$21,$3f,$75,$59,$c1,$43,$87,$a7,$b2,$76,$dc,$f6
      db $f4,$c4,$15,$62,$33,$59,$d7,$f6,$9e,$78,$d9,$d0,$d0,$31,$71,$0d,$19,$5c,$4f,$8e,$36,$0f,$4a,$3e,$27
      db $bc,$5b,$7f,$c8,$05,$4f,$88,$79,$e3,$ec,$0f,$52,$b9,$b8,$e7,$ca,$f6,$e0,$eb,$7d,$fa,$4f,$56,$9a,$25
      db $20,$50,$3d,$fd,$93,$09,$87,$25,$1a,$4a,$4d,$10,$45,$48,$3d,$dd,$80,$b5,$3d,$ef,$3d,$1a,$cb,$b5,$28
      db $80,$1a,$35,$02,$09,$1c,$4c,$c8,$3d,$1a,$90,$ed,$f5,$2d,$d4,$00,$18,$6b,$c7,$8e,$7d,$8b,$a5,$c9,$11
      db $51,$85,$08,$53,$dd,$2f,$e2,$e2,$10,$13,$65,$7f,$07,$65,$ad,$64,$6b,$dc,$3d,$37,$74,$d9,$a8,$0b,$d1
      db $6f,$ef,$02,$7b,$00,$ef,$65,$e1,$5f,$eb,$1b,$74,$0c,$72,$e1,$41,$25,$ba,$c9,$ec,$d5,$7f,$4e,$50,$76
      db $89,$51,$f9,$aa,$f2,$82,$a6,$8a,$14,$1e,$09,$54,$c1,$0a,$7d,$02,$8b,$21,$b6,$ba,$6d,$9b,$7e,$da,$37
      db $05,$18,$b5,$d2,$1a,$40,$1a,$03,$b0,$29,$09,$e8,$63,$7b,$93,$cf,$ff,$a8,$ba,$21,$eb,$63,$2d,$63,$38
      db $01,$9e,$bd,$35,$ad,$aa,$b3,$87,$9f,$34,$cd,$d7,$fb,$33,$9e,$28,$e6,$50,$1a,$ca,$90,$88,$3f,$a7,$8d
      db $76,$7c,$e1,$6e,$7b,$8e,$75,$ea,$ce,$c4,$93,$a2,$c7,$72,$7e,$bd,$d2,$db,$46,$15,$89,$a8,$d9,$8c,$7a
      db $b8,$e5,$ec,$8e,$9b,$be,$9d,$37,$56,$d0,$cc,$9a,$10,$79,$fd,$f2,$9e,$08,$3e,$fe,$c2,$bd,$53,$b6,$3f
      db $ac,$e5,$d1,$50,$15,$ab,$2e,$92,$25,$49,$d5,$54,$ce,$8e,$88,$20,$e3,$61,$ff,$9a,$ea,$6d,$f4,$25,$a5
      db $52,$d5,$6a,$19,$ef,$28,$45,$66,$96,$33,$2a,$4a,$58,$e6,$78,$76,$cc,$e9,$8a,$92,$95,$8f,$cc,$49,$7e
      db $57,$bd,$1f,$87,$24,$0a,$4c,$5f,$3f,$7e,$3a,$00,$81,$09,$cb,$dc,$20,$2e,$c5,$57,$01,$4a,$0b,$38,$e5
      db $70,$45,$52,$39,$31,$9b,$48,$92,$30,$44,$cf,$f1,$c3,$e1,$b7,$01,$fe,$72,$7b,$90,$41,$07,$c3,$d0,$11
      db $de,$69,$17,$62,$1b,$85,$cc,$97,$d4,$7a,$f5,$66,$f0,$34,$7c,$af,$30,$63,$d0,$44,$4d,$59,$09,$5e,$de
      db $0d,$18,$0a,$a7,$18,$37,$fe,$3a,$28,$c7,$d8,$db,$06,$c3,$84,$ad,$30,$80,$b8,$4c,$a7,$ab,$d9,$93,$f4
      db $72,$b4,$60,$45,$5a,$c8,$e5,$78,$0c,$cf,$a4,$b8,$4e,$c9,$02,$d8,$bb,$af,$ae,$48,$a1,$6c,$a8,$83,$b9
      db $ae,$83,$66,$77,$9c,$83,$23,$0b,$68,$bd,$ca,$34,$04,$9e,$11,$65,$6d,$e5,$3f,$59,$1f,$42,$18,$b9,$f3
      db $3d,$7c,$15,$12,$68,$db,$e1,$df,$52,$3e,$b0,$e7,$7f,$49,$fc,$cf,$a3,$47,$82,$91,$d8,$d0,$66,$4b,$bf
      db $dd,$0e,$9b,$db,$aa,$fd,$3e,$3f,$4d,$ea,$04,$b7,$18,$28,$7a,$f7,$7e,$26,$46,$f8,$53,$36,$db,$9a,$c3
      db $71,$9e,$d5,$44,$1b,$59,$cb,$0d,$c9,$d9,$75,$0e,$f7,$34,$3a,$01,$3c,$4a,$43,$71,$f3,$00,$3f,$e8,$ed
      db $4e,$43,$e4,$9a,$43,$fb,$42,$d9,$9e,$e7,$fe,$f3,$b6,$df,$67,$28,$24,$73,$c2,$f6,$45,$57,$a8,$ee,$c4
      db $cb,$e2,$14,$56,$eb,$49,$1e,$e5,$1e,$6f,$dd,$8e,$f5,$ba,$cd,$ce,$4b,$ac,$f3,$04,$56,$5e,$60,$09,$b8
      db $a5,$fd,$0b,$8c,$4b,$5a,$10,$ab,$69,$f1,$9a,$8a,$ce,$c4,$9d,$53,$33,$1e,$0b,$bc,$47,$1a,$84,$90,$8d
      db $ed,$e4,$94,$79,$c1,$a8,$6e,$f9,$92,$fe,$d8,$d4,$06,$9a,$c4,$06,$04,$72,$bb,$7e,$8f,$49,$6f,$76,$38
      db $b8,$70,$f2,$51,$10,$85,$96,$42,$84,$84,$e1,$b6,$43,$3b,$1b,$b5,$6a,$fa,$8a,$99,$2b,$d9,$d3,$b6,$a1
      db $aa,$17,$ef,$af,$fa,$62,$19,$39,$30,$ff,$a7,$6f,$20,$15,$fc,$3d,$f0,$21,$98,$1e,$1e,$fe,$18,$00,$d8
      db $13,$74,$7f,$60,$d1,$bd,$8e,$07,$ec,$1d,$d1,$6b,$81,$35,$cd,$bf,$33,$20,$d0,$9a,$cc,$fc,$07,$aa,$dc
      db $6c,$20,$10,$61,$cf,$00,$17,$97,$dd,$e8,$15,$ef,$af,$7a,$2b,$89,$15,$51,$90,$a6,$6f,$d6,$0b,$98,$53
      db $96,$3d,$23,$fc,$b7,$7f,$68,$bd,$1d,$3b,$ce,$21,$e2,$6f,$70,$94,$d9,$be,$70,$f2,$2c,$e3,$9d,$2d,$1d
      db $7c,$95,$51,$bf,$36,$6e,$5e,$ba,$c9,$98,$62,$c7,$77,$e1,$21,$1a,$83,$9e,$64,$57,$e5,$78,$91,$9b,$96
      db $63,$88,$2d,$15,$dc,$23,$35,$55,$64,$72,$40,$b7,$93,$2e,$42,$eb,$38,$6e,$e0,$81,$74,$04,$6a,$18,$92
      db $1d,$5d,$2a,$67,$ad,$df,$c6,$6d,$b8,$2a,$77,$02,$59,$8b,$42,$68,$71,$0c,$91,$8c,$fa,$16,$87,$c0,$82
      db $7c,$2f,$a3,$12,$a6,$68,$09,$47,$5b,$bf,$84,$06,$ae,$4f,$47,$42,$e9,$69,$37,$e8,$61,$bc,$83,$ca,$1a
      db $3c,$5a,$77,$2d,$f6,$a0,$d5,$ce,$41,$bf,$a8,$78,$b3,$a6,$d3,$f3,$5f,$d4,$94,$04,$57,$80,$69,$c1,$a0
      db $fe,$a8,$00,$ad,$71,$03,$79,$b4,$09,$e6,$be,$23,$09,$dd,$cf,$e1,$90,$70,$d8,$13,$01,$98,$6a,$01,$ad
      db $7c,$05,$de,$8a,$30,$3d,$61,$b8,$00,$18,$16,$2a,$8c,$37,$1f,$58,$84,$79,$a7,$b5,$c8,$ce,$3a,$98,$d4
      db $f0,$b7,$65,$30,$b0,$85,$8d,$2e,$f2,$fb,$95,$a4,$2f,$2d,$b7,$ba,$ff,$1f,$68,$73,$96,$47,$5d,$9b,$c1
      db $79,$02,$52,$df,$00,$62,$0e,$5c,$82,$8f,$5f,$24,$9f,$09,$21,$b8,$dc,$97,$dc,$93,$67,$6e,$46,$b0,$56
      db $ed,$34,$e7,$ae,$3f,$3b,$55,$15,$5e,$02,$3f,$29,$6d,$76,$0f,$5f,$34,$d3,$6b,$43,$f1,$07,$94,$3b,$d7
      db $ac,$58,$79,$be,$28,$a1,$45,$36,$5b,$14,$44,$cf,$1b,$f5,$71,$f9,$d5,$b8,$53,$b1,$0d,$8a,$af,$f3,$b7
      db $71,$b2,$0d,$f5,$e6,$8c,$8d,$24,$98,$e0,$84,$99,$c3,$be,$a3,$90,$18,$00,$38,$0e,$8e,$73,$8d,$f4,$fe
      db $bd,$73,$ea,$f0,$9d,$cd,$d6,$b1,$4b,$46,$d7,$2c,$19,$e7,$93,$e8,$cd,$d0,$b4,$e0,$89,$d0,$26,$6f,$26
      db $ef,$b5,$44,$63,$a7,$6f,$c3,$dc,$c5,$75,$8b,$47,$1f,$b1,$c0,$31,$1e,$ed,$e3,$06,$da,$c8,$68,$64,$11
      db $6c,$9b,$ac,$6f,$9a,$8d,$9b,$ad,$15,$d2,$2a,$cb,$ef,$25,$1b,$9e,$71,$a8,$dd,$b4,$99,$52,$70,$b2,$14
      db $c3,$fe,$3b,$91,$a3,$77,$9f,$75,$5c,$c2,$87,$53,$34,$13,$71,$8a,$4e,$6e,$f6,$ea,$6e,$91,$94,$2b,$25
      db $49,$ba,$2f,$97,$b8,$c1,$28,$7e,$b5,$7c,$1a,$8b,$54,$66,$98,$fb,$e3,$38,$a6,$a1,$bb,$88,$b0,$22,$70
      db $ae,$10,$c9,$0b,$bd,$a9,$47,$80,$ea,$a7,$3c,$b4,$29,$20,$f0,$18,$6e,$a6,$fa,$5a,$93,$63,$f9,$45,$54
      db $2e,$99,$16,$f5,$8a,$09,$a8,$be,$69,$ce,$8e,$b8,$52,$ea,$65,$61,$bb,$d5,$58,$ba,$71,$cb,$fb,$f4,$fd
      db $d4,$bd,$f6,$8f,$c1,$1b,$da,$5d,$fe,$82,$96,$60,$39,$2a,$45,$61,$ec,$21,$80,$66,$c2,$79,$ab,$4e,$cf
      db $b9,$64,$fa,$64,$3f,$56,$a7,$8d,$35,$e5,$cb,$3c,$2d,$f6,$40,$8a,$e2,$d4,$65,$b5,$6b,$0e,$80,$5f,$69
      db $9f,$74,$73,$cf,$b5,$9f,$1f,$4b,$4a,$07,$e7,$c9,$73,$eb,$7c,$ae,$26,$d6,$b5,$ad,$1c,$96,$a2,$3d,$53
      db $38,$0a,$6b,$1c,$78,$06,$69,$79,$91,$ec,$2b,$3f,$7b,$64,$fe,$a6,$fa,$34,$f1,$23,$4a,$0e,$d5,$f3,$3e
      db $35,$59,$8a,$5e,$44,$cb,$8a,$1e,$17,$c0,$64,$7b,$5d,$58,$15,$7d,$e5,$83,$91,$38,$f4,$d7,$a3,$05,$41
      db $5b,$75,$22,$3d,$1b,$6e,$80,$e8,$58,$49,$41,$59,$3c,$2f,$70,$fa,$e7,$85,$4f,$24,$15,$e8,$a5,$c9,$c6
      db $bb,$2a,$4e,$81,$ca,$86,$cf,$db,$66,$c7,$98,$c8,$7d,$7d,$67,$19,$be,$c9,$b7,$3c,$c6,$5d,$4c,$f6,$4a
      db $2c,$55,$7f,$7d,$40,$d5,$8d,$20,$a2,$ae,$8f,$2e,$ca,$14,$5d,$c4,$70,$82,$b6,$7e,$2a,$f3,$14,$59,$f3
      db $2b,$64,$97,$d3,$7a,$05,$d4,$4e,$0c,$37,$23,$8d,$e7,$56,$18,$a4,$c8,$3e,$64,$7a,$9e,$a4,$30,$be,$cf
      db $9e,$f3,$94,$48,$73,$99,$1d,$d3,$ef,$8c,$aa,$2d,$b8,$df,$b0,$ca,$4f,$51,$92,$37,$94,$da,$59,$04,$f9
      db $a0,$28,$ab,$82,$04,$37,$5b,$6c,$89,$1a,$61,$4b,$7f,$42,$41,$c8,$59,$ac,$e6,$91,$82,$99,$11,$70,$37
      db $56,$1b,$37,$b3,$c7,$08,$32,$2b,$96,$ef,$bd,$f7,$46,$d6,$23,$50,$33,$34,$80,$5f,$da,$ed,$38,$ee,$f7
      db $68,$9a,$9f,$8e,$96,$7f,$d1,$4f,$8d,$42,$91,$2f,$2d,$33,$75,$5c,$ba,$b3,$aa,$3e,$99,$21,$2e,$e3,$59
      db $84,$ab,$af,$cf,$1b,$d1,$f4,$a6,$f9,$18,$d5,$0b,$d7,$67,$ab,$b0,$9d,$c6,$fd,$93,$98,$dc,$1d,$03,$83
      db $63,$79,$d6,$c4,$d5,$fe,$1c,$85,$9c,$6b,$d4,$df,$57,$50,$10,$32,$c2,$13,$ad,$38,$b3,$0d,$34,$91,$9a
      db $80,$45,$6b,$e1,$38,$6c,$01,$0a,$25,$66,$0e,$87,$41,$ae,$27,$ed,$33,$9b,$a8,$12,$1b,$cf,$ce,$e0,$8d
      db $48,$2a,$e0,$80,$46,$87,$52,$c9,$dc,$89,$fe,$18,$45,$58,$2c,$79,$e4,$cf,$f7,$2e,$ce,$9e,$4a,$cd,$11
    @@skip:
    mov ax,offset @@table
    mov word ptr [PiBytes],ax
    mov ax,cs
    mov word ptr [PiBytes+2],ax
    {$ifdef debug}
      mov ax,offset @@skip
      mov word ptr [PSkip],ax
      mov ax,cs
      mov word ptr [PSkip+2],ax
    {$endif}
  end;
  AddrPiBytes := PiBytes;
  {$ifdef debug}
    assert(MAX_TCBITS = 8*(longint(PSkip)-longint(PiBytes)), 'Length Pi table');
  {$endif}
end;


{---------------------------------------------------------------------------}
function AddrLn2Bytes: pointer;
  {-Pointer to bits of ln(2), big endian format}
var
  L2Bytes: pointer;
{$ifdef debug}
  PSkip: pointer;
{$endif}
begin
  asm
    jmp @@skip
    @@table:
      {calculated with t_ciln2}
      db $b1,$72,$17,$f7,$d1,$cf,$79,$ab,$c9,$e3,$b3,$98,$03,$f2,$f6,$af,$40,$f3,$43,$26,$72,$98,$b6,$2d,$8a
      db $0d,$17,$5b,$8b,$aa,$fa,$2b,$e7,$b8,$76,$20,$6d,$eb,$ac,$98,$55,$95,$52,$fb,$4a,$fa,$1b,$10,$ed,$2e
      db $ae,$35,$c1,$38,$21,$44,$27,$57,$3b,$29,$11,$69,$b8,$25,$3e,$96,$ca,$16,$22,$4a,$e8,$c5,$1a,$cb,$da
      db $11,$31,$7c,$38,$7e,$b9,$ea,$9b,$c3,$b1,$36,$60,$3b,$25,$6f,$a0,$ec,$76,$57,$f7,$4b,$72,$ce,$87,$b1
      db $9d,$65,$48,$ca,$f5,$df,$a6,$bd,$38,$30,$32,$48,$65,$5f,$a1,$87,$2f,$20,$e3,$a2,$da,$2d,$97,$c5,$0f
      db $3f,$d5,$c6,$07,$f4,$ca,$11,$fb,$5b,$fb,$90,$61,$0d,$30,$f8,$8f,$e5,$51,$a2,$ee,$56,$9d,$6d,$fc,$1e
      db $fa,$15,$7d,$2e,$23,$de,$14,$00,$b3,$96,$17,$46,$07,$75,$db,$89,$90,$e5,$c9,$43,$e7,$32,$b4,$79,$cd
      db $33,$cc,$cc,$4e,$65,$93,$93,$51,$4c,$4c,$1a,$1e,$0b,$d1,$d6,$09,$5d,$25,$66,$9b,$33,$35,$64,$a3,$37
      db $6a,$9c,$7f,$8a,$5e,$14,$8e,$82,$07,$4d,$b6,$01,$5c,$fe,$7a,$a3,$0c,$48,$0a,$54,$17,$35,$0d,$2c,$95
      db $5d,$51,$79,$b1,$e1,$7b,$9d,$ae,$31,$3c,$db,$6c,$60,$6c,$b1,$07,$8f,$73,$5d,$1b,$2d,$b3,$1b,$5f,$50
      db $b5,$18,$50,$64,$c1,$8b,$4d,$16,$2d,$b3,$b3,$65,$85,$3d,$75,$98,$a1,$95,$1a,$e2,$73,$ee,$55,$70,$b6
      db $c6,$8f,$96,$98,$34,$96,$d4,$e6,$d3,$30,$af,$88,$9b,$44,$a0,$25,$54,$73,$1c,$dc,$8e,$a1,$72,$93,$d1
      db $22,$8a,$4e,$f9,$8d,$6f,$51,$77,$fb,$cf,$07,$55,$26,$8a,$5c,$1f,$95,$38,$b9,$82,$61,$af,$fd,$44,$6b
      db $1c,$a3,$cf,$5e,$92,$22,$b8,$8c,$66,$d3,$c5,$42,$21,$83,$ed,$c9,$94,$21,$09,$0b,$bb,$16,$fa,$f3,$d9
      db $49,$f2,$36,$e0,$2b,$20,$ce,$e8,$86,$b9,$05,$c1,$28,$d5,$3d,$0b,$d2,$f9,$62,$13,$63,$19,$6a,$f5,$03
      db $02,$00,$60,$e4,$99,$08,$39,$1a,$0c,$57,$33,$9b,$a2,$be,$ba,$7d,$05,$2a,$c5,$b6,$1c,$c4,$e9,$20,$7c
      db $ef,$2f,$0c,$e2,$d7,$37,$39,$58,$d7,$62,$26,$58,$90,$1e,$64,$6a,$95,$18,$44,$60,$dc,$4e,$74,$87,$15
      db $6e,$0c,$29,$24,$13,$d5,$e3,$61,$c1,$69,$6d,$d2,$4a,$ae,$bd,$47,$38,$26,$fd,$a0,$c2,$38,$b9,$0a,$b1
      db $11,$bb,$bd,$67,$c7,$24,$97,$2c,$d1,$8b,$fb,$bd,$9d,$42,$6c,$47,$20,$96,$e7,$61,$15,$c0,$5f,$6f,$7c
      db $eb,$ac,$9f,$45,$ae,$ce,$cb,$72,$f1,$9c,$38,$33,$9d,$8f,$68,$26,$25,$0d,$ea,$89,$1e,$f0,$7a,$ff,$f3
      db $a8,$92,$37,$4e,$17,$5e,$b4,$af,$c8,$da,$ad,$d8,$85,$db,$6a,$b0,$3a,$49,$bd,$0d,$c0,$b1,$b3,$1d,$8a
      db $0e,$23,$fa,$c5,$e5,$76,$7d,$f9,$58,$84,$e0,$64,$25,$a4,$15,$26,$fa,$c5,$1c,$3e,$a8,$44,$9f,$e8,$f7
      db $0e,$dd,$06,$2b,$1a,$63,$a6,$c4,$c6,$0c,$52,$ab,$33,$16,$1e,$23,$84,$38,$89,$7a,$39,$ce,$78,$b6,$3c
      db $9f,$36,$4f,$5b,$8a,$ef,$22,$ec,$2f,$ee,$6e,$08,$50,$ec,$a4,$2d,$06,$fb,$0c,$75,$df,$54,$97,$e0,$0c
      db $55,$4b,$03,$d7,$d2,$87,$4a,$00,$0c,$a8,$f5,$8d,$94,$f0,$34,$1c,$be,$2e,$c9,$21,$56,$c9,$f9,$49,$db
      db $4a,$93,$16,$f2,$81,$50,$1e,$53,$da,$ec,$3f,$64,$f1,$b7,$83,$15,$4c,$60,$32,$0e,$2f,$f7,$93,$33,$ce
      db $35,$73,$fa,$cc,$5f,$dc,$f1,$17,$85,$90,$31,$55,$bb,$d9,$0f,$02,$3b,$22,$02,$24,$fc,$d8,$47,$1b,$f4
      db $f4,$45,$f0,$a8,$8a,$14,$f0,$cd,$97,$6e,$a3,$54,$bb,$20,$cd,$b5,$cc,$b3,$db,$23,$92,$88,$d5,$86,$55
      db $4e,$2a,$0e,$8a,$6f,$e5,$1a,$8c,$fa,$a7,$2e,$f2,$ad,$8a,$43,$dc,$42,$12,$b2,$10,$b7,$79,$df,$e4,$9d
      db $73,$07,$cc,$84,$65,$32,$e4,$b9,$69,$4e,$da,$d1,$62,$af,$05,$3b,$17,$51,$f3,$a3,$d0,$91,$f6,$56,$65
      db $81,$54,$12,$b5,$e8,$c2,$02,$46,$10,$69,$ac,$14,$b9,$58,$78,$49,$34,$b8,$d6,$cc,$e1,$da,$a5,$05,$37
      db $01,$1a,$a4,$fb,$42,$b9,$a3,$de,$f4,$1b,$da,$1f,$85,$ef,$6f,$db,$f2,$f2,$d8,$9d,$2a,$4b,$18,$35,$27
      db $8f,$d9,$40,$57,$89,$f4,$56,$81,$2b,$55,$28,$79,$a6,$16,$86,$95,$c1,$29,$63,$b0,$ff,$01,$ea,$ab,$73
      db $e5,$b5,$c1,$58,$53,$18,$e7,$62,$4f,$14,$a5,$1a,$4a,$02,$6b,$68,$08,$29,$20,$57,$fd,$99,$b6,$6d,$c0
      db $85,$a9,$8a,$c8,$d8,$ca,$f9,$ee,$ee,$a9,$8a,$24,$00,$ca,$c9,$5f,$26,$0f,$d1,$00,$36,$f9,$f9,$10,$96
      db $ac,$31,$95,$22,$0a,$1a,$35,$6b,$2a,$73,$b7,$ea,$ad,$af,$6d,$60,$58,$71,$ef,$7a,$fb,$80,$bc,$42,$34
      db $33,$56,$2e,$94,$b1,$2d,$fa,$b4,$14,$45,$15,$79,$df,$59,$ea,$e0,$51,$70,$70,$62,$40,$12,$a8,$29,$62
      db $c5,$9c,$ab,$34,$7f,$83,$04,$d8,$89,$65,$9e,$5a,$91,$39,$db,$14,$ef,$cc,$30,$85,$2b,$e3,$e8,$fc,$99
      db $f1,$4d,$1d,$82,$2d,$d6,$e2,$f7,$67,$97,$e3,$02,$19,$c8,$aa,$9c,$e8,$84,$8a,$88,$6e,$b3,$c8,$7b,$72
      db $95,$98,$80,$12,$e8,$31,$41,$86,$ed,$ba,$f8,$68,$56,$cc,$d3,$c3,$b6,$ee,$94,$e6,$2f,$11,$0a,$67,$83
      db $d2,$aa,$e8,$9c,$cc,$3b,$76,$fc,$43,$5a,$0c,$e1,$34,$c2,$83,$8f,$d5,$71,$ec,$6c,$13,$66,$a9,$92,$cb
      db $b9,$ac,$40,$7d,$db,$6c,$13,$a4,$b8,$d1,$ec,$f7,$56,$7e,$b0,$97,$1c,$c9,$0b,$55,$18,$56,$9f,$14,$4e
      db $67,$eb,$e9,$b4,$26,$98,$fe,$a7,$9d,$89,$d5,$c5,$ed,$40,$ac,$5e,$37,$01,$d7,$d7,$72,$53,$77,$cf,$06
      db $56,$90,$7f,$b9,$b1,$b1,$6e,$a8,$91,$1a,$fb,$f1,$ae,$5a,$66,$20,$3d,$62,$fd,$1e,$70,$93,$43,$5b,$9c
      db $27,$77,$36,$a7,$0f,$a8,$60,$1c,$f6,$86,$8a,$05,$5b,$22,$38,$67,$7a,$2b,$fb,$bd,$84,$3b,$fa,$18,$73
      db $f0,$c4,$46,$b0,$1b,$2a,$e0,$e9,$8e,$0e,$15,$27,$a9,$00,$b1,$af,$5e,$75,$f8,$7c,$7c,$d1,$7a,$f8,$04
      db $d9,$33,$d6,$f7,$e1,$b9,$e1,$90,$3d,$71,$c7,$bb,$a0,$28,$11,$37,$60,$90,$dd,$61,$73,$35,$df,$dd,$42
      db $4f,$2b,$66,$1c,$d8,$50,$63,$03,$4e,$34,$1b,$06,$e2,$11,$97,$7b,$07,$5a,$8b,$78,$08,$df,$43,$bd,$8e
      db $ef,$1f,$d9,$67,$8c,$c0,$b5,$e9,$f6,$0a,$3e,$b8,$17,$47,$f8,$7e,$57,$09,$46,$8d,$78,$eb,$d2,$da,$01
      db $16,$e6,$b6,$5a,$eb,$3b,$e7,$7e,$e2,$36,$fd,$c3,$3b,$c8,$e7,$df,$1f,$fc,$2e,$22,$88,$f8,$ca,$9a,$ee
      db $5b,$60,$89,$9d,$52,$24,$7f,$4c,$e0,$b6,$ff,$d8,$b9,$47,$81,$76,$42,$c6,$f2,$24,$f7,$0f,$88,$25,$5e
      db $1c,$08,$39,$cb,$cb,$dd,$82,$6c,$5d,$37,$70,$06,$7e,$3e,$08,$eb,$67,$9c,$1d,$b7,$ff,$c1,$68,$42,$23
      db $d7,$c6,$e6,$81,$15,$a2,$83,$22,$91,$ff,$14,$fc,$9d,$b2,$5d,$a9,$be,$cc,$b4,$c2,$75,$33,$e7,$47,$d9
      db $04,$7f,$55,$c9,$01,$af,$f6,$4a,$09,$c4,$5b,$e2,$c7,$3e,$6f,$dc,$29,$69,$6e,$22,$68,$a5,$e0,$c9,$2f
      db $51,$81,$95,$db,$bd,$18,$c4,$ef,$70,$2d,$6c,$55,$01,$31,$6d,$ed,$1f,$3e,$55,$85,$f6,$dd,$d1,$3a,$05
      db $95,$46,$94,$87,$0e,$71,$c1,$94,$cb,$c4,$39,$62,$e1,$12,$ee,$67,$99,$0c,$36,$31,$21,$77,$d1,$b7,$b1
      db $a7,$69,$f8,$87,$32,$4b,$53,$de,$1a,$4c,$99,$c9,$a0,$92,$c1,$06,$c3,$ce,$35,$b9,$48,$cb,$52,$53,$d6
      db $6a,$e9,$9d,$7e,$fb,$ea,$3d,$9e,$7f,$58,$62,$17,$ae,$b4,$06,$8a,$ee,$de,$38,$4f,$a8,$8b,$87,$fd,$03
      db $63,$81,$a7,$43,$42,$da,$7a,$6e,$d2,$65,$8a,$de,$a5,$ee,$41,$4c,$bc,$1b,$b5,$f5,$05,$d2,$20,$a7,$04
      db $3a,$07,$42,$28,$1b,$03,$f8,$1f,$c6,$47,$39,$30,$ff,$7c,$84,$7e,$90,$a3,$73,$f5,$a9,$a5,$27,$bd,$44
      db $47,$ad,$b7,$d1,$9c,$45,$e2,$71,$5d,$3d,$15,$c2,$98,$57,$0b,$15,$2c,$ec,$2f,$bd,$58,$2d,$c4,$6a,$e7
      db $26,$e4,$9e,$3c,$7c,$33,$4b,$5e,$fd,$b4,$c6,$fa,$45,$07,$ba,$00,$26,$76,$1a,$68,$0b,$77,$f6,$ca,$b5
      db $ed,$37,$6d,$10,$58,$39,$f0,$31,$92,$22,$be,$8f,$db,$67,$92,$68,$9f,$f1,$58,$a5,$99,$08,$f1,$67,$07
      db $76,$73,$8a,$f2,$0f,$3c,$cf,$6d,$58,$2b,$a8,$c6,$12,$0d,$a6,$a6,$49,$a5,$57,$83,$ce,$13,$4a,$42,$bb
      db $af,$ae,$7d,$ac,$1d,$53,$87,$bf,$01,$8d,$2b,$31,$01,$22,$34,$78,$59,$28,$dc,$db,$04,$1e,$93,$25,$b4
      db $af,$62,$1a,$8a,$55,$5e,$27,$6a,$31,$41,$f6,$01,$e3,$bd,$2b,$cb,$98,$56,$b2,$9f,$ad,$c8,$ec,$1b,$2f
      db $45,$a1,$62,$99,$e5,$ac,$fa,$4d,$76,$8e,$e2,$fe,$52,$71,$57,$ff,$cd,$4a,$70,$aa,$1e,$df,$64,$10,$11
      db $0b,$d6,$15,$e2,$57,$a2,$03,$f7,$6e,$d0,$e7,$90,$70,$d2,$0a,$75,$1f,$cc,$43,$a7,$c4,$11,$79,$05,$7f
      db $c4,$50,$dc,$a7,$a9,$02,$47,$84,$be,$09,$f6,$09,$ee,$c9,$04,$43,$85,$0a,$ca,$3b,$ed,$5f,$3d,$14,$af
      db $70,$7b,$b0,$95,$af,$93,$1c,$f8,$31,$7d,$02,$a3,$73,$4a,$d2,$cc,$11,$4b,$ff,$03,$d0,$60,$4a,$99,$8e
      db $01,$95,$0b,$82,$e6,$86,$ce,$94,$a8,$38,$ee,$11,$68,$96,$88,$d0,$fd,$96,$7d,$06,$bd,$a5,$b7,$2a,$a6
      db $13,$fa,$22,$0b,$bd,$ec,$99,$5c,$53,$ef,$ad,$e3,$56,$ed,$90,$b7,$a9,$43,$fd,$28,$0f,$ce,$7d,$f5,$1d
      db $60,$7b,$1b,$85,$53,$5b,$4c,$e9,$34,$3f,$67,$1d,$48,$77,$fc,$71,$94,$cc,$c4,$cd,$fe,$fc,$08,$ea,$6d
      db $a2,$46,$40,$bb,$e2,$ba,$0f,$2b,$9f,$73,$ad,$3d,$da,$75,$60,$93,$22,$47,$ff,$56,$56,$34,$67,$f3,$20
      db $18,$e1,$4f,$e4,$6c,$75,$a4,$4a,$4d,$5d,$7e,$8d,$ab,$6b,$b6,$db,$f8,$bc,$4e,$6d,$07,$f9,$e7,$d9,$93
      db $d8,$99,$57,$23,$81,$57,$fb,$ef,$fa,$03,$45,$b5,$aa,$b3,$d1,$fe,$ed,$cb,$09,$f1,$4c,$40,$25,$7b,$2b
      db $7f,$30,$8e,$b7,$99,$ab,$4c,$95,$d1,$a1,$ed,$94,$bd,$da,$0b,$62,$5f,$39,$c4,$c8,$6f,$c5,$ee,$57,$48
      db $04,$1d,$64,$7a,$ef,$cb,$7a,$f2,$26,$fe,$13,$82,$99,$7d,$87,$b2,$21,$99,$e9,$12,$c3,$e3,$f5,$3d,$b5
      db $a7,$20,$8e,$03,$52,$bb,$66,$b3,$79,$77,$8c,$ee,$2e,$c9,$af,$2d,$87,$31,$11,$9f,$38,$bd,$9d,$90,$d4
      db $3e,$05,$4d,$8b,$71,$fc,$0c,$0d,$10,$99,$0e,$84,$c3,$32,$02,$ec,$c9,$fe,$d5,$1f,$62,$63,$93,$cc,$c5
      db $66,$84,$03,$67,$ae,$0b,$74,$cb,$4d,$bf,$e4,$ce,$bd,$7f,$e7,$4b,$8d,$e2,$aa,$91,$76,$bd,$b5,$00,$61
      db $3e,$e2,$ac,$34,$e4,$e9,$e7,$87,$b4,$c2,$9e,$7f,$b2,$08,$a9,$dd,$9c,$92,$f0,$02,$ca,$30,$f8,$33,$f5
      db $60,$93,$c1,$84,$8b,$de,$22,$a0,$bd,$ad,$10,$22,$21,$80,$95,$f2,$b8,$63,$b0,$6c,$ca,$87,$9f,$0b,$c8
      db $d7,$73,$a5,$28,$1e,$ef,$a8,$11,$77,$4a,$66,$64,$b9,$f1,$b1,$2a,$25,$c5,$4e,$b9,$d9,$07,$73,$5d,$44
      db $1c,$77,$e8,$42,$7c,$da,$02,$b7,$50,$8d,$45,$49,$5b,$4e,$c1,$b2,$e6,$b8,$5e,$8d,$75,$1a,$8f,$7d,$d2
      db $71,$ee,$0e,$c8,$dc,$e8,$b6,$35,$96,$53,$3f,$c2,$41,$fe,$15,$a9,$cf,$a5,$ab,$d5,$73,$5a,$ed,$99,$93
      db $7f,$74,$09,$36,$9a,$1b,$26,$32,$1b,$d1,$35,$0e,$cb,$f2,$1c,$45,$4c,$1d,$a1,$19,$c7,$12,$16,$d7,$3e
      db $8e,$c5,$dd,$5c,$59,$27,$9d,$16,$f2,$d3,$ff,$84,$47,$cd,$6a,$cd,$1f,$8d,$6b,$54,$71,$95,$49,$a7,$c2
      db $bb,$38,$ae,$f7,$e6,$7c,$20,$08,$bc,$d7,$89,$24,$ba,$55,$75,$db,$ae,$31,$f6,$54,$9f,$33,$7c,$9c,$24
      db $49,$07,$04,$a3,$44,$0c,$51,$3e,$f2,$b9,$c7,$30,$c6,$f6,$4c,$03,$1c,$91,$3b,$81,$86,$c5,$ee,$43,$a2
      db $62,$7e,$f4,$52,$a9,$43,$58,$b2,$69,$65,$a8,$48,$d4,$12,$9a,$f7,$2e,$d3,$b4,$ec,$0d,$1a,$20,$60,$62
      db $00,$67,$86,$2b,$47,$de,$f3,$6e,$a6,$6b,$c4,$81,$a4,$32,$68,$77,$8c,$0e,$17,$6d,$3a,$2f,$7c,$12,$a6
      db $9d,$d6,$54,$31,$4a,$fc,$a2,$07,$a0,$c1,$f9,$00,$a2,$c7,$ac,$b9,$aa,$69,$58,$70,$19,$31,$03,$d8,$87
      db $b8,$d0,$d1,$6b,$ff,$66,$22,$9b,$1d,$5c,$71,$e4,$0b,$b1,$96,$e5,$a5,$a8,$7a,$ea,$56,$00,$1f,$36,$aa
      db $98,$b3,$4a,$93,$53,$f8,$87,$5c,$a6,$6f,$cf,$6e,$80,$eb,$eb,$0c,$22,$fc,$51,$10,$40,$d7,$66,$f3,$8a
      db $91,$d1,$8e,$b2,$c5,$f0,$2e,$9a,$aa,$ae,$d8,$da,$f4,$9b,$dc,$90,$a7,$c8,$11,$7a,$a4,$ac,$c8,$a6,$02
      db $89,$ca,$3c,$b5,$b5,$44,$57,$e1,$39,$1c,$8f,$7d,$31,$05,$4e,$5b,$58,$f5,$0a,$c7,$6e,$c5,$66,$20,$cc
      db $a6,$48,$97,$2f,$6d,$c0,$8e,$00,$ce,$59,$d5,$db,$3a,$89,$08,$50,$24,$a9,$b4,$bf,$7f,$b0,$13,$ec,$a9
      db $2e,$5c,$1c,$83,$68,$f1,$d8,$c1,$3d,$87,$b3,$77,$e6,$f3,$ca,$18,$06,$0a,$fe,$25,$42,$be,$41,$c4,$83
      db $be,$5d,$ca,$1a,$3f,$c1,$0c,$c4,$65,$da,$62,$74,$04,$38,$27,$96,$76,$04,$f4,$09,$a1,$fb,$ae,$b8,$a5
      db $e4,$f1,$fe,$54,$41,$d2,$86,$43,$fd,$2d,$6d,$64,$da,$55,$b7,$46,$a4,$f3,$d2,$18,$7e,$fa,$de,$66,$c0
      db $bf,$8e,$db,$27,$1e,$34,$d7,$1a,$d4,$3d,$43,$e5,$ba,$ca,$7f,$76,$9e,$2b,$a1,$c2,$72,$a4,$9d,$e8,$19
      db $10,$e8,$b6,$02,$e9,$67,$1b,$e8,$20,$31,$49,$1d,$74,$fb,$17,$05,$1b,$db,$41,$1a,$86,$c5,$89,$7c,$13
      db $3a,$47,$48,$55,$97,$ee,$a6,$9d,$95,$a1,$26,$8b,$fe,$49,$14,$09,$e6,$e3,$f9,$d2,$aa,$2d,$fb,$55,$70
      db $09,$7e,$e1,$d4,$2c,$07,$ee,$5a,$2d,$67,$3d,$d9,$ac,$ad,$6b,$02,$69,$39,$40,$56,$c8,$0d,$c2,$b5,$b6
      db $c2,$39,$b3,$55,$a0,$bf,$aa,$a4,$d8,$b1,$2b,$86,$f4,$27,$b5,$bf,$c1,$59,$f1,$2d,$27,$32,$fb,$9d,$53
      db $6b,$06,$2e,$90,$4d,$70,$a7,$8a,$e8,$8f,$14,$2d,$27,$57,$ad,$ef,$78,$3b,$74,$a7,$6c,$7e,$2a,$d8,$45
      db $62,$a4,$78,$e0,$8a,$97,$8d,$cc,$2c,$0e,$e3,$b6,$0d,$5a,$b4,$3f,$ed,$28,$6d,$8a,$c9,$92,$c9,$91,$8e
      db $5f,$23,$22,$e2,$2e,$a4,$8d,$7d,$e9,$50,$45,$46,$fe,$52,$3c,$f5,$9f,$15,$d5,$7e,$0c,$6a,$27,$c0,$09
      db $cb,$a9,$2f,$dd,$34,$51,$5a,$d0,$21,$96,$48,$65,$1f,$ec,$89,$51,$ef,$24,$cd,$15,$f0,$76,$8d,$8e,$8b
      db $fc,$8b,$fd,$b1,$cc,$86,$75,$4b,$d0,$32,$3d,$e0,$69,$22,$b8,$aa,$bf,$3f,$e7,$58,$20,$36,$0a,$54,$29
      db $d9,$f2,$5b,$d5,$fc,$8e,$8e,$ac,$20,$1f,$37,$56,$27,$aa,$36,$b8,$f2,$7d,$e0,$ca,$84,$dc,$43,$36,$e0
      db $d4,$47,$df,$a2,$11,$0d,$c0,$89,$06,$a1,$4e,$9d,$85,$94,$9a,$f3,$59,$8f,$5e,$65,$5e,$16,$3c,$ab,$15
      db $91,$da,$d6,$f6,$e8,$64,$7c,$b9,$c1,$d8,$3f,$bb,$f5,$92,$24,$a0,$dd,$88,$3c,$2b,$04,$53,$2b,$a5,$44
      db $ab,$77,$90,$7c,$6a,$64,$e0,$b3,$d6,$70,$ac,$71,$23,$38,$41,$a3,$43,$34,$23,$98,$05,$58,$59,$b9,$7d
      db $52,$9a,$98,$60,$dd,$f1,$09,$cf,$8b,$7c,$aa,$50,$5d,$4d,$b8,$3c,$22,$a3,$89,$1b,$37,$57,$f5,$b4,$37
      db $86,$25,$0e,$dc,$71,$7f,$f1,$c7,$ed,$d3,$c8,$10,$3f,$a0,$01,$75,$3f,$de,$36,$b8,$34,$e1,$4d,$3c,$c3
      db $b3,$4f,$47,$23,$24,$5a,$9e,$db,$3a,$c2,$25,$e3,$ed,$7f,$4b,$b3,$ed,$18,$87,$d9,$c4,$ef,$8f,$cd,$34
      db $f8,$b4,$a6,$f9,$72,$c2,$f0,$6b,$a8,$d4,$92,$1d,$51,$ff,$b0,$28,$31,$a6,$31,$32,$60,$ed,$a6,$00,$42
      db $16,$e1,$79,$46,$df,$af,$e1,$c5,$2c,$ba,$83,$41,$73,$14,$2b,$53,$fd,$25,$78,$9b,$0b,$07,$45,$0d,$41
      db $77,$f4,$b3,$04,$96,$1f,$58,$22,$e8,$b4,$5d,$b0,$97,$5f,$0b,$df,$4e,$5c,$8f,$91,$d5,$44,$c9,$89,$81
      db $4a,$7e,$88,$6a,$72,$64,$3b,$d5,$14,$05,$10,$dc,$b9,$24,$c1,$c9,$3a,$ef,$e6,$0f,$53,$9c,$86,$4f,$95
      db $51,$8e,$85,$c4,$71,$0c,$08,$5e,$8f,$77,$8c,$24,$d2,$ce,$70,$b6,$9e,$4c,$61,$b7,$15,$d8,$97,$bb,$30
      db $63,$fa,$b3,$12,$2d,$0b,$f7,$e1,$48,$8b,$57,$ee,$58,$22,$a5,$5d,$8d,$0a,$fa,$46,$cd,$ee,$c3,$91,$d6
      db $88,$1a,$0d,$c4,$d3,$ea,$65,$23,$ba,$5b,$98,$03,$9c,$a9,$63,$a1,$e0,$f7,$2c,$b6,$25,$9a,$13,$d6,$e3
      db $e5,$2e,$c8,$b7,$e4,$18,$b7,$d1,$e4,$fc,$f1,$cb,$e3,$b9,$49,$4f,$a8,$dc,$54,$e1,$c4,$96,$db,$32,$14
      db $33,$9b,$b3,$b0,$4c,$e2,$73,$77,$ea,$aa,$cf,$a1,$21,$05,$1d,$16,$dd,$79,$85,$26,$74,$ba,$8a,$aa,$57
      db $03,$a9,$2f,$2a,$86,$c8,$f5,$7e,$20,$c7,$25,$17,$c5,$a9,$99,$df,$3d,$a5,$9d,$3d,$a4,$cb,$ed,$c4,$7b
      db $57,$98,$f4,$ab,$e7,$5b,$7c,$e9,$81,$b0,$d4,$fc,$5b,$51,$a7,$e5,$a1,$69,$67,$e3,$ec,$50,$b0,$37,$7a
      db $d8,$81,$53,$f9,$19,$7e,$a0,$b9,$6e,$85,$86,$c8,$bc,$30,$f8,$82,$36,$e7,$6b,$b7,$7e,$28,$82,$2c,$6a
      db $00,$30,$94,$fb,$70,$03,$ad,$a1,$b6,$db,$b5,$4c,$2f,$c7,$9f,$61,$1b,$f4,$30,$f8,$84,$82,$a4,$48,$51
      db $60,$43,$b7,$2e,$a3,$25,$7f,$ff,$0c,$7b,$9d,$8e,$6b,$ce,$01,$6d,$58,$76,$f7,$fd,$61,$33,$f8,$9a,$cc
      db $5f,$bf,$2e,$81,$d6,$a6,$56,$96,$78,$f4,$23,$c0,$5f,$0f,$af,$df,$23,$2a,$a7,$86,$f5,$2f,$3d,$0a,$43
      db $5f,$23,$18,$6f,$9f,$f8,$91,$79,$4b,$ce,$3f,$9d,$92,$8e,$1e,$81,$02,$00,$c7,$f4,$6c,$5b,$7a,$c5,$61
      db $82,$be,$cb,$db,$d7,$43,$fa,$b4,$e7,$00,$e2,$f4,$55,$ae,$f8,$a0,$32,$e9,$29,$51,$ad,$98,$bb,$99,$1c
      db $de,$1a,$09,$b3,$cd,$4f,$70,$2e,$93,$a3,$65,$71,$99,$cd,$1f,$00,$86,$dc,$8b,$1c,$cc,$4f,$2a,$80,$91
      db $ea,$1d,$f0,$fd,$88,$9f,$df,$05,$d2,$c0,$fd,$18,$e6,$1e,$78,$2d,$f0,$72,$bb,$8b,$e5,$11,$a7,$e8,$41
      db $2b,$53,$3a,$9a,$d6,$3e,$28,$d3,$de,$c7,$60,$ec,$c1,$ee,$b7,$56,$f9,$36,$c3,$63,$be,$71,$35,$01,$ec
      db $05,$a1,$33,$45,$97,$af,$64,$96,$e0,$0e,$36,$48,$b6,$bf,$94,$0b,$ed,$fa,$10,$e6,$93,$fb,$89,$55,$a3
      db $44,$86,$9f,$d9,$f1,$05,$a4,$57,$eb,$a9,$20,$58,$1f,$48,$b4,$7a,$d4,$18,$df,$52,$68,$17,$71,$fc,$b6
      db $34,$af,$ed,$a9,$53,$0e,$cc,$68,$bb,$83,$7c,$d0,$33,$0a,$39,$43,$60,$c1,$d8,$3d,$14,$c2,$6e,$f2,$ea
      db $02,$56,$84,$ac,$9d,$6a,$48,$c8,$1b,$0d,$75,$9d,$4f,$ca,$8c,$60,$af,$f6,$a8,$87,$ed,$39,$3f,$8f,$f9
      db $bb,$07,$05,$fe,$ce,$f8,$7e,$65,$b0,$37,$39,$3f,$b8,$0f,$75,$d0,$78,$a3,$cc,$af,$86,$d0,$3a,$30,$e3
      db $60,$7b,$b1,$ea,$6b,$3c,$48,$54,$12,$70,$dc,$b3,$ae,$84,$18,$a6,$28,$28,$23,$02,$c6,$86,$19,$f3,$25
      db $9a,$19,$10,$74,$21,$05,$8f,$a6,$3f,$f2,$79,$0c,$af,$19,$0c,$ac,$de,$24,$4b,$f8,$43,$4c,$ea,$9f,$2f
      db $9c,$ed,$ca,$29,$b2,$3c,$93,$c7,$5c,$33,$b6,$32,$21,$87,$c3,$76,$ad,$3a,$e1,$9b,$17,$29,$64,$1d,$83
      db $83,$22,$ae,$3a,$f9,$80,$84,$d0,$70,$3f,$c9,$b3,$46,$d2,$8f,$a8,$9c,$4d,$2b,$ee,$8f,$26,$ba,$f1,$6a
      db $a5,$39,$75,$2f,$84,$5e,$c6,$3b,$ca,$48,$c5,$78,$47,$3e,$ad,$af,$df,$94,$37,$fb,$6b,$ee,$d1,$5c,$18
      db $53,$cc,$78,$01,$08,$51,$66,$d8,$a0,$3b,$a0,$13,$4e,$87,$14,$2f,$af,$19,$4e,$fe,$30,$47,$44,$f4,$8b
      db $1f,$94,$72,$47,$98,$35,$db,$c8,$4e,$37,$57,$51,$2e,$b7,$43,$54,$6c,$f3,$72,$b2,$e0,$a6,$72,$4e,$7c
    @@skip:
    mov ax,offset @@table
    mov word ptr [L2Bytes],ax
    mov ax,cs
    mov word ptr [L2Bytes+2],ax
    {$ifdef debug}
      mov ax,offset @@skip
      mov word ptr [PSkip],ax
      mov ax,cs
      mov word ptr [PSkip+2],ax
    {$endif}
  end;
  AddrLn2Bytes := L2Bytes;
  {$ifdef debug}
    assert(MAX_TCBITS = 8*(longint(PSkip)-longint(L2Bytes)), 'Length ln2 table');
  {$endif}
end;


{$ifdef MPC_E1Ln10Tab}

{---------------------------------------------------------------------------}
function AddrLn10Bytes: pointer;
  {-Pointer to bits of ln(10), big endian format}
var
  L10Bytes: pointer;
{$ifdef debug}
  PSkip: pointer;
{$endif}
begin
  asm
    jmp @@skip
    @@table:
      {calculated with t_ciln10}
      db $93,$5d,$8d,$dd,$aa,$a8,$ac,$16,$ea,$56,$d6,$2b,$82,$d3,$0a,$28,$e2,$8f,$ec,$f9,$da,$5d,$f9,$0e,$83
      db $c6,$1e,$82,$01,$f0,$2d,$72,$96,$2f,$02,$d7,$b1,$a8,$10,$5c,$cc,$70,$cb,$c0,$2c,$5f,$0d,$68,$2c,$62
      db $24,$18,$41,$0b,$e2,$da,$fb,$8f,$78,$84,$02,$e5,$16,$d6,$78,$2c,$f8,$a2,$8a,$8c,$91,$1e,$76,$5a,$a6
      db $c3,$b0,$d8,$31,$fb,$ef,$66,$ce,$b0,$4a,$b3,$c6,$fa,$51,$61,$bb,$49,$d2,$19,$c7,$bb,$ca,$67,$b3,$5b
      db $23,$60,$50,$85,$8e,$93,$36,$8d,$44,$78,$9c,$4f,$5b,$08,$b0,$57,$d5,$ed,$e2,$0f,$46,$9e,$a5,$8e,$93
      db $05,$e9,$81,$e2,$47,$8f,$ca,$ad,$3a,$ee,$98,$9c,$d5,$b4,$2e,$6a,$27,$16,$19,$a4,$7e,$cb,$26,$97,$8c
      db $5d,$4f,$db,$1d,$28,$ea,$57,$d4,$fd,$c0,$e4,$0b,$f3,$cc,$1e,$14,$12,$6a,$45,$76,$5c,$de,$26,$83,$39
      db $db,$f4,$7f,$a9,$6d,$eb,$27,$10,$60,$af,$88,$48,$6e,$a9,$b7,$40,$1e,$3d,$fd,$3c,$51,$74,$8e,$6d,$6e
      db $38,$48,$c8,$d2,$5f,$af,$1b,$ca,$e8,$80,$47,$f1,$7b,$0d,$9b,$50,$a9,$49,$ea,$aa,$df,$69,$e8,$a5,$f7
      db $7e,$37,$60,$4e,$94,$39,$60,$e3,$8a,$57,$00,$ff,$de,$2d,$b1,$ad,$6b,$fb,$ff,$d8,$21,$ba,$0a,$4c,$b0
      db $46,$6d,$61,$ba,$64,$8e,$ef,$99,$c8,$e5,$f6,$97,$4f,$36,$39,$82,$a7,$8c,$a4,$5d,$df,$c8,$09,$42,$61
      db $78,$19,$12,$7a,$6e,$3b,$70,$fc,$da,$2d,$73,$2d,$47,$b5,$e4,$b1,$c8,$c0,$e5,$a1,$0a,$aa,$66,$04,$a5
      db $32,$4e,$c3,$dc,$bc,$64,$ea,$80,$6e,$19,$85,$66,$1f,$1d,$36,$6c,$20,$66,$38,$34,$4d,$5e,$84,$3f,$20
      db $64,$2b,$97,$0a,$62,$d1,$8e,$47,$8f,$7b,$d5,$8f,$cd,$08,$32,$4a,$7b,$32,$a6,$de,$f8,$5a,$05,$eb,$56
      db $32,$3a,$42,$1e,$f5,$e0,$b0,$04,$10,$a0,$a0,$d9,$c2,$60,$79,$4a,$97,$6f,$f6,$ff,$36,$3d,$b0,$0b,$6b
      db $33,$f4,$2c,$58,$de,$f8,$a3,$c5,$2d,$ed,$69,$b1,$3d,$c1,$a0,$37,$30,$b6,$52,$4d,$c1,$8c,$16,$7e,$86
      db $99,$d6,$d2,$0e,$a2,$de,$fd,$2b,$d0,$06,$f8,$b4,$be,$14,$5a,$2a,$df,$3c,$cb,$b3,$18,$9d,$a4,$9d,$bc
      db $12,$61,$c8,$b3,$e4,$da,$ad,$6a,$36,$ce,$cc,$b2,$d5,$ae,$5b,$89,$bf,$75,$2f,$b5,$df,$b3,$53,$ff,$30
      db $65,$c4,$0c,$fc,$ee,$c8,$1b,$e5,$a9,$a9,$67,$fd,$dc,$57,$c4,$b8,$33,$01,$00,$6b,$f0,$62,$4b,$40,$ed
      db $7a,$56,$c6,$cd,$cd,$a2,$d6,$fe,$91,$38,$8e,$9e,$3e,$48,$a9,$3f,$5f,$5e,$3b,$6e,$b4,$b8,$1c,$4a,$5b
      db $53,$d4,$9e,$a6,$8e,$66,$8a,$ea,$ba,$83,$c7,$f8,$fb,$5f,$06,$c3,$58,$ac,$8f,$70,$fa,$9d,$8c,$59,$8c
      db $57,$45,$02,$ba,$f5,$4c,$96,$c8,$49,$11,$f0,$04,$82,$d0,$95,$1a,$0a,$f0,$22,$ab,$ba,$b0,$80,$ec,$97
      db $ef,$d3,$67,$1e,$4e,$0e,$52,$f1,$66,$b6,$cd,$5c,$d2,$26,$0d,$c6,$77,$95,$2e,$1e,$34,$a3,$f7,$99,$67
      db $7f,$2c,$1d,$48,$f1,$29,$44,$b6,$c5,$2b,$a1,$30,$7e,$70,$4d,$67,$f9,$1c,$10,$35,$e4,$4e,$92,$7c,$63
      db $03,$cf,$12,$bf,$e2,$cd,$2e,$31,$f8,$ee,$48,$43,$34,$4d,$51,$b0,$f3,$7d,$a4,$2b,$9f,$0b,$0f,$d9,$13
      db $4f,$b2,$d9,$f8,$15,$e4,$90,$d9,$66,$28,$3f,$23,$96,$27,$66,$ec,$ea,$b1,$e4,$f3,$b5,$fc,$86,$46,$81
      db $27,$e2,$b6,$06,$d1,$0d,$3a,$45,$f4,$b6,$b7,$76,$10,$2d,$2f,$db,$b4,$20,$80,$c8,$fa,$84,$d0,$ff,$9f
      db $45,$c5,$8a,$ef,$38,$db,$24,$10,$fd,$1f,$1c,$eb,$ad,$73,$3b,$22,$81,$52,$ca,$5f,$36,$dd,$f2,$9d,$aa
      db $54,$43,$34,$b8,$de,$ea,$f6,$59,$4e,$46,$27,$13,$1e,$d4,$85,$b4,$6a,$08,$22,$e1,$28,$db,$47,$1c,$a5
      db $39,$38,$a8,$44,$c3,$be,$f7,$f3,$52,$15,$c8,$b3,$82,$bc,$4e,$3e,$4c,$6f,$15,$62,$85,$f5,$4c,$17,$ab
      db $40,$8e,$cc,$bf,$7f,$5e,$d1,$6a,$b3,$f6,$ce,$d2,$84,$6d,$f4,$57,$e1,$4f,$bb,$45,$d9,$c5,$64,$6a,$d4
      db $97,$ac,$69,$74,$94,$14,$5d,$e3,$2e,$93,$90,$71,$28,$d2,$63,$d5,$21,$79,$ef,$b4,$24,$d6,$46,$51,$d6
      db $eb,$c0,$c9,$f0,$bb,$58,$3a,$44,$c6,$41,$2c,$84,$85,$bb,$29,$a6,$4d,$31,$a2,$cd,$92,$95,$44,$69,$a3
      db $2b,$1a,$bd,$f7,$f5,$20,$2c,$a4,$aa,$6c,$93,$2e,$9b,$53,$cf,$38,$5a,$b1,$36,$27,$41,$f3,$56,$5d,$e9
      db $c0,$65,$60,$09,$90,$1c,$88,$ab,$bd,$d8,$74,$ef,$cf,$73,$3f,$76,$1a,$d4,$35,$f3,$c0,$83,$fd,$6b,$8e
      db $e0,$0b,$ef,$11,$c7,$c5,$52,$a8,$9d,$58,$ce,$4a,$21,$d7,$1e,$54,$f2,$41,$57,$f6,$c7,$d4,$62,$23,$16
      db $e9,$89,$56,$d7,$45,$00,$27,$de,$cb,$d3,$98,$d8,$4b,$98,$b3,$6a,$07,$24,$c2,$5c,$db,$23,$77,$60,$e9
      db $32,$4b,$68,$75,$23,$e5,$06,$8e,$da,$d9,$33,$92,$19,$7f,$00,$b8,$53,$a3,$26,$b3,$30,$c4,$44,$65,$12
      db $92,$96,$34,$bc,$06,$70,$e1,$77,$80,$6d,$e3,$38,$da,$c4,$55,$37,$49,$2a,$e1,$9a,$dd,$83,$cf,$45,$00
      db $0f,$5b,$42,$3b,$ce,$64,$97,$d2,$09,$e3,$0e,$18,$a1,$3c,$bf,$06,$87,$67,$97,$31,$03,$d9,$48,$53,$66
      db $81,$50,$6b,$ba,$2e,$93,$a9,$a4,$7d,$d5,$9d,$3f,$f1,$7c,$d7,$46,$8c,$20,$75,$be,$55,$2a,$43,$47,$b4
      db $a6,$38,$ef,$d4,$9a,$2e,$66,$3e,$f2,$7d,$99,$88,$92,$30,$5b,$c1,$9c,$9b,$2b,$41,$9e,$48,$b6,$3b,$76
      db $37,$51,$37,$a7,$57,$65,$d3,$40,$5d,$db,$3c,$fd,$a4,$6d,$82,$da,$f4,$7f,$79,$cc,$1e,$36,$10,$af,$77
      db $aa,$42,$d4,$47,$8d,$de,$e6,$07,$47,$37,$06,$25,$3d,$b0,$e0,$3f,$bc,$ed,$76,$c3,$c0,$68,$06,$fb,$76
      db $bf,$77,$e9,$c5,$8e,$3d,$a5,$0c,$dc,$d5,$00,$6d,$a9,$d4,$a3,$d3,$c3,$6f,$b0,$37,$ab,$ab,$f4,$d2,$ce
      db $4d,$a1,$f8,$8a,$8d,$9f,$48,$7d,$88,$d7,$ce,$99,$6e,$eb,$7e,$4a,$b9,$40,$6a,$2f,$73,$2b,$5a,$dd,$24
      db $81,$8d,$60,$93,$61,$f7,$b2,$7d,$f5,$7b,$ae,$5d,$07,$bc,$a4,$9f,$9e,$13,$cb,$41,$1a,$89,$64,$14,$d3
      db $3a,$27,$50,$3c,$00,$b4,$80,$c9,$b6,$cf,$ef,$b8,$ff,$46,$8e,$39,$d7,$d3,$8c,$2d,$1b,$66,$d6,$b4,$9b
      db $55,$fb,$da,$6e,$c6,$a2,$6c,$65,$a2,$99,$85,$60,$8b,$03,$78,$42,$18,$28,$5a,$10,$f0,$75,$88,$7c,$d5
      db $0d,$f7,$c4,$cf,$b8,$bc,$8c,$7b,$ee,$0c,$a9,$25,$ba,$83,$fc,$68,$4c,$52,$eb,$f3,$d9,$be,$03,$78,$15
      db $fc,$14,$9b,$76,$de,$ef,$0a,$32,$97,$42,$9f,$be,$3f,$28,$14,$c5,$3a,$a7,$cf,$88,$db,$48,$cb,$14,$3e
      db $82,$07,$da,$b6,$44,$b8,$36,$25,$05,$59,$3f,$e7,$3f,$79,$3b,$35,$e7,$4b,$1c,$b6,$94,$b2,$8d,$f6,$d2
      db $ed,$7e,$95,$90,$0b,$04,$a7,$e2,$af,$f7,$e9,$a8,$45,$b3,$2b,$1b,$a2,$7c,$89,$06,$1f,$d2,$be,$26,$96
      db $b4,$ca,$23,$1c,$94,$7f,$8d,$bb,$02,$91,$b0,$78,$ca,$33,$d0,$69,$18,$83,$66,$3d,$e4,$61,$e2,$97,$91
      db $6e,$d6,$0d,$64,$b8,$88,$0b,$3e,$25,$7a,$f3,$6e,$d7,$b4,$ff,$b0,$b1,$db,$b7,$ef,$be,$15,$42,$43,$35
      db $5b,$c2,$14,$0c,$89,$30,$91,$c6,$4d,$cd,$c5,$4f,$46,$05,$d9,$bd,$e0,$40,$33,$29,$ec,$4d,$a2,$0f,$dc
      db $75,$b8,$2f,$ab,$09,$6b,$6a,$b8,$8f,$20,$2c,$e6,$68,$d8,$e7,$f8,$87,$b1,$ea,$3e,$73,$8f,$30,$48,$03
      db $ee,$25,$0f,$b5,$88,$35,$c7,$ac,$c7,$a3,$3d,$89,$1d,$a4,$e3,$f5,$66,$c2,$a2,$10,$95,$d9,$cc,$a9,$44
      db $67,$a5,$24,$c8,$5e,$95,$63,$87,$54,$7d,$84,$53,$ad,$12,$c2,$71,$a1,$f3,$59,$fe,$41,$78,$da,$d0,$3a
      db $20,$f2,$a2,$e7,$dc,$3f,$1f,$c6,$ce,$27,$fe,$b6,$19,$9c,$d5,$66,$39,$16,$4f,$4d,$c8,$b3,$ed,$5b,$b8
      db $e5,$10,$3e,$61,$ef,$75,$96,$b8,$43,$94,$95,$1a,$2e,$47,$61,$c7,$b2,$c1,$5e,$33,$f0,$e2,$3e,$04,$ec
      db $7d,$b9,$db,$9a,$8f,$9d,$45,$ec,$74,$36,$5f,$e6,$56,$20,$f7,$c3,$72,$b1,$57,$fb,$1a,$bd,$72,$8d,$83
      db $81,$db,$d2,$9d,$dc,$98,$ae,$55,$cf,$3c,$81,$9f,$b4,$c9,$02,$5c,$ee,$d5,$aa,$9a,$a3,$54,$35,$a6,$44
      db $82,$43,$1d,$6b,$12,$4d,$8c,$0e,$2d,$56,$c2,$54,$d1,$6f,$a2,$67,$c7,$22,$b4,$32,$27,$1a,$ff,$f0,$9d
      db $6c,$f2,$55,$04,$c1,$75,$7a,$c8,$50,$a8,$35,$7d,$ec,$d9,$61,$55,$f0,$3c,$d7,$58,$76,$d2,$17,$92,$18
      db $26,$db,$3b,$4d,$fa,$cb,$fb,$c9,$75,$85,$ce,$84,$f2,$06,$9f,$cb,$d9,$ab,$4f,$cd,$ae,$f3,$9c,$ba,$47
      db $43,$f8,$b8,$de,$dd,$3a,$8b,$39,$74,$ec,$51,$f9,$43,$fc,$52,$ef,$22,$02,$81,$2f,$ef,$39,$a6,$fe,$85
      db $9c,$cb,$53,$79,$76,$1c,$bd,$dd,$e2,$f0,$f8,$da,$f9,$30,$e5,$a0,$42,$86,$82,$d6,$1b,$77,$7c,$81,$dd
      db $0e,$61,$a1,$f3,$18,$de,$8e,$f8,$56,$5b,$fe,$1b,$71,$90,$da,$79,$eb,$95,$dd,$14,$d2,$4e,$0f,$a2,$94
      db $8a,$5a,$f4,$0c,$75,$2e,$a8,$f0,$df,$84,$d9,$01,$ab,$f4,$90,$16,$70,$67,$df,$c1,$67,$19,$70,$2d,$e9
      db $a3,$94,$ee,$7b,$37,$b6,$c5,$e9,$cb,$d4,$42,$46,$80,$30,$11,$d1,$7e,$5c,$fa,$5a,$ab,$cb,$ec,$1e,$6f
      db $44,$b8,$5a,$cb,$f0,$c3,$ec,$8a,$0a,$6e,$a0,$70,$90,$34,$e0,$fa,$4c,$4f,$27,$0f,$ca,$5a,$ff,$ff,$d1
      db $54,$5a,$22,$fd,$64,$2e,$a8,$74,$dd,$5d,$a9,$4a,$d1,$2f,$0a,$c9,$72,$78,$0e,$85,$2b,$c0,$7d,$21,$72
      db $0f,$6a,$46,$d9,$4b,$e8,$0c,$a7,$28,$57,$0b,$c8,$98,$72,$de,$8f,$67,$f1,$70,$88,$e8,$77,$8b,$f9,$b3
      db $6d,$72,$98,$ad,$3c,$62,$88,$31,$38,$b8,$c6,$f8,$e5,$45,$fb,$cb,$78,$30,$9b,$9a,$b3,$08,$d3,$71,$19
      db $6a,$1d,$27,$d3,$03,$b3,$af,$12,$c0,$66,$db,$be,$69,$92,$5f,$e4,$68,$13,$c8,$2d,$6f,$c6,$9b,$1e,$e3
      db $0c,$2e,$d0,$f0,$14,$56,$9b,$cd,$74,$e5,$f7,$0c,$6c,$72,$06,$f0,$09,$be,$63,$d5,$54,$e5,$88,$07,$7d
      db $ac,$9c,$bd,$ef,$4e,$d7,$f8,$57,$ec,$2a,$d3,$07,$de,$71,$60,$fd,$62,$36,$17,$80,$e8,$99,$53,$9a,$86
      db $ab,$e3,$1e,$c7,$77,$5f,$d6,$50,$46,$80,$28,$7e,$1e,$40,$40,$ef,$20,$89,$22,$f3,$a5,$35,$25,$f6,$ac
      db $55,$9f,$83,$af,$a5,$28,$af,$b8,$c5,$aa,$90,$05,$fa,$68,$cf,$9d,$fc,$dc,$de,$00,$82,$79,$ff,$7c,$97
      db $92,$93,$7b,$8f,$0e,$ab,$d2,$e6,$ab,$45,$35,$d5,$8f,$ba,$35,$31,$bc,$87,$6d,$b0,$bf,$45,$54,$69,$e6
      db $5b,$0f,$4f,$15,$07,$bf,$48,$6e,$b2,$16,$7a,$82,$c9,$2c,$fe,$25,$1c,$cd,$7a,$b9,$26,$7a,$7c,$9b,$84
      db $95,$9b,$57,$eb,$24,$5b,$af,$8b,$4f,$a7,$75,$e7,$c9,$88,$bb,$af,$e3,$62,$15,$6a,$3e,$ef,$aa,$04,$ea
      db $d5,$0c,$b7,$75,$73,$bd,$f2,$b6,$32,$4a,$88,$15,$7a,$9b,$ad,$6d,$be,$e7,$24,$12,$eb,$da,$4e,$ac,$53
      db $6a,$ed,$d9,$c8,$92,$6e,$6f,$ae,$18,$90,$b3,$80,$8c,$d5,$5e,$44,$9f,$aa,$07,$54,$dd,$df,$1f,$6e,$d2
      db $1e,$a0,$67,$d6,$d0,$13,$ee,$b9,$91,$31,$f0,$bc,$f6,$2f,$16,$31,$df,$82,$7c,$03,$22,$4d,$22,$35,$79
      db $07,$68,$d2,$b8,$8d,$5a,$b9,$7b,$46,$b3,$b0,$d4,$fb,$3b,$4d,$57,$45,$bc,$be,$6b,$85,$20,$91,$56,$cb
      db $da,$64,$bb,$df,$02,$fc,$0c,$f9,$64,$f8,$3e,$13,$22,$66,$f5,$bd,$df,$80,$65,$9a,$e3,$d3,$a5,$81,$1f
      db $b9,$f4,$64,$4d,$11,$79,$84,$51,$d2,$94,$bd,$4b,$45,$1b,$94,$28,$38,$3a,$fe,$6a,$97,$75,$2f,$3c,$d7
      db $c0,$00,$d1,$c6,$68,$48,$cd,$22,$43,$4b,$20,$f6,$a9,$3d,$5a,$07,$38,$37,$27,$62,$e9,$b7,$ae,$c3,$e3
      db $40,$ea,$b8,$2b,$87,$c2,$d1,$7b,$46,$1d,$86,$3a,$c6,$3e,$30,$07,$0c,$cb,$f5,$98,$05,$a4,$90,$7c,$77
      db $26,$e5,$5a,$3a,$68,$49,$3d,$0a,$3f,$09,$77,$ed,$77,$90,$b1,$fe,$5b,$98,$30,$20,$c3,$20,$ed,$d9,$e1
      db $fd,$d7,$5d,$60,$14,$17,$67,$c5,$71,$ed,$68,$62,$20,$db,$c1,$81,$00,$53,$39,$a3,$26,$62,$46,$06,$42
      db $b0,$5e,$87,$bc,$58,$50,$ee,$74,$22,$59,$c2,$9f,$fb,$82,$f5,$31,$12,$04,$3c,$9b,$45,$89,$c1,$b5,$7b
      db $31,$34,$29,$2d,$ca,$7a,$a0,$0c,$c5,$01,$5a,$25,$43,$1e,$55,$11,$31,$ea,$55,$29,$58,$0c,$69,$4d,$ef
      db $b9,$57,$23,$3d,$76,$c4,$24,$da,$0d,$97,$ca,$b3,$c4,$cc,$4b,$5b,$47,$ee,$c0,$dc,$4d,$72,$8f,$f3,$10
      db $8c,$49,$e2,$0f,$aa,$08,$03,$5e,$d3,$cf,$41,$22,$a2,$c1,$d9,$95,$2d,$45,$0b,$26,$7e,$64,$95,$9c,$41
      db $4a,$74,$41,$4e,$c4,$01,$94,$ab,$1c,$6e,$2a,$68,$0b,$4d,$1a,$37,$52,$91,$1d,$59,$af,$9a,$8d,$3a,$d5
      db $40,$4d,$a3,$0d,$88,$ff,$4d,$d6,$78,$20,$f6,$39,$7e,$5f,$a0,$36,$c9,$a9,$41,$07,$bb,$fc,$01,$d5,$77
      db $a5,$61,$9b,$d5,$55,$4d,$28,$f7,$a5,$74,$5d,$98,$86,$54,$4b,$06,$ec,$5c,$02,$76,$0e,$dc,$a7,$35,$14
      db $71,$25,$67,$7b,$73,$c6,$b2,$a1,$3f,$d7,$a3,$46,$a8,$54,$14,$8d,$d8,$bf,$38,$21,$ff,$98,$dc,$18,$52
      db $7d,$af,$90,$63,$e9,$c7,$90,$28,$8d,$44,$84,$3a,$25,$88,$db,$ef,$c9,$42,$0c,$20,$76,$62,$90,$62,$27
      db $41,$30,$3d,$c8,$16,$66,$74,$b0,$57,$d4,$f7,$1c,$cf,$02,$1f,$85,$b5,$7d,$4e,$16,$cb,$38,$d4,$c5,$24
      db $a0,$f1,$9a,$4e,$f4,$85,$ec,$de,$39,$b2,$59,$4c,$2d,$3a,$c7,$28,$b6,$b6,$d9,$53,$94,$62,$36,$6a,$71
      db $a8,$18,$c5,$3a,$72,$f1,$fd,$94,$bc,$65,$90,$33,$ea,$11,$31,$f8,$37,$1d,$7d,$f6,$9b,$79,$f9,$72,$9b
      db $bb,$94,$8e,$97,$e9,$00,$c9,$e6,$f3,$32,$b6,$61,$9b,$2a,$73,$d0,$bd,$12,$60,$89,$b0,$ad,$93,$f1,$96
      db $a4,$f6,$3a,$b9,$3f,$ab,$4c,$64,$61,$08,$45,$b1,$6d,$ca,$88,$13,$65,$f5,$c1,$32,$d8,$75,$7b,$ad,$65
      db $39,$ec,$28,$30,$a7,$f3,$09,$e9,$91,$17,$57,$39,$e1,$4e,$fb,$d9,$8d,$7e,$89,$be,$50,$35,$90,$f1,$33
      db $0c,$0e,$58,$5c,$54,$98,$76,$87,$4a,$38,$ff,$5a,$03,$0f,$5a,$89,$42,$56,$d8,$c1,$11,$50,$89,$3c,$df
      db $3d,$7d,$a1,$8a,$91,$5b,$40,$64,$d5,$c6,$ff,$ed,$dc,$27,$d0,$a1,$d1,$0f,$11,$a3,$44,$27,$ba,$e7,$90
      db $45,$13,$df,$cc,$02,$7c,$a8,$93,$0a,$a3,$a6,$2b,$f3,$f5,$37,$5a,$0b,$fb,$b1,$22,$4f,$01,$57,$a6,$23
      db $ba,$a8,$2c,$b0,$23,$fe,$e3,$08,$5d,$04,$84,$3f,$9e,$2e,$1c,$85,$6e,$6b,$29,$ef,$5a,$fd,$bb,$71,$27
      db $01,$54,$65,$6f,$d0,$94,$16,$19,$e6,$81,$b8,$68,$75,$35,$95,$ec,$6e,$6f,$e7,$71,$96,$42,$f6,$0b,$44
      db $e9,$3b,$d3,$0e,$cf,$88,$11,$c4,$6d,$dc,$43,$b2,$5f,$7a,$ed,$84,$85,$93,$58,$88,$48,$2b,$90,$e8,$ae
      db $61,$9f,$3d,$be,$59,$60,$fa,$44,$cb,$15,$b5,$55,$ca,$55,$c5,$1c,$ee,$22,$25,$96,$a5,$60,$e0,$46,$04
      db $b1,$64,$3c,$06,$83,$51,$7c,$b9,$28,$8d,$e3,$7f,$a5,$af,$cd,$b3,$f6,$c4,$f4,$95,$d3,$a6,$ed,$00,$7a
      db $29,$c0,$cf,$86,$04,$60,$59,$60,$61,$7a,$ae,$fb,$29,$7c,$5d,$72,$e3,$11,$a4,$65,$75,$63,$0b,$d9,$b2
      db $74,$6f,$e8,$c4,$34,$36,$b0,$e8,$76,$78,$f2,$1f,$08,$0e,$c1,$64,$08,$75,$52,$5b,$76,$3c,$9f,$bb,$5c
      db $2d,$1d,$ba,$0e,$4f,$af,$50,$25,$95,$5d,$b5,$fa,$99,$49,$c9,$bb,$df,$9b,$f4,$66,$00,$81,$40,$de,$b1
      db $29,$a3,$85,$d4,$88,$37,$c2,$14,$81,$94,$c9,$70,$a1,$23,$ed,$86,$bf,$17,$74,$41,$4c,$38,$4b,$37,$55
      db $4a,$6c,$19,$2c,$e2,$ab,$77,$f8,$82,$ab,$58,$41,$c0,$c0,$d5,$d9,$91,$56,$96,$a0,$f8,$0a,$ee,$ce,$c2
      db $80,$84,$65,$0a,$3b,$49,$4a,$40,$83,$24,$5d,$2d,$30,$bd,$1b,$da,$8b,$3e,$50,$6f,$27,$c0,$76,$17,$58
      db $a3,$89,$da,$ba,$6b,$45,$d3,$73,$54,$e9,$ad,$f1,$2f,$69,$36,$c8,$9a,$9f,$b5,$8c,$27,$be,$3b,$00,$82
      db $5d,$54,$f0,$fe,$23,$39,$b8,$ff,$91,$6f,$1e,$47,$53,$8c,$55,$3f,$08,$91,$1e,$fe,$f9,$af,$e2,$0b,$b5
      db $18,$e8,$7f,$39,$a0,$f2,$3b,$bd,$f0,$ac,$e8,$ad,$d1,$8a,$2e,$bd,$c2,$99,$cb,$0d,$9b,$ba,$74,$ff,$77
      db $4d,$17,$60,$f4,$0a,$d5,$c2,$e7,$97,$16,$22,$49,$92,$b4,$ea,$dc,$38,$ec,$7a,$27,$c8,$1d,$af,$70,$cb
      db $1e,$6f,$44,$cd,$65,$60,$34,$6f,$d0,$07,$f5,$d5,$1c,$36,$8b,$6c,$c5,$c2,$dc,$5f,$e5,$55,$8e,$9b,$67
      db $70,$24,$e5,$38,$91,$e3,$fd,$a4,$89,$da,$43,$0c,$9d,$96,$15,$74,$ba,$7f,$a8,$75,$00,$2e,$a8,$44,$7c
      db $79,$41,$bc,$e5,$e7,$e2,$06,$98,$79,$85,$a3,$8c,$07,$0c,$8c,$40,$22,$cd,$62,$cc,$59,$34,$73,$23,$50
      db $a2,$2d,$ec,$29,$ac,$5e,$26,$16,$15,$56,$1d,$3a,$d5,$dc,$02,$06,$79,$03,$51,$aa,$9b,$ff,$20,$6d,$75
      db $b0,$70,$5b,$eb,$6a,$de,$ad,$6e,$d3,$96,$bc,$33,$40,$a2,$c0,$78,$93,$e5,$aa,$6a,$e8,$9a,$43,$7e,$22
      db $89,$ae,$21,$c1,$b9,$a8,$82,$a1,$21,$b6,$9e,$9f,$11,$b5,$72,$96,$dc,$35,$0b,$7e,$09,$67,$c7,$52,$52
      db $02,$40,$df,$1b,$4d,$6d,$5b,$05,$fa,$22,$78,$ce,$99,$0f,$47,$49,$cc,$38,$78,$43,$d6,$59,$32,$f1,$2c
      db $21,$11,$5f,$06,$1e,$fa,$2f,$f9,$25,$42,$f3,$f9,$55,$f0,$89,$15,$f8,$4d,$08,$a9,$04,$0b,$c3,$da,$e5
      db $a5,$4f,$fa,$5b,$70,$0c,$db,$c0,$43,$0e,$0a,$1b,$45,$f1,$7d,$19,$36,$d6,$fd,$ef,$c1,$b3,$70,$46,$22
      db $22,$cf,$6f,$50,$44,$de,$f0,$7f,$63,$1e,$ae,$3b,$f9,$65,$b6,$15,$5b,$c9,$08,$18,$0b,$bd,$47,$fa,$5a
      db $b0,$84,$2d,$4a,$0f,$05,$c5,$42,$7a,$e5,$a7,$27,$84,$1a,$ad,$37,$0e,$59,$bb,$cf,$57,$75,$21,$19,$30
      db $24,$b6,$c4,$d9,$cd,$23,$07,$30,$8f,$03,$99,$d3,$ce,$b7,$1f,$3b,$dd,$fa,$3f,$9e,$23,$62,$e7,$7c,$cf
      db $49,$13,$d2,$27,$c9,$9a,$25,$70,$ad,$64,$ab,$66,$3d,$a5,$7e,$65,$5d,$9e,$f5,$a8,$96,$c8,$d3,$82,$a3
      db $47,$6f,$c2,$0c,$63,$01,$bd,$e1,$83,$22,$67,$c9,$bf,$2b,$b3,$22,$2a,$7c,$b4,$68,$93,$da,$83,$47,$47
      db $ed,$e5,$11,$1a,$b3,$8e,$78,$ca,$dc,$b0,$3f,$01,$2d,$25,$ab,$29,$42,$0d,$18,$d3,$5c,$f6,$40,$ee,$9a
      db $d2,$10,$07,$69,$7d,$be,$a8,$93,$b6,$43,$8e,$b6,$fb,$39,$39,$2a,$21,$e3,$f4,$19,$ee,$80,$55,$80,$01
      db $cb,$a7,$ac,$f6,$9f,$26,$a5,$d6,$34,$02,$88,$cd,$d2,$37,$07,$3f,$42,$62,$ed,$c6,$e5,$9b,$80,$b7,$e1
      db $4f,$7c,$f4,$44,$22,$23,$f2,$93,$5a,$ac,$55,$c0,$b8,$93,$b9,$59,$eb,$c7,$b4,$4e,$e7,$60,$ce,$e3,$35
      db $74,$28,$37,$69,$b2,$aa,$c4,$32,$a2,$d7,$b9,$f3,$df,$cb,$1b,$12,$47,$1e,$4b,$ec,$3b,$3b,$b5,$fe,$f8
      db $50,$83,$50,$30,$11,$24,$5f,$de,$74,$3b,$5c,$1e,$06,$63,$15,$c0,$d9,$2c,$42,$f1,$50,$64,$c8,$98,$ef
      db $d0,$17,$67,$f0,$69,$dd,$09,$6d,$18,$ac,$cc,$8a,$a7,$ed,$38,$f6,$43,$97,$4e,$f0,$cb,$02,$8e,$7b,$3b
      db $51,$ce,$cb,$d5,$4c,$f7,$7c,$0e,$98,$cd,$c1,$11,$72,$02,$47,$32,$c4,$58,$d9,$6b,$cf,$7c,$3d,$8d,$78
      db $b7,$db,$13,$1e,$d6,$f1,$c4,$eb,$c4,$ef,$9e,$03,$10,$57,$5e,$0b,$5c,$a0,$8e,$81,$9f,$84,$91,$7b,$5b
      db $04,$90,$99,$bf,$76,$65,$98,$6b,$56,$88,$29,$36,$73,$14,$e2,$17,$47,$2f,$a5,$bb,$aa,$06,$7d,$28,$1e
    @@skip:
    mov ax,offset @@table
    mov word ptr [L10Bytes],ax
    mov ax,cs
    mov word ptr [L10Bytes+2],ax
    {$ifdef debug}
      mov ax,offset @@skip
      mov word ptr [PSkip],ax
      mov ax,cs
      mov word ptr [PSkip+2],ax
    {$endif}
  end;
  AddrLn10Bytes := L10Bytes;
  {$ifdef debug}
    assert(MAX_TCBITS = 8*(longint(PSkip)-longint(L10Bytes)), 'Length ln10 table');
  {$endif}
end;


{---------------------------------------------------------------------------}
function AddrEBytes: pointer;
  {-Pointer to bits of exp(1), big endian format}
var
  EBytes: pointer;
{$ifdef debug}
  PSkip: pointer;
{$endif}
begin
  asm
    jmp @@skip
    @@table:
      {calculated with t_ciexp1}
      db $ad,$f8,$54,$58,$a2,$bb,$4a,$9a,$af,$dc,$56,$20,$27,$3d,$3c,$f1,$d8,$b9,$c5,$83,$ce,$2d,$36,$95,$a9
      db $e1,$36,$41,$14,$64,$33,$fb,$cc,$93,$9d,$ce,$24,$9b,$3e,$f9,$7d,$2f,$e3,$63,$63,$0c,$75,$d8,$f6,$81
      db $b2,$02,$ae,$c4,$61,$7a,$d3,$df,$1e,$d5,$d5,$fd,$65,$61,$24,$33,$f5,$1f,$5f,$06,$6e,$d0,$85,$63,$65
      db $55,$3d,$ed,$1a,$f3,$b5,$57,$13,$5e,$7f,$57,$c9,$35,$98,$4f,$0c,$70,$e0,$e6,$8b,$77,$e2,$a6,$89,$da
      db $f3,$ef,$e8,$72,$1d,$f1,$58,$a1,$36,$ad,$e7,$35,$30,$ac,$ca,$4f,$48,$3a,$79,$7a,$bc,$0a,$b1,$82,$b3
      db $24,$fb,$61,$d1,$08,$a9,$4b,$b2,$c8,$e3,$fb,$b9,$6a,$da,$b7,$60,$d7,$f4,$68,$1d,$4f,$42,$a3,$de,$39
      db $4d,$f4,$ae,$56,$ed,$e7,$63,$72,$bb,$19,$0b,$07,$a7,$c8,$ee,$0a,$6d,$70,$9e,$02,$fc,$e1,$cd,$f7,$e2
      db $ec,$c0,$34,$04,$cd,$28,$34,$2f,$61,$91,$72,$fe,$9c,$e9,$85,$83,$ff,$8e,$4f,$12,$32,$ee,$f2,$81,$83
      db $c3,$fe,$3b,$1b,$4c,$6f,$ad,$73,$3b,$b5,$fc,$bc,$2e,$c2,$20,$05,$c5,$8e,$f1,$83,$7d,$16,$83,$b2,$c6
      db $f3,$4a,$26,$c1,$b2,$ef,$fa,$88,$6b,$42,$38,$61,$1f,$cf,$dc,$de,$35,$5b,$3b,$65,$19,$03,$5b,$bc,$34
      db $f4,$de,$f9,$9c,$02,$38,$61,$b4,$6f,$c9,$d6,$e6,$c9,$07,$7a,$d9,$1d,$26,$91,$f7,$f7,$ee,$59,$8c,$b0
      db $fa,$c1,$86,$d9,$1c,$ae,$fe,$13,$09,$85,$13,$92,$70,$b4,$13,$0c,$93,$bc,$43,$79,$44,$f4,$fd,$44,$52
      db $e2,$d7,$4d,$d3,$64,$f2,$e2,$1e,$71,$f5,$4b,$ff,$5c,$ae,$82,$ab,$9c,$9d,$f6,$9e,$e8,$6d,$2b,$c5,$22
      db $36,$3a,$0d,$ab,$c5,$21,$97,$9b,$0d,$ea,$da,$1d,$bf,$9a,$42,$d5,$c4,$48,$4e,$0a,$bc,$d0,$6b,$fa,$53
      db $dd,$ef,$3c,$1b,$20,$ee,$3f,$d5,$9d,$7c,$25,$e4,$1d,$2b,$66,$9e,$1e,$f1,$6e,$6f,$52,$c3,$16,$4d,$f4
      db $fb,$79,$30,$e9,$e4,$e5,$88,$57,$b6,$ac,$7d,$5f,$42,$d6,$9f,$6d,$18,$77,$63,$cf,$1d,$55,$03,$40,$04
      db $87,$f5,$5b,$a5,$7e,$31,$cc,$7a,$71,$35,$c8,$86,$ef,$b4,$31,$8a,$ed,$6a,$1e,$01,$2d,$9e,$68,$32,$a9
      db $07,$60,$0a,$91,$81,$30,$c4,$6d,$c7,$78,$f9,$71,$ad,$00,$38,$09,$29,$99,$a3,$33,$cb,$8b,$7a,$1a,$1d
      db $b9,$3d,$71,$40,$00,$3c,$2a,$4e,$ce,$a9,$f9,$8d,$0a,$cc,$0a,$82,$91,$cd,$ce,$c9,$7d,$cf,$8e,$c9,$b5
      db $5a,$7f,$88,$a4,$6b,$4d,$b5,$a8,$51,$f4,$41,$82,$e1,$c6,$8a,$00,$7e,$5e,$0d,$d9,$02,$0b,$fd,$64,$b6
      db $45,$03,$6c,$7a,$4e,$67,$7d,$2c,$38,$53,$2a,$3a,$23,$ba,$44,$42,$ca,$f5,$3e,$a6,$3b,$b4,$54,$32,$9b
      db $76,$24,$c8,$91,$7b,$dd,$64,$b1,$c0,$fd,$4c,$b3,$8e,$8c,$33,$4c,$70,$1c,$3a,$cd,$ad,$06,$57,$fc,$cf
      db $ec,$71,$9b,$1f,$5c,$3e,$4e,$46,$04,$1f,$38,$81,$47,$fb,$4c,$fd,$b4,$77,$a5,$24,$71,$f7,$a9,$a9,$69
      db $10,$b8,$55,$32,$2e,$db,$63,$40,$d8,$a0,$0e,$f0,$92,$35,$05,$11,$e3,$0a,$be,$c1,$ff,$f9,$e3,$a2,$6e
      db $7f,$b2,$9f,$8c,$18,$30,$23,$c3,$58,$7e,$38,$da,$00,$77,$d9,$b4,$76,$3e,$4e,$4b,$94,$b2,$bb,$c1,$94
      db $c6,$65,$1e,$77,$ca,$f9,$92,$ee,$aa,$c0,$23,$2a,$28,$1b,$f6,$b3,$a7,$39,$c1,$22,$61,$16,$82,$0a,$e8
      db $db,$58,$47,$a6,$7c,$be,$f9,$c9,$09,$1b,$46,$2d,$53,$8c,$d7,$2b,$03,$74,$6a,$e7,$7f,$5e,$62,$29,$2c
      db $31,$15,$62,$a8,$46,$50,$5d,$c8,$2d,$b8,$54,$33,$8a,$e4,$9f,$52,$35,$c9,$5b,$91,$17,$8c,$cf,$2d,$d5
      db $ca,$ce,$f4,$03,$ec,$9d,$18,$10,$c6,$27,$2b,$04,$5b,$3b,$71,$f9,$dc,$6b,$80,$d6,$3f,$dd,$4a,$8e,$9a
      db $db,$1e,$69,$62,$a6,$95,$26,$d4,$31,$61,$c1,$a4,$1d,$57,$0d,$79,$38,$da,$d4,$a4,$0e,$32,$9c,$cf,$f4
      db $6a,$aa,$36,$ad,$00,$4c,$f6,$00,$c8,$38,$1e,$42,$5a,$31,$d9,$51,$ae,$64,$fd,$b2,$3f,$ce,$c9,$50,$9d
      db $43,$68,$7f,$eb,$69,$ed,$d1,$cc,$5e,$0b,$8c,$c3,$bd,$f6,$4b,$10,$ef,$86,$b6,$31,$42,$a3,$ab,$88,$29
      db $55,$5b,$2f,$74,$7c,$93,$26,$65,$cb,$2c,$0f,$1c,$c0,$1b,$d7,$02,$29,$38,$88,$39,$d2,$af,$05,$e4,$54
      db $50,$4a,$c7,$8b,$75,$82,$82,$28,$46,$c0,$ba,$35,$c3,$5f,$5c,$59,$16,$0c,$c0,$46,$fd,$82,$51,$54,$1f
      db $c6,$8c,$9c,$86,$b0,$22,$bb,$70,$99,$87,$6a,$46,$0e,$74,$51,$a8,$a9,$31,$09,$70,$3f,$ee,$1c,$21,$7e
      db $6c,$38,$26,$e5,$2c,$51,$aa,$69,$1e,$0e,$42,$3c,$fc,$99,$e9,$e3,$16,$50,$c1,$21,$7b,$62,$48,$16,$cd
      db $ad,$9a,$95,$f9,$d5,$b8,$01,$94,$88,$d9,$c0,$a0,$a1,$fe,$30,$75,$a5,$77,$e2,$31,$83,$f8,$1d,$4a,$3f
      db $2f,$a4,$57,$1e,$fc,$8c,$e0,$ba,$8a,$4f,$e8,$b6,$85,$5d,$fe,$72,$b0,$a6,$6e,$de,$d2,$fb,$ab,$fb,$e5
      db $8a,$30,$fa,$fa,$be,$1c,$5d,$71,$a8,$7e,$2f,$74,$1e,$f8,$c1,$fe,$86,$fe,$a6,$bb,$fd,$e5,$30,$67,$7f
      db $0d,$97,$d1,$1d,$49,$f7,$a8,$44,$3d,$08,$22,$e5,$06,$a9,$f4,$61,$4e,$01,$1e,$2a,$94,$83,$8f,$f8,$8c
      db $d6,$8c,$8b,$b7,$c5,$1e,$ef,$6d,$49,$ea,$8a,$b4,$f2,$c3,$df,$5b,$b4,$e0,$73,$5a,$b0,$d6,$87,$49,$2f
      db $e2,$6d,$d4,$06,$58,$16,$bb,$a7,$7e,$ae,$97,$3f,$3b,$40,$ce,$e8,$84,$0a,$82,$f6,$f8,$ed,$27,$5c,$9c
      db $be,$27,$82,$74,$02,$39,$75,$6f,$66,$48,$e4,$d8,$a1,$87,$ad,$09,$8a,$5f,$16,$10,$4e,$5d,$45,$51,$cf
      db $3c,$a8,$f5,$b7,$96,$31,$2e,$c6,$e4,$6b,$65,$ea,$fc,$0a,$3a,$59,$97,$69,$3b,$3a,$03,$77,$04,$f8,$37
      db $d0,$c8,$bb,$68,$3f,$4e,$26,$c5,$d7,$a3,$44,$34,$23,$14,$8c,$29,$ad,$5d,$b2,$53,$f1,$4a,$d3,$9d,$2c
      db $b8,$d0,$83,$a4,$0b,$20,$bb,$9e,$ba,$e0,$17,$0b,$ad,$d9,$45,$fa,$af,$fe,$e1,$dd,$cd,$e3,$0e,$01,$b2
      db $47,$41,$fc,$42,$75,$9d,$49,$f0,$61,$5a,$0d,$12,$05,$53,$5a,$48,$58,$41,$d5,$6f,$bc,$c2,$e0,$09,$f9
      db $7a,$2c,$13,$96,$4b,$a2,$cb,$a8,$d5,$28,$98,$7a,$ae,$d2,$14,$59,$1c,$b5,$1a,$69,$9a,$09,$3c,$93,$51
      db $2c,$57,$38,$50,$bf,$2d,$94,$c1,$fc,$87,$36,$ae,$f5,$a1,$63,$31,$eb,$8f,$3b,$81,$ab,$99,$2f,$b6,$46
      db $81,$04,$b9,$bc,$ce,$e8,$13,$ea,$d5,$ee,$39,$48,$59,$72,$a0,$74,$31,$f4,$46,$0e,$17,$5b,$48,$c2,$91
      db $f8,$7a,$2d,$4e,$bc,$36,$69,$ea,$b0,$94,$9e,$d1,$b1,$00,$9e,$10,$6a,$8e,$16,$90,$31,$b1,$45,$eb,$e3
      db $b4,$9a,$3d,$1a,$66,$43,$5b,$6b,$8a,$da,$3d,$8b,$f7,$6d,$bc,$c0,$e9,$d6,$62,$fa,$35,$6d,$c6,$8e,$a0
      db $0d,$c4,$b1,$f9,$ae,$c6,$f2,$f0,$32,$1e,$fa,$96,$a3,$b8,$fa,$38,$7a,$39,$ba,$ea,$67,$01,$1f,$6d,$7d
      db $b8,$4d,$c9,$e5,$b7,$09,$8d,$39,$59,$54,$b6,$03,$17,$ee,$be,$09,$aa,$70,$f7,$0b,$06,$7a,$65,$a7,$89
      db $d9,$6f,$e2,$07,$ba,$a2,$fd,$9c,$14,$07,$da,$8e,$73,$25,$79,$4d,$64,$45,$fe,$7f,$b0,$2b,$7e,$de,$e5
      db $65,$4a,$6f,$4b,$1e,$f9,$76,$d9,$5c,$1f,$13,$29,$37,$a1,$a4,$24,$bc,$00,$a7,$67,$8d,$49,$1b,$d2,$54
      db $e9,$f8,$1d,$79,$46,$02,$1a,$3e,$96,$f5,$82,$8c,$a2,$cf,$fa,$94,$c2,$13,$45,$d3,$27,$7a,$32,$3d,$a8
      db $65,$f5,$98,$d6,$6d,$a6,$88,$a4,$b4,$97,$c4,$bc,$6c,$c8,$8e,$4a,$c4,$0d,$6a,$05,$62,$84,$3e,$55,$83
      db $fc,$64,$9f,$89,$a6,$fc,$a9,$b8,$59,$12,$95,$90,$f2,$9c,$2c,$bd,$98,$78,$a8,$f2,$84,$d3,$94,$57,$a0
      db $71,$8b,$44,$a6,$e8,$ed,$b5,$9b,$38,$ee,$b6,$55,$0d,$b0,$57,$f3,$9b,$b3,$35,$4a,$ab,$49,$c4,$11,$f7
      db $bd,$56,$c2,$10,$cd,$52,$47,$e1,$96,$64,$31,$1c,$59,$98,$46,$bb,$77,$37,$12,$e0,$1e,$84,$6f,$82,$13
      db $61,$6c,$3b,$ea,$02,$f4,$c7,$c2,$80,$87,$91,$60,$09,$60,$b4,$7d,$81,$6c,$6b,$8f,$8c,$9d,$2b,$90,$94
      db $d7,$68,$9d,$3d,$ff,$b5,$f1,$68,$e1,$a7,$33,$6f,$29,$63,$15,$37,$49,$26,$1a,$96,$4f,$c6,$2e,$e3,$10
      db $8a,$47,$9e,$59,$32,$64,$f4,$53,$2e,$b0,$d8,$ea,$00,$68,$81,$23,$53,$8f,$1b,$e3,$b3,$e9,$d7,$f3,$34
      db $02,$69,$92,$6b,$d7,$b0,$83,$13,$5f,$b5,$e1,$0d,$ed,$1a,$da,$ab,$d6,$e7,$6d,$2b,$7d,$00,$b3,$d9,$01
      db $4c,$04,$63,$a6,$14,$af,$90,$80,$fc,$8b,$2e,$da,$37,$fc,$af,$ce,$29,$ba,$b8,$2d,$b4,$39,$ee,$ff,$93
      db $74,$e5,$f5,$09,$00,$7d,$85,$06,$6c,$7c,$e3,$00,$e2,$f4,$92,$eb,$4b,$cc,$1c,$70,$b8,$51,$1c,$87,$fd
      db $ca,$19,$94,$c3,$86,$f1,$fd,$b7,$27,$d8,$85,$f8,$96,$16,$b0,$ff,$df,$23,$ec,$7a,$1a,$08,$cf,$58,$b7
      db $a1,$79,$b7,$dc,$ef,$f4,$72,$07,$91,$59,$b3,$a5,$8d,$65,$40,$8b,$0e,$e4,$41,$59,$0b,$c1,$ca,$50,$e5
      db $10,$6f,$36,$60,$fa,$97,$c6,$38,$6f,$33,$53,$b3,$15,$b2,$ad,$0e,$43,$7d,$85,$56,$c8,$6f,$62,$ad,$f1
      db $cb,$c0,$0d,$88,$6d,$34,$60,$0d,$eb,$39,$11,$bb,$04,$83,$94,$85,$34,$61,$8b,$ac,$17,$1a,$ed,$5f,$7b
      db $5e,$26,$c7,$66,$06,$8c,$6e,$5a,$8c,$ec,$d4,$83,$9d,$99,$17,$73,$9b,$15,$07,$82,$65,$6d,$ec,$56,$b6
      db $00,$23,$1f,$5a,$e7,$24,$25,$7c,$15,$8f,$bb,$fe,$c0,$77,$9b,$13,$ff,$6a,$36,$f5,$90,$ff,$40,$ff,$d7
      db $17,$17,$42,$be,$d1,$f8,$ce,$d0,$73,$26,$c0,$53,$6d,$49,$b2,$b9,$3f,$55,$05,$7d,$5c,$b1,$3a,$46,$04
      db $c9,$d2,$92,$a1,$43,$87,$d8,$cd,$0e,$30,$54,$19,$68,$c4,$53,$fb,$73,$73,$ca,$cd,$93,$a0,$70,$56,$2b
      db $58,$06,$30,$89,$85,$94,$81,$df,$88,$31,$2c,$5a,$97,$81,$eb,$1e,$44,$a5,$f6,$87,$5f,$c8,$ae,$4e,$de
      db $ef,$57,$c1,$2f,$24,$a9,$d6,$17,$6b,$49,$cf,$13,$37,$9e,$f1,$ed,$9e,$cc,$a8,$e0,$82,$9e,$71,$e1,$d7
      db $45,$15,$17,$1b,$44,$7d,$2c,$2c,$d1,$be,$b7,$a3,$ba,$7b,$34,$27,$a3,$83,$78,$6f,$dc,$27,$86,$13,$2a
      db $42,$4d,$c0,$62,$13,$dd,$9f,$1c,$85,$9f,$fa,$43,$f1,$c1,$e1,$0e,$70,$6f,$95,$e5,$2e,$f3,$10,$93,$6e
      db $ab,$07,$d3,$40,$fb,$06,$cd,$f1,$f4,$4d,$b1,$26,$9d,$b0,$16,$bb,$bb,$5c,$ef,$91,$ec,$21,$c3,$96,$94
      db $44,$57,$4d,$31,$77,$2d,$d2,$a0,$40,$77,$ca,$d6,$ab,$fe,$67,$83,$c1,$cf,$af,$d8,$c7,$68,$a7,$96,$c7
      db $dd,$22,$15,$f0,$69,$f0,$7f,$ab,$98,$85,$3e,$7a,$de,$65,$48,$3a,$3c,$50,$2d,$b1,$27,$08,$c4,$19,$d0
      db $49,$ed,$a2,$47,$79,$53,$f4,$39,$17,$9a,$50,$58,$96,$00,$f4,$49,$b0,$e5,$43,$03,$6f,$dc,$ad,$8a,$6a
      db $74,$a9,$bb,$b3,$d3,$01,$3d,$37,$ae,$b2,$13,$fe,$e1,$4e,$e0,$60,$06,$cf,$67,$6f,$46,$47,$00,$12,$3c
      db $f0,$43,$06,$df,$00,$d7,$de,$79,$35,$89,$c4,$aa,$66,$71,$71,$a9,$b3,$d1,$f5,$70,$e0,$2e,$c7,$7d,$49
      db $36,$79,$df,$f1,$ee,$f7,$fb,$8f,$8f,$9d,$ea,$d7,$91,$7d,$8c,$b1,$7e,$e8,$4e,$f5,$5e,$cf,$31,$9c,$3f
      db $7e,$82,$c2,$fd,$92,$a5,$81,$24,$ab,$aa,$85,$2e,$c5,$5f,$f5,$22,$64,$8d,$a3,$27,$f4,$28,$ba,$fe,$51
      db $38,$2c,$e5,$85,$37,$2f,$d1,$4e,$be,$5d,$73,$ac,$f6,$94,$1f,$2e,$d0,$20,$c2,$08,$1d,$72,$40,$19,$cb
      db $bc,$85,$2c,$17,$19,$d1,$c3,$8e,$3e,$cc,$5d,$ab,$86,$69,$70,$57,$ee,$9b,$ce,$15,$a4,$ea,$81,$21,$36
      db $5c,$c7,$93,$49,$2d,$d1,$81,$a4,$b0,$3a,$10,$60,$e8,$d1,$c8,$6c,$5f,$16,$3c,$e0,$67,$37,$ec,$78,$02
      db $3a,$bf,$3b,$57,$e7,$54,$ed,$de,$36,$9c,$58,$50,$e2,$97,$76,$12,$cd,$83,$c1,$bc,$30,$50,$aa,$74,$2b
      db $9f,$c5,$46,$a2,$77,$5d,$c1,$89,$46,$17,$d6,$72,$4d,$d6,$36,$1c,$a8,$0e,$b5,$a5,$b8,$23,$4b,$f8,$c3
      db $40,$62,$18,$74,$6b,$27,$32,$29,$9c,$26,$5c,$c9,$24,$12,$f2,$21,$cc,$8e,$ea,$57,$23,$74,$12,$e6,$f8
      db $c4,$9d,$a6,$8c,$79,$c7,$a6,$22,$8c,$ef,$47,$23,$bc,$09,$28,$e6,$0f,$a1,$17,$7b,$f2,$bc,$00,$e9,$a2
      db $23,$77,$89,$88,$e1,$17,$d6,$1a,$fd,$37,$22,$3d,$04,$5d,$5f,$a1,$ca,$03,$15,$07,$60,$34,$40,$3a,$d1
      db $1b,$0d,$be,$f5,$32,$a8,$37,$7b,$da,$32,$bb,$aa,$ec,$c8,$a1,$96,$15,$f6,$76,$97,$36,$20,$33,$29,$12
      db $1e,$29,$0b,$2e,$79,$47,$c6,$7f,$55,$42,$ea,$8a,$59,$8c,$40,$7c,$a7,$44,$91,$32,$fe,$db,$32,$00,$7c
      db $fb,$d5,$9b,$90,$fe,$31,$ad,$bf,$d7,$d9,$30,$fa,$53,$0e,$dd,$67,$c0,$94,$58,$97,$bc,$1b,$e2,$bf,$c0
      db $12,$0d,$de,$f3,$18,$e0,$cb,$3e,$33,$3a,$9d,$05,$60,$24,$15,$ce,$e2,$25,$52,$67,$2c,$29,$b5,$33,$1a
      db $11,$d6,$dc,$44,$1d,$29,$57,$f4,$d8,$92,$81,$ad,$0e,$aa,$b7,$31,$3c,$da,$ad,$09,$0b,$33,$7b,$cd,$bf
      db $91,$f3,$15,$a0,$93,$a7,$2c,$a0,$c7,$b8,$78,$f2,$58,$e2,$e3,$7f,$ec,$de,$83,$f4,$a6,$13,$4b,$8b,$0c
      db $f4,$de,$2a,$67,$03,$f4,$3e,$fc,$ed,$91,$0c,$6b,$75,$55,$6f,$9b,$8a,$47,$e6,$fd,$d4,$e8,$40,$d5,$58
      db $1d,$bd,$9b,$2e,$87,$0e,$f3,$71,$59,$8b,$d8,$e7,$08,$9f,$52,$b4,$f9,$74,$47,$15,$58,$6c,$2a,$c0,$99
      db $12,$06,$66,$42,$3f,$19,$80,$a0,$a7,$66,$2e,$09,$61,$ac,$11,$e4,$0f,$d9,$25,$66,$df,$ab,$4d,$26,$55
      db $02,$d2,$d9,$c7,$31,$b7,$53,$be,$bb,$92,$3a,$97,$58,$ad,$ac,$28,$6d,$65,$54,$50,$71,$d5,$7c,$68,$11
      db $81,$35,$0c,$ac,$9e,$66,$d3,$bd,$1b,$a1,$14,$b6,$22,$11,$a4,$89,$11,$05,$be,$db,$4c,$6a,$10,$eb,$37
      db $d3,$3d,$e8,$7b,$70,$3d,$4b,$72,$e3,$86,$7f,$cb,$23,$e4,$b9,$00,$e5,$d9,$7a,$4a,$7e,$15,$63,$f5,$53
      db $ba,$34,$54,$4f,$57,$a9,$fb,$07,$df,$a0,$4a,$df,$dc,$ce,$1e,$81,$dd,$31,$42,$f4,$f3,$b7,$71,$44,$4d
      db $c2,$86,$0e,$06,$ba,$19,$fe,$12,$db,$ce,$5b,$ba,$94,$59,$15,$2b,$af,$b9,$2e,$c0,$34,$bf,$04,$43,$11
      db $26,$e2,$64,$75,$8e,$b6,$b7,$a5,$52,$01,$7c,$22,$7c,$4b,$ac,$51,$df,$54,$96,$e0,$dc,$ec,$d0,$d3,$0f
      db $ad,$5a,$de,$8b,$c4,$63,$f7,$33,$00,$52,$10,$36,$fb,$90,$ef,$b2,$6a,$28,$d1,$d1,$f7,$ae,$80,$27,$7a
      db $4b,$55,$a7,$00,$b3,$5c,$b1,$8b,$9c,$4d,$4f,$92,$a4,$26,$3d,$1c,$eb,$89,$0c,$a5,$2e,$fc,$66,$d4,$a1
      db $65,$3e,$e7,$31,$ea,$ef,$92,$6b,$34,$ef,$a0,$8f,$e0,$cb,$ab,$e3,$cf,$99,$04,$75,$fb,$60,$e6,$3f,$39
      db $e3,$98,$07,$a1,$b3,$cc,$a2,$62,$9e,$57,$51,$ef,$c4,$c5,$20,$87,$e2,$81,$d5,$88,$42,$5e,$26,$bf,$08
      db $c5,$f8,$f7,$8a,$4b,$3a,$e3,$e9,$ee,$2a,$13,$59,$9c,$f6,$3d,$42,$fd,$cc,$90,$1a,$9e,$f1,$4e,$e7,$65
      db $46,$3d,$0e,$d1,$fb,$a1,$22,$8f,$73,$2b,$63,$c8,$20,$f2,$98,$58,$64,$35,$96,$93,$ca,$98,$3d,$4f,$03
      db $8e,$a5,$bb,$98,$d2,$cd,$78,$7c,$b3,$e9,$96,$13,$b1,$15,$af,$a8,$06,$d0,$3b,$cc,$c4,$22,$a3,$7e,$23
      db $66,$79,$21,$d3,$12,$26,$0a,$ba,$0e,$96,$51,$98,$c6,$82,$20,$a0,$84,$8c,$9e,$aa,$92,$b0,$60,$6d,$e5
      db $77,$a4,$7f,$99,$fc,$be,$fb,$7b,$51,$45,$76,$7a,$b4,$2f,$cf,$7b,$0a,$33,$07,$9b,$ab,$18,$3c,$9b,$33
      db $03,$05,$41,$b1,$84,$d9,$7e,$3c,$c0,$c9,$68,$d8,$b7,$74,$ad,$15,$40,$1c,$2d,$e8,$ee,$c9,$cb,$2d,$6d
      db $1a,$73,$f8,$ea,$08,$a7,$0c,$d3,$5a,$9b,$35,$c0,$8d,$0a,$cd,$eb,$b5,$f8,$1c,$3d,$48,$4e,$1b,$d1,$bb
      db $01,$3e,$99,$69,$0d,$da,$db,$36,$49,$db,$af,$37,$5d,$25,$0b,$e2,$dc,$ea,$1c,$7f,$40,$44,$d9,$52,$8c
      db $7e,$e3,$0d,$9b,$ca,$df,$1d,$d0,$20,$51,$f7,$a1,$0e,$e5,$d8,$41,$43,$40,$9c,$88,$bd,$eb,$00,$a8,$5d
      db $b4,$5b,$1f,$e0,$fb,$40,$1a,$b4,$eb,$6e,$ea,$21,$ee,$6c,$1b,$dd,$8b,$6c,$80,$65,$be,$c9,$d8,$e5,$f8
      db $81,$ff,$e8,$de,$d1,$96,$fb,$a9,$10,$e9,$a8,$43,$58,$a2,$a4,$33,$a3,$3b,$62,$b1,$97,$6e,$f5,$d4,$2b
      db $19,$02,$7d,$3c,$31,$4f,$85,$10,$e3,$a8,$e7,$9c,$47,$52,$32,$cc,$c1,$c8,$72,$03,$c0,$f4,$57,$5f,$51
      db $18,$dc,$f6,$b5,$33,$cf,$e4,$85,$c4,$63,$55,$26,$96,$17,$91,$46,$ce,$71,$39,$06,$13,$77,$a1,$7f,$1b
      db $af,$81,$7a,$b2,$03,$9f,$8d,$23,$95,$02,$a0,$51,$1a,$39,$1c,$3c,$50,$f8,$b3,$17,$eb,$7c,$9f,$3c,$b7
      db $7e,$7a,$06,$09,$20,$3b,$94,$27,$d5,$a0,$4e,$e0,$75,$16,$46,$04,$34,$d1,$37,$98,$43,$54,$c1,$2e,$12
      db $3b,$c9,$d6,$ce,$e3,$85,$87,$85,$04,$e2,$ce,$8b,$dc,$96,$45,$0c,$eb,$13,$d0,$5d,$58,$51,$d3,$4f,$fd
      db $ca,$11,$8e,$dd,$0b,$6e,$8d,$c6,$b2,$ec,$a4,$ac,$09,$d6,$b5,$6f,$e2,$08,$30,$64,$a1,$01,$19,$9d,$f6
      db $6a,$9a,$7b,$e9,$e6,$d2,$40,$93,$99,$2f,$60,$fa,$24,$1f,$3f,$0f,$91,$2c,$34,$47,$dd,$32,$fa,$67,$e6
      db $92,$49,$fc,$fb,$bd,$e2,$24,$af,$cb,$dd,$5d,$2c,$2b,$00,$be,$e8,$b9,$f8,$ab,$62,$63,$85,$67,$f8,$65
      db $42,$3f,$3f,$c7,$7a,$2c,$27,$cf,$39,$57,$36,$b9,$19,$2c,$cf,$aa,$a8,$7c,$3a,$4d,$7b,$43,$14,$8c,$6f
      db $8d,$d8,$0c,$0f,$60,$72,$c1,$6a,$7c,$8b,$a0,$f7,$58,$71,$05,$c1,$18,$bd,$88,$dd,$3f,$27,$af,$2b,$b8
      db $ef,$34,$52,$4b,$8a,$d7,$57,$99,$1c,$ff,$05,$20,$41,$21,$82,$e1,$db,$5a,$14,$09,$b5,$4f,$51,$c5,$06
      db $31,$b2,$50,$b9,$23,$2a,$23,$8a,$eb,$6f,$14,$be,$8a,$ff,$cb,$9c,$2d,$4f,$ea,$ef,$39,$26,$6e,$b1,$f6
      db $26,$31,$d0,$e6,$03,$af,$5b,$4f,$15,$39,$8a,$09,$36,$d2,$62,$92,$95,$6e,$f0,$f8,$55,$a3,$ac,$c6,$3a
      db $b3,$4f,$e5,$ee,$37,$f3,$cc,$51,$ad,$04,$d7,$8d,$54,$40,$7b,$25,$0b,$c8,$79,$eb,$ec,$45,$c9,$80,$7a
      db $b2,$f5,$d6,$04,$b6,$90,$8a,$85,$e4,$a8,$e9,$13,$60,$9b,$3a,$a7,$38,$80,$b1,$aa,$6f,$c7,$4b,$67,$60
      db $e2,$b5,$38,$ab,$ae,$18,$8d,$26,$5d,$c7,$26,$a8,$48,$38,$d9,$9f,$c9,$a4,$5c,$84,$b0,$6e,$13,$c2,$1d
      db $96,$cb,$2a,$f2,$aa,$26,$3d,$78,$cd,$8d,$22,$bd,$75,$28,$d3,$f5,$57,$dc,$6a,$26,$86,$ca,$c2,$54,$ae
      db $f2,$2c,$74,$7b,$e1,$a8,$58,$d2,$af,$ca,$8f,$0e,$a4,$9e,$e0,$c7,$79,$37,$bc,$31,$42,$2d,$e2,$55,$28
      db $f1,$1b,$06,$d5,$29,$11,$8f,$0c,$04,$9e,$74,$ac,$39,$92,$c5,$e3,$aa,$6a,$73,$e9,$55,$f3,$0e,$72,$a4
      db $f8,$ff,$bf,$3f,$81,$96,$fb,$f5,$43,$29,$28,$af,$4e,$6d,$46,$ca,$b0,$45,$98,$49,$28,$1c,$63,$c3,$26
      db $33,$7d,$78,$9e,$27,$10,$6e,$88,$fb,$85,$7e,$63,$04,$55,$ff,$b4,$58,$9f,$a6,$84,$23,$10,$e9,$6b,$a2
      db $fe,$d2,$f4,$77,$8f,$fd,$5f,$8d,$c5,$0e,$51,$ab,$b6,$2e,$e5,$a4,$6c,$43,$9d,$5e,$e9,$d4,$0b,$e3,$97
      db $ac,$dc,$2e,$3b,$a6,$4b,$ab,$07,$5a,$55,$55,$04,$73,$42,$ee,$18,$ad,$64,$1a,$a3,$c2,$bb,$14,$8d,$8e
      db $a0,$37,$c0,$62,$0c,$1d,$0b,$f1,$69,$c6,$e7,$a5,$49,$c6,$ce,$ca,$d4,$ca,$16,$f6,$26,$da,$5c,$1b,$72
      db $d2,$86,$22,$f6,$11,$13,$69,$98,$d6,$0f,$f6,$e1,$38,$65,$e3,$d1,$7c,$bd,$81,$9c,$1d,$ab,$79,$e3,$ba
    @@skip:
    mov ax,offset @@table
    mov word ptr [EBytes],ax
    mov ax,cs
    mov word ptr [EBytes+2],ax
    {$ifdef debug}
      mov ax,offset @@skip
      mov word ptr [PSkip],ax
      mov ax,cs
      mov word ptr [PSkip+2],ax
    {$endif}
  end;
  AddrEBytes := EBytes;
  {$ifdef debug}
    assert(MAX_TCBITS = 8*(longint(PSkip)-longint(EBytes)), 'Length e table');
  {$endif}
end;

{$endif}


end.

