unit mp_rc32;

{Bit tables for Pi, ln(2), ln(10), exp(1)}

interface

{$i STD.INC}

(*************************************************************************

 DESCRIPTION   :  bit tables for Pi, ln(2), ln(10), exp(1)

 REQUIREMENTS  :  BP7, D1-D7/D9-D10/D12/D17-D18, FPC, VP

 REMARK        :  ln(10), exp(1) only if MPC_E1Ln10Tab is defined
                  last bits of short tables are NOT rounded, may be off by 1

 Version  Date      Author      Modification
 -------  --------  -------     ------------------------------------------
 0.00.10  25.01.08  W.Ehrhardt  Initial version: BP7

 1.16.00  12.06.10  we          Table for e = exp(1)
 1.16.01  13.06.10  we          Check table sizes, MAX_TCBITS moved to mp_types
 1.16.02  13.06.10  we          Uses MPC_FPrec30K for smaller tables

 1.20.00  15.01.12  we          Arrays moved from code to data for 32/64 bit
 1.20.01  14.02.12  we          ln(2) always available; new name MPC_E1Ln10Tab
**************************************************************************)


(*-------------------------------------------------------------------------
 (C) Copyright 2008-2012 Wolfgang Ehrhardt

 This software is provided 'as-is', without any express or implied warranty.
 In no event will the authors be held liable for any damages arising from
 the use of this software.

 Permission is granted to anyone to use this software for any purpose,
 including commercial applications, and to alter it and redistribute it
 freely, subject to the following restrictions:

 1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software in
    a product, an acknowledgment in the product documentation would be
    appreciated but is not required.

 2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

 3. This notice may not be removed or altered from any source distribution.
----------------------------------------------------------------------------*)

{$i mp_conf.inc}

function AddrPiBytes: pointer;
  {-Pointer to bits of Pi, big endian format}

function AddrLn2Bytes: pointer;
  {-Pointer to bits of ln(2), big endian format}

{$ifdef MPC_E1Ln10Tab}
function AddrEBytes: pointer;
  {-Pointer to bits of exp(1), big endian format}

function AddrLn10Bytes: pointer;
  {-Pointer to bits of ln(10), big endian format}
{$endif}


implementation


uses
  mp_types;

const
  PiBytes: packed array[1..MAX_TCBITS div 8] of byte = (
    $c9,$0f,$da,$a2,$21,$68,$c2,$34,$c4,$c6,$62,$8b,$80,$dc,$1c,$d1,$29,$02,$4e,$08,$8a,$67,$cc,$74,$02,
    $0b,$be,$a6,$3b,$13,$9b,$22,$51,$4a,$08,$79,$8e,$34,$04,$dd,$ef,$95,$19,$b3,$cd,$3a,$43,$1b,$30,$2b,
    $0a,$6d,$f2,$5f,$14,$37,$4f,$e1,$35,$6d,$6d,$51,$c2,$45,$e4,$85,$b5,$76,$62,$5e,$7e,$c6,$f4,$4c,$42,
    $e9,$a6,$37,$ed,$6b,$0b,$ff,$5c,$b6,$f4,$06,$b7,$ed,$ee,$38,$6b,$fb,$5a,$89,$9f,$a5,$ae,$9f,$24,$11,
    $7c,$4b,$1f,$e6,$49,$28,$66,$51,$ec,$e4,$5b,$3d,$c2,$00,$7c,$b8,$a1,$63,$bf,$05,$98,$da,$48,$36,$1c,
    $55,$d3,$9a,$69,$16,$3f,$a8,$fd,$24,$cf,$5f,$83,$65,$5d,$23,$dc,$a3,$ad,$96,$1c,$62,$f3,$56,$20,$85,
    $52,$bb,$9e,$d5,$29,$07,$70,$96,$96,$6d,$67,$0c,$35,$4e,$4a,$bc,$98,$04,$f1,$74,$6c,$08,$ca,$18,$21,
    $7c,$32,$90,$5e,$46,$2e,$36,$ce,$3b,$e3,$9e,$77,$2c,$18,$0e,$86,$03,$9b,$27,$83,$a2,$ec,$07,$a2,$8f,
    $b5,$c5,$5d,$f0,$6f,$4c,$52,$c9,$de,$2b,$cb,$f6,$95,$58,$17,$18,$39,$95,$49,$7c,$ea,$95,$6a,$e5,$15,
    $d2,$26,$18,$98,$fa,$05,$10,$15,$72,$8e,$5a,$8a,$aa,$c4,$2d,$ad,$33,$17,$0d,$04,$50,$7a,$33,$a8,$55,
    $21,$ab,$df,$1c,$ba,$64,$ec,$fb,$85,$04,$58,$db,$ef,$0a,$8a,$ea,$71,$57,$5d,$06,$0c,$7d,$b3,$97,$0f,
    $85,$a6,$e1,$e4,$c7,$ab,$f5,$ae,$8c,$db,$09,$33,$d7,$1e,$8c,$94,$e0,$4a,$25,$61,$9d,$ce,$e3,$d2,$26,
    $1a,$d2,$ee,$6b,$f1,$2f,$fa,$06,$d9,$8a,$08,$64,$d8,$76,$02,$73,$3e,$c8,$6a,$64,$52,$1f,$2b,$18,$17,
    $7b,$20,$0c,$bb,$e1,$17,$57,$7a,$61,$5d,$6c,$77,$09,$88,$c0,$ba,$d9,$46,$e2,$08,$e2,$4f,$a0,$74,$e5,
    $ab,$31,$43,$db,$5b,$fc,$e0,$fd,$10,$8e,$4b,$82,$d1,$20,$a9,$21,$08,$01,$1a,$72,$3c,$12,$a7,$87,$e6,
    $d7,$88,$71,$9a,$10,$bd,$ba,$5b,$26,$99,$c3,$27,$18,$6a,$f4,$e2,$3c,$1a,$94,$68,$34,$b6,$15,$0b,$da,
    $25,$83,$e9,$ca,$2a,$d4,$4c,$e8,$db,$bb,$c2,$db,$04,$de,$8e,$f9,$2e,$8e,$fc,$14,$1f,$be,$ca,$a6,$28,
    $7c,$59,$47,$4e,$6b,$c0,$5d,$99,$b2,$96,$4f,$a0,$90,$c3,$a2,$23,$3b,$a1,$86,$51,$5b,$e7,$ed,$1f,$61,
    $29,$70,$ce,$e2,$d7,$af,$b8,$1b,$dd,$76,$21,$70,$48,$1c,$d0,$06,$91,$27,$d5,$b0,$5a,$a9,$93,$b4,$ea,
    $98,$8d,$8f,$dd,$c1,$86,$ff,$b7,$dc,$90,$a6,$c0,$8f,$4d,$f4,$35,$c9,$34,$02,$84,$92,$36,$c3,$fa,$b4,
    $d2,$7c,$70,$26,$c1,$d4,$dc,$b2,$60,$26,$46,$de,$c9,$75,$1e,$76,$3d,$ba,$37,$bd,$f8,$ff,$94,$06,$ad,
    $9e,$53,$0e,$e5,$db,$38,$2f,$41,$30,$01,$ae,$b0,$6a,$53,$ed,$90,$27,$d8,$31,$17,$97,$27,$b0,$86,$5a,
    $89,$18,$da,$3e,$db,$eb,$cf,$9b,$14,$ed,$44,$ce,$6c,$ba,$ce,$d4,$bb,$1b,$db,$7f,$14,$47,$e6,$cc,$25,
    $4b,$33,$20,$51,$51,$2b,$d7,$af,$42,$6f,$b8,$f4,$01,$37,$8c,$d2,$bf,$59,$83,$ca,$01,$c6,$4b,$92,$ec,
    $f0,$32,$ea,$15,$d1,$72,$1d,$03,$f4,$82,$d7,$ce,$6e,$74,$fe,$f6,$d5,$5e,$70,$2f,$46,$98,$0c,$82,$b5,
    $a8,$40,$31,$90,$0b,$1c,$9e,$59,$e7,$c9,$7f,$be,$c7,$e8,$f3,$23,$a9,$7a,$7e,$36,$cc,$88,$be,$0f,$1d,
    $45,$b7,$ff,$58,$5a,$c5,$4b,$d4,$07,$b2,$2b,$41,$54,$aa,$cc,$8f,$6d,$7e,$bf,$48,$e1,$d8,$14,$cc,$5e,
    $d2,$0f,$80,$37,$e0,$a7,$97,$15,$ee,$f2,$9b,$e3,$28,$06,$a1,$d5,$8b,$b7,$c5,$da,$76,$f5,$50,$aa,$3d,
    $8a,$1f,$bf,$f0,$eb,$19,$cc,$b1,$a3,$13,$d5,$5c,$da,$56,$c9,$ec,$2e,$f2,$96,$32,$38,$7f,$e8,$d7,$6e,
    $3c,$04,$68,$04,$3e,$8f,$66,$3f,$48,$60,$ee,$12,$bf,$2d,$5b,$0b,$74,$74,$d6,$e6,$94,$f9,$1e,$6d,$be,
    $11,$59,$74,$a3,$92,$6f,$12,$fe,$e5,$e4,$38,$77,$7c,$b6,$a9,$32,$df,$8c,$d8,$be,$c4,$d0,$73,$b9,$31,
    $ba,$3b,$c8,$32,$b6,$8d,$9d,$d3,$00,$74,$1f,$a7,$bf,$8a,$fc,$47,$ed,$25,$76,$f6,$93,$6b,$a4,$24,$66,
    $3a,$ab,$63,$9c,$5a,$e4,$f5,$68,$34,$23,$b4,$74,$2b,$f1,$c9,$78,$23,$8f,$16,$cb,$e3,$9d,$65,$2d,$e3,
    $fd,$b8,$be,$fc,$84,$8a,$d9,$22,$22,$2e,$04,$a4,$03,$7c,$07,$13,$eb,$57,$a8,$1a,$23,$f0,$c7,$34,$73,
    $fc,$64,$6c,$ea,$30,$6b,$4b,$cb,$c8,$86,$2f,$83,$85,$dd,$fa,$9d,$4b,$7f,$a2,$c0,$87,$e8,$79,$68,$33,
    $03,$ed,$5b,$dd,$3a,$06,$2b,$3c,$f5,$b3,$a2,$78,$a6,$6d,$2a,$13,$f8,$3f,$44,$f8,$2d,$df,$31,$0e,$e0,
    $74,$ab,$6a,$36,$45,$97,$e8,$99,$a0,$25,$5d,$c1,$64,$f3,$1c,$c5,$08,$46,$85,$1d,$f9,$ab,$48,$19,$5d,
    $ed,$7e,$a1,$b1,$d5,$10,$bd,$7e,$e7,$4d,$73,$fa,$f3,$6b,$c3,$1e,$cf,$a2,$68,$35,$90,$46,$f4,$eb,$87,
    $9f,$92,$40,$09,$43,$8b,$48,$1c,$6c,$d7,$88,$9a,$00,$2e,$d5,$ee,$38,$2b,$c9,$19,$0d,$a6,$fc,$02,$6e,
    $47,$95,$58,$e4,$47,$56,$77,$e9,$aa,$9e,$30,$50,$e2,$76,$56,$94,$df,$c8,$1f,$56,$e8,$80,$b9,$6e,$71,
    $60,$c9,$80,$dd,$98,$a5,$73,$ea,$44,$72,$06,$5a,$13,$9c,$d2,$90,$6c,$d1,$cb,$72,$9e,$c5,$2a,$52,$86,
    $d4,$40,$14,$a6,$94,$ca,$45,$75,$83,$d5,$cf,$ef,$26,$f1,$b9,$0a,$d8,$29,$1d,$a0,$79,$9d,$00,$02,$2e,
    $9b,$ed,$55,$c6,$fa,$47,$fc,$a5,$bb,$1a,$ca,$83,$76,$45,$6d,$98,$d9,$48,$79,$ee,$7e,$6d,$bf,$cd,$01,
    $4b,$b1,$61,$55,$99,$14,$ec,$0b,$57,$6a,$67,$e3,$e8,$42,$2e,$91,$e6,$5b,$a1,$41,$da,$92,$de,$9c,$3a,
    $6d,$6c,$ca,$51,$36,$dd,$42,$4b,$b1,$06,$49,$88,$eb,$5b,$a9,$ac,$12,$69,$f7,$df,$67,$3b,$98,$2e,$23,
    $fb,$6c,$99,$bb,$2a,$a3,$1c,$5a,$66,$85,$ff,$d5,$99,$14,$9b,$30,$ac,$67,$b8,$46,$4d,$80,$a9,$5d,$42,
    $53,$0a,$68,$16,$44,$d0,$39,$06,$0e,$8f,$8f,$d5,$26,$26,$96,$d0,$a7,$59,$5a,$e3,$f9,$35,$a6,$7d,$cf,
    $f5,$a8,$74,$a7,$01,$fb,$fa,$0c,$3d,$53,$4b,$4e,$39,$bc,$09,$57,$70,$53,$37,$48,$21,$a1,$1c,$3a,$c9,
    $98,$e0,$ba,$71,$80,$87,$b3,$17,$82,$5a,$1a,$cf,$cf,$ae,$bb,$f2,$4f,$25,$c6,$05,$1a,$da,$9c,$28,$5a,
    $1f,$cd,$61,$14,$a8,$38,$a1,$ad,$e7,$14,$c1,$6a,$94,$01,$cd,$cf,$81,$e1,$07,$1f,$f7,$ab,$97,$23,$9f,
    $51,$3b,$15,$c5,$bc,$ae,$2c,$0e,$b6,$8d,$fc,$14,$03,$03,$7c,$07,$07,$c1,$00,$80,$2c,$ff,$eb,$83,$3d,
    $46,$8f,$2d,$5d,$2c,$89,$60,$de,$96,$37,$02,$48,$6f,$74,$64,$44,$fe,$5f,$2a,$4b,$fd,$a5,$0c,$91,$dc,
    $c8,$bd,$51,$c0,$4e,$b9,$79,$60,$4d,$f0,$b6,$b7,$32,$2d,$5d,$8d,$26,$bc,$f7,$69,$ea,$51,$18,$51,$83,
    $f4,$00,$c3,$bb,$32,$31,$cf,$a9,$1d,$47,$90,$78,$8e,$33,$66,$4e,$fa,$83,$8b,$cc,$a0,$2e,$e8,$46,$0f,
    $ac,$cc,$53,$95,$22,$ce,$13,$db,$6e,$42,$1b,$d0,$83,$40,$fd,$82,$81,$2f,$cb,$2e,$04,$a4,$09,$25,$df,
    $1e,$55,$9e,$6c,$1c,$af,$2b,$e2,$6b,$f7,$a6,$9d,$c7,$f6,$64,$c2,$04,$2c,$e2,$eb,$84,$b7,$33,$cf,$cb,
    $95,$44,$9c,$87,$cb,$9a,$dc,$49,$14,$06,$b7,$79,$a7,$e1,$33,$61,$de,$96,$11,$c6,$1d,$02,$36,$85,$ef,
    $27,$e6,$af,$3a,$52,$df,$63,$3b,$1e,$bb,$0e,$b6,$e1,$47,$7e,$98,$c2,$50,$d9,$b1,$19,$30,$f4,$bb,$c7,
    $06,$11,$cc,$85,$76,$42,$37,$50,$ce,$cd,$c9,$30,$ae,$85,$84,$a8,$53,$50,$ca,$99,$71,$14,$54,$25,$00,
    $00,$84,$ce,$b9,$37,$5c,$77,$fe,$27,$84,$0c,$53,$95,$60,$6b,$1d,$f5,$97,$c4,$46,$66,$c1,$0d,$55,$bc,
    $75,$e8,$f1,$da,$cf,$04,$46,$0e,$d6,$49,$29,$42,$7c,$a3,$f9,$bb,$65,$fc,$7e,$fe,$a7,$ae,$af,$cb,$07,
    $85,$4f,$1b,$a1,$b8,$d1,$5c,$3a,$ba,$5b,$ec,$61,$83,$97,$82,$96,$8f,$8a,$ac,$dd,$c7,$f9,$c7,$13,$8f,
    $41,$be,$8a,$59,$77,$2e,$66,$79,$c7,$43,$e0,$0f,$a2,$75,$94,$99,$b2,$09,$4b,$93,$32,$5e,$27,$04,$2c,
    $da,$b1,$85,$43,$ae,$a5,$38,$ba,$9e,$29,$7f,$0f,$14,$c7,$82,$8b,$7d,$3c,$bd,$d3,$a9,$cd,$87,$4a,$cf,
    $46,$4e,$49,$83,$c6,$70,$9e,$58,$14,$88,$e9,$c2,$3d,$c4,$c4,$ad,$ba,$eb,$7f,$9b,$ba,$b0,$c7,$d9,$b8,
    $ef,$11,$65,$69,$9e,$f2,$20,$ec,$5f,$cd,$f4,$40,$63,$3f,$ca,$30,$cc,$b7,$7b,$ef,$9b,$16,$a9,$59,$56,
    $08,$61,$5a,$2a,$e6,$00,$bb,$b3,$a9,$43,$f6,$cb,$e5,$4e,$ca,$bb,$df,$6b,$56,$db,$8b,$e1,$05,$48,$6d,
    $8a,$0a,$41,$d8,$5c,$3b,$37,$51,$dd,$58,$67,$c5,$44,$04,$f3,$2a,$0c,$3a,$d8,$6f,$65,$80,$cd,$3f,$87,
    $aa,$80,$d8,$f3,$ed,$5c,$d7,$24,$13,$1c,$28,$8e,$75,$67,$a7,$82,$f2,$ea,$b7,$85,$3b,$b3,$21,$af,$18,
    $18,$8b,$29,$e7,$2a,$d7,$2a,$ec,$bc,$e1,$1b,$99,$22,$c7,$ab,$c6,$6f,$7c,$32,$a8,$08,$da,$6e,$59,$56,
    $ae,$d4,$10,$1a,$16,$8c,$8f,$0a,$ad,$2c,$cc,$67,$ba,$75,$70,$08,$6e,$3d,$e6,$d5,$02,$c6,$61,$d7,$e8,
    $26,$65,$7d,$e6,$5f,$98,$8f,$5f,$6a,$3e,$0d,$e2,$26,$a5,$f8,$cb,$5d,$c4,$7b,$64,$d7,$c5,$9a,$04,$a0,
    $43,$8d,$62,$0a,$71,$f9,$87,$f5,$a5,$b7,$b7,$e8,$5e,$16,$2e,$a6,$55,$fd,$61,$29,$46,$c8,$9c,$98,$e6,
    $e0,$f0,$ff,$c6,$b0,$91,$a5,$b3,$6c,$c2,$ba,$d4,$cb,$8c,$15,$23,$f6,$52,$39,$1b,$6f,$0c,$4a,$16,$3a,
    $fc,$bb,$cd,$31,$bf,$fa,$bf,$8a,$3b,$58,$7b,$9f,$0f,$1c,$d7,$52,$85,$36,$7a,$19,$2d,$f8,$d0,$84,$17,
    $45,$08,$0f,$84,$f8,$11,$7b,$b8,$ad,$a8,$ea,$aa,$fa,$b6,$db,$13,$c5,$7e,$b2,$d3,$f4,$31,$d0,$bd,$10,
    $bb,$da,$ae,$ed,$59,$53,$ce,$c7,$50,$73,$48,$41,$76,$07,$9e,$67,$a1,$a1,$53,$71,$f9,$12,$d1,$da,$8f,
    $60,$58,$94,$33,$d8,$a8,$7c,$96,$e3,$49,$91,$bf,$22,$20,$e8,$30,$71,$ed,$a8,$df,$c5,$49,$30,$da,$72,
    $dd,$24,$91,$e1,$22,$82,$d5,$a4,$ac,$a1,$42,$56,$ef,$c0,$2b,$46,$52,$27,$45,$18,$ac,$5d,$08,$e0,$83,
    $80,$16,$10,$a3,$4a,$83,$15,$7d,$7a,$87,$6b,$7d,$0f,$88,$cf,$dc,$18,$4c,$dc,$bc,$24,$a3,$64,$df,$90,
    $75,$97,$fb,$3c,$5b,$08,$8e,$f6,$df,$37,$8d,$d6,$72,$fb,$9d,$18,$10,$21,$7c,$a9,$f3,$9d,$cc,$9b,$a9,
    $81,$e0,$21,$06,$7e,$14,$27,$ba,$3b,$f6,$15,$58,$76,$65,$cd,$6a,$5a,$69,$ea,$b1,$43,$01,$b0,$96,$81,
    $2a,$ff,$20,$02,$f2,$b7,$27,$91,$1e,$8b,$b0,$d1,$4d,$21,$3f,$75,$59,$c1,$43,$87,$a7,$b2,$76,$dc,$f6,
    $f4,$c4,$15,$62,$33,$59,$d7,$f6,$9e,$78,$d9,$d0,$d0,$31,$71,$0d,$19,$5c,$4f,$8e,$36,$0f,$4a,$3e,$27,
    $bc,$5b,$7f,$c8,$05,$4f,$88,$79,$e3,$ec,$0f,$52,$b9,$b8,$e7,$ca,$f6,$e0,$eb,$7d,$fa,$4f,$56,$9a,$25,
    $20,$50,$3d,$fd,$93,$09,$87,$25,$1a,$4a,$4d,$10,$45,$48,$3d,$dd,$80,$b5,$3d,$ef,$3d,$1a,$cb,$b5,$28,
    $80,$1a,$35,$02,$09,$1c,$4c,$c8,$3d,$1a,$90,$ed,$f5,$2d,$d4,$00,$18,$6b,$c7,$8e,$7d,$8b,$a5,$c9,$11,
    $51,$85,$08,$53,$dd,$2f,$e2,$e2,$10,$13,$65,$7f,$07,$65,$ad,$64,$6b,$dc,$3d,$37,$74,$d9,$a8,$0b,$d1,
    $6f,$ef,$02,$7b,$00,$ef,$65,$e1,$5f,$eb,$1b,$74,$0c,$72,$e1,$41,$25,$ba,$c9,$ec,$d5,$7f,$4e,$50,$76,
    $89,$51,$f9,$aa,$f2,$82,$a6,$8a,$14,$1e,$09,$54,$c1,$0a,$7d,$02,$8b,$21,$b6,$ba,$6d,$9b,$7e,$da,$37,
    $05,$18,$b5,$d2,$1a,$40,$1a,$03,$b0,$29,$09,$e8,$63,$7b,$93,$cf,$ff,$a8,$ba,$21,$eb,$63,$2d,$63,$38,
    $01,$9e,$bd,$35,$ad,$aa,$b3,$87,$9f,$34,$cd,$d7,$fb,$33,$9e,$28,$e6,$50,$1a,$ca,$90,$88,$3f,$a7,$8d,
    $76,$7c,$e1,$6e,$7b,$8e,$75,$ea,$ce,$c4,$93,$a2,$c7,$72,$7e,$bd,$d2,$db,$46,$15,$89,$a8,$d9,$8c,$7a,
    $b8,$e5,$ec,$8e,$9b,$be,$9d,$37,$56,$d0,$cc,$9a,$10,$79,$fd,$f2,$9e,$08,$3e,$fe,$c2,$bd,$53,$b6,$3f,
    $ac,$e5,$d1,$50,$15,$ab,$2e,$92,$25,$49,$d5,$54,$ce,$8e,$88,$20,$e3,$61,$ff,$9a,$ea,$6d,$f4,$25,$a5,
    $52,$d5,$6a,$19,$ef,$28,$45,$66,$96,$33,$2a,$4a,$58,$e6,$78,$76,$cc,$e9,$8a,$92,$95,$8f,$cc,$49,$7e,
    $57,$bd,$1f,$87,$24,$0a,$4c,$5f,$3f,$7e,$3a,$00,$81,$09,$cb,$dc,$20,$2e,$c5,$57,$01,$4a,$0b,$38,$e5,
    $70,$45,$52,$39,$31,$9b,$48,$92,$30,$44,$cf,$f1,$c3,$e1,$b7,$01,$fe,$72,$7b,$90,$41,$07,$c3,$d0,$11,
    $de,$69,$17,$62,$1b,$85,$cc,$97,$d4,$7a,$f5,$66,$f0,$34,$7c,$af,$30,$63,$d0,$44,$4d,$59,$09,$5e,$de,
    $0d,$18,$0a,$a7,$18,$37,$fe,$3a,$28,$c7,$d8,$db,$06,$c3,$84,$ad,$30,$80,$b8,$4c,$a7,$ab,$d9,$93,$f4,
    $72,$b4,$60,$45,$5a,$c8,$e5,$78,$0c,$cf,$a4,$b8,$4e,$c9,$02,$d8,$bb,$af,$ae,$48,$a1,$6c,$a8,$83,$b9,
    $ae,$83,$66,$77,$9c,$83,$23,$0b,$68,$bd,$ca,$34,$04,$9e,$11,$65,$6d,$e5,$3f,$59,$1f,$42,$18,$b9,$f3,
    $3d,$7c,$15,$12,$68,$db,$e1,$df,$52,$3e,$b0,$e7,$7f,$49,$fc,$cf,$a3,$47,$82,$91,$d8,$d0,$66,$4b,$bf,
    $dd,$0e,$9b,$db,$aa,$fd,$3e,$3f,$4d,$ea,$04,$b7,$18,$28,$7a,$f7,$7e,$26,$46,$f8,$53,$36,$db,$9a,$c3,
    $71,$9e,$d5,$44,$1b,$59,$cb,$0d,$c9,$d9,$75,$0e,$f7,$34,$3a,$01,$3c,$4a,$43,$71,$f3,$00,$3f,$e8,$ed,
    $4e,$43,$e4,$9a,$43,$fb,$42,$d9,$9e,$e7,$fe,$f3,$b6,$df,$67,$28,$24,$73,$c2,$f6,$45,$57,$a8,$ee,$c4,
    $cb,$e2,$14,$56,$eb,$49,$1e,$e5,$1e,$6f,$dd,$8e,$f5,$ba,$cd,$ce,$4b,$ac,$f3,$04,$56,$5e,$60,$09,$b8,
    $a5,$fd,$0b,$8c,$4b,$5a,$10,$ab,$69,$f1,$9a,$8a,$ce,$c4,$9d,$53,$33,$1e,$0b,$bc,$47,$1a,$84,$90,$8d,
    $ed,$e4,$94,$79,$c1,$a8,$6e,$f9,$92,$fe,$d8,$d4,$06,$9a,$c4,$06,$04,$72,$bb,$7e,$8f,$49,$6f,$76,$38,
    $b8,$70,$f2,$51,$10,$85,$96,$42,$84,$84,$e1,$b6,$43,$3b,$1b,$b5,$6a,$fa,$8a,$99,$2b,$d9,$d3,$b6,$a1,
    $aa,$17,$ef,$af,$fa,$62,$19,$39,$30,$ff,$a7,$6f,$20,$15,$fc,$3d,$f0,$21,$98,$1e,$1e,$fe,$18,$00,$d8,
    $13,$74,$7f,$60,$d1,$bd,$8e,$07,$ec,$1d,$d1,$6b,$81,$35,$cd,$bf,$33,$20,$d0,$9a,$cc,$fc,$07,$aa,$dc,
    $6c,$20,$10,$61,$cf,$00,$17,$97,$dd,$e8,$15,$ef,$af,$7a,$2b,$89,$15,$51,$90,$a6,$6f,$d6,$0b,$98,$53,
    $96,$3d,$23,$fc,$b7,$7f,$68,$bd,$1d,$3b,$ce,$21,$e2,$6f,$70,$94,$d9,$be,$70,$f2,$2c,$e3,$9d,$2d,$1d,
    $7c,$95,$51,$bf,$36,$6e,$5e,$ba,$c9,$98,$62,$c7,$77,$e1,$21,$1a,$83,$9e,$64,$57,$e5,$78,$91,$9b,$96,
    $63,$88,$2d,$15,$dc,$23,$35,$55,$64,$72,$40,$b7,$93,$2e,$42,$eb,$38,$6e,$e0,$81,$74,$04,$6a,$18,$92,
    $1d,$5d,$2a,$67,$ad,$df,$c6,$6d,$b8,$2a,$77,$02,$59,$8b,$42,$68,$71,$0c,$91,$8c,$fa,$16,$87,$c0,$82,
    $7c,$2f,$a3,$12,$a6,$68,$09,$47,$5b,$bf,$84,$06,$ae,$4f,$47,$42,$e9,$69,$37,$e8,$61,$bc,$83,$ca,$1a,
    $3c,$5a,$77,$2d,$f6,$a0,$d5,$ce,$41,$bf,$a8,$78,$b3,$a6,$d3,$f3,$5f,$d4,$94,$04,$57,$80,$69,$c1,$a0,
    $fe,$a8,$00,$ad,$71,$03,$79,$b4,$09,$e6,$be,$23,$09,$dd,$cf,$e1,$90,$70,$d8,$13,$01,$98,$6a,$01,$ad,
    $7c,$05,$de,$8a,$30,$3d,$61,$b8,$00,$18,$16,$2a,$8c,$37,$1f,$58,$84,$79,$a7,$b5,$c8,$ce,$3a,$98,$d4,
    $f0,$b7,$65,$30,$b0,$85,$8d,$2e,$f2,$fb,$95,$a4,$2f,$2d,$b7,$ba,$ff,$1f,$68,$73,$96,$47,$5d,$9b,$c1,
    $79,$02,$52,$df,$00,$62,$0e,$5c,$82,$8f,$5f,$24,$9f,$09,$21,$b8,$dc,$97,$dc,$93,$67,$6e,$46,$b0,$56,
    $ed,$34,$e7,$ae,$3f,$3b,$55,$15,$5e,$02,$3f,$29,$6d,$76,$0f,$5f,$34,$d3,$6b,$43,$f1,$07,$94,$3b,$d7,
    $ac,$58,$79,$be,$28,$a1,$45,$36,$5b,$14,$44,$cf,$1b,$f5,$71,$f9,$d5,$b8,$53,$b1,$0d,$8a,$af,$f3,$b7,
    $71,$b2,$0d,$f5,$e6,$8c,$8d,$24,$98,$e0,$84,$99,$c3,$be,$a3,$90,$18,$00,$38,$0e,$8e,$73,$8d,$f4,$fe,
    $bd,$73,$ea,$f0,$9d,$cd,$d6,$b1,$4b,$46,$d7,$2c,$19,$e7,$93,$e8,$cd,$d0,$b4,$e0,$89,$d0,$26,$6f,$26,
    $ef,$b5,$44,$63,$a7,$6f,$c3,$dc,$c5,$75,$8b,$47,$1f,$b1,$c0,$31,$1e,$ed,$e3,$06,$da,$c8,$68,$64,$11,
    $6c,$9b,$ac,$6f,$9a,$8d,$9b,$ad,$15,$d2,$2a,$cb,$ef,$25,$1b,$9e,$71,$a8,$dd,$b4,$99,$52,$70,$b2,$14,
    $c3,$fe,$3b,$91,$a3,$77,$9f,$75,$5c,$c2,$87,$53,$34,$13,$71,$8a,$4e,$6e,$f6,$ea,$6e,$91,$94,$2b,$25,
    $49,$ba,$2f,$97,$b8,$c1,$28,$7e,$b5,$7c,$1a,$8b,$54,$66,$98,$fb,$e3,$38,$a6,$a1,$bb,$88,$b0,$22,$70,
    $ae,$10,$c9,$0b,$bd,$a9,$47,$80,$ea,$a7,$3c,$b4,$29,$20,$f0,$18,$6e,$a6,$fa,$5a,$93,$63,$f9,$45,$54,
    $2e,$99,$16,$f5,$8a,$09,$a8,$be,$69,$ce,$8e,$b8,$52,$ea,$65,$61,$bb,$d5,$58,$ba,$71,$cb,$fb,$f4,$fd,
    $d4,$bd,$f6,$8f,$c1,$1b,$da,$5d,$fe,$82,$96,$60,$39,$2a,$45,$61,$ec,$21,$80,$66,$c2,$79,$ab,$4e,$cf,
    $b9,$64,$fa,$64,$3f,$56,$a7,$8d,$35,$e5,$cb,$3c,$2d,$f6,$40,$8a,$e2,$d4,$65,$b5,$6b,$0e,$80,$5f,$69,
    $9f,$74,$73,$cf,$b5,$9f,$1f,$4b,$4a,$07,$e7,$c9,$73,$eb,$7c,$ae,$26,$d6,$b5,$ad,$1c,$96,$a2,$3d,$53,
    $38,$0a,$6b,$1c,$78,$06,$69,$79,$91,$ec,$2b,$3f,$7b,$64,$fe,$a6,$fa,$34,$f1,$23,$4a,$0e,$d5,$f3,$3e,
    $35,$59,$8a,$5e,$44,$cb,$8a,$1e,$17,$c0,$64,$7b,$5d,$58,$15,$7d,$e5,$83,$91,$38,$f4,$d7,$a3,$05,$41,
    $5b,$75,$22,$3d,$1b,$6e,$80,$e8,$58,$49,$41,$59,$3c,$2f,$70,$fa,$e7,$85,$4f,$24,$15,$e8,$a5,$c9,$c6,
    $bb,$2a,$4e,$81,$ca,$86,$cf,$db,$66,$c7,$98,$c8,$7d,$7d,$67,$19,$be,$c9,$b7,$3c,$c6,$5d,$4c,$f6,$4a,
    $2c,$55,$7f,$7d,$40,$d5,$8d,$20,$a2,$ae,$8f,$2e,$ca,$14,$5d,$c4,$70,$82,$b6,$7e,$2a,$f3,$14,$59,$f3,
    $2b,$64,$97,$d3,$7a,$05,$d4,$4e,$0c,$37,$23,$8d,$e7,$56,$18,$a4,$c8,$3e,$64,$7a,$9e,$a4,$30,$be,$cf,
    $9e,$f3,$94,$48,$73,$99,$1d,$d3,$ef,$8c,$aa,$2d,$b8,$df,$b0,$ca,$4f,$51,$92,$37,$94,$da,$59,$04,$f9,
    $a0,$28,$ab,$82,$04,$37,$5b,$6c,$89,$1a,$61,$4b,$7f,$42,$41,$c8,$59,$ac,$e6,$91,$82,$99,$11,$70,$37,
    $56,$1b,$37,$b3,$c7,$08,$32,$2b,$96,$ef,$bd,$f7,$46,$d6,$23,$50,$33,$34,$80,$5f,$da,$ed,$38,$ee,$f7,
    $68,$9a,$9f,$8e,$96,$7f,$d1,$4f,$8d,$42,$91,$2f,$2d,$33,$75,$5c,$ba,$b3,$aa,$3e,$99,$21,$2e,$e3,$59,
    $84,$ab,$af,$cf,$1b,$d1,$f4,$a6,$f9,$18,$d5,$0b,$d7,$67,$ab,$b0,$9d,$c6,$fd,$93,$98,$dc,$1d,$03,$83,
    $63,$79,$d6,$c4,$d5,$fe,$1c,$85,$9c,$6b,$d4,$df,$57,$50,$10,$32,$c2,$13,$ad,$38,$b3,$0d,$34,$91,$9a,
    $80,$45,$6b,$e1,$38,$6c,$01,$0a,$25,$66,$0e,$87,$41,$ae,$27,$ed,$33,$9b,$a8,$12,$1b,$cf,$ce,$e0,$8d,
    $48,$2a,$e0,$80,$46,$87,$52,$c9,$dc,$89,$fe,$18,$45,$58,$2c,$79,$e4,$cf,$f7,$2e,$ce,$9e,$4a,$cd,$11
  {$ifndef MPC_FPrec30K}
   ,$49,$6f,$68,$22,$0e,$78,$54,$73,$9e,$52,$cb,$cc,$b2,$6d,$e8,$07,$ee,$b2,$78,$07,$32,$1f,$af,$31,$f4,
    $7d,$b3,$c0,$44,$70,$e8,$7a,$2a,$b1,$c6,$a4,$21,$d2,$75,$13,$ef,$66,$b4,$36,$b7,$b2,$f5,$42,$b6,$8e,
    $00,$ce,$70,$ca,$b1,$a4,$4d,$99,$e3,$7e,$4c,$5f,$38,$2c,$4a,$d3,$fd,$e7,$96,$6d,$d0,$fd,$6e,$ce,$bc,
    $b5,$46,$7f,$c9,$f6,$51,$67,$2f,$e5,$c8,$8b,$05,$79,$bf,$0a,$83,$e4,$7f,$1c,$66,$e5,$05,$49,$7e,$b9,
    $64,$d8,$73,$ad,$a7,$3a,$f0,$aa,$19,$16,$44,$ae,$aa,$34,$6d,$b0,$41,$d7,$b8,$c1,$5a,$83,$04,$34,$94,
    $19,$72,$c0,$e9,$10,$b8,$3b,$1b,$83,$85,$a6,$36,$ce,$d3,$26,$28,$2f,$8c,$9e,$3a,$59,$27,$c7,$e5,$4c,
    $b8,$34,$e4,$b7,$f4,$e8,$0d,$0a,$e2,$5c,$7c,$87,$86,$c2,$9d,$10,$52,$e8,$cd,$23,$31,$6f,$9c,$48,$30,
    $dd,$8c,$b6,$37,$cd,$67,$e3,$66,$e6,$4b,$cb,$b9,$82,$db,$d1,$c3,$f8,$fc,$47,$79,$53,$36,$95,$07,$b6,
    $b6,$24,$73,$98,$9e,$73,$f3,$4f,$9f,$9b,$c5,$86,$2c,$59,$bf,$4b,$07,$41,$61,$23,$f4,$b1,$7d,$be,$c8,
    $a6,$7d,$48,$fc,$d5,$e9,$8c,$9d,$88,$e4,$ea,$0d,$4c,$55,$b3,$33,$40,$ab,$3c,$20,$58,$96,$9d,$7a,$ed,
    $5b,$85,$8d,$a5,$e2,$34,$9c,$f3,$37,$a5,$98,$a4,$a0,$6e,$52,$74,$13,$14,$24,$06,$dc,$71,$95,$85,$39,
    $b1,$b1,$ef,$4c,$9e,$85,$02,$91,$78,$74,$01,$b0,$fc,$9e,$e6,$b2,$6a,$94,$ff,$58,$aa,$03,$c0,$2e,$c9,
    $6f,$f8,$8d,$6f,$74,$bd,$9c,$44,$9a,$41,$6c,$81,$00,$88,$ad,$b2,$f3,$df,$33,$5d,$a7,$0a,$d4,$c4,$4f,
    $b0,$05,$90,$38,$f4,$ce,$2a,$ef,$58,$09,$51,$eb,$7c,$2e,$8e,$08,$27,$3d,$d1,$b3,$9d,$9d,$eb,$e8,$71,
    $48,$1d,$58,$18,$21,$72,$ff,$93,$a2,$ba,$23,$76,$1e,$aa,$be,$6c,$13,$3e,$6a,$9f,$86,$52,$30,$97,$00,
    $be,$e2,$a3,$00,$70,$da,$b9,$35,$ba,$f8,$7e,$64,$35,$3e,$1a,$69,$97,$37,$a8,$0f,$c2,$49,$4b,$70,$82,
    $39,$a7,$ed,$d3,$98,$4c,$b3,$9d,$f8,$96,$d5,$e3,$f7,$f8,$ce,$b0,$dc,$b9,$ae,$0e,$b2,$c0,$88,$00,$8e,
    $5e,$9b,$b1,$be,$d6,$ff,$f3,$f5,$37,$53,$2f,$d7,$b4,$7d,$0f,$f9,$60,$8f,$bd,$3a,$08,$cb,$45,$4a,$bc,
    $39,$c6,$32,$5c,$16,$6f,$66,$08,$07,$d2,$a7,$58,$b9,$e9,$4a,$6e,$a2,$78,$49,$23,$4e,$fe,$21,$95,$b1,
    $44,$53,$43,$af,$13,$3b,$85,$80,$d3,$62,$8e,$48,$39,$1e,$20,$ba,$6b,$a3,$ef,$78,$ea,$f7,$07,$db,$69,
    $47,$94,$97,$6c,$ae,$b8,$40,$7e,$1b,$9e,$9b,$67,$1a,$2a,$49,$c2,$3f,$36,$4a,$4f,$2f,$03,$2c,$0f,$21,
    $be,$2a,$2b,$4b,$0b,$c0,$10,$93,$ba,$f2,$b2,$d1,$4b,$62,$73,$1c,$7f,$35,$67,$1f,$e4,$5f,$c1,$88,$af,
    $1b,$62,$82,$2c,$60,$d3,$48,$4c,$9a,$21,$32,$a0,$b8,$ea,$b2,$fc,$dd,$e1,$bc,$be,$8b,$2a,$db,$8f,$73,
    $94,$d6,$b4,$7c,$82,$b1,$81,$f1,$ae,$38,$53,$d7,$ad,$0e,$23,$9d,$d4,$05,$29,$99,$59,$99,$7d,$ed,$92,
    $90,$f9,$2e,$8e,$0f,$40,$6c,$b9,$04,$1e,$63,$ac,$a6,$1b,$e4,$27,$83,$29,$07,$de,$cd,$dd,$cb,$04,$98,
    $0c,$21,$42,$21,$c6,$31,$39,$f4,$6f,$50,$19,$7f,$f3,$a0,$e4,$bf,$62,$aa,$8d,$b4,$4a,$ed,$32,$32,$74,
    $26,$53,$ec,$2d,$c5,$3e,$5a,$06,$3e,$69,$4e,$66,$28,$28,$5e,$31,$89,$a1,$2a,$07,$a2,$ba,$5c,$bd,$ae,
    $10,$97,$ad,$9e,$8a,$75,$21,$95,$46,$f5,$c6,$6b,$cc,$b0,$62,$75,$45,$15,$41,$77,$20,$75,$4f,$92,$10,
    $93,$b6,$ad,$e5,$bb,$d1,$a8,$12,$63,$c0,$d9,$9f,$7b,$b7,$80,$eb,$02,$ac,$f1,$25,$dc,$cf,$54,$c5,$aa,
    $24,$4c,$2a,$23,$f3,$25,$81,$11,$02,$b3,$ba,$e2,$4e,$9d,$c9,$6e,$0a,$c3,$87,$bd,$a7,$4c,$0a,$97,$23,
    $b9,$ee,$13,$7b,$d6,$8c,$6c,$25,$b2,$7a,$fe,$23,$54,$4b,$5e,$23,$9f,$90,$00,$bb,$a1,$f8,$0a,$bd,$b0,
    $d6,$28,$6e,$c0,$ba,$35,$eb,$f7,$e7,$ec,$39,$de,$43,$a5,$0a,$8e,$cf,$06,$ae,$b1,$bf,$e9,$eb,$e7,$7d,
    $e5,$be,$4c,$86,$ee,$65,$00,$5d,$2a,$2a,$3b,$48,$85,$8b,$33,$fc,$6e,$e6,$76,$aa,$35,$54,$69,$35,$79,
    $12,$fb,$37,$78,$fb,$2a,$03,$71,$42,$40,$e4,$fb,$bc,$9c,$94,$8f,$4c,$75,$23,$8e,$87,$08,$93,$ad,$97,
    $98,$14,$b0,$e9,$08,$42,$70,$cb,$c1,$4b,$b8,$e2,$3b,$67,$df,$aa,$64,$71,$8b,$e5,$dd,$ed,$56,$e8,$05,
    $43,$2f,$28,$ce,$bb,$19,$49,$77,$cc,$60,$e0,$d0,$ea,$73,$89,$a4,$d8,$ab,$62,$c0,$4d,$05,$02,$e3,$7d,
    $73,$e0,$47,$a7,$fd,$56,$45,$9f,$c1,$59,$0e,$04,$bd,$38,$16,$22,$94,$ac,$32,$ee,$3a,$51,$3b,$d6,$c5,
    $a8,$dc,$f1,$3b,$68,$5f,$7f,$3f,$ba,$bd,$52,$f2,$ef,$a1,$dc,$f8,$f4,$b1,$4c,$77,$34,$15,$71,$19,$ca,
    $54,$9d,$d3,$ce,$95,$f2,$9a,$f0,$11,$9f,$4e,$8e,$c9,$1d,$e1,$09,$6d,$e6,$47,$a6,$b7,$60,$97,$09,$b9,
    $14,$b2,$3b,$bf,$2b,$37,$87,$a1,$20,$ce,$bc,$d8,$48,$47,$40,$c5,$cc,$b0,$4c,$73,$2b,$6c,$91,$f9,$81,
    $af,$a3,$1c,$4a,$ce,$7c,$62,$2d,$3b,$ce,$4e,$a7,$f3,$71,$71,$5d,$54,$5a,$47,$fd,$a6,$53,$ce,$60,$a7,
    $2c,$04,$40,$59,$5c,$cc,$d0,$f2,$fa,$d8,$74,$f4,$2d,$11,$3c,$c2,$bb,$ea,$2b,$9c,$dd,$4a,$8e,$87,$27,
    $6d,$2d,$c2,$45,$35,$aa,$c9,$43,$21,$4e,$dc,$ca,$12,$57,$e5,$23,$f4,$a9,$3b,$63,$9b,$3d,$d4,$79,$30,
    $c8,$2e,$dd,$76,$72,$a8,$2c,$e2,$e9,$58,$98,$93,$a1,$2c,$0f,$fb,$aa,$01,$db,$9d,$28,$1f,$f9,$60,$0e,
    $78,$03,$0d,$bf,$f6,$be,$00,$dc,$c4,$d6,$2e,$79,$9c,$6e,$76,$b1,$33,$87,$2c,$96,$c4,$3b,$53,$74,$f5,
    $6c,$7f,$3c,$ad,$20,$11,$8d,$3d,$5e,$49,$7a,$b1,$00,$2a,$6b,$15,$7a,$9c,$a2,$4c,$b7,$c1,$81,$07,$41,
    $57,$4c,$7d,$40,$b1,$4e,$70,$b8,$ca,$e2,$76,$76,$d6,$f3,$02,$a6,$30,$16,$ff,$5b,$c6,$c9,$41,$88,$8b,
    $88,$6f,$83,$98,$25,$ce,$c1,$3b,$35,$52,$99,$ed,$53,$f9,$8e,$29,$bb,$59,$85,$66,$06,$a4,$42,$97,$64,
    $a4,$a3,$6b,$1b,$f1,$a5,$f9,$cf,$18,$eb,$51,$5b,$b7,$d7,$d1,$5e,$a0,$51,$54,$61,$d6,$99,$33,$4c,$26,
    $7c,$5a,$6d,$7c,$62,$a3,$1c,$fb,$82,$d7,$95,$cd,$a3,$db,$1e,$7d,$2e,$de,$96,$56,$49,$aa,$ad,$27,$b1,
    $a2,$b2,$3f,$3e,$e0,$05,$b2,$f6,$e2,$ef,$07,$d1,$da,$60,$b1,$c4,$61,$71,$f6,$9e,$96,$20,$45,$1d,$f3,
    $59,$fe,$b4,$75,$d9,$36,$6d,$32,$04,$0a,$54,$17,$34,$27,$69,$46,$ec,$7c,$51,$e5,$45,$9f,$60,$00,$d9,
    $c1,$1b,$6b,$c7,$6b,$28,$68,$89,$1e,$c8,$c4,$4c,$06,$95,$1e,$47,$66,$71,$9e,$93,$ed,$f3,$c9,$dd,$12,
    $69,$02,$79,$5d,$24,$8d,$72,$69,$5f,$97,$9f,$d4,$02,$ae,$69,$8a,$a2,$b5,$49,$0a,$9a,$e8,$4c,$de,$16,
    $67,$36,$8c,$d1,$85,$21,$d0,$11,$d0,$ca,$2e,$82,$3a,$e0,$75,$47,$cc,$92,$22,$5a,$a0,$01,$f6,$17,$43,
    $db,$a3,$76,$a0,$94,$2f,$6b,$f3,$8b,$b8,$10,$a2,$5f,$b8,$08,$97,$c0,$0d,$84,$8e,$e8,$63,$e4,$09,$8c,
    $50,$1d,$9e,$09,$00,$d6,$8e,$d5,$f8,$b5,$63,$9e,$2b,$b4,$7a,$43,$71,$80,$24,$30,$57,$a9,$89,$18,$25,
    $46,$6d,$cb,$b0,$fe,$19,$8f,$77,$3b,$fb,$d5,$d0,$8c,$86,$50,$65,$6d,$88,$32,$48,$0f,$fe,$64,$dc,$03,
    $b0,$69,$12,$9f,$2e,$bb,$7a,$50,$7e,$62,$b5,$31,$3e,$3d,$0f,$c9,$bc,$18,$37,$42,$03,$ae,$77,$c7,$66,
    $cc,$d8,$62,$15,$7b,$ad,$cd,$1b,$15,$b4,$35,$0c,$e8,$35,$b9,$06,$0a,$21,$6c,$87,$6b,$ed,$84,$7c,$13,
    $51,$bd,$d4,$33,$72,$e4,$50,$1e,$d1,$cc,$f5,$e8,$ac,$d0,$c0,$9a,$6b,$14,$ad,$48,$1d,$cf,$23,$24,$42,
    $e7,$dd,$a0,$39,$7d,$b1,$e6,$b6,$1d,$3f,$65,$f3,$f9,$b7,$ad,$e3,$ec,$c9,$e7,$b8,$a8,$5d,$0a,$19,$c1,
    $4c,$d9,$16,$a9,$55,$51,$0e,$02,$13,$c7,$f2,$37,$04,$5b,$31,$00,$b5,$3c,$51,$85,$d8,$60,$f3,$67,$b8,
    $47,$56,$e7,$86,$57,$49,$48,$fb,$cb,$e3,$81,$f6,$0d,$9d,$9f,$11,$20,$46,$4e,$2b,$4d,$9b,$27,$f6,$ac,
    $01,$96,$cb,$16,$f5,$bd,$98,$fd,$f4,$43,$9e,$44,$12,$f4,$75,$f0,$f9,$25,$d0,$cb,$74,$15,$a4,$cb,$e3,
    $c8,$21,$e0,$09,$69,$c2,$3a,$40,$d1,$f3,$6b,$08,$48,$1d,$4e,$8c,$80,$67,$1a,$f8,$7f,$40,$fd,$ec,$ca,
    $c2,$ae,$14,$54,$35,$d6,$a3,$c7,$a3,$ef,$dc,$3b,$38,$31,$9f,$7e,$5c,$7c,$94,$39,$06,$ae,$e6,$fc,$0f,
    $c5,$fc,$42,$3b,$6f,$f7,$1d,$8d,$f7,$35,$52,$92,$33,$39,$5c,$20,$04,$6e,$cd,$62,$f1,$c2,$fe,$e0,$65,
    $3e,$46,$f4,$20,$5a,$c4,$16,$cf,$f9,$90,$57,$1b,$17,$50,$8e,$74,$0e,$b5,$9a,$20,$73,$0e,$e5,$7b,$4a,
    $ee,$a6,$cb,$7d,$9d,$df,$6e,$01,$5f,$83,$86,$8c,$1a,$a7,$1c,$ec,$8a,$c3,$db,$eb,$93,$05,$1d,$cc,$93,
    $a1,$41,$07,$22,$8c,$7c,$6c,$c7,$2c,$0f,$d4,$ce,$0e,$8f,$9c,$3c,$00,$ef,$68,$f5,$c8,$54,$1f,$05,$a6,
    $43,$f0,$37,$3b,$12,$3a,$c6,$db,$96,$a9,$dd,$e7,$71,$f9,$5d,$43,$25,$24,$3f,$3a,$2d,$c2,$ed,$a7,$09,
    $18,$b7,$96,$3e,$75,$b8,$a1,$b2,$43,$67,$8e,$4e,$55,$2f,$6d,$7c,$a9,$3f,$68,$72,$b5,$f1,$9b,$7b,$7f,
    $11,$d1,$99,$34,$a1,$45,$38,$84,$0e,$28,$d2,$b9,$69,$83,$32,$20,$a5,$49,$b5,$d2,$21,$22,$16,$9f,$6a,
    $aa,$c3,$89,$6a,$b8,$ca,$36,$b3,$cf,$5d,$72,$97,$24,$c5,$6c,$1c,$6a,$3e,$1a,$9e,$a1,$a6,$8a,$8e,$dc,
    $f0,$91,$25,$9b,$8c,$a0,$d2,$c9,$1f,$7b,$8e,$ac,$e8,$f8,$46,$03,$8f,$a1,$92,$e1,$58,$e7,$58,$a2,$2c,
    $cd,$8f,$ec,$e9,$d9,$f9,$52,$e1,$1e,$d7,$8e,$55,$02,$05,$9d,$4e,$37,$a5,$d0,$6c,$35,$b6,$3b,$5d,$f5,
    $c4,$1d,$8d,$ae,$5b,$3d,$5f,$53,$16,$c8,$19,$bf,$ec,$c5,$e1,$df,$d2,$7f,$c1,$1e,$34,$9d,$66,$bc,$4b,
    $03,$99,$37,$3e,$15,$1e,$d6,$7c,$49,$ef,$3b,$0f,$46,$33,$a1,$6e,$21,$06,$3f,$85,$3c,$1f,$3d,$ad,$29,
    $60,$9f,$b0,$7b,$4a,$24,$f7,$a2,$97,$f9,$a2,$d3,$84,$4b,$5c,$d1,$d5,$cb,$32,$eb,$e1,$99,$e0,$00,$b9,
    $8b,$55,$f8,$a6,$53,$b5,$ec,$71,$fa,$e9,$00,$6a,$b4,$d8,$0d,$1c,$3d,$95,$63,$c6,$ec,$68,$7a,$7f,$1f,
    $a6,$66,$55,$9c,$ac,$28,$be,$95,$c0,$85,$6d,$37,$8a,$93,$c8,$c2,$23,$3c,$be,$5f,$b2,$4e,$15,$a1,$c5,
    $98,$69,$3f,$f3,$3d,$06,$73,$c4,$73,$3e,$b2,$f3,$ae,$f2,$8a,$38,$de,$13,$75,$0f,$84,$54,$37,$b8,$ef,
    $41,$82,$e8,$20,$88,$f0,$f6,$52,$f6,$8e,$31,$e3,$66,$9d,$5d,$53,$00,$1f,$b0,$2a,$d6,$3b,$ee,$d7,$30,
    $a1,$f4,$3c,$19,$3a,$97,$21,$70,$0d,$a2,$ab,$ca,$b4,$84,$c3,$aa,$b0,$f3,$73,$9d,$00,$90,$7e,$c0,$0d,
    $88,$af,$ea,$92,$ae,$d6,$0e,$d6,$25,$d8,$aa,$5a,$86,$b4,$1b,$1f,$d5,$ff,$d4,$fc,$51,$37,$59,$fc,$11,
    $34,$70,$a0,$a0,$a2,$f6,$80,$e2,$15,$23,$41,$35,$13,$93,$6a,$cb,$6c,$01,$cd,$3b,$b5,$1b,$a1,$02,$1b,
    $bf,$38,$af,$4e,$72,$7e,$c6,$d4,$22,$62,$05,$71,$9f,$9e,$71,$27,$f7,$b7,$c1,$44,$36,$82,$ca,$be,$2c,
    $ca,$0e,$41,$b2,$46,$bc,$1f,$c9,$3c,$ef,$3e,$c4,$1d,$83,$2d,$27,$c2,$14,$34,$b3,$25,$f6,$3b,$18,$ce,
    $56,$e7,$6c,$c7,$50,$a9,$f3,$b0,$19,$3e,$83,$8e,$63,$51,$e6,$b2,$ad,$c1,$ef,$85,$42,$8a,$3c,$dd,$53,
    $5c,$a3,$1c,$a8,$fd,$4f,$e0,$b5,$6f,$7b,$f6,$19,$af,$c8,$bd,$c2,$d9,$57,$a6,$80,$e9,$bc,$47,$58,$d2,
    $9c,$4c,$bd,$a4,$fa,$0c,$e9,$57,$a4,$9d,$2d,$00,$57,$d3,$8d,$5a,$96,$77,$37,$67,$47,$a1,$7b,$c4,$df,
    $50,$c2,$a9,$af,$82,$c3,$c4,$a8,$4f,$a8,$c0,$64,$5a,$4b,$64,$68,$79,$da,$47,$30,$a1,$4b,$6b,$b9,$cc,
    $03,$f6,$bc,$e1,$83,$90,$22,$b0,$ca,$9a,$94,$32,$1d,$a6,$5b,$94,$8e,$e3,$01,$3e,$06,$d0,$5f,$e1,$65,
    $6a,$93,$31,$60,$bf,$2c,$ee,$38,$8f,$08,$55,$bf,$33,$5c,$6b,$b3,$4c,$aa,$57,$1c,$1e,$49,$0e,$26,$f4,
    $41,$1d,$50,$65,$45,$b1,$df,$cf,$7d,$fd,$69,$ac,$4e,$13,$5b,$f2,$c8,$bc,$42,$ef,$1c,$f2,$8e,$c7,$b1,
    $3d,$3d,$2e,$77,$b9,$be,$e4,$e9,$19,$fd,$f5,$8e,$08,$84,$22,$68,$50,$17,$3f,$4b,$ed,$a9,$52,$f1,$86,
    $2d,$a5,$bc,$ee,$c9,$9c,$a2,$7f,$86,$3a,$e1,$55,$e5,$bc,$26,$15,$34,$77,$27,$a2,$7f,$2b,$c4,$1b,$f6,
    $fd,$68,$8e,$15,$c6,$c6,$05,$72,$82,$9b,$df,$2d,$45,$44,$cd,$4f,$e2,$b1,$0e,$50,$74,$58,$02,$59,$ee,
    $9e,$a0,$c2,$b3,$5b,$3a,$0b,$0a,$6a,$c5,$18,$71,$2f,$97,$b3,$ef,$dd,$59,$ca,$3b,$85,$94,$74,$a0,$d4,
    $3f,$bc,$b6,$69,$59,$40,$c4,$58,$34,$f3,$2c,$1d,$9e,$12,$20,$be,$8c,$9c,$9f,$71,$d4,$8e,$d0,$27,$61,
    $7a,$7d,$18,$59,$f2,$7b,$cc,$d6,$ff,$b7,$84,$39,$4e,$4d,$80,$a5,$73,$57,$83,$cd,$1e,$a7,$70,$36,$80,
    $50,$ab,$24,$ff,$0c,$b0,$ef,$b2,$45,$c5,$9e,$2e,$53,$1e,$2a,$0a,$df,$0b,$1d,$23,$ce,$0a,$02,$57,$81,
    $93,$58,$b2,$7d,$27,$2c,$af,$a7,$27,$c0,$49,$8c,$85,$21,$72,$d1,$dc,$f9,$18,$c3,$89,$34,$d2,$46,$df,
    $4c,$ba,$17,$45,$7b,$9d,$aa,$58,$bd,$d9,$37,$ac,$57,$51,$5d,$36,$e3,$4c,$82,$65,$84,$3c,$ec,$ef,$49,
    $69,$e2,$92,$87,$8c,$38,$e9,$fe,$5d,$04,$6d,$80,$13,$36,$91,$2f,$d7,$a6,$12,$1c,$47,$d4,$41,$2b,$78,
    $52,$8e,$ca,$b2,$33,$c1,$21,$08,$0b,$c1,$cc,$94,$2f,$97,$c7,$73,$16,$7d,$62,$cd,$3d,$50,$42,$d2,$c5,
    $cb,$b7,$09,$f2,$2d,$e6,$32,$79,$fe,$e9,$ef,$ba,$04,$c7,$18,$cf,$6a,$75,$e5,$13,$e9,$64,$ed,$9a,$12,
    $83,$1a,$fa,$89,$21,$96,$1a,$8c,$fc,$42,$8e,$9d,$bd,$f2,$52,$42,$f3,$00,$40,$24,$3b,$6b,$19,$0f,$f8,
    $da,$a8,$ce,$95,$24,$28,$f5,$7e,$2d,$23,$3c,$ff,$39,$bd,$4f,$be,$0d,$0e,$77,$1b,$bb,$72,$de,$8a,$b5,
    $dc,$d5,$c0,$8a,$ef,$69,$39,$14,$e3,$6d,$b0,$c4,$e2,$37,$d9,$14,$e9,$7a,$7a,$89,$17,$71,$4b,$46,$62,
    $af,$9a,$1a,$5a,$f6,$e3,$65,$b8,$86,$e4,$aa,$e3,$0f,$6e,$0d,$7f,$f3,$b7,$f2,$ab,$a5,$1f,$8e,$d2,$68,
    $ad,$2f,$4a,$63,$33,$0d,$a8,$f5,$c6,$4e,$e4,$8b,$56,$9f,$b4,$31,$45,$87,$79,$33,$65,$02,$f8,$19,$ca,
    $31,$3b,$d7,$2e,$03,$48,$f9,$cd,$c2,$0a,$71,$3f,$5b,$e4,$fd,$84,$8c,$13,$8c,$33,$c1,$41,$f1,$d3,$02,
    $70,$13,$06,$cf,$65,$0f,$57,$b1,$80,$f8,$4a,$46,$51,$7f,$74,$55,$c4,$4d,$23,$ed,$72,$cd,$b6,$e8,$71,
    $3f,$9a,$48,$c9,$28,$ce,$c3,$92,$65,$09,$7c,$be,$a1,$cc,$67,$16,$1f,$4b,$87,$78,$ef,$de,$9d,$8f,$46,
    $87,$4e,$58,$03,$65,$39,$22,$00,$64,$1c,$31,$ee,$bb,$7f,$44,$dc,$cf,$30,$58,$0a,$65,$d9,$f8,$fb,$78,
    $45,$82,$f1,$d5,$a1,$11,$2a,$ef,$58,$29,$1f,$63,$88,$31,$62,$68,$18,$cd,$48,$b2,$c0,$3a,$1e,$05,$45,
    $92,$b3,$a4,$f3,$1a,$e7,$97,$48,$28,$4a,$ff,$8f,$44,$4d,$c7,$ba,$d6,$12,$91,$0d,$a9,$b7,$e3,$52,$ed,
    $12,$0c,$71,$a1,$4b,$23,$0a,$9c,$47,$ad,$0e,$5b,$6d,$a3,$00,$ea,$7e,$e2,$7c,$ff,$05,$c3,$99,$3a,$9d,
    $ad,$6c,$23,$6f,$83,$97,$e0,$17,$2b,$4d,$9d,$4c,$2b,$27,$1b,$3d,$a3,$8d,$62,$46,$8c,$e2,$05,$3b,$5e,
    $4e,$7a,$08,$52,$da,$f6,$cc,$81,$ac,$79,$29,$98,$1c,$b1,$12,$5f,$98,$bf,$05,$2e,$d9,$9f,$d3,$b2,$68,
    $15,$92,$21,$5f,$66,$56,$3b,$db,$33,$89,$21,$f7,$6d,$e7,$87,$eb,$12,$5a,$1f,$bd,$ef,$ec,$4c,$82,$5c,
    $bd,$1c,$8f,$3d,$99,$49,$4a,$74,$a4,$48,$a3,$5c,$c2,$25,$0a,$fc,$03,$7a,$5f,$79,$65,$a4,$a3,$3d,$c0,
    $5f,$f2,$68,$22,$fe,$44,$06,$ae,$70,$45,$5a,$3d,$64,$af,$ad,$22,$93,$b3,$a4,$70,$6e,$74,$02,$f0,$d0,
    $2c,$67,$b6,$0a,$c1,$10,$d7,$4d,$4a,$c2,$f6,$3f,$3b,$cd,$51,$3f,$f6,$ee,$43,$d7,$3b,$ab,$2e,$6c,$9b,
    $06,$05,$04,$c2,$3c,$34,$e4,$8f,$a4,$83,$43,$21,$19,$f6,$57,$bb,$8f,$4a,$24,$bb,$b2,$ef,$e5,$e4,$76,
    $b1,$aa,$fe,$88,$41,$f2,$85,$fb,$ac,$b5,$07,$c2,$85,$6e,$14,$d8,$45,$fd,$fe,$38,$e3,$bf,$45,$e1,$1f,
    $16,$59,$4a,$67,$d1,$54,$4a,$24,$08,$21,$38,$f0,$67,$21,$0b,$68,$3f,$1f,$d0,$e1,$50,$8d,$54,$0c,$28,
    $0a,$c2,$6c,$82,$ff,$68,$19,$eb,$70,$c8,$1f,$a3,$88,$a5,$ad,$ee,$30,$eb,$88,$0d,$34,$43,$7d,$09,$99,
    $22,$bb,$19,$8d,$d3,$1a,$93,$2b,$c0,$87,$17,$8e,$09,$d5,$b7,$c7,$f6,$ac,$e5,$e2,$4b,$89,$63,$4f,$ee,
    $fa,$c9,$fc,$96,$4f,$28,$8b,$00,$bd,$4e,$15,$5f,$1d,$87,$ef,$57,$47,$e4,$d5,$96,$f4,$a3,$07,$7e,$4f,
    $39,$cd,$66,$6e,$2a,$91,$04,$61,$22,$7b,$c3,$83,$b6,$01,$09,$ba,$69,$83,$f1,$56,$e8,$69,$cb,$56,$4a,
    $a5,$07,$b8,$6e,$f2,$bc,$f9,$3a,$a1,$ae,$d8,$e6,$26,$22,$8b,$1b,$d5,$65,$35,$3b,$dc,$74,$f7,$5a,$2c,
    $f8,$b8,$4a,$d7,$78,$e8,$fd,$d5,$f5,$fb,$85,$66,$69,$30,$38,$63,$66,$65,$fe,$0b,$6f,$4c,$aa,$31,$a0,
    $f2,$c6,$ca,$68,$16,$65,$4e,$93,$e4,$b6,$04,$ae,$c8,$a2,$6a,$61,$57,$62,$23,$19,$df,$5b,$bc,$d0,$76,
    $8c,$86,$8a,$d3,$ff,$12,$6c,$60,$86,$f4,$80,$95,$f9,$f8,$9b,$d0,$4c,$e5,$f2,$cc,$5c,$a2,$00,$72,$ce,
    $43,$0f,$2d,$a1,$6c,$87,$2c,$fe,$e8,$5e,$23,$6e,$28,$49,$75,$00,$6e,$d8,$ef,$fe,$a0,$44,$bc,$67,$50,
    $a8,$98,$20,$af,$81,$79,$38,$6b,$0b,$ea,$64,$df,$c8,$49,$6a,$dd,$b9,$78,$c5,$78,$89,$9b,$02,$6f,$77,
    $16,$e2,$d9,$b5,$9c,$7c,$4d,$5f,$cd,$50,$64,$fe,$af,$2c,$0e,$8f,$27,$5a,$0b,$5f,$da,$60,$c4,$e2,$e8,
    $10,$0e,$7f,$c5,$03,$f7,$0c,$4a,$82,$8e,$53,$a3,$24,$12,$06,$88,$a4,$3b,$2e,$7e,$47,$40,$4a,$33,$52,
    $79,$eb,$d1,$47,$b0,$03,$da,$b6,$f7,$97,$ff,$f0,$42,$b1,$4f,$23,$1d,$7e,$9f,$73,$e6,$40,$7f,$26,$33,
    $2d,$13,$29,$bc,$da,$f5,$e9,$60,$82,$72,$5a,$30,$2b,$08,$ef,$3f,$46,$93,$e8,$b8,$5f,$f2,$1b,$1b,$ec,
    $6c,$3a,$5f,$27,$b6,$40,$7d,$80,$7a,$be,$7c,$25,$7c,$42,$06,$86,$fa,$ef,$cd,$f6,$c9,$b3,$43,$68,$e6,
    $c7,$c1,$53,$1b,$9c,$f7,$d7,$ec,$fc,$3e,$8f,$ae,$1a,$86,$62,$c9,$84,$c7,$32,$b1,$56,$49,$08,$a4,$d7,
    $af,$65,$7a,$b3,$4a,$6f,$d8,$5f,$be,$8b,$d0,$68,$17,$bf,$fa,$0a,$de,$6a,$1a,$90,$3b,$d2,$19,$f2,$b9,
    $e7,$0e,$6e,$44,$5e,$84,$96,$61,$b8,$df,$4c,$b3,$8e,$76,$f2,$d4,$4f,$da,$0d,$e3,$ad,$a1,$12,$84,$df,
    $31,$68,$9f,$29,$e6,$3e,$43,$01,$88,$2a,$00,$d5,$fb,$04,$29,$7a,$e1,$0c,$b9,$5c,$0c,$44,$77,$ba,$16,
    $7c,$dc,$af,$8d,$51,$58,$20,$99,$5b,$c8,$68,$b1,$5c,$51,$45,$2a,$6d,$f8,$71,$54,$fa,$9a,$34,$b5,$77,
    $5b,$e4,$c1,$ee,$e2,$85,$2d,$c6,$50,$81,$ed,$44,$17,$20,$61,$34,$f0,$e8,$4b,$77,$1d,$d4,$67,$ca,$59,
    $cf,$60,$56,$33,$ac,$f0,$cb,$28,$c3,$dd,$fa,$60,$c8,$4a,$39,$75,$f8,$a5,$ae,$ea,$ac,$f0,$db,$c4,$23,
    $00,$9c,$06,$de,$5e,$67,$94,$ba,$1d,$7c,$73,$fe,$b7,$fe,$c0,$8e,$9a,$39,$db,$2c,$24,$a8,$5f,$13,$ce,
    $24,$e0,$09,$99,$26,$05,$48,$45,$5d,$f6,$99,$37,$dc,$c3,$8a,$e1,$e9,$3f,$46,$97,$93,$c5,$05,$a7,$3f,
    $3c,$b8,$1d,$a9,$97,$e5,$db,$98,$d7,$37,$2b,$55,$e4,$db,$80,$5f,$a1,$08,$48,$fd,$2b,$73,$f8,$f9,$d8,
    $5d,$a2,$73,$a7,$1c,$a4,$6c,$d2,$8c,$4a,$35,$89,$26,$ff,$bd,$ab,$fb,$66,$1d,$17,$eb,$66,$f2,$a8,$39,
    $22,$14,$28,$c5,$8d,$7e,$e2,$c1,$bf,$55,$ef,$43,$4c,$98,$86,$45,$d1,$b7,$e9,$3a,$16,$40,$79,$eb,$a7,
    $56,$91,$4c,$f6,$22,$9b,$48,$7a,$9c,$24,$0b,$79,$4b,$91,$dc,$44,$09,$35,$dc,$d7,$89,$f1,$20,$d2,$38,
    $5f,$c0,$50,$64,$39,$bd,$b7,$c3,$54,$31,$9c,$00,$4a,$16,$dc,$3b,$2a,$e9,$a8,$07,$26,$04,$45,$2b,$cf,
    $87,$75,$1b,$f5,$87,$8f,$15,$a0,$8c,$1d,$be,$8f,$59,$0b,$cb,$ad,$69,$f1,$f1,$89,$5d,$59,$56,$a6,$71,
    $46,$48,$33,$7b,$f1,$78,$1f,$47,$26,$58,$10,$38,$c1,$21,$91,$3b,$d7,$f0,$ae,$bc,$e0,$48,$d7,$d6,$56,
    $75,$09,$ac,$52,$17,$6e,$fc,$52,$ee,$0a,$cd,$03,$28,$aa,$df,$a3,$0c,$54,$56,$40,$6e,$e7,$76,$32,$4c,
    $41,$c5,$a4,$dc,$07,$0b,$c3,$f4,$e2,$88,$99,$75,$9d,$3b,$30,$e5,$7e,$fe,$54,$72,$f9,$d3,$37,$92,$be,
    $45,$b7,$a1,$78,$f8,$32,$cc,$c4,$22,$c5,$b0,$05,$0e,$80,$41,$bf,$fc,$49,$6d,$6c,$f2,$d1,$7d,$01,$de,
    $79,$ce,$27,$d7,$34,$e8,$d9,$d6,$a0,$2c,$79,$3b,$6c,$5c,$66,$50,$db,$a8,$16,$8f,$8d,$de,$52,$4e,$8f,
    $53,$b3,$78,$02,$cd,$28,$3f,$e9,$25,$59,$fd,$9a,$3e,$a4,$cd,$af,$2b,$88,$80,$78,$70,$5f,$3a,$b0,$d9,
    $43,$e5,$ce,$b6,$35,$50,$c3,$6e,$e3,$41,$3e,$48,$41,$15,$90,$7b,$73,$8e,$b0,$9a,$7e,$de,$1a,$bf,$dc,
    $0a,$8b,$2b,$2d,$3d,$88,$a0,$06,$3b,$ac,$f8,$81,$54,$88,$ed,$ae,$21,$36,$9e,$e4,$fd,$a7,$2c,$5c,$95,
    $98,$73,$b6,$14,$b5,$95,$9d,$2e,$c1,$91,$1f,$44,$36,$5d,$a2,$12,$93,$53,$8b,$a1,$55,$8a,$58,$eb,$ee,
    $3f,$5d,$cf,$ac,$19,$f1,$8b,$b3,$2c,$c5,$ad,$c5,$6e,$0b,$13,$35,$5c,$12,$ff,$12,$32,$54,$56,$9a,$5a,
    $aa,$a4,$72,$8e,$8b,$f1,$b2,$5e,$d2,$07,$84,$89,$fb,$ba,$96,$ab,$83,$c7,$f3,$86,$c6,$c0,$52,$71,$8c,
    $7f,$76,$2b,$f6,$7d,$91,$99,$8e,$59,$14,$ce,$3c,$fa,$e0,$e4,$b2,$0c,$c6,$24,$56,$d4,$dd,$20,$e6,$2e,
    $dc,$46,$4d,$eb,$ea,$c2,$79,$57,$f0,$cb,$fc,$6c,$f0,$7f,$87,$6b,$28,$e7,$6f,$9b,$e1,$ed,$82,$2f,$b4,
    $ac,$6c,$70,$0d,$b1,$55,$08,$b6,$ab,$c9,$32,$bd,$3d,$4f,$7e,$15,$39,$56,$88,$49,$1c,$76,$5f,$4b,$41,
    $dd,$e6,$c9,$55,$8e,$e1,$7b,$61,$47,$37,$25,$76,$f9,$1b,$fe,$59,$85,$8b,$04,$fa,$4d,$02,$21,$ae,$93,
    $61,$7d,$dd,$bb,$13,$f0,$dd,$5f,$33,$1e,$47,$1f,$59,$c5,$c7,$60,$ff,$a4,$fa,$15,$59,$41,$0a,$31,$69,
    $75,$a6,$42,$46,$92,$bc,$af,$25,$6d,$d0,$89,$c9,$ab,$81,$7a,$4d,$f3,$1b,$96,$2d,$ba,$fd,$a9,$9e,$7f,
    $27,$6f,$1a,$8a,$27,$e0,$c7,$6c,$24,$28,$ff,$b6,$41,$b6,$21,$cc,$54,$f6,$23,$45,$c1,$a3,$0b,$f2,$c9,
    $57,$67,$39,$bc,$d8,$c8,$b9,$7c,$3a,$66,$54,$17,$b1,$1d,$df,$47,$4a,$33,$a8,$c6,$85,$7f,$0f,$d1,$0c,
    $90,$3b,$c1,$e7,$13,$c9,$b0,$ca,$fe,$70,$b2,$e2,$7c,$2d,$03,$80,$d3,$c4,$36,$f1,$2f,$20,$f7,$9b,$78,
    $d2,$b9,$72,$e1,$e6,$3b,$0d,$18,$cb,$61,$94,$73,$1b,$2b,$da,$f3,$91,$e5,$ef,$66,$32,$a2,$09,$b0,$a3,
    $60,$bb,$71,$ef,$34,$ec,$8a,$be,$d5,$bf,$ec,$8a,$02,$fe,$43,$ea,$94,$ee,$9c,$2e,$c4,$fc,$6e,$23,$26,
    $fb,$9d,$7e,$19,$76,$d9,$6e,$5d,$1d,$80,$30,$9e,$91,$dc,$51,$cd,$9f,$94,$ac,$e4,$74,$e6,$84,$31,$8c,
    $8a,$22,$65,$9e,$b5,$99,$d4,$87,$f9,$f9,$a3,$67,$d0,$6c,$53,$54,$17,$58,$23,$62,$8a,$a3,$57,$b9,$1c,
    $86,$5e,$de,$a7,$83,$50,$38,$2d,$f6,$13,$61,$a3,$eb,$bd,$92,$a7,$d0,$2e,$0c,$02,$16,$91,$6f,$cd,$f1,
    $99,$dd,$f7,$49,$bf,$91,$92,$52,$5e,$33,$70,$cd,$ab,$04,$db,$43,$bc,$b4,$24,$f5,$8f,$54,$90,$16,$4a,
    $63,$dd,$63,$8e,$77,$ad,$67,$ec,$1e,$a7,$d4,$21,$f6,$5e,$96,$af,$e9,$ba,$a2,$51,$40,$56,$7a,$6c,$95,
    $c2,$e0,$42,$45,$95,$b3,$c8,$eb,$22,$f7,$5f,$25,$1d,$be,$8d,$0a,$03,$34,$cf,$00,$9d,$27,$56,$2a,$30,
    $53,$53,$73,$93,$66,$13,$5d,$2f,$3a,$8b,$7e,$6e,$97,$ba,$20,$7f,$7e,$c7,$50,$fe,$a5,$34,$32,$bb,$00,
    $e3,$ef,$8f,$de,$39,$2f,$03,$20,$dc,$9f,$18,$29,$03,$0e,$ed,$80,$a0,$ff,$46,$87,$df,$7e,$20,$d9,$1f,
    $77,$d6,$9b,$a9,$0e,$91,$78,$b1,$7e,$84,$59,$19,$df,$44,$b0,$89,$08,$a4,$b6,$0c,$a2,$59,$f1,$92,$ec,
    $54,$5b,$3c,$bf,$77,$7f,$00,$fe,$94,$b5,$5c,$f7,$7c,$b6,$86,$17,$db,$f3,$bd,$54,$ca,$03,$27,$eb,$a2,
    $fa,$f8,$70,$d7,$11,$92,$31,$ab,$23,$88,$55,$8e,$7c,$eb,$37,$6a,$bf,$c8,$fb,$10,$99,$5c,$3b,$bf,$d6,
    $75,$25,$06,$70,$ef,$82,$f5,$be,$16,$7e,$c8,$f3,$84,$21,$c5,$dd,$76,$86,$ae,$99,$22,$99,$b3,$32,$02,
    $31,$14,$6f,$90,$7a,$94,$68,$a3,$f6,$0d,$37,$d7,$a4,$e1,$20,$7d,$d3,$d8,$40,$20,$5b,$4c,$d0,$29,$19,
    $9c,$15,$20,$92,$71,$d7,$94,$48,$2e,$9e,$5c,$08,$9e,$68,$01,$d2,$1b,$c1,$9c,$4c,$f8,$e7,$1b,$da,$35,
    $29,$5b,$db,$7a,$16,$fd,$9c,$26,$f0,$0e,$ae,$a9,$bf,$06,$9c,$5e,$25,$88,$5e,$54,$a4,$dd,$c4,$41,$33,
    $e7,$f5,$b1,$84,$fd,$d1,$eb,$6d,$39,$79,$12,$f6,$43,$e2,$26,$53,$f5,$4b,$7c,$5d,$35,$a6,$4c,$01,$0a,
    $04,$c0,$4b,$00,$45,$3c,$98,$81,$f4,$0a,$9b,$8a,$65,$34,$07,$aa,$32,$a2,$d0,$a2,$79,$0a,$22,$cb,$a7,
    $ed,$df,$b2,$30,$81,$8a,$ad,$7b,$d1,$07,$11,$2e,$40,$8c,$05,$ec,$93,$4c,$f8,$fd,$f2,$71,$52,$44,$9a,
    $d8,$a7,$f1,$44,$f4,$80,$a3,$5c,$23,$ff,$0e,$26,$6f,$d2,$9a,$00,$0b,$ea,$7e,$65,$66,$ca,$f6,$ef,$de,
    $71,$61,$0a,$9a,$b8,$ea,$6b,$c3,$7b,$df,$ce,$58,$77,$fe,$53,$83,$cb,$f0,$cd,$d5,$80,$bd,$b2,$ce,$02,
    $43,$bf,$c9,$fe,$97,$ed,$0d,$87,$6b,$49,$fc,$41,$71,$bf,$de,$a1,$a9,$e2,$71,$a8,$a5,$1e,$5e,$e6,$7f,
    $61,$67,$20,$43,$4f,$99,$41,$95,$33,$57,$eb,$af,$6a,$a4,$79,$e9,$20,$2d,$c0,$54,$94,$4e,$83,$8e,$fc,
    $f5,$48,$97,$58,$84,$8c,$16,$82,$cb,$8b,$41,$35,$b8,$2a,$66,$2f,$00,$1c,$d3,$2d,$13,$8a,$f1,$37,$da,
    $48,$ef,$c9,$d0,$63,$05,$1a,$c3,$63,$9e,$6e,$f0,$88,$0b,$13,$d4,$d5,$3b,$33,$dc,$2f,$ef,$4c,$01,$15,
    $1f,$6a,$7f,$0c,$be,$4a,$12,$9f,$9f,$d2,$6a,$e2,$5c,$5f,$13,$14,$9c,$cd,$22,$bb,$29,$d2,$bd,$ed,$15,
    $9a,$9f,$49,$d3,$d0,$45,$13,$9c,$b1,$95,$da,$d7,$a5,$37,$8c,$7a,$23,$ff,$cb,$22,$71,$2c,$89,$65,$34,
    $04,$e2,$a5,$d8,$fb,$f9,$e9,$14,$fa,$9b,$77,$ed,$84,$7b,$2d,$1c,$8b,$f8,$dc,$fe,$06,$71,$2e,$23,$f0,
    $f7,$d8,$24,$fa,$1e,$76,$61,$c7,$94,$2c,$82,$ad,$81,$8f,$fe,$08,$ea,$c9,$07,$5d,$4f,$8d,$80,$0b,$f8,
    $28,$96,$45,$56,$0a,$68,$8d,$39,$50,$0a,$8e,$97,$bd,$81,$b8,$5b,$1c,$ce,$63,$bb,$54,$0a,$e9,$e4,$f7,
    $e8,$b9,$33,$4f,$54,$39,$f2,$40,$91,$54,$89,$bb,$a7,$30,$06,$59,$b9,$e0,$f1,$7e,$a9,$28,$0e,$ee,$39,
    $89,$c1,$da,$9a,$a4,$99,$cc,$df,$b3,$fc,$e7,$de,$87,$cc,$24,$43,$3e,$e3,$f1,$88,$ac,$fb,$51,$0e,$fc,
    $a1,$cc,$9f,$32,$5d,$c1,$05,$96,$23,$af,$21,$c9,$ce,$ff,$48,$eb,$4b,$8f,$5e,$77,$27,$e9,$1b,$cc,$fe,
    $55,$37,$0f,$77,$ea,$a2,$15,$44,$d7,$8a,$4d,$7b,$d3,$a6,$02,$82,$39,$32,$b7,$86,$c2,$c2,$ba,$0a,$71,
    $2e,$60,$cb,$49,$5f,$89,$94,$23,$2c,$38,$76,$54,$40,$53,$81,$68,$7b,$fb,$c0,$ca,$9f,$5c,$f8,$95,$fd,
    $39,$19,$0d,$8f,$b7,$6d,$7a,$af,$e4,$94,$63,$2f,$ac,$b9,$df,$7a,$1f,$ef,$a8,$b9,$dd,$2c,$72,$1c,$dc,
    $3e,$95,$20,$84,$b0,$b4,$ff,$5a,$2f,$34,$e0,$3f,$7f,$af,$d6,$59,$70,$d2,$4d,$b8,$2f,$8a,$60,$96,$6c,
    $f0,$9e,$40,$2d,$c7,$0e,$ef,$79,$64,$52,$f9,$20,$09,$f2,$ea,$f0,$8c,$26,$ae,$18,$8f,$44,$3c,$77,$6e,
    $19,$cd,$4f,$f9,$d2,$c8,$f8,$20,$85,$34,$64,$50,$f3,$40,$6e,$f6,$66,$e3,$c9,$ff,$b9,$a1,$cc,$c5,$ea,
    $43,$e7,$ca,$2f,$80,$08,$90,$e5,$78,$cc,$9a,$86,$97,$42,$61,$7a,$73,$7f,$22,$a0,$ee,$df,$62,$f7,$56,
    $f7,$2f,$ed,$88,$e0,$12,$2b,$17,$6c,$21,$a5,$d3,$12,$89,$4c,$4e,$2a,$12,$9b,$b9,$57,$7e,$6d,$aa,$bf,
    $e1,$f4,$6a,$aa,$dd,$1f,$62,$0b,$40,$7a,$b1,$45,$8e,$6c,$75,$e1,$9c,$20,$1c,$2c,$12,$4a,$cb,$5a,$b2,
    $ef,$98,$32,$80,$df,$a7,$ed,$12,$c0,$ee,$df,$37,$75,$1b,$35,$b8,$ad,$cd,$57,$c5,$8a,$06,$26,$54,$b8,
    $3c,$93,$bc,$6a,$c6,$ee,$40,$32,$7b,$49,$97,$38,$b4,$13,$3c,$8e,$5b,$a7,$a5,$7a,$42,$df,$0a,$f1,$c7,
    $b7,$ef,$ca,$ff,$40,$8e,$6f,$2a,$86,$b8,$b5,$69,$9e,$b1,$d7,$42,$78,$1b,$04,$10,$b2,$64,$52,$f5,$e9,
    $da,$2e,$57,$44,$5a,$94,$15,$df,$7d,$53,$56,$e9,$ff,$64,$c0,$aa,$56,$04,$3b,$ec,$d7,$5f,$11,$34,$13,
    $0a,$10,$33,$4e,$ef,$fa,$28,$00,$1b,$87,$42,$e4,$6f,$e0,$c5,$9b,$f2,$52,$fa,$b7,$38,$a8,$89,$98,$4a,
    $7c,$f7,$b1,$9b,$65,$29,$4a,$db,$7a,$36,$09,$66,$32,$03,$56,$b6,$e8,$d7,$47,$a7,$f2,$65,$db,$9c,$31,
    $bd,$e3,$32,$c7,$70,$6b,$17,$e2,$75,$42,$86,$83,$d4,$05,$22,$6e,$9a,$af,$89,$9d,$68,$d0,$4b,$a4,$72,
    $8b,$58,$e5,$c1,$71,$c2,$87,$84,$3d,$7b,$2c,$73,$7e,$31,$bd,$35,$31,$db,$28,$a3,$8c,$9b,$07,$c6,$58,
    $38,$7f,$ee,$8f,$7f,$6a,$e9,$ed,$d2,$50,$5f,$32,$cb,$af,$95,$cd,$3d,$06,$ed,$ed,$bf,$c6,$ce,$20,$fc,
    $72,$6a,$c4,$f4,$ae,$c6,$59,$59,$a8,$e3,$7e,$9e,$d4,$48,$e0,$99,$63,$83,$a9,$64,$d6,$ed,$75,$28,$4a,
    $a6,$fd,$e5,$83,$ee,$a8,$95,$57,$02,$dc,$0f,$94,$96,$94,$eb,$b5,$94,$ac,$ac,$e4,$86,$06,$22,$3c,$27,
    $a4,$7a,$e7,$08,$31,$e1,$4b,$2d,$6d,$bf,$90,$4b,$54,$4b,$97,$f1,$bd,$ff,$3a,$32,$1d,$eb,$af,$c1,$15,
    $99,$39,$55,$0e,$13,$eb,$a1,$b4,$cf,$df,$f2,$cb,$4a,$58,$51,$5e,$ba,$aa,$86,$44,$6d,$ec,$49,$ed,$3f,
    $01,$c1,$68,$8b,$51,$67,$70,$38,$6f,$7e,$87,$02,$ff,$7b,$a6,$3e,$1e,$e1,$ea,$7e,$0f,$0c,$9a,$97,$bc,
    $fe,$f3,$69,$a1,$b3,$35,$4e,$98,$18,$84,$24,$98,$7c,$b2,$db,$79,$6b,$7c,$e7,$10,$c9,$8e,$34,$f9,$6c,
    $90,$86,$a8,$53,$7d,$6e,$30,$f8,$a9,$23,$41,$2d,$6a,$cd,$4e,$b4,$c9,$5a,$cc,$01,$54,$4b,$0a,$4f,$9e,
    $d7,$35,$97,$b4,$74,$50,$92,$84,$94,$d9,$6a,$50,$4b,$5f,$6a,$fe,$6a,$66,$a7,$bc,$28,$8b,$9c,$fd,$05,
    $13,$05,$2c,$2c,$a3,$44,$2d,$36,$bb,$7a,$b3,$bb,$47,$19,$d2,$00,$a2,$46,$bc,$65,$c2,$1a,$c4,$06,$29,
    $33,$e0,$f8,$af,$ea,$fb,$4d,$c4,$9d,$cd,$40,$bd,$ba,$9b,$b9,$4f,$3b,$98,$aa,$f4,$7b,$84,$f9,$c3,$ec,
    $02,$31,$d3,$bd,$0a,$e6,$99,$ab,$70,$6c,$df,$5a,$2f,$4c,$73,$d5,$ee,$94,$3f,$87,$5f,$8c,$d0,$49,$65,
    $57,$87,$43,$d5,$7f,$c6,$7e,$98,$18,$2f,$10,$bb,$e2,$88,$17,$59,$7d,$b2,$62,$3f,$8a,$53,$45,$99,$95,
    $fe,$f4,$1b,$d8,$fa,$d1,$b8,$db,$bd,$19,$11,$29,$33,$6f,$a0,$a2,$d4,$99,$1c,$a2,$f6,$14,$bc,$6f,$2b,
    $83,$6d,$f7,$5c,$d5,$bc,$a9,$9a,$2f,$a9,$b8,$90,$35,$c3,$0e,$d5,$d4,$23,$2d,$4b,$10,$10,$62,$5b,$8e,
    $65,$e6,$32,$12,$64,$95,$5d,$8b,$fc,$41,$fb,$55,$df,$96,$52,$9f,$8b,$ef,$2c,$ba,$e3,$e6,$ba,$f2,$03,
    $ca,$ce,$3c,$9a,$90,$f9,$b7,$66,$4f,$f8,$14,$f2,$ef,$5d,$e9,$f9,$44,$8a,$ba,$ab,$79,$e4,$a2,$c5,$b4,
    $48,$49,$5e,$42,$56,$f5,$db,$db,$8d,$92,$b9,$d1,$71,$72,$3b,$1e,$e6,$9c,$e7,$86,$4a,$c0,$00,$9f,$53,
    $c7,$c5,$71,$cd,$c1,$11,$7f,$cd,$5a,$76,$93,$62,$7c,$b2,$08,$21,$e1,$d3,$4c,$00,$6d,$f2,$ac,$83,$72,
    $01,$23,$a1,$7f,$f3,$df,$79,$2a,$d7,$4e,$e9,$e5,$3d,$56,$86,$6b,$79,$c7,$52,$16,$fa,$fd,$43,$1d,$b6,
    $ba,$4d,$f0,$8d,$52,$9a,$7f,$d9,$99,$6c,$3f,$71,$78,$9f,$e5,$41,$7f,$7b,$16,$d5,$c1,$e9,$07,$2d,$55,
    $88,$9b,$96,$3c,$0d,$0b,$e7,$79,$1e,$ca,$48,$97,$49,$2f,$90,$01,$c2,$2c,$16,$0b,$68,$35,$5a,$72,$ba,
    $b2,$be,$d8,$9b,$62,$2d,$b6,$68,$8f,$76,$bf,$12,$77,$9b,$03,$95,$3e,$30,$a2,$2b,$9e,$8d,$96,$85,$84,
    $03,$3e,$1b,$5f,$28,$06,$d0,$01,$de,$6d,$0a,$77,$3f,$ff,$f6,$d1,$7a,$3f,$80,$ec,$e1,$55,$62,$0f,$d6,
    $86,$31,$30,$64,$0d,$82,$03,$93,$7a,$60,$f0,$12,$29,$d8,$67,$93,$57,$5c,$08,$ee,$ba,$86,$d1,$12,$3e,
    $b8,$67,$7c,$64,$82,$9e,$98,$bf,$e6,$f9,$76,$a3,$58,$c3,$24,$4e,$a1,$15,$ec,$f0,$4c,$5e,$ab,$46,$6c,
    $49,$79,$0a,$b1,$83,$6a,$17,$bd,$a9,$3f,$06,$11,$4f,$55,$f2,$f8,$5b,$8d,$16,$cf,$83,$70,$61,$3a,$44,
    $34,$d6,$62,$7a,$56,$2c,$0a,$9c,$e3,$29,$da,$88,$4d,$0a,$b8,$c5,$4f,$5b,$26,$9b,$68,$e9,$ad,$64,$30,
    $e4,$a5,$3c,$ac,$98,$7f,$f6,$7f,$17,$71,$ea,$ef,$02,$82,$17,$84,$5c,$63,$ce,$f9,$bf,$41,$66,$2f,$e1,
    $54,$8c,$c7,$28,$5c,$60,$fb,$31,$c0,$b2,$aa,$45,$47,$05,$c7,$c0,$a9,$ba,$73,$4a,$3f,$09,$8d,$c7,$5e,
    $73,$a7,$f2,$81,$b2,$17,$7e,$64,$37,$e3,$51,$c1,$09,$0f,$52,$97,$07,$d5,$bf,$a5,$96,$cf,$18,$0c,$d1,
    $eb,$77,$0d,$e9,$94,$07,$2b,$8d,$3a,$d7,$ae,$e6,$a5,$c4,$df,$50,$47,$d3,$7b,$fa,$7f,$a2,$c7,$9b,$62,
    $e7,$64,$a9,$f7,$56,$22,$01,$46,$ef,$5f,$79,$c2,$0a,$81,$7d,$44,$f3,$40,$bc,$cb,$7c,$de,$98,$24,$00,
    $93,$7e,$6a,$8f,$a7,$4a,$9f,$a0,$a7,$3f,$7c,$98,$74,$95,$51,$9e,$4a,$c4,$16,$46,$c1,$0b,$2a,$f3,$14,
    $55,$87,$2e,$9a,$a4,$bc,$f2,$70,$8d,$0c,$76,$be,$83,$00,$49,$96,$09,$0b,$c5,$4e,$22,$a3,$25,$f4,$71,
    $b8,$47,$5d,$6a,$61,$89,$c2,$b8,$c5,$00,$2b,$58,$9a,$7d,$d0,$36,$bd,$78,$46,$b3,$49,$8a,$7b,$57,$1f,
    $5e,$2e,$3a,$55,$44,$fe,$35,$30,$a6,$d0,$85,$91,$57,$ac,$1f,$e7,$b7,$a5,$b9,$06,$ce,$63,$9d,$03,$d1,
    $fd,$36,$ea,$9a,$bc,$88,$5a,$f8,$c9,$39,$4b,$b5,$4d,$18,$8b,$ef,$e8,$3e,$26,$de,$bc,$db,$e9,$eb,$51,
    $67,$52,$11,$99,$c6,$5c,$ef,$23,$00,$44,$68,$21,$de,$6f,$ef,$df,$a3,$cc,$be,$45,$08,$e6,$9f,$00,$e7,
    $e9,$a3,$72,$f3,$17,$2c,$b1,$de,$4d,$3e,$74,$67,$b8,$88,$05,$3b,$fe,$5e,$fd,$bc,$e0,$c6,$28,$78,$56,
    $dd,$8b,$42,$9d,$aa,$e5,$d9,$c6,$11,$3d,$f4,$3d,$f6,$40,$3b,$46,$7d,$56,$1c,$9b,$f1,$01,$c0,$af,$bf,
    $7e,$05,$09,$cf,$01,$2b,$46,$a8,$83,$a7,$fb,$29,$66,$53,$9f,$d3,$eb,$84,$c0,$30,$c3,$63,$25,$57,$7f,
    $f0,$7a,$4a,$2d,$9f,$d4,$4c,$87,$5e,$69,$20,$20,$17,$37,$72,$ab,$08,$5e,$b1,$14,$00,$03,$9d,$a9,$31,
    $01,$f9,$ac,$45,$1d,$fe,$ca,$6e,$87,$ed,$84,$5e,$cb,$b6,$d9,$fb,$f7,$e4,$24,$67,$3d,$7c,$90,$09,$98,
    $21,$c2,$64,$37,$b9,$a5,$4f,$51,$56,$77,$28,$ae,$f8,$74,$7b,$f8,$06,$bf,$00,$8c,$87,$2e,$63,$ef,$53,
    $59,$34,$f8,$53,$c8,$c2,$81,$99,$c4,$4c,$83,$aa,$2a,$7d,$c6,$ac,$c1,$1c,$86,$0c,$a4,$73,$8c,$94,$3a,
    $16,$d5,$58,$d6,$34,$66,$dd,$55,$d0,$d2,$70,$18,$64,$22,$b5,$ff,$46,$a9,$f9,$3f,$1e,$98,$70,$07,$a5,
    $24,$be,$cc,$3a,$4c,$a3,$b9,$b6,$27,$ea,$8f,$66,$8b,$be,$2d,$70,$6d,$f0,$2c,$2e,$8d,$f6,$07,$f7,$f7,
    $55,$3a,$cc,$9c,$27,$34,$9d,$01,$ba,$c7,$fe,$7d,$cc,$e3,$2b,$8c,$7e,$2e,$0b,$8b,$e1,$7f,$5c,$9e,$ac,
    $0c,$fe,$90,$35,$c0,$7e,$f2,$ed,$d0,$0c,$80,$f7,$9d,$f7,$58,$35,$10,$8e,$e8,$d3,$21,$72,$54,$63,$a1,
    $40,$05,$9e,$9c,$24,$c7,$fb,$a1,$80,$29,$6b,$6e,$e8,$06,$5f,$35,$4a,$61,$98,$98,$98,$e8,$ec,$e0,$75,
    $34,$4a,$24,$4f,$34,$04,$b6,$9c,$f6,$9a,$d0,$77,$4d,$30,$6b,$68,$be,$e3,$65,$f6,$86,$6c,$59,$fb,$66,
    $ed,$20,$4b,$53,$16,$ed,$b3,$69,$d9,$a7,$17,$ac,$2a,$d0,$4b,$b4,$b4,$f0,$34,$e0,$a0,$2f,$2b,$79,$25,
    $87,$e6,$6f,$80,$1c,$dd,$72,$78,$73,$4f,$c3,$46,$10,$9f,$9a,$cb,$e3,$87,$cc,$6b,$41,$e4,$5a,$c2,$f2,
    $d1,$64,$10,$85,$2a,$59,$82,$03,$7a,$93,$f9,$10,$0a,$ae,$0c,$59,$36,$8b,$13,$b1,$b5,$2c,$fb,$06,$ac,
    $f8,$0d,$c6,$a9,$00,$dc,$0f,$31,$28,$b5,$87,$27,$ab,$f7,$e4,$e0,$9d,$ec,$d8,$33,$19,$2a,$02,$58,$8b,
    $b4,$72,$88,$8e,$83,$43,$10,$71,$87,$f0,$72,$dc,$da,$c7,$41,$f4,$3d,$d8,$c7,$d9,$33,$5b,$11,$ca,$c8,
    $dc,$58,$5a,$5f,$15,$1f,$44,$07,$78,$c6,$48,$2e,$2a,$97,$57,$56,$82,$2e,$13,$61,$e0,$fe,$89,$a1,$1d,
    $7b,$b0,$d2,$b4,$92,$01,$77,$aa,$52,$35,$3b,$f5,$1a,$03,$78,$42,$08,$51,$47,$d0,$42,$9b,$d7,$2c,$97,
    $5e,$d6,$0c,$2e,$ba,$37,$f0,$6c,$f7,$69,$4b,$ba,$b0,$e7,$ea,$79,$39,$b9,$e0,$0a,$56,$a8,$7f,$6a,$73,
    $01,$4a,$48,$1d,$98,$ce,$f5,$b0,$61,$3d,$14,$a1,$8e,$f3,$34,$fc,$03,$03,$fe,$84,$e9,$22,$b0,$39,$f0,
    $42,$c1,$c7,$63,$ce,$2c,$1a,$7f,$1e,$5f,$c5,$02,$83,$21,$0e,$d5,$6c,$84,$0b,$ca,$db,$cb,$2e,$9a,$99,
    $6a,$fd,$2f,$04,$fc,$f5,$5e,$ae,$60,$7a,$7b,$4d,$af,$22,$80,$56,$43,$7b,$78,$a2,$14,$ea,$62,$02,$8c,
    $16,$91,$64,$9e,$ed,$6b,$58,$2f,$b0,$9d,$63,$25,$0d,$13,$d7,$30,$b4,$21,$0b,$ff,$e8,$6d,$5a,$f2,$44,
    $16,$27,$64,$46,$4a,$54,$41,$20,$bd,$ae,$16,$f2,$c6,$47,$17,$a3,$02,$ab,$23,$c7,$54,$eb,$b8,$01,$2c,
    $e9,$d8,$2c,$9b,$11,$c5,$c6,$b8,$48,$bb,$94,$23,$e2,$09,$62,$30,$eb,$de,$47,$4e,$ac,$79,$e1,$de,$1f,
    $c8,$e8,$9d,$02,$8f,$5a,$7d,$5d,$3e,$22,$55,$8e,$9d,$50,$bb,$c0,$9d,$6a,$47,$53,$1f,$c0,$a2,$2c,$3d,
    $2f,$42,$da,$82,$55,$41,$71,$5e,$d8,$3f,$37,$04,$eb,$0e,$09,$43,$30,$9f,$03,$db,$76,$3f,$b9,$00,$e5,
    $dd,$74,$10,$2f,$53,$e3,$02,$2c,$36,$90,$2f,$34,$6a,$d1,$87,$a9,$c2,$ec,$99,$93,$ec,$d8,$6c,$41,$bf,
    $e2,$30,$5f,$1f,$b2,$ef,$7c,$f2,$ce,$5f,$d6,$8d,$2f,$63,$01,$8b,$1e,$42,$5f,$9c,$77,$10,$26,$43,$1f,
    $8a,$1d,$0d,$c0,$f8,$e1,$5a,$d3,$81,$2a,$cc,$91,$34,$b8,$1f,$ee,$32,$7c,$e0,$a0,$85,$cf,$c8,$6b,$ea,
    $f1,$72,$a2,$7e,$30,$ce,$a8,$96,$fe,$ba,$78,$6e,$22,$dd,$fa,$30,$2e,$22,$2c,$2c,$bb,$a3,$5f,$a6,$48,
    $82,$c8,$53,$62,$f9,$32,$15,$11,$2c,$09,$a9,$3c,$bc,$a6,$a1,$78,$62,$06,$91,$69,$5c,$c4,$17,$37,$af,
    $3d,$24,$e6,$9a,$e4,$8e,$8b,$fe,$a9,$b6,$73,$51,$3a,$e2,$ed,$1e,$53,$61,$f7,$c2,$98,$02,$52,$cf,$2d,
    $be,$0f,$00,$d9,$85,$9e,$68,$7c,$85,$fa,$25,$8f,$15,$fc,$11,$56,$e9,$dc,$d2,$3b,$59,$bc,$d9,$25,$31,
    $ea,$82,$c8,$3a,$bf,$61,$7a,$e4,$64,$47,$22,$42,$60,$3b,$e8,$d3,$03,$6f,$68,$47,$a0,$96,$d8,$25,$76,
    $cf,$86,$4f,$6d,$f3,$8f,$97,$c6,$75,$4a,$87,$8d,$1d,$ec,$33,$20,$92,$27,$6c,$0e,$0e,$0d,$db,$32,$0f,
    $3b,$c8,$2c,$2f,$27,$e1,$3a,$5c,$32,$c5,$12,$03,$b1,$88,$80,$09,$18,$89,$e9,$29,$57,$b8,$18,$de,$bd,
    $a7,$ac,$0f,$d7,$a9,$d1,$ce,$7e,$a1,$d9,$64,$65,$ad,$fc,$e0,$7b,$3b,$bf,$95,$0e,$ec,$98,$c1,$04,$02,
    $74,$33,$cd,$cd,$cc,$58,$a0,$da,$c9,$3b,$b0,$57,$a5,$2d,$26,$24,$8b,$6e,$ad,$f9,$58,$ae,$29,$3f,$2a,
    $51,$d4,$24,$9a,$a2,$03,$0d,$19,$e2,$09,$fc,$1e,$36,$6b,$6b,$42,$a3,$47,$42,$15,$1b,$cc,$15,$1c,$0b,
    $4f,$78,$55,$7f,$fe,$4a,$8b,$68,$47,$f6,$6c,$1a,$c0,$4a,$93,$25,$80,$25,$85,$65,$6f,$50,$b5,$32,$87,
    $fe,$19,$ec,$6c,$6d,$c1,$a8,$6d,$94,$b1,$18,$1b,$07,$01,$a0,$d8,$50,$33,$1a,$1c,$e4,$fa,$55,$c7,$d6,
    $62,$df,$ec,$c6,$27,$40,$40,$21,$00,$c6,$5b,$71,$b8,$c2,$77,$4f,$e5,$8b,$b8,$94,$13,$93,$e8,$0c,$eb,
    $d9,$5b,$b7,$67,$07,$0a,$26,$36,$87,$3d,$c1,$6e,$3a,$f9,$fb,$55,$a8,$cd,$7e,$77,$a4,$83,$81,$15,$85,
    $db,$98,$cb,$56,$45,$5c,$51,$0d,$ee,$00,$db,$ef,$79,$1f,$81,$23,$6d,$a7,$09,$56,$d6,$0d,$47,$5d,$78,
    $eb,$46,$3e,$85,$71,$0d,$6e,$23,$e3,$e9,$e7,$9e,$b3,$4b,$bd,$3c,$6b,$6c,$8b,$11,$10,$c2,$a9,$bd,$99,
    $ed,$de,$1f,$79,$54,$5f,$d6,$9c,$81,$6a,$ac,$ca,$65,$10,$56,$25,$26,$5b,$ff,$6b,$75,$c0,$19,$ae,$db,
    $37,$71,$63,$5a,$5c,$cd,$57,$6f,$32,$25,$8c,$73,$d1,$0c,$7f,$52,$67,$a2,$1d,$76,$d2,$5f,$5f,$e4,$a9,
    $e1,$70,$28,$67,$84,$96,$65,$5f,$36,$0a,$09,$01,$aa,$93,$cf,$f9,$c3,$28,$a8,$88,$22,$4d,$d3,$1c,$06,
    $b6,$79,$64,$8c,$a8,$54,$bc,$4b,$e9,$bd,$61,$90,$b5,$6c,$7a,$5e,$1d,$47,$6e,$60,$f5,$19,$9d,$1b,$f0,
    $18,$ae,$49,$ed,$33,$75,$29,$00,$58,$f4,$c6,$3b,$e9,$cb,$53,$a3,$8b,$01,$ce,$70,$ac,$4d,$41,$9b,$a1,
    $4a,$23,$4b,$d7,$b1,$55,$2e,$d6,$7a,$4c,$82,$43,$85,$65,$2c,$47,$66,$6a,$ba,$4d,$0e,$5d,$5d,$04,$6d,
    $8d,$57,$af,$9c,$d4,$b4,$35,$71,$54,$0d,$b4,$0c,$f5,$33,$86,$b3,$d8,$b9,$15,$5d,$95,$dc,$3b,$74,$d6,
    $cc,$b4,$21,$e7,$3b,$19,$52,$9f,$dd,$6a,$47,$95,$68,$e1,$f6,$e9,$df,$33,$9a,$ee,$a1,$b1,$69,$6f,$50,
    $11,$4c,$c4,$b3,$0d,$32,$1c,$f3,$d2,$b4,$ac,$86,$f6,$be,$d5,$eb,$2e,$ff,$fe,$0a,$a1,$95,$21,$4c,$a5,
    $07,$97,$25,$fd,$71,$9a,$82,$fe,$3a,$29,$7a,$e8,$90,$98,$f3,$9d,$fd,$fa,$db,$79,$4b,$a2,$98,$26,$7e,
    $e7,$2d,$43,$07,$51,$6a,$d8,$44,$6c,$37,$2c,$1f,$f9,$4b,$fc,$c0,$21,$d1,$e1,$96,$56,$df,$61,$ba,$81,
    $3a,$57,$c0,$99,$c3,$2e,$9c,$9c,$9c,$37,$36,$89,$91,$f7,$2b,$1d,$56,$f3,$16,$1c,$80,$0a,$5a,$8d,$b1,
    $22,$b7,$2c,$b0,$07,$22,$e1,$4b,$d5,$65,$0d,$d4,$46,$19,$fb,$2a,$22,$36,$85,$69,$ba,$f8,$4b,$c6,$b2,
    $9a,$ec,$05,$45,$b1,$e6,$e9,$5b,$16,$75,$54,$49,$94,$36,$a1,$7f,$89,$42,$19,$07,$c0,$3b,$ec,$f6,$04,
    $20,$a1,$92,$20,$3e,$1c,$de,$ec,$d4,$d0,$9a,$9d,$b2,$58,$83,$c4,$4c,$f3,$5c,$15,$96,$c3,$0e,$11,$d5,
    $fd,$d3,$0e,$ba,$6a,$71,$cf,$0c,$56,$3a,$4d,$05,$cf,$f6,$24,$cb,$aa,$df,$fc,$86,$15,$bd,$bb,$9b,$92,
    $3e,$a7,$17,$48,$ae,$fc,$35,$59,$1f,$68,$61,$dd,$ce,$84,$b6,$3e,$36,$45,$d7,$54,$2d,$73,$07,$62,$7c,
    $9b,$29,$7d,$da,$eb,$4d,$40,$46,$0a,$20,$e0,$bb,$cb,$a0,$82,$6a,$62,$54,$5c,$1e,$90,$a6,$b0,$38,$1b,
    $a2,$58,$10,$07,$e8,$e7,$aa,$ce,$31,$d9,$be,$03,$6c,$e1,$d7,$24,$22,$71,$44,$b6,$0c,$f0,$86,$2c,$8a,
    $58,$01,$14,$74,$08,$8d,$d6,$0d,$f3,$69,$ac,$65,$fb,$f4,$cc,$41,$84,$3b,$25,$b6,$50,$9a,$80,$f7,$66,
    $7c,$69,$9a,$2a,$5a,$92,$6f,$55,$65,$62,$2e,$71,$69,$20,$b7,$bc,$f3,$f3,$8a,$6a,$84,$48,$f3,$83,$33,
    $ba,$31,$74,$6f,$b7,$fa,$e3,$d3,$27,$f5,$72,$50,$7e,$90,$fe,$05,$5c,$50,$0e,$c0,$d4,$0d,$fe,$35,$be,
    $5a,$61,$5d,$c6,$db,$2b,$bd,$6c,$33,$ab,$bc,$50,$1f,$e9,$c9,$b6,$ba,$00,$c8,$ef,$15,$a1,$fb,$b9,$d9,
    $b6,$44,$d4,$10,$e2,$ac,$60,$cd,$4b,$58,$c5,$f6,$08,$35,$11,$c7,$eb,$e1,$34,$66,$e7,$5a,$ed,$a2,$db,
    $c1,$71,$54,$5e,$1e,$83,$60,$b2,$e6,$8b,$f4,$3b,$42,$a7,$27,$e9,$a6,$e0,$d5,$4c,$f3,$d4,$a5,$c2,$32,
    $2d,$8c,$69,$61,$ab,$9b,$9f,$66,$a6,$80,$87,$23,$4d,$dd,$b8,$5b,$51,$90,$8e,$af,$78,$99,$da,$3c,$95,
    $be,$8a,$16,$b4,$41,$93,$47,$60,$1c,$7b,$02,$e1,$f7,$40,$32,$8a,$5b,$b0,$c5,$15,$59,$92,$ea,$9d,$c4,
    $9a,$dc,$61,$24,$8f,$3a,$92,$2e,$02,$97,$73,$bb,$c3,$ed,$f3,$6f,$98,$fa,$3b,$a7,$34,$7e,$3c,$6e,$9a,
    $74,$58,$c3,$2c,$63,$73,$be,$7d,$b5,$a6,$b3,$33,$fb,$07,$97,$02,$e2,$e9,$f1,$80,$f5,$b0,$5d,$c6,$64,
    $23,$63,$3f,$28,$49,$7c,$3f,$06,$08,$6d,$01,$02,$d1,$66,$0e,$96,$1d,$e4,$9d,$f4,$df,$db,$d0,$2d,$f8,
    $03,$81,$71,$c3,$43,$75,$c8,$cb,$81,$81,$0a,$92,$0d,$40,$f4,$44,$79,$b1,$d6,$53,$8b,$4d,$ef,$2c,$15,
    $dd,$2e,$cc,$49,$76,$1e,$3c,$bc,$de,$c9,$b6,$5a,$19,$98,$3b,$11,$9c,$9a,$31,$2e,$18,$c1,$2f,$2d,$81,
    $a5,$6d,$b0,$bf,$95,$ef,$43,$c7,$cd,$81,$02,$fa,$56,$4d,$c5,$f9,$d0,$b9,$03,$1a,$b2,$cf,$31,$20,$13,
    $25,$13,$de,$88,$11,$81,$02,$38,$b8,$6c,$18,$25,$69,$9c,$34,$25,$16,$f9,$67,$ce,$8c,$08,$15,$ec,$db,
    $ff,$dc,$ac,$76,$85,$9c,$94,$37,$30,$d5,$07,$01,$59,$00,$9c,$11,$44,$1e,$2d,$2d,$da,$a8,$c8,$04,$ff,
    $03,$66,$91,$ab,$bb,$b3,$60,$25,$0f,$5e,$50,$3b,$37,$c4,$c1,$cc,$6d,$28,$65,$eb,$4f,$59,$e6,$4a,$28,
    $10,$94,$13,$d7,$06,$27,$e9,$47,$21,$13,$a1,$eb,$f4,$4d,$d2,$09,$a4,$e8,$ae,$b7,$ec,$3e,$bf,$a8,$80,
    $07,$e1,$fa,$9f,$b0,$7b,$1c,$8c,$96,$b9,$d1,$9d,$26,$15,$42,$98,$91,$81,$e3,$ab,$d8,$e7,$62,$e5,$3f,
    $35,$80,$e4,$d1,$51,$27,$71,$7c,$74,$24,$87,$84,$ae,$fc,$f8,$6a,$09,$e3,$49,$ef,$c5,$25,$36,$4e,$aa,
    $db,$87,$f6,$a8,$81,$e3,$29,$c1,$5b,$61,$d2,$3f,$a0,$e4,$fb,$79,$61,$fd,$ea,$9f,$c3,$44,$1f,$30,$bd,
    $a1,$42,$17,$ca,$dc,$ea,$c0,$4e,$aa,$23,$34,$bd,$13,$f9,$ca,$3b,$9d,$25,$36,$c6,$a4,$24,$79,$eb,$47,
    $1e,$a7,$70,$e8,$02,$d4,$3e,$0d,$f3,$32,$82,$ae,$5b,$d3,$eb,$d2,$cd,$64,$71,$dd,$a3,$b3,$bd,$ee,$d7,
    $ad,$8b,$d8,$8e,$2e,$15,$9d,$9b,$1f,$9d,$87,$ea,$01,$31,$e4,$ec,$a2,$da,$75,$b8,$7b,$46,$7d,$24,$1a,
    $33,$6d,$d2,$44,$12,$d3,$13,$76,$4e,$45,$35,$7a,$ad,$39,$c5,$36,$f7,$33,$b1,$73,$21,$6d,$70,$18,$14,
    $1f,$f6,$cc,$90,$97,$03,$65,$f8,$2a,$90,$26,$0b,$01,$24,$47,$0a,$30,$8b,$30,$27,$96,$ef,$ac,$8a,$3a,
    $c4,$b4,$3b,$95,$42,$b9,$53,$1f,$f3,$69,$36,$f9,$0a,$43,$84,$64,$b2,$39,$fb,$8a,$9b,$4d,$6a,$17,$79,
    $97,$12,$e8,$af,$6c,$5c,$a8,$c9,$39,$de,$15,$80,$e5,$3a,$a5,$b0,$ac,$9e,$90,$f8,$ee,$37,$8e,$9a,$21,
    $47,$29,$8a,$93,$a5,$80,$97,$15,$54,$c3,$f8,$ed,$95,$c2,$61,$e4,$21,$49,$d0,$c1,$e8,$a6,$55,$7b,$c1,
    $e9,$44,$a3,$44,$f9,$27,$5c,$94,$09,$f6,$ff,$38,$34,$47,$c2,$8d,$ad,$c1,$0a,$a1,$81,$30,$2e,$66,$33,
    $f9,$d2,$0c,$82,$5a,$30,$d2,$4a,$94,$c6,$59,$88,$9d,$2a,$c6,$29,$60,$09,$ba,$61,$ac,$aa,$01,$60,$72,
    $6e,$e3,$98,$7d,$af,$a9,$75,$46,$1a,$2d,$df,$be,$e7,$bb,$e4,$64,$81,$91,$22,$f6,$65,$2f,$82,$8a,$2a,
    $dc,$ac,$e7,$93,$c8,$07,$c5,$f8,$ea,$49,$39,$05,$d2,$53,$0a,$d0,$25,$22,$fc,$60,$7c,$6b,$52,$05,$db,
    $b2,$13,$9b,$7e,$b3,$a2,$95,$05,$9f,$80,$fa,$b9,$fa,$f3,$35,$d9,$41,$2b,$6d,$60,$33,$e6,$bb,$f5,$fb,
    $32,$57,$c4,$14,$5b,$f0,$60,$8b,$05,$ad,$36,$f2,$9f,$8a,$bd,$ed,$93,$0b,$07,$a4,$a8,$ff,$62,$c7,$8b,
    $11,$fa,$62,$bd,$00,$b6,$e3,$d0,$6c,$d7,$b3,$19,$92,$28,$fd,$0f,$98,$c1,$36,$ee,$ad,$f5,$cb,$df,$f0,
    $6a,$ba,$be,$42,$20,$03,$88,$87,$5e,$92,$51,$5d,$cb,$56,$cb,$0b,$81,$88,$bf,$1f,$43,$ae,$a5,$90,$89,
    $1a,$c1,$6c,$8d,$8c,$02,$9e,$c3,$d1,$52,$1a,$d8,$1b,$44,$c5,$df,$db,$f3,$a2,$48,$c4,$b4,$72,$c3,$c7,
    $db,$7f,$31,$81,$aa,$9e,$39,$4c,$cd,$de,$d2,$21,$35,$ad,$4b,$d2,$a3,$27,$8a,$98,$26,$6d,$cf,$f4,$2b,
    $38,$1c,$eb,$1b,$05,$68,$06,$4e,$ae,$71,$c5,$b7,$f0,$ae,$76,$ad,$cc,$7d,$65,$0d,$19,$5c,$2f,$6f,$bb,
    $bd,$dd,$5c,$b1,$2a,$8f,$5f,$cc,$56,$15,$4c,$e9,$c9,$32,$5b,$55,$d5,$9d,$41,$23,$52,$51,$6c,$92,$19,
    $3a,$d2,$84,$cc,$8c,$76,$16,$4b,$30,$35,$71,$cf,$6c,$17,$94,$90,$25,$8d,$ff,$e7,$cf,$00,$20,$f7,$6c,
    $98,$b3,$cc,$a7,$ed,$32,$8b,$1e,$be,$8c,$ba,$dc,$d0,$32,$00,$06,$11,$28,$2f,$af,$30,$ff,$c2,$bd,$39,
    $a7,$f6,$8c,$82,$41,$25,$3c,$96,$f1,$ae,$1d,$67,$35,$1c,$d7,$3a,$f5,$e5,$30,$d3,$e6,$66,$b0,$c5,$7b,
    $12,$ac,$e0,$65,$99,$96,$01,$08,$9d,$58,$2a,$b5,$b8,$46,$59,$9b,$d4,$ec,$70,$b4,$39,$73,$ff,$69,$4a,
    $19,$e3,$18,$d7,$66,$aa,$57,$8a,$c1,$ab,$e6,$07,$ce,$2b,$23,$48,$4e,$eb,$c5,$1d,$8f,$98,$62,$60,$31,
    $c1,$86,$13,$f6,$01,$97,$ed,$22,$fe,$fb,$56,$3c,$8d,$4c,$2f,$dc,$1c,$bd,$16,$c9,$58,$d2,$5b,$3c,$4f,
    $6a,$25,$29,$c4,$2c,$2a,$ae,$6d,$4f,$40,$35,$72,$53,$41,$9a,$42,$94,$13,$a7,$ec,$06,$23,$e6,$28,$50,
    $5b,$23,$f5,$85,$da,$80,$62,$76,$ca,$e2,$75,$b1,$65,$cc,$1c,$ac,$34,$a5,$90,$71,$6b,$dc,$c9,$54,$32,
    $a6,$47,$c5,$29,$33,$51,$c9,$a1,$75,$1f,$fc,$3a,$cc,$16,$65,$02,$a3,$da,$b5,$2e,$c1,$ac,$b3,$f0,$66,
    $89,$b2,$ab,$fc,$9f,$4c,$65,$96,$c9,$2e,$b1,$d4,$f2,$a3,$1d,$25,$42,$f4,$03,$80,$06,$b4,$0d,$4c,$72,
    $16,$98,$7d,$06,$d3,$d8,$ef,$f0,$1a,$79,$5b,$a9,$90,$7f,$22,$d6,$3c,$0e,$e7,$e0,$89,$25,$14,$ce,$11,
    $72,$39,$8a,$21,$27,$0b,$0d,$68,$1f,$aa,$8a,$cc,$47,$49,$2f,$61,$e9,$b2,$aa,$20,$23,$b9,$18,$4c,$a5,
    $a6,$bd,$1c,$7f,$bb,$ab,$4f,$2a,$0f,$bf,$f0,$14,$3c,$a4,$bc,$a3,$eb,$7f,$a5,$52,$3b,$92,$91,$17,$a5,
    $78,$ae,$5a,$9f,$c3,$35,$d0,$34,$ce,$2b,$13,$a7,$88,$b2,$f6,$e8,$0b,$2b,$3b,$81,$8b,$bc,$2d,$1f,$b5,
    $28,$99,$54,$65,$45,$8b,$7f,$9b,$dd,$c8,$63,$a5,$26,$ef,$9b,$8b,$17,$86,$32,$f6,$16,$11,$5e,$ba,$1a,
    $ba,$e3,$dc,$05,$83,$13,$0c,$bf,$30,$0d,$76,$86,$c4,$5d,$4e,$00,$8b,$30,$50,$55,$6d,$11,$89,$0b,$a5,
    $45,$83,$ae,$ce,$a4,$70,$17,$db,$70,$5c,$b3,$92,$84,$e4,$55,$7e,$17,$a5,$a3,$47,$4a,$f8,$88,$0b,$6a,
    $83,$74,$44,$b9,$a6,$90,$d6,$42,$d5,$15,$c7,$5c,$41,$aa,$84,$5d,$e0,$61,$73,$df,$21,$ce,$a0,$f4,$ad,
    $c3,$82,$05,$0d,$7f,$1d,$99,$44,$46,$51,$0f,$98,$51,$de,$f3,$6f,$8d,$06,$91,$1f,$ef,$cc,$55,$08,$1e,
    $ec,$4c,$83,$12,$31,$1f,$d2,$af,$69,$b7,$8d,$e3,$80,$a4,$d8,$41,$f3,$bc,$80,$9c,$f3,$a5,$36,$e5,$d6,
    $e0,$b7,$8b,$9d,$2d,$ed,$70,$ca,$29,$14,$84,$10,$82,$bf,$57,$6c,$19,$3f,$c3,$39,$62,$28,$4f,$40,$c3,
    $fa,$85,$14,$7c,$90,$ed,$ce,$0f,$41,$4f,$94,$ab,$83,$a3,$bc,$96,$7a,$c4,$24,$82,$ab,$4e,$b4,$7a,$5d,
    $15,$1e,$2d,$18,$81,$6d,$03,$1d,$da,$9a,$bc,$95,$54,$e0,$9e,$5c,$5e,$69,$68,$cb,$bf,$85,$49,$3b,$f3,
    $16,$3e,$5f,$3e,$22,$31,$ef,$01,$e6,$7c,$b3,$69,$0e,$e9,$01,$77,$6e,$dd,$6d,$51,$dd,$8d,$99,$dd,$cc,
    $bd,$80,$4c,$67,$b0,$86,$23,$5c,$c1,$70,$c1,$30,$f2,$72,$41,$73,$a7,$3c,$7f,$9e,$a1,$02,$1b,$66,$06,
    $0c,$ff,$f2,$40,$92,$2b,$fb,$fb,$17,$e4,$db,$9f,$82,$dc,$fc,$a9,$66,$5d,$4c,$ff,$95,$f4,$f4,$af,$2a,
    $1c,$9f,$08,$74,$c0,$95,$1e,$2e,$69,$7b,$2d,$b4,$ad,$4c,$84,$e9,$c6,$7b,$18,$64,$50,$73,$d9,$15,$d4,
    $64,$78,$13,$99,$4c,$4e,$ba,$4e,$7a,$61,$ff,$31,$9f,$72,$38,$c4,$77,$08,$2d,$2f,$e6,$32,$a0,$ec,$63,
    $cf,$f8,$f1,$ac,$66,$91,$68,$fd,$f6,$f9,$0c,$c2,$7c,$8a,$72,$21,$49,$ee,$df,$85,$e6,$26,$f6,$41,$94,
    $e5,$7b,$54,$24,$8e,$be,$91,$c4,$cf,$2e,$1f,$cb,$de,$74,$cc,$55,$bc,$a8,$2b,$ad,$53,$8e,$46,$4a,$20,
    $a7,$b5,$c9,$11,$65,$db,$a1,$4e,$95,$c5,$5b,$09,$d3,$b2,$c9,$9d,$d0,$89,$77,$3c,$07,$54,$63,$59,$36,
    $42,$9d,$ad,$ea,$a7,$36,$4e,$f7,$df,$7f,$19,$d4,$61,$f0,$77,$7a,$18,$34,$96,$c9,$d1,$05,$e5,$ce,$66,
    $a8,$cd,$29,$21,$2b,$89,$69,$f6,$33,$0d,$04,$dd,$0f,$ba,$81,$1f,$0e,$19,$90,$be,$14,$68,$e7,$c3,$f8,
    $1f,$9e,$13,$c9,$91,$61,$59,$ad,$85,$99,$fa,$d6,$8d,$94,$0e,$6d,$cc,$83,$35,$70,$a1,$dd,$1f,$10,$b1,
    $40,$18,$9a,$62,$08,$df,$15,$06,$40,$33,$55,$46,$fa,$a3,$c1,$87,$2c,$ba,$db,$44,$02,$71,$7e,$3f,$91,
    $19,$f3,$a7,$79,$ae,$13,$11,$d2,$41,$f9,$fa,$4c,$35,$37,$a6,$80,$dd,$20,$08,$42,$50,$4d,$47,$40,$68,
    $b3,$45,$9c,$79,$79,$fa,$8d,$b9,$35,$2e,$bb,$8c,$61,$bf,$72,$02,$86,$26,$e0,$50,$56,$fc,$17,$97,$0b,
    $3a,$b5,$44,$c1,$ec,$42,$fd,$2a,$c6,$fc,$24,$77,$66,$8d,$80,$19,$39,$fb,$fd,$e0,$00,$0b,$63,$f5,$9e,
    $40,$cd,$96,$4b,$2d,$9d,$56,$99,$ed,$fc,$6a,$5d,$0a,$27,$0c,$b6,$b0,$c4,$0d,$a2,$d0,$f1,$7d,$f9,$1b,
    $5d,$89,$a9,$60,$93,$fd,$e8,$70,$10,$94,$ef,$8a,$66,$db,$92,$19,$a2,$75,$a9,$a8,$d9,$90,$2c,$39,$fb,
    $44,$3b,$f3,$86,$ce,$da,$4c,$65,$5e,$69,$31,$d4,$47,$bb,$cb,$89,$75,$b9,$cd,$14,$d6,$82,$8e,$79,$35,
    $c3,$50,$16,$2e,$5f,$2e,$a2,$3f,$cf,$e9,$ec,$52,$c9,$33,$71,$b3,$65,$ec,$c9,$5d,$fd,$a7,$b9,$f4,$bd,
    $15,$ba,$b7,$a0,$a9,$d9,$fe,$c0,$04,$2b,$9e,$9b,$0d,$15,$35,$0d,$69,$91,$f3,$cb,$3f,$90,$53,$d7,$e4,
    $e7,$78,$79,$ac,$e9,$59,$1d,$87,$35,$2d,$7d,$2e,$3e,$77,$8c,$9c,$07,$67,$ce,$21,$65,$59,$24,$9c,$02,
    $04,$9a,$f2,$53,$b4,$81,$32,$8e,$6a,$0b,$4f,$b9,$28,$09,$bf,$c9,$8a,$f9,$30,$73,$60,$f6,$05,$c7,$b7,
    $17,$ab,$c4,$6c,$b9,$9c,$de,$31,$13,$ee,$36,$ef,$90,$e0,$86,$c8,$a7,$b5,$6e,$57,$3d,$c1,$26,$ab,$0d,
    $8d,$72,$f9,$07,$13,$77,$6c,$de,$e6,$bf,$41,$7c,$32,$fd,$6d,$1a,$47,$34,$f4,$da,$5d,$b0,$18,$bb,$43,
    $ff,$a6,$76,$4d,$b6,$d1,$e3,$99,$53,$66,$c8,$bc,$d3,$6f,$a2,$c4,$fc,$b9,$f3,$2d,$f7,$0b,$78,$9c,$c6,
    $c1,$a1,$7e,$2f,$53,$f2,$f7,$e5,$fe,$80,$be,$ad,$c1,$16,$c7,$41,$57,$e5,$ea,$53,$8c,$76,$dd,$e2,$e1,
    $ca,$20,$b7,$d7,$f2,$01,$55,$df,$6d,$a5,$75,$d3,$e6,$8c,$37,$92,$83,$3c,$12,$70,$29,$cd,$17,$e9,$65,
    $ae,$54,$27,$d5,$7d,$8d,$20,$cc,$21,$34,$2b,$ef,$88,$92,$ab,$61,$32,$91,$4e,$f2,$08,$41,$7a,$66,$d3,
    $56,$27,$52,$80,$bf,$b3,$cc,$4d,$be,$ee,$a0,$64,$8c,$86,$af,$65,$f4,$51,$b7,$e7,$d3,$8a,$ee,$7e,$01,
    $02,$58,$b6,$5b,$68,$3b,$73,$d4,$74,$df,$65,$88,$7d,$e3,$7b,$28,$e4,$10,$a8,$5b,$bf,$65,$10,$7e,$14,
    $f0,$54,$12,$d5,$e7,$44,$09,$3f,$60,$95,$8c,$14,$74,$d1,$d1,$96,$22,$cb,$3a,$06,$9a,$fa,$9b,$56,$37,
    $d8,$b2,$06,$4c,$40,$a4,$d4,$74,$b9,$e4,$13,$b7,$6c,$0e,$a2,$0d,$17,$f2,$f6,$f9,$88,$c1,$f6,$e2,$eb,
    $74,$81,$cc,$b9,$6e,$e2,$49,$74,$c9,$bd,$0a,$43,$77,$1b,$3e,$2c,$71,$66,$02,$dc,$92,$20,$9f,$c1,$a8,
    $00,$1d,$9e,$7b,$85,$a1,$97,$92,$3a,$c7,$ce,$ea,$dd,$93,$63,$fa,$33,$cf,$2c,$3a,$12,$3d,$52,$cd,$4e,
    $25,$5b,$67,$1b,$bb,$3b,$0e,$59,$5c,$fb,$1c,$73,$82,$0a,$04,$0e,$a1,$be,$fb,$85,$f6,$21,$16,$b7,$79,
    $83,$4e,$bf,$c3,$c0,$1f,$2f,$a2,$74,$c0,$ce,$95,$43,$ca,$84,$4b,$82,$06,$54,$be,$10,$a4,$e8,$25,$d3,
    $c3,$c9,$16,$f4,$29,$79,$ff,$9e,$64,$52,$16,$71,$80,$d5,$22,$60,$ab,$3a,$bf,$58,$3d,$5e,$78,$48,$cf,
    $d0,$24,$16,$46,$6f,$be,$ac,$77,$fc,$e3,$c0,$86,$14,$71,$0c,$35,$a2,$bf,$ec,$0e,$a8,$8a,$ed,$cb,$90,
    $5b,$ea,$a7,$57,$ab,$4e,$91,$3f,$a0,$1e,$59,$12,$8b,$a5,$58,$7e,$e2,$e0,$06,$51,$0c,$c0,$28,$78,$b0,
    $15,$33,$3d,$36,$02,$13,$52,$5f,$ff,$de,$84,$30,$cf,$a2,$ca,$e2,$0a,$39,$ca,$34,$6c,$f7,$ac,$ca,$77,
    $54,$b0,$0c,$6e,$22,$d0,$fd,$81,$b1,$7f,$ad,$08,$4b,$9d,$b9,$b7,$18,$25,$64,$cc,$93,$99,$04,$6a,$44,
    $6a,$94,$4e,$2f,$a8,$42,$77,$af,$33,$16,$ee,$8e,$3c,$66,$1f,$bb,$2e,$28,$bf,$3f,$28,$9e,$24,$7b,$ee,
    $3e,$c3,$0b,$bd,$60,$45,$e5,$8d,$81,$89,$69,$2b,$99,$2c,$61,$b2,$7c,$f3,$cb,$f2,$f0,$f1,$63,$03,$fd,
    $7a,$9d,$03,$e6,$26,$09,$e5,$49,$59,$2a,$70,$54,$3f,$ee,$a3,$39,$89,$91,$7a,$01,$0f,$44,$6f,$ef,$cf,
    $f3,$30,$d8,$66,$cf,$0d,$5c,$f4,$43,$da,$d8,$34,$46,$05,$6a,$38,$48,$20,$b5,$d8,$90,$45,$24,$91,$7c,
    $69,$e8,$26,$5c,$9c,$6b,$50,$29,$0c,$47,$65,$50,$05,$b0,$62,$cd,$f4,$ff,$ef,$a8,$f2,$a7,$dd,$72,$de,
    $26,$f7,$32,$34,$55,$0d,$15,$e9,$c6,$60,$7d,$e1,$1e,$50,$2e,$4b,$95,$e0,$8e,$19,$7b,$a4,$60,$d0,$5d,
    $b4,$68,$73,$c9,$97,$fc,$a9,$5e,$75,$be,$bb,$6d,$9a,$fe,$1b,$2c,$64,$78,$d6,$2c,$39,$72,$28,$25,$93,
    $c1,$2b,$90,$1f,$43,$72,$16,$57,$4e,$4d,$17,$7f,$e4,$7a,$75,$8d,$4b,$b0,$6e,$75,$25,$77,$5b,$f9,$fa,
    $f6,$20,$56,$f3,$c3,$b7,$fb,$6c,$07,$f0,$0a,$fc,$ae,$ca,$8a,$9b,$e9,$f3,$4b,$c7,$3a,$94,$e5,$63,$c9,
    $c1,$a0,$d6,$49,$00,$dd,$88,$18,$c3,$c9,$99,$4a,$d4,$24,$4c,$0e,$f9,$0f,$fe,$e2,$21,$60,$48,$84,$0c,
    $81,$6b,$6a,$7c,$34,$d9,$02,$88,$a0,$1c,$56,$34,$de,$9d,$00,$e2,$c5,$e9,$ea,$aa,$04,$86,$55,$b9,$6c,
    $8e,$72,$56,$b8,$cb,$bc,$68,$52,$d1,$0a,$9f,$be,$31,$e0,$38,$51,$27,$29,$3c,$5a,$b7,$52,$39,$81,$e6,
    $e7,$5d,$c5,$29,$1d,$66,$a7,$cd,$ab,$6d,$7f,$38,$07,$01,$4d,$11,$3f,$7b,$d4,$49,$88,$a7,$87,$16,$89,
    $ea,$5d,$05,$3c,$e5,$c5,$60,$09,$e5,$92,$ee,$8c,$cd,$5d,$c6,$4f,$9d,$bc,$e4,$e0,$14,$0f,$0e,$34,$22,
    $67,$b9,$69,$d5,$be,$2a,$0e,$1b,$61,$34,$a5,$d1,$8c,$06,$80,$49,$1b,$03,$e2,$94,$d0,$15,$53,$74,$72,
    $1a,$e6,$97,$00,$b9,$e9,$6a,$11,$ab,$ab,$65,$2b,$d0,$53,$48,$3f,$f9,$b3,$9e,$ad,$cb,$d0,$9d,$6c,$5c,
    $07,$4d,$e0,$02,$aa,$2d,$53,$8e,$47,$1e,$5b,$b2,$2a,$08,$19,$13,$36,$cc,$95,$ec,$b2,$40,$cd,$c8,$a2,
    $8d,$be,$a7,$3b,$b9,$d3,$98,$92,$e1,$c5,$75,$6e,$44,$34,$45,$cd,$bd,$14,$37,$ec,$e1,$02,$ce,$02,$21,
    $dc,$cd,$b4,$2a,$58,$89,$b0,$7c,$6c,$75,$fa,$bc,$7b,$e4,$59,$28,$8d,$2a,$fa,$c7,$31,$b5,$bd,$57,$24,
    $de,$1e,$06,$d4,$54,$a6,$e1,$47,$8d,$43,$5c,$42,$11,$71,$97,$75,$45,$91,$d2,$3f,$b4,$1e,$64,$c9,$ce,
    $63,$60,$f5,$73,$66,$2c,$dc,$87,$23,$61,$ef,$69,$89,$c8,$7d,$b5,$e7,$f0,$00,$f5,$e5,$0c,$dd,$6c,$c5,
    $58,$ea,$e4,$ae,$20,$ef,$1f,$95,$e7,$64,$db,$b3,$0f,$68,$f4,$7a,$dc,$9b,$83,$ce,$08,$6b,$b6,$87,$a5,
    $6f,$69,$8d,$e1,$df,$be,$95,$4a,$6c,$a6,$4d,$d0,$c8,$c0,$fc,$1f,$53,$74,$27,$bd,$25,$c3,$40,$5d,$c2,
    $b5,$cc,$f6,$69,$d5,$76,$b4,$5c,$f9,$07,$f2,$33,$9b,$09,$8c,$ae,$25,$b8,$27,$08,$3d,$76,$18,$8f,$a4,
    $65,$96,$e4,$37,$32,$54,$c1,$23,$48,$eb,$cc,$e7,$ca,$85,$1d,$8f,$f4,$12,$ec,$8f,$06,$22,$57,$34,$ae,
    $a3,$e7,$cf,$a5,$7c,$8b,$3a,$98,$4d,$d2,$d6,$8e,$76,$d2,$6e,$e7,$7a,$34,$19,$25,$55,$37,$4c,$90,$0d,
    $a1,$bd,$e6,$f5,$2a,$90,$7c,$24,$8d,$17,$9e,$cf,$d2,$e6,$01,$6e,$48,$43,$24,$6c,$6d,$ca,$25,$a4,$26,
    $d0,$1d,$c2,$9c,$5e,$f5,$1c,$59,$46,$59,$a1,$70,$ee,$8a,$60,$67,$a5,$27,$70,$7d,$a3,$fd,$4d,$66,$06,
    $9e,$6f,$72,$6c,$8f,$6a,$2a,$e8,$c5,$f5,$8d,$8b,$d0,$90,$e5,$7a,$36,$46,$31,$74,$a7,$89,$2a,$6b,$e5
 {$endif}
 );

{---------------------------------------------------------------------------}
function AddrPiBytes: pointer;
  {-Pointer to bits of Pi, big endian format}
begin
  AddrPiBytes := @PiBytes;
end;


{---------------------------------------------------------------------------}
{---------------------------------------------------------------------------}
const
  L2Bytes: packed array[1..MAX_TCBITS div 8] of byte = ( {calculated with t_ciln2}
    $b1,$72,$17,$f7,$d1,$cf,$79,$ab,$c9,$e3,$b3,$98,$03,$f2,$f6,$af,$40,$f3,$43,$26,$72,$98,$b6,$2d,$8a,
    $0d,$17,$5b,$8b,$aa,$fa,$2b,$e7,$b8,$76,$20,$6d,$eb,$ac,$98,$55,$95,$52,$fb,$4a,$fa,$1b,$10,$ed,$2e,
    $ae,$35,$c1,$38,$21,$44,$27,$57,$3b,$29,$11,$69,$b8,$25,$3e,$96,$ca,$16,$22,$4a,$e8,$c5,$1a,$cb,$da,
    $11,$31,$7c,$38,$7e,$b9,$ea,$9b,$c3,$b1,$36,$60,$3b,$25,$6f,$a0,$ec,$76,$57,$f7,$4b,$72,$ce,$87,$b1,
    $9d,$65,$48,$ca,$f5,$df,$a6,$bd,$38,$30,$32,$48,$65,$5f,$a1,$87,$2f,$20,$e3,$a2,$da,$2d,$97,$c5,$0f,
    $3f,$d5,$c6,$07,$f4,$ca,$11,$fb,$5b,$fb,$90,$61,$0d,$30,$f8,$8f,$e5,$51,$a2,$ee,$56,$9d,$6d,$fc,$1e,
    $fa,$15,$7d,$2e,$23,$de,$14,$00,$b3,$96,$17,$46,$07,$75,$db,$89,$90,$e5,$c9,$43,$e7,$32,$b4,$79,$cd,
    $33,$cc,$cc,$4e,$65,$93,$93,$51,$4c,$4c,$1a,$1e,$0b,$d1,$d6,$09,$5d,$25,$66,$9b,$33,$35,$64,$a3,$37,
    $6a,$9c,$7f,$8a,$5e,$14,$8e,$82,$07,$4d,$b6,$01,$5c,$fe,$7a,$a3,$0c,$48,$0a,$54,$17,$35,$0d,$2c,$95,
    $5d,$51,$79,$b1,$e1,$7b,$9d,$ae,$31,$3c,$db,$6c,$60,$6c,$b1,$07,$8f,$73,$5d,$1b,$2d,$b3,$1b,$5f,$50,
    $b5,$18,$50,$64,$c1,$8b,$4d,$16,$2d,$b3,$b3,$65,$85,$3d,$75,$98,$a1,$95,$1a,$e2,$73,$ee,$55,$70,$b6,
    $c6,$8f,$96,$98,$34,$96,$d4,$e6,$d3,$30,$af,$88,$9b,$44,$a0,$25,$54,$73,$1c,$dc,$8e,$a1,$72,$93,$d1,
    $22,$8a,$4e,$f9,$8d,$6f,$51,$77,$fb,$cf,$07,$55,$26,$8a,$5c,$1f,$95,$38,$b9,$82,$61,$af,$fd,$44,$6b,
    $1c,$a3,$cf,$5e,$92,$22,$b8,$8c,$66,$d3,$c5,$42,$21,$83,$ed,$c9,$94,$21,$09,$0b,$bb,$16,$fa,$f3,$d9,
    $49,$f2,$36,$e0,$2b,$20,$ce,$e8,$86,$b9,$05,$c1,$28,$d5,$3d,$0b,$d2,$f9,$62,$13,$63,$19,$6a,$f5,$03,
    $02,$00,$60,$e4,$99,$08,$39,$1a,$0c,$57,$33,$9b,$a2,$be,$ba,$7d,$05,$2a,$c5,$b6,$1c,$c4,$e9,$20,$7c,
    $ef,$2f,$0c,$e2,$d7,$37,$39,$58,$d7,$62,$26,$58,$90,$1e,$64,$6a,$95,$18,$44,$60,$dc,$4e,$74,$87,$15,
    $6e,$0c,$29,$24,$13,$d5,$e3,$61,$c1,$69,$6d,$d2,$4a,$ae,$bd,$47,$38,$26,$fd,$a0,$c2,$38,$b9,$0a,$b1,
    $11,$bb,$bd,$67,$c7,$24,$97,$2c,$d1,$8b,$fb,$bd,$9d,$42,$6c,$47,$20,$96,$e7,$61,$15,$c0,$5f,$6f,$7c,
    $eb,$ac,$9f,$45,$ae,$ce,$cb,$72,$f1,$9c,$38,$33,$9d,$8f,$68,$26,$25,$0d,$ea,$89,$1e,$f0,$7a,$ff,$f3,
    $a8,$92,$37,$4e,$17,$5e,$b4,$af,$c8,$da,$ad,$d8,$85,$db,$6a,$b0,$3a,$49,$bd,$0d,$c0,$b1,$b3,$1d,$8a,
    $0e,$23,$fa,$c5,$e5,$76,$7d,$f9,$58,$84,$e0,$64,$25,$a4,$15,$26,$fa,$c5,$1c,$3e,$a8,$44,$9f,$e8,$f7,
    $0e,$dd,$06,$2b,$1a,$63,$a6,$c4,$c6,$0c,$52,$ab,$33,$16,$1e,$23,$84,$38,$89,$7a,$39,$ce,$78,$b6,$3c,
    $9f,$36,$4f,$5b,$8a,$ef,$22,$ec,$2f,$ee,$6e,$08,$50,$ec,$a4,$2d,$06,$fb,$0c,$75,$df,$54,$97,$e0,$0c,
    $55,$4b,$03,$d7,$d2,$87,$4a,$00,$0c,$a8,$f5,$8d,$94,$f0,$34,$1c,$be,$2e,$c9,$21,$56,$c9,$f9,$49,$db,
    $4a,$93,$16,$f2,$81,$50,$1e,$53,$da,$ec,$3f,$64,$f1,$b7,$83,$15,$4c,$60,$32,$0e,$2f,$f7,$93,$33,$ce,
    $35,$73,$fa,$cc,$5f,$dc,$f1,$17,$85,$90,$31,$55,$bb,$d9,$0f,$02,$3b,$22,$02,$24,$fc,$d8,$47,$1b,$f4,
    $f4,$45,$f0,$a8,$8a,$14,$f0,$cd,$97,$6e,$a3,$54,$bb,$20,$cd,$b5,$cc,$b3,$db,$23,$92,$88,$d5,$86,$55,
    $4e,$2a,$0e,$8a,$6f,$e5,$1a,$8c,$fa,$a7,$2e,$f2,$ad,$8a,$43,$dc,$42,$12,$b2,$10,$b7,$79,$df,$e4,$9d,
    $73,$07,$cc,$84,$65,$32,$e4,$b9,$69,$4e,$da,$d1,$62,$af,$05,$3b,$17,$51,$f3,$a3,$d0,$91,$f6,$56,$65,
    $81,$54,$12,$b5,$e8,$c2,$02,$46,$10,$69,$ac,$14,$b9,$58,$78,$49,$34,$b8,$d6,$cc,$e1,$da,$a5,$05,$37,
    $01,$1a,$a4,$fb,$42,$b9,$a3,$de,$f4,$1b,$da,$1f,$85,$ef,$6f,$db,$f2,$f2,$d8,$9d,$2a,$4b,$18,$35,$27,
    $8f,$d9,$40,$57,$89,$f4,$56,$81,$2b,$55,$28,$79,$a6,$16,$86,$95,$c1,$29,$63,$b0,$ff,$01,$ea,$ab,$73,
    $e5,$b5,$c1,$58,$53,$18,$e7,$62,$4f,$14,$a5,$1a,$4a,$02,$6b,$68,$08,$29,$20,$57,$fd,$99,$b6,$6d,$c0,
    $85,$a9,$8a,$c8,$d8,$ca,$f9,$ee,$ee,$a9,$8a,$24,$00,$ca,$c9,$5f,$26,$0f,$d1,$00,$36,$f9,$f9,$10,$96,
    $ac,$31,$95,$22,$0a,$1a,$35,$6b,$2a,$73,$b7,$ea,$ad,$af,$6d,$60,$58,$71,$ef,$7a,$fb,$80,$bc,$42,$34,
    $33,$56,$2e,$94,$b1,$2d,$fa,$b4,$14,$45,$15,$79,$df,$59,$ea,$e0,$51,$70,$70,$62,$40,$12,$a8,$29,$62,
    $c5,$9c,$ab,$34,$7f,$83,$04,$d8,$89,$65,$9e,$5a,$91,$39,$db,$14,$ef,$cc,$30,$85,$2b,$e3,$e8,$fc,$99,
    $f1,$4d,$1d,$82,$2d,$d6,$e2,$f7,$67,$97,$e3,$02,$19,$c8,$aa,$9c,$e8,$84,$8a,$88,$6e,$b3,$c8,$7b,$72,
    $95,$98,$80,$12,$e8,$31,$41,$86,$ed,$ba,$f8,$68,$56,$cc,$d3,$c3,$b6,$ee,$94,$e6,$2f,$11,$0a,$67,$83,
    $d2,$aa,$e8,$9c,$cc,$3b,$76,$fc,$43,$5a,$0c,$e1,$34,$c2,$83,$8f,$d5,$71,$ec,$6c,$13,$66,$a9,$92,$cb,
    $b9,$ac,$40,$7d,$db,$6c,$13,$a4,$b8,$d1,$ec,$f7,$56,$7e,$b0,$97,$1c,$c9,$0b,$55,$18,$56,$9f,$14,$4e,
    $67,$eb,$e9,$b4,$26,$98,$fe,$a7,$9d,$89,$d5,$c5,$ed,$40,$ac,$5e,$37,$01,$d7,$d7,$72,$53,$77,$cf,$06,
    $56,$90,$7f,$b9,$b1,$b1,$6e,$a8,$91,$1a,$fb,$f1,$ae,$5a,$66,$20,$3d,$62,$fd,$1e,$70,$93,$43,$5b,$9c,
    $27,$77,$36,$a7,$0f,$a8,$60,$1c,$f6,$86,$8a,$05,$5b,$22,$38,$67,$7a,$2b,$fb,$bd,$84,$3b,$fa,$18,$73,
    $f0,$c4,$46,$b0,$1b,$2a,$e0,$e9,$8e,$0e,$15,$27,$a9,$00,$b1,$af,$5e,$75,$f8,$7c,$7c,$d1,$7a,$f8,$04,
    $d9,$33,$d6,$f7,$e1,$b9,$e1,$90,$3d,$71,$c7,$bb,$a0,$28,$11,$37,$60,$90,$dd,$61,$73,$35,$df,$dd,$42,
    $4f,$2b,$66,$1c,$d8,$50,$63,$03,$4e,$34,$1b,$06,$e2,$11,$97,$7b,$07,$5a,$8b,$78,$08,$df,$43,$bd,$8e,
    $ef,$1f,$d9,$67,$8c,$c0,$b5,$e9,$f6,$0a,$3e,$b8,$17,$47,$f8,$7e,$57,$09,$46,$8d,$78,$eb,$d2,$da,$01,
    $16,$e6,$b6,$5a,$eb,$3b,$e7,$7e,$e2,$36,$fd,$c3,$3b,$c8,$e7,$df,$1f,$fc,$2e,$22,$88,$f8,$ca,$9a,$ee,
    $5b,$60,$89,$9d,$52,$24,$7f,$4c,$e0,$b6,$ff,$d8,$b9,$47,$81,$76,$42,$c6,$f2,$24,$f7,$0f,$88,$25,$5e,
    $1c,$08,$39,$cb,$cb,$dd,$82,$6c,$5d,$37,$70,$06,$7e,$3e,$08,$eb,$67,$9c,$1d,$b7,$ff,$c1,$68,$42,$23,
    $d7,$c6,$e6,$81,$15,$a2,$83,$22,$91,$ff,$14,$fc,$9d,$b2,$5d,$a9,$be,$cc,$b4,$c2,$75,$33,$e7,$47,$d9,
    $04,$7f,$55,$c9,$01,$af,$f6,$4a,$09,$c4,$5b,$e2,$c7,$3e,$6f,$dc,$29,$69,$6e,$22,$68,$a5,$e0,$c9,$2f,
    $51,$81,$95,$db,$bd,$18,$c4,$ef,$70,$2d,$6c,$55,$01,$31,$6d,$ed,$1f,$3e,$55,$85,$f6,$dd,$d1,$3a,$05,
    $95,$46,$94,$87,$0e,$71,$c1,$94,$cb,$c4,$39,$62,$e1,$12,$ee,$67,$99,$0c,$36,$31,$21,$77,$d1,$b7,$b1,
    $a7,$69,$f8,$87,$32,$4b,$53,$de,$1a,$4c,$99,$c9,$a0,$92,$c1,$06,$c3,$ce,$35,$b9,$48,$cb,$52,$53,$d6,
    $6a,$e9,$9d,$7e,$fb,$ea,$3d,$9e,$7f,$58,$62,$17,$ae,$b4,$06,$8a,$ee,$de,$38,$4f,$a8,$8b,$87,$fd,$03,
    $63,$81,$a7,$43,$42,$da,$7a,$6e,$d2,$65,$8a,$de,$a5,$ee,$41,$4c,$bc,$1b,$b5,$f5,$05,$d2,$20,$a7,$04,
    $3a,$07,$42,$28,$1b,$03,$f8,$1f,$c6,$47,$39,$30,$ff,$7c,$84,$7e,$90,$a3,$73,$f5,$a9,$a5,$27,$bd,$44,
    $47,$ad,$b7,$d1,$9c,$45,$e2,$71,$5d,$3d,$15,$c2,$98,$57,$0b,$15,$2c,$ec,$2f,$bd,$58,$2d,$c4,$6a,$e7,
    $26,$e4,$9e,$3c,$7c,$33,$4b,$5e,$fd,$b4,$c6,$fa,$45,$07,$ba,$00,$26,$76,$1a,$68,$0b,$77,$f6,$ca,$b5,
    $ed,$37,$6d,$10,$58,$39,$f0,$31,$92,$22,$be,$8f,$db,$67,$92,$68,$9f,$f1,$58,$a5,$99,$08,$f1,$67,$07,
    $76,$73,$8a,$f2,$0f,$3c,$cf,$6d,$58,$2b,$a8,$c6,$12,$0d,$a6,$a6,$49,$a5,$57,$83,$ce,$13,$4a,$42,$bb,
    $af,$ae,$7d,$ac,$1d,$53,$87,$bf,$01,$8d,$2b,$31,$01,$22,$34,$78,$59,$28,$dc,$db,$04,$1e,$93,$25,$b4,
    $af,$62,$1a,$8a,$55,$5e,$27,$6a,$31,$41,$f6,$01,$e3,$bd,$2b,$cb,$98,$56,$b2,$9f,$ad,$c8,$ec,$1b,$2f,
    $45,$a1,$62,$99,$e5,$ac,$fa,$4d,$76,$8e,$e2,$fe,$52,$71,$57,$ff,$cd,$4a,$70,$aa,$1e,$df,$64,$10,$11,
    $0b,$d6,$15,$e2,$57,$a2,$03,$f7,$6e,$d0,$e7,$90,$70,$d2,$0a,$75,$1f,$cc,$43,$a7,$c4,$11,$79,$05,$7f,
    $c4,$50,$dc,$a7,$a9,$02,$47,$84,$be,$09,$f6,$09,$ee,$c9,$04,$43,$85,$0a,$ca,$3b,$ed,$5f,$3d,$14,$af,
    $70,$7b,$b0,$95,$af,$93,$1c,$f8,$31,$7d,$02,$a3,$73,$4a,$d2,$cc,$11,$4b,$ff,$03,$d0,$60,$4a,$99,$8e,
    $01,$95,$0b,$82,$e6,$86,$ce,$94,$a8,$38,$ee,$11,$68,$96,$88,$d0,$fd,$96,$7d,$06,$bd,$a5,$b7,$2a,$a6,
    $13,$fa,$22,$0b,$bd,$ec,$99,$5c,$53,$ef,$ad,$e3,$56,$ed,$90,$b7,$a9,$43,$fd,$28,$0f,$ce,$7d,$f5,$1d,
    $60,$7b,$1b,$85,$53,$5b,$4c,$e9,$34,$3f,$67,$1d,$48,$77,$fc,$71,$94,$cc,$c4,$cd,$fe,$fc,$08,$ea,$6d,
    $a2,$46,$40,$bb,$e2,$ba,$0f,$2b,$9f,$73,$ad,$3d,$da,$75,$60,$93,$22,$47,$ff,$56,$56,$34,$67,$f3,$20,
    $18,$e1,$4f,$e4,$6c,$75,$a4,$4a,$4d,$5d,$7e,$8d,$ab,$6b,$b6,$db,$f8,$bc,$4e,$6d,$07,$f9,$e7,$d9,$93,
    $d8,$99,$57,$23,$81,$57,$fb,$ef,$fa,$03,$45,$b5,$aa,$b3,$d1,$fe,$ed,$cb,$09,$f1,$4c,$40,$25,$7b,$2b,
    $7f,$30,$8e,$b7,$99,$ab,$4c,$95,$d1,$a1,$ed,$94,$bd,$da,$0b,$62,$5f,$39,$c4,$c8,$6f,$c5,$ee,$57,$48,
    $04,$1d,$64,$7a,$ef,$cb,$7a,$f2,$26,$fe,$13,$82,$99,$7d,$87,$b2,$21,$99,$e9,$12,$c3,$e3,$f5,$3d,$b5,
    $a7,$20,$8e,$03,$52,$bb,$66,$b3,$79,$77,$8c,$ee,$2e,$c9,$af,$2d,$87,$31,$11,$9f,$38,$bd,$9d,$90,$d4,
    $3e,$05,$4d,$8b,$71,$fc,$0c,$0d,$10,$99,$0e,$84,$c3,$32,$02,$ec,$c9,$fe,$d5,$1f,$62,$63,$93,$cc,$c5,
    $66,$84,$03,$67,$ae,$0b,$74,$cb,$4d,$bf,$e4,$ce,$bd,$7f,$e7,$4b,$8d,$e2,$aa,$91,$76,$bd,$b5,$00,$61,
    $3e,$e2,$ac,$34,$e4,$e9,$e7,$87,$b4,$c2,$9e,$7f,$b2,$08,$a9,$dd,$9c,$92,$f0,$02,$ca,$30,$f8,$33,$f5,
    $60,$93,$c1,$84,$8b,$de,$22,$a0,$bd,$ad,$10,$22,$21,$80,$95,$f2,$b8,$63,$b0,$6c,$ca,$87,$9f,$0b,$c8,
    $d7,$73,$a5,$28,$1e,$ef,$a8,$11,$77,$4a,$66,$64,$b9,$f1,$b1,$2a,$25,$c5,$4e,$b9,$d9,$07,$73,$5d,$44,
    $1c,$77,$e8,$42,$7c,$da,$02,$b7,$50,$8d,$45,$49,$5b,$4e,$c1,$b2,$e6,$b8,$5e,$8d,$75,$1a,$8f,$7d,$d2,
    $71,$ee,$0e,$c8,$dc,$e8,$b6,$35,$96,$53,$3f,$c2,$41,$fe,$15,$a9,$cf,$a5,$ab,$d5,$73,$5a,$ed,$99,$93,
    $7f,$74,$09,$36,$9a,$1b,$26,$32,$1b,$d1,$35,$0e,$cb,$f2,$1c,$45,$4c,$1d,$a1,$19,$c7,$12,$16,$d7,$3e,
    $8e,$c5,$dd,$5c,$59,$27,$9d,$16,$f2,$d3,$ff,$84,$47,$cd,$6a,$cd,$1f,$8d,$6b,$54,$71,$95,$49,$a7,$c2,
    $bb,$38,$ae,$f7,$e6,$7c,$20,$08,$bc,$d7,$89,$24,$ba,$55,$75,$db,$ae,$31,$f6,$54,$9f,$33,$7c,$9c,$24,
    $49,$07,$04,$a3,$44,$0c,$51,$3e,$f2,$b9,$c7,$30,$c6,$f6,$4c,$03,$1c,$91,$3b,$81,$86,$c5,$ee,$43,$a2,
    $62,$7e,$f4,$52,$a9,$43,$58,$b2,$69,$65,$a8,$48,$d4,$12,$9a,$f7,$2e,$d3,$b4,$ec,$0d,$1a,$20,$60,$62,
    $00,$67,$86,$2b,$47,$de,$f3,$6e,$a6,$6b,$c4,$81,$a4,$32,$68,$77,$8c,$0e,$17,$6d,$3a,$2f,$7c,$12,$a6,
    $9d,$d6,$54,$31,$4a,$fc,$a2,$07,$a0,$c1,$f9,$00,$a2,$c7,$ac,$b9,$aa,$69,$58,$70,$19,$31,$03,$d8,$87,
    $b8,$d0,$d1,$6b,$ff,$66,$22,$9b,$1d,$5c,$71,$e4,$0b,$b1,$96,$e5,$a5,$a8,$7a,$ea,$56,$00,$1f,$36,$aa,
    $98,$b3,$4a,$93,$53,$f8,$87,$5c,$a6,$6f,$cf,$6e,$80,$eb,$eb,$0c,$22,$fc,$51,$10,$40,$d7,$66,$f3,$8a,
    $91,$d1,$8e,$b2,$c5,$f0,$2e,$9a,$aa,$ae,$d8,$da,$f4,$9b,$dc,$90,$a7,$c8,$11,$7a,$a4,$ac,$c8,$a6,$02,
    $89,$ca,$3c,$b5,$b5,$44,$57,$e1,$39,$1c,$8f,$7d,$31,$05,$4e,$5b,$58,$f5,$0a,$c7,$6e,$c5,$66,$20,$cc,
    $a6,$48,$97,$2f,$6d,$c0,$8e,$00,$ce,$59,$d5,$db,$3a,$89,$08,$50,$24,$a9,$b4,$bf,$7f,$b0,$13,$ec,$a9,
    $2e,$5c,$1c,$83,$68,$f1,$d8,$c1,$3d,$87,$b3,$77,$e6,$f3,$ca,$18,$06,$0a,$fe,$25,$42,$be,$41,$c4,$83,
    $be,$5d,$ca,$1a,$3f,$c1,$0c,$c4,$65,$da,$62,$74,$04,$38,$27,$96,$76,$04,$f4,$09,$a1,$fb,$ae,$b8,$a5,
    $e4,$f1,$fe,$54,$41,$d2,$86,$43,$fd,$2d,$6d,$64,$da,$55,$b7,$46,$a4,$f3,$d2,$18,$7e,$fa,$de,$66,$c0,
    $bf,$8e,$db,$27,$1e,$34,$d7,$1a,$d4,$3d,$43,$e5,$ba,$ca,$7f,$76,$9e,$2b,$a1,$c2,$72,$a4,$9d,$e8,$19,
    $10,$e8,$b6,$02,$e9,$67,$1b,$e8,$20,$31,$49,$1d,$74,$fb,$17,$05,$1b,$db,$41,$1a,$86,$c5,$89,$7c,$13,
    $3a,$47,$48,$55,$97,$ee,$a6,$9d,$95,$a1,$26,$8b,$fe,$49,$14,$09,$e6,$e3,$f9,$d2,$aa,$2d,$fb,$55,$70,
    $09,$7e,$e1,$d4,$2c,$07,$ee,$5a,$2d,$67,$3d,$d9,$ac,$ad,$6b,$02,$69,$39,$40,$56,$c8,$0d,$c2,$b5,$b6,
    $c2,$39,$b3,$55,$a0,$bf,$aa,$a4,$d8,$b1,$2b,$86,$f4,$27,$b5,$bf,$c1,$59,$f1,$2d,$27,$32,$fb,$9d,$53,
    $6b,$06,$2e,$90,$4d,$70,$a7,$8a,$e8,$8f,$14,$2d,$27,$57,$ad,$ef,$78,$3b,$74,$a7,$6c,$7e,$2a,$d8,$45,
    $62,$a4,$78,$e0,$8a,$97,$8d,$cc,$2c,$0e,$e3,$b6,$0d,$5a,$b4,$3f,$ed,$28,$6d,$8a,$c9,$92,$c9,$91,$8e,
    $5f,$23,$22,$e2,$2e,$a4,$8d,$7d,$e9,$50,$45,$46,$fe,$52,$3c,$f5,$9f,$15,$d5,$7e,$0c,$6a,$27,$c0,$09,
    $cb,$a9,$2f,$dd,$34,$51,$5a,$d0,$21,$96,$48,$65,$1f,$ec,$89,$51,$ef,$24,$cd,$15,$f0,$76,$8d,$8e,$8b,
    $fc,$8b,$fd,$b1,$cc,$86,$75,$4b,$d0,$32,$3d,$e0,$69,$22,$b8,$aa,$bf,$3f,$e7,$58,$20,$36,$0a,$54,$29,
    $d9,$f2,$5b,$d5,$fc,$8e,$8e,$ac,$20,$1f,$37,$56,$27,$aa,$36,$b8,$f2,$7d,$e0,$ca,$84,$dc,$43,$36,$e0,
    $d4,$47,$df,$a2,$11,$0d,$c0,$89,$06,$a1,$4e,$9d,$85,$94,$9a,$f3,$59,$8f,$5e,$65,$5e,$16,$3c,$ab,$15,
    $91,$da,$d6,$f6,$e8,$64,$7c,$b9,$c1,$d8,$3f,$bb,$f5,$92,$24,$a0,$dd,$88,$3c,$2b,$04,$53,$2b,$a5,$44,
    $ab,$77,$90,$7c,$6a,$64,$e0,$b3,$d6,$70,$ac,$71,$23,$38,$41,$a3,$43,$34,$23,$98,$05,$58,$59,$b9,$7d,
    $52,$9a,$98,$60,$dd,$f1,$09,$cf,$8b,$7c,$aa,$50,$5d,$4d,$b8,$3c,$22,$a3,$89,$1b,$37,$57,$f5,$b4,$37,
    $86,$25,$0e,$dc,$71,$7f,$f1,$c7,$ed,$d3,$c8,$10,$3f,$a0,$01,$75,$3f,$de,$36,$b8,$34,$e1,$4d,$3c,$c3,
    $b3,$4f,$47,$23,$24,$5a,$9e,$db,$3a,$c2,$25,$e3,$ed,$7f,$4b,$b3,$ed,$18,$87,$d9,$c4,$ef,$8f,$cd,$34,
    $f8,$b4,$a6,$f9,$72,$c2,$f0,$6b,$a8,$d4,$92,$1d,$51,$ff,$b0,$28,$31,$a6,$31,$32,$60,$ed,$a6,$00,$42,
    $16,$e1,$79,$46,$df,$af,$e1,$c5,$2c,$ba,$83,$41,$73,$14,$2b,$53,$fd,$25,$78,$9b,$0b,$07,$45,$0d,$41,
    $77,$f4,$b3,$04,$96,$1f,$58,$22,$e8,$b4,$5d,$b0,$97,$5f,$0b,$df,$4e,$5c,$8f,$91,$d5,$44,$c9,$89,$81,
    $4a,$7e,$88,$6a,$72,$64,$3b,$d5,$14,$05,$10,$dc,$b9,$24,$c1,$c9,$3a,$ef,$e6,$0f,$53,$9c,$86,$4f,$95,
    $51,$8e,$85,$c4,$71,$0c,$08,$5e,$8f,$77,$8c,$24,$d2,$ce,$70,$b6,$9e,$4c,$61,$b7,$15,$d8,$97,$bb,$30,
    $63,$fa,$b3,$12,$2d,$0b,$f7,$e1,$48,$8b,$57,$ee,$58,$22,$a5,$5d,$8d,$0a,$fa,$46,$cd,$ee,$c3,$91,$d6,
    $88,$1a,$0d,$c4,$d3,$ea,$65,$23,$ba,$5b,$98,$03,$9c,$a9,$63,$a1,$e0,$f7,$2c,$b6,$25,$9a,$13,$d6,$e3,
    $e5,$2e,$c8,$b7,$e4,$18,$b7,$d1,$e4,$fc,$f1,$cb,$e3,$b9,$49,$4f,$a8,$dc,$54,$e1,$c4,$96,$db,$32,$14,
    $33,$9b,$b3,$b0,$4c,$e2,$73,$77,$ea,$aa,$cf,$a1,$21,$05,$1d,$16,$dd,$79,$85,$26,$74,$ba,$8a,$aa,$57,
    $03,$a9,$2f,$2a,$86,$c8,$f5,$7e,$20,$c7,$25,$17,$c5,$a9,$99,$df,$3d,$a5,$9d,$3d,$a4,$cb,$ed,$c4,$7b,
    $57,$98,$f4,$ab,$e7,$5b,$7c,$e9,$81,$b0,$d4,$fc,$5b,$51,$a7,$e5,$a1,$69,$67,$e3,$ec,$50,$b0,$37,$7a,
    $d8,$81,$53,$f9,$19,$7e,$a0,$b9,$6e,$85,$86,$c8,$bc,$30,$f8,$82,$36,$e7,$6b,$b7,$7e,$28,$82,$2c,$6a,
    $00,$30,$94,$fb,$70,$03,$ad,$a1,$b6,$db,$b5,$4c,$2f,$c7,$9f,$61,$1b,$f4,$30,$f8,$84,$82,$a4,$48,$51,
    $60,$43,$b7,$2e,$a3,$25,$7f,$ff,$0c,$7b,$9d,$8e,$6b,$ce,$01,$6d,$58,$76,$f7,$fd,$61,$33,$f8,$9a,$cc,
    $5f,$bf,$2e,$81,$d6,$a6,$56,$96,$78,$f4,$23,$c0,$5f,$0f,$af,$df,$23,$2a,$a7,$86,$f5,$2f,$3d,$0a,$43,
    $5f,$23,$18,$6f,$9f,$f8,$91,$79,$4b,$ce,$3f,$9d,$92,$8e,$1e,$81,$02,$00,$c7,$f4,$6c,$5b,$7a,$c5,$61,
    $82,$be,$cb,$db,$d7,$43,$fa,$b4,$e7,$00,$e2,$f4,$55,$ae,$f8,$a0,$32,$e9,$29,$51,$ad,$98,$bb,$99,$1c,
    $de,$1a,$09,$b3,$cd,$4f,$70,$2e,$93,$a3,$65,$71,$99,$cd,$1f,$00,$86,$dc,$8b,$1c,$cc,$4f,$2a,$80,$91,
    $ea,$1d,$f0,$fd,$88,$9f,$df,$05,$d2,$c0,$fd,$18,$e6,$1e,$78,$2d,$f0,$72,$bb,$8b,$e5,$11,$a7,$e8,$41,
    $2b,$53,$3a,$9a,$d6,$3e,$28,$d3,$de,$c7,$60,$ec,$c1,$ee,$b7,$56,$f9,$36,$c3,$63,$be,$71,$35,$01,$ec,
    $05,$a1,$33,$45,$97,$af,$64,$96,$e0,$0e,$36,$48,$b6,$bf,$94,$0b,$ed,$fa,$10,$e6,$93,$fb,$89,$55,$a3,
    $44,$86,$9f,$d9,$f1,$05,$a4,$57,$eb,$a9,$20,$58,$1f,$48,$b4,$7a,$d4,$18,$df,$52,$68,$17,$71,$fc,$b6,
    $34,$af,$ed,$a9,$53,$0e,$cc,$68,$bb,$83,$7c,$d0,$33,$0a,$39,$43,$60,$c1,$d8,$3d,$14,$c2,$6e,$f2,$ea,
    $02,$56,$84,$ac,$9d,$6a,$48,$c8,$1b,$0d,$75,$9d,$4f,$ca,$8c,$60,$af,$f6,$a8,$87,$ed,$39,$3f,$8f,$f9,
    $bb,$07,$05,$fe,$ce,$f8,$7e,$65,$b0,$37,$39,$3f,$b8,$0f,$75,$d0,$78,$a3,$cc,$af,$86,$d0,$3a,$30,$e3,
    $60,$7b,$b1,$ea,$6b,$3c,$48,$54,$12,$70,$dc,$b3,$ae,$84,$18,$a6,$28,$28,$23,$02,$c6,$86,$19,$f3,$25,
    $9a,$19,$10,$74,$21,$05,$8f,$a6,$3f,$f2,$79,$0c,$af,$19,$0c,$ac,$de,$24,$4b,$f8,$43,$4c,$ea,$9f,$2f,
    $9c,$ed,$ca,$29,$b2,$3c,$93,$c7,$5c,$33,$b6,$32,$21,$87,$c3,$76,$ad,$3a,$e1,$9b,$17,$29,$64,$1d,$83,
    $83,$22,$ae,$3a,$f9,$80,$84,$d0,$70,$3f,$c9,$b3,$46,$d2,$8f,$a8,$9c,$4d,$2b,$ee,$8f,$26,$ba,$f1,$6a,
    $a5,$39,$75,$2f,$84,$5e,$c6,$3b,$ca,$48,$c5,$78,$47,$3e,$ad,$af,$df,$94,$37,$fb,$6b,$ee,$d1,$5c,$18,
    $53,$cc,$78,$01,$08,$51,$66,$d8,$a0,$3b,$a0,$13,$4e,$87,$14,$2f,$af,$19,$4e,$fe,$30,$47,$44,$f4,$8b,
    $1f,$94,$72,$47,$98,$35,$db,$c8,$4e,$37,$57,$51,$2e,$b7,$43,$54,$6c,$f3,$72,$b2,$e0,$a6,$72,$4e,$7c
  {$ifndef MPC_FPrec30K}
   ,$9f,$6f,$a7,$6c,$c3,$6c,$ba,$d4,$4a,$c2,$59,$a1,$3b,$e4,$b8,$4d,$0b,$dc,$ad,$36,$57,$66,$1d,$e3,$6d,
    $b9,$c1,$6b,$5c,$79,$bb,$6f,$43,$ce,$c9,$31,$c9,$35,$31,$41,$2c,$64,$d4,$15,$3a,$86,$f3,$12,$19,$11,
    $0e,$e4,$b7,$64,$6e,$a7,$1f,$40,$7a,$73,$88,$54,$26,$de,$a4,$81,$8e,$ca,$d4,$93,$24,$37,$15,$79,$d8,
    $57,$af,$30,$96,$b7,$39,$57,$e2,$4b,$d0,$a8,$92,$6f,$63,$d5,$01,$67,$6a,$c4,$63,$8b,$18,$91,$32,$1b,
    $01,$b8,$12,$4a,$56,$af,$bc,$59,$1d,$7c,$a3,$04,$6b,$99,$36,$f8,$78,$61,$3e,$d1,$98,$8e,$d5,$f1,$2b,
    $d8,$44,$00,$32,$80,$53,$a0,$9f,$f2,$06,$7d,$4f,$bd,$bb,$c7,$43,$33,$d9,$1b,$13,$22,$ac,$40,$6e,$f8,
    $b4,$bd,$c4,$1d,$bd,$08,$8b,$f2,$9b,$b4,$69,$d2,$23,$e4,$b8,$0c,$40,$c1,$17,$4f,$7e,$ae,$d2,$9f,$c4,
    $b6,$68,$79,$f9,$5f,$aa,$f7,$42,$70,$1b,$ee,$1c,$86,$47,$d8,$8a,$79,$e7,$58,$31,$e5,$41,$ea,$66,$f5,
    $c4,$ab,$1a,$2c,$67,$69,$95,$02,$c3,$66,$a0,$ff,$ef,$29,$88,$c3,$2f,$65,$23,$cc,$49,$cb,$25,$09,$19,
    $2e,$f7,$6c,$be,$8b,$3d,$b8,$22,$c0,$dd,$b5,$76,$9f,$fd,$a6,$9b,$c6,$f9,$76,$a0,$74,$2c,$b9,$82,$2a,
    $32,$1e,$15,$7f,$d4,$d8,$06,$a3,$9e,$3b,$2a,$1f,$48,$96,$1e,$49,$f5,$fe,$a5,$ba,$f8,$ca,$df,$b7,$5a,
    $d5,$f9,$53,$b1,$25,$a9,$b7,$7f,$ae,$48,$9a,$5b,$43,$08,$4d,$f4,$44,$c8,$0e,$eb,$77,$64,$57,$25,$61,
    $0e,$d3,$e0,$e7,$1a,$ef,$de,$4b,$37,$74,$00,$4d,$b1,$f3,$d4,$fb,$58,$63,$24,$00,$17,$ff,$11,$fd,$8f,
    $77,$55,$ac,$30,$da,$6a,$48,$8e,$cf,$1b,$7f,$61,$85,$e3,$5d,$ff,$88,$65,$1f,$bb,$15,$bf,$85,$75,$cf,
    $71,$5b,$71,$c5,$45,$2d,$cc,$b1,$32,$fd,$e4,$09,$11,$75,$da,$81,$3a,$81,$4c,$48,$95,$95,$0b,$70,$d1,
    $ae,$8b,$11,$f0,$2f,$d9,$22,$93,$60,$03,$63,$29,$3e,$b3,$5f,$0c,$17,$c6,$16,$46,$2a,$04,$24,$be,$65,
    $16,$f7,$10,$4e,$08,$48,$f7,$52,$33,$34,$d4,$25,$78,$2e,$ab,$58,$e0,$de,$20,$af,$04,$f1,$cb,$f5,$c2,
    $00,$88,$cf,$9e,$92,$55,$29,$f4,$8a,$92,$4b,$1e,$af,$0d,$6f,$71,$8c,$e0,$ac,$4f,$ae,$f4,$61,$a4,$7c,
    $70,$87,$54,$b4,$4c,$9b,$99,$47,$a3,$c0,$32,$22,$c4,$a1,$37,$bd,$74,$38,$5b,$a9,$7c,$80,$d7,$81,$a6,
    $4e,$46,$c3,$95,$4b,$f1,$93,$92,$02,$8d,$92,$48,$b8,$30,$f0,$e6,$23,$a8,$ae,$cb,$5f,$14,$50,$24,$af,
    $16,$b3,$fe,$20,$5a,$e2,$ea,$61,$8d,$f0,$02,$94,$ef,$05,$6e,$81,$ff,$60,$2c,$17,$fc,$9d,$b8,$99,$b7,
    $21,$a7,$d0,$92,$49,$24,$af,$fb,$20,$96,$10,$07,$2a,$eb,$52,$e9,$d7,$7c,$43,$16,$30,$0e,$cc,$76,$de,
    $33,$87,$1d,$91,$ae,$08,$9e,$3d,$c9,$24,$8d,$94,$13,$03,$1e,$bf,$3c,$f5,$59,$28,$e4,$3c,$0f,$46,$c7,
    $0e,$7c,$13,$5f,$a9,$84,$57,$79,$11,$7a,$88,$53,$55,$fd,$35,$22,$f3,$74,$e9,$33,$21,$06,$4c,$c1,$ad,
    $8d,$79,$50,$4e,$fe,$d0,$c7,$0f,$a5,$5b,$ea,$c4,$ad,$f7,$e2,$7f,$0d,$5a,$62,$63,$47,$7b,$3d,$00,$59,
    $68,$00,$f8,$ac,$78,$a4,$aa,$db,$1e,$1c,$54,$46,$87,$db,$9a,$c8,$13,$a6,$af,$b5,$16,$64,$1a,$bd,$c8,
    $c6,$b7,$8a,$3b,$2d,$3f,$a5,$43,$8c,$fd,$f0,$4e,$8c,$fc,$48,$a5,$a0,$0a,$f1,$60,$b2,$60,$ca,$70,$5c,
    $e5,$23,$fd,$00,$18,$f6,$c7,$5f,$54,$9a,$7c,$f7,$51,$c2,$6c,$60,$01,$23,$3d,$87,$94,$44,$5f,$17,$69,
    $7b,$1a,$3b,$08,$56,$bb,$28,$24,$54,$24,$27,$d1,$8f,$27,$0d,$0b,$4a,$2f,$55,$a0,$3d,$eb,$a4,$a9,$66,
    $b2,$06,$d3,$cf,$0c,$ec,$cb,$35,$63,$bd,$c9,$bd,$79,$8a,$a1,$16,$5a,$28,$05,$01,$b3,$bf,$ed,$a8,$f5,
    $94,$fd,$6e,$48,$5e,$66,$2e,$6f,$f3,$2b,$9d,$dd,$87,$38,$1b,$44,$19,$d9,$99,$84,$69,$7c,$26,$59,$6a,
    $94,$3f,$89,$e6,$22,$5a,$4e,$a5,$27,$a2,$c5,$23,$3c,$16,$98,$f7,$93,$43,$d4,$09,$41,$ad,$5e,$b3,$dd,
    $e4,$ba,$34,$78,$89,$a5,$55,$71,$b3,$de,$b3,$f0,$fc,$40,$27,$1f,$e3,$b8,$17,$ee,$e8,$27,$4d,$07,$4a,
    $1b,$b0,$41,$5e,$f1,$d9,$f2,$1e,$98,$e0,$f6,$d9,$bc,$94,$d4,$ab,$71,$87,$84,$76,$d6,$70,$62,$f0,$46,
    $f6,$84,$bd,$b8,$38,$b1,$73,$80,$4b,$03,$0e,$4f,$4f,$eb,$08,$fe,$a9,$f8,$51,$83,$65,$51,$36,$0b,$e4,
    $af,$e7,$67,$ba,$20,$f9,$57,$9d,$dd,$c0,$38,$fc,$36,$69,$b9,$df,$94,$31,$e4,$50,$0c,$eb,$48,$12,$08,
    $a3,$27,$37,$27,$42,$d9,$09,$0a,$c5,$7f,$88,$60,$38,$ab,$28,$9a,$72,$fd,$69,$a0,$38,$97,$ac,$43,$06,
    $1f,$96,$0f,$ba,$b8,$da,$6e,$35,$e6,$5c,$c8,$19,$2a,$c7,$b2,$ee,$67,$3c,$57,$e7,$69,$98,$03,$11,$ac,
    $3e,$34,$5b,$b7,$70,$01,$06,$40,$32,$7b,$1b,$01,$07,$4b,$d0,$22,$05,$50,$20,$36,$ea,$b9,$47,$bb,$d0,
    $e2,$d5,$00,$d3,$d0,$13,$f8,$dc,$93,$bb,$76,$12,$01,$59,$24,$fb,$68,$b0,$4e,$3c,$e1,$2c,$56,$63,$77,
    $4a,$bf,$77,$b0,$6f,$ad,$49,$77,$22,$d0,$23,$54,$bc,$92,$be,$e6,$70,$d1,$b4,$aa,$55,$39,$96,$7d,$8e,
    $62,$b2,$12,$b8,$83,$e7,$e7,$b2,$0b,$3d,$29,$e8,$af,$5c,$17,$0e,$d5,$23,$29,$f8,$7f,$65,$92,$0a,$6a,
    $5a,$6c,$4b,$bb,$08,$7f,$c5,$82,$a3,$51,$7e,$c6,$e7,$bd,$24,$3d,$92,$79,$3a,$01,$1a,$d5,$04,$f0,$20,
    $cd,$7a,$78,$0b,$72,$ef,$8d,$c5,$75,$ad,$3a,$02,$c0,$71,$c2,$0b,$1e,$f0,$12,$23,$8e,$44,$c1,$18,$90,
    $7e,$14,$eb,$d5,$85,$a8,$76,$05,$b8,$92,$3a,$b9,$14,$76,$ca,$76,$98,$94,$a6,$3f,$17,$ed,$7a,$16,$8e,
    $fc,$2a,$fe,$50,$f0,$49,$5b,$df,$8d,$58,$a2,$f7,$cc,$4d,$43,$57,$88,$3b,$4b,$36,$ea,$30,$00,$09,$34,
    $7f,$3b,$ca,$a4,$3e,$97,$99,$f7,$72,$e2,$58,$56,$41,$4f,$89,$9d,$89,$75,$6c,$e5,$93,$7a,$06,$8a,$e2,
    $a9,$e7,$32,$c6,$e5,$2f,$dc,$58,$0b,$d9,$d9,$c3,$87,$dc,$2e,$50,$c9,$42,$6e,$1e,$55,$28,$fb,$79,$8a,
    $48,$71,$1b,$ea,$1e,$4e,$23,$c3,$15,$bb,$89,$97,$83,$af,$14,$12,$bd,$af,$55,$11,$82,$7e,$40,$9c,$25,
    $df,$80,$95,$bb,$d3,$eb,$cf,$a1,$54,$dc,$76,$d6,$3b,$28,$71,$1b,$95,$d6,$c8,$46,$7b,$02,$90,$e7,$f5,
    $93,$6b,$e4,$8e,$a5,$77,$95,$ea,$dc,$e8,$25,$52,$16,$fb,$9d,$35,$6a,$2d,$5c,$12,$7e,$01,$97,$fa,$81,
    $58,$f1,$9e,$a5,$9a,$57,$a0,$0c,$1c,$f5,$95,$05,$89,$d8,$22,$80,$ab,$31,$b6,$8f,$46,$34,$22,$a9,$7e,
    $ed,$35,$c3,$04,$1f,$09,$f2,$2f,$20,$3c,$72,$4a,$17,$bb,$1e,$13,$b0,$0d,$a5,$e5,$67,$33,$b5,$fb,$3f,
    $0d,$84,$6d,$3c,$45,$14,$de,$d9,$28,$51,$79,$91,$ac,$82,$a3,$4f,$61,$06,$d6,$47,$68,$e6,$db,$8d,$a5,
    $a2,$0f,$64,$44,$4f,$35,$76,$e1,$b1,$19,$82,$63,$fc,$36,$85,$bb,$f4,$ce,$42,$47,$68,$6e,$44,$c8,$80,
    $b7,$cf,$87,$fd,$f1,$4c,$76,$8c,$63,$fe,$cc,$f6,$8d,$08,$04,$df,$93,$3c,$08,$35,$a4,$31,$ff,$53,$f8,
    $89,$2c,$17,$41,$00,$41,$f7,$b6,$94,$f9,$ce,$31,$74,$fc,$e9,$87,$ac,$60,$69,$f5,$08,$be,$90,$cd,$97,
    $c2,$78,$d1,$59,$de,$8a,$bf,$fc,$cf,$47,$84,$ec,$c8,$7f,$83,$44,$a9,$35,$be,$97,$41,$a1,$f7,$a8,$f0,
    $b8,$15,$8a,$62,$f5,$fe,$3a,$d9,$d1,$f1,$c3,$bf,$22,$e6,$1c,$70,$e7,$49,$20,$cb,$be,$7c,$88,$63,$ba,
    $18,$80,$b1,$fa,$1a,$c9,$5d,$d4,$0e,$ae,$b0,$c3,$ac,$63,$8a,$42,$3b,$62,$96,$4d,$64,$c7,$4d,$fd,$42,
    $db,$24,$7e,$20,$6d,$b8,$4f,$7c,$26,$c1,$ce,$ac,$aa,$15,$00,$e0,$fb,$ba,$1d,$7a,$de,$36,$07,$d0,$68,
    $ed,$c7,$d2,$60,$64,$b2,$71,$1d,$6f,$1d,$03,$68,$c5,$bf,$80,$5a,$f8,$be,$96,$18,$1c,$aa,$ce,$6d,$88,
    $b6,$bf,$14,$2d,$5c,$3d,$de,$63,$11,$5c,$69,$20,$c8,$31,$19,$69,$54,$9c,$1f,$57,$20,$5d,$e7,$17,$eb,
    $15,$dd,$87,$88,$32,$f5,$19,$21,$e5,$21,$50,$da,$cb,$22,$b2,$ef,$51,$1f,$b4,$c1,$bd,$25,$4b,$aa,$74,
    $fc,$19,$08,$6e,$31,$f4,$3e,$1b,$ab,$72,$ee,$20,$9a,$71,$68,$9c,$80,$29,$06,$5b,$21,$a4,$7f,$34,$a1,
    $0f,$f7,$14,$c2,$1d,$20,$27,$dc,$82,$b7,$80,$8a,$15,$91,$67,$fe,$cf,$c6,$3a,$3a,$7d,$2f,$76,$50,$45,
    $95,$3a,$4e,$bc,$0c,$59,$0c,$1a,$c4,$b1,$d3,$85,$9f,$29,$bd,$00,$0d,$de,$73,$4f,$91,$a8,$33,$c4,$15,
    $50,$0a,$e5,$94,$7c,$fa,$35,$7f,$ec,$f5,$d2,$b3,$e1,$d7,$1a,$ce,$e5,$c2,$b0,$27,$c4,$73,$22,$72,$bd,
    $1f,$36,$6d,$9c,$18,$92,$c7,$32,$1c,$4a,$6e,$70,$15,$84,$75,$4e,$b7,$b8,$1c,$db,$21,$25,$04,$46,$15,
    $f5,$24,$67,$59,$42,$0b,$e4,$a9,$ac,$bc,$6b,$68,$10,$30,$93,$5c,$5c,$ce,$ac,$4e,$ad,$1e,$c3,$56,$dc,
    $07,$f8,$5f,$17,$0a,$b8,$d7,$71,$f2,$19,$60,$cd,$1f,$60,$4c,$a8,$ad,$a4,$5a,$6c,$b1,$a5,$1b,$cf,$7b,
    $ee,$a7,$21,$66,$4b,$b6,$13,$6b,$cd,$2e,$ba,$4e,$f1,$f5,$79,$31,$61,$26,$da,$c1,$fe,$50,$c4,$09,$32,
    $47,$6b,$d8,$cd,$8a,$f1,$48,$fa,$74,$c6,$50,$1b,$17,$9a,$c0,$d5,$70,$74,$d6,$13,$dc,$4f,$c4,$27,$af,
    $24,$35,$4d,$55,$82,$8f,$86,$5f,$0c,$84,$87,$03,$b3,$62,$38,$00,$2a,$08,$19,$ee,$e3,$17,$a9,$db,$7a,
    $8b,$84,$51,$ee,$f8,$9c,$78,$37,$72,$63,$79,$d0,$21,$31,$0d,$e5,$95,$c1,$28,$81,$28,$d1,$92,$d0,$a0,
    $6f,$75,$40,$29,$5b,$2a,$84,$b6,$72,$bf,$07,$5c,$73,$f8,$81,$2b,$5b,$18,$66,$f7,$e7,$0c,$cf,$d6,$af,
    $b1,$ae,$e9,$a6,$f8,$2b,$23,$79,$96,$4e,$1e,$b7,$43,$59,$dc,$66,$c5,$10,$d0,$08,$ac,$9b,$10,$b8,$73,
    $0d,$a1,$d7,$25,$35,$80,$f2,$49,$86,$ad,$4c,$b5,$b2,$95,$5e,$35,$da,$b7,$ad,$f9,$1d,$f5,$2d,$f4,$a5,
    $da,$24,$43,$d8,$c2,$64,$30,$fd,$f4,$54,$01,$25,$20,$19,$fe,$71,$6a,$62,$55,$b2,$3a,$38,$53,$22,$2c,
    $93,$95,$a2,$c0,$eb,$a2,$5e,$5d,$3a,$fc,$8d,$1d,$03,$dc,$81,$f3,$cb,$65,$1e,$1c,$d6,$d4,$ae,$d5,$71,
    $4b,$ab,$7d,$b8,$f4,$2b,$26,$e0,$12,$4e,$af,$37,$1c,$dd,$a8,$b1,$3e,$fc,$b4,$3e,$4a,$06,$2b,$a5,$46,
    $6c,$8c,$9c,$a2,$73,$6c,$8f,$8b,$6d,$7b,$dc,$16,$58,$a5,$76,$e3,$43,$3c,$8b,$51,$a1,$f5,$90,$58,$8f,
    $df,$21,$3e,$f9,$68,$3f,$eb,$19,$a7,$38,$ff,$84,$0b,$7f,$ce,$d3,$6f,$a3,$fd,$6d,$35,$c4,$1a,$4b,$da,
    $64,$7f,$93,$4c,$16,$49,$aa,$db,$28,$f8,$d1,$91,$4c,$e2,$bf,$c9,$0d,$07,$e1,$ac,$90,$e3,$86,$83,$94,
    $2a,$f3,$85,$0c,$a1,$3c,$43,$70,$4f,$ef,$7d,$26,$1c,$a2,$b2,$50,$17,$a3,$74,$cb,$da,$26,$a1,$31,$0b,
    $f3,$ce,$5a,$ab,$de,$92,$92,$dc,$91,$92,$e8,$81,$1b,$bd,$57,$24,$8a,$fa,$74,$86,$c7,$65,$73,$54,$fa,
    $e6,$7b,$96,$29,$55,$e6,$bc,$9b,$19,$cb,$d3,$dd,$0c,$b7,$99,$b0,$fe,$fd,$e0,$3b,$c8,$e6,$9f,$94,$b2,
    $72,$d6,$20,$5e,$10,$b3,$4c,$79,$2b,$9c,$a5,$5b,$cc,$de,$49,$6c,$68,$ec,$6a,$8b,$ff,$70,$a8,$db,$c5,
    $7d,$d4,$cc,$9d,$ad,$36,$09,$60,$ff,$8e,$3e,$31,$de,$2a,$4d,$97,$77,$cf,$8d,$2c,$26,$17,$15,$45,$62,
    $d2,$97,$a1,$7c,$86,$1c,$ed,$a2,$57,$31,$ac,$03,$ee,$52,$d8,$a2,$0e,$63,$3a,$25,$1b,$a8,$bf,$05,$f9,
    $6c,$5e,$eb,$c1,$32,$0e,$f4,$f0,$6d,$ca,$47,$0a,$80,$ba,$1d,$29,$ec,$82,$e0,$f2,$05,$54,$82,$ee,$86,
    $c9,$94,$61,$8c,$ab,$11,$82,$1a,$fe,$fe,$8f,$8c,$fb,$15,$15,$78,$a5,$8f,$20,$c9,$bc,$80,$a2,$53,$95,
    $3e,$3b,$57,$dd,$ad,$76,$10,$7b,$80,$5b,$e8,$bd,$71,$21,$91,$67,$32,$73,$ed,$3c,$19,$8b,$48,$cd,$14,
    $65,$5a,$cf,$6e,$16,$f9,$9c,$ce,$6f,$73,$af,$ec,$68,$a1,$5d,$72,$e5,$a6,$8d,$4b,$a2,$af,$8b,$31,$b6,
    $44,$3b,$87,$2a,$18,$dd,$cd,$79,$dc,$4d,$3d,$36,$f6,$87,$63,$2a,$3e,$9f,$a1,$52,$3b,$bf,$d3,$b3,$2a,
    $10,$e4,$a1,$1c,$0f,$0b,$20,$b3,$e1,$ea,$f9,$b8,$04,$4c,$21,$24,$24,$cb,$e4,$27,$7c,$be,$e6,$6b,$61,
    $55,$f8,$53,$66,$9f,$10,$63,$ee,$9b,$c3,$cd,$19,$ef,$ae,$5d,$3e,$56,$e0,$18,$ca,$ee,$83,$10,$69,$5c,
    $4e,$8b,$73,$86,$29,$df,$65,$26,$41,$1b,$da,$81,$0b,$81,$a8,$62,$b2,$5f,$9f,$1e,$eb,$4b,$9f,$20,$20,
    $95,$04,$63,$2c,$38,$2d,$aa,$bf,$5f,$04,$b3,$d5,$52,$b7,$e7,$b0,$fc,$7f,$21,$aa,$64,$08,$75,$87,$cc,
    $dd,$4a,$90,$fc,$35,$e5,$49,$27,$7a,$73,$88,$3d,$dd,$83,$1e,$17,$e3,$84,$20,$34,$8c,$4b,$72,$d0,$03,
    $a7,$a3,$ea,$a6,$28,$5d,$20,$b0,$3d,$99,$a2,$a0,$5b,$ab,$29,$49,$f1,$a9,$5f,$3f,$3a,$d2,$02,$34,$de,
    $b3,$77,$b9,$8c,$8b,$66,$80,$5a,$56,$57,$88,$0d,$76,$64,$8f,$78,$f0,$bc,$e5,$cc,$c7,$d5,$f5,$e2,$97,
    $0d,$ad,$f4,$78,$a6,$6d,$10,$61,$b2,$08,$1f,$cc,$2c,$9f,$03,$a9,$72,$a6,$b0,$78,$4c,$b8,$f4,$2d,$a7,
    $f0,$9a,$63,$93,$64,$d2,$4a,$0c,$8e,$6d,$de,$42,$c6,$63,$18,$2f,$62,$7f,$a6,$a3,$3b,$09,$75,$91,$8c,
    $d2,$6e,$fe,$d6,$a5,$4e,$26,$af,$f3,$56,$19,$36,$32,$7d,$ba,$03,$a3,$e3,$05,$e6,$76,$32,$23,$e7,$51,
    $05,$4a,$ec,$1f,$e6,$be,$4b,$1c,$34,$2d,$57,$4b,$d2,$10,$93,$e5,$8f,$93,$44,$8d,$78,$26,$0e,$00,$47,
    $3c,$e4,$3f,$d5,$bd,$81,$67,$b5,$be,$ea,$18,$23,$1c,$15,$91,$86,$56,$89,$c6,$40,$21,$86,$d0,$40,$92,
    $64,$d1,$6f,$09,$a8,$9e,$91,$2c,$f6,$a1,$3c,$d9,$d4,$99,$5a,$f3,$6a,$f8,$ac,$0e,$cf,$95,$39,$15,$2a,
    $8b,$c0,$76,$c1,$33,$ab,$40,$30,$3c,$f8,$af,$08,$fd,$70,$c4,$c6,$8e,$ba,$19,$fc,$2c,$d6,$57,$60,$dd,
    $eb,$4c,$3b,$b1,$7f,$38,$61,$32,$87,$cf,$9b,$42,$6f,$bd,$fa,$01,$62,$88,$57,$b7,$5d,$9e,$52,$29,$23,
    $af,$7b,$6e,$8b,$ab,$ae,$82,$f4,$7f,$0d,$d4,$3c,$c9,$18,$77,$7e,$d2,$05,$75,$d2,$04,$88,$30,$01,$09,
    $92,$83,$5b,$31,$d9,$58,$08,$d2,$cf,$9a,$f8,$cd,$62,$a4,$0f,$19,$2e,$b1,$2f,$b5,$76,$2a,$c3,$ac,$d0,
    $50,$3f,$7e,$d3,$a1,$5d,$cd,$62,$91,$97,$59,$6e,$25,$21,$77,$ae,$ab,$50,$cc,$87,$99,$83,$88,$fc,$63,
    $4b,$4d,$61,$7a,$93,$e8,$39,$ed,$4e,$54,$c2,$69,$37,$56,$9f,$23,$23,$38,$f0,$2c,$60,$cd,$21,$ea,$4e,
    $b6,$7d,$93,$7e,$8a,$f2,$d4,$ba,$7b,$ba,$37,$b4,$eb,$cb,$2a,$87,$09,$3f,$0f,$ff,$1c,$8d,$4e,$53,$48,
    $fd,$ba,$24,$71,$38,$da,$57,$13,$2d,$af,$a2,$12,$0a,$0e,$22,$eb,$da,$33,$dd,$f0,$a6,$08,$88,$7a,$e7,
    $d9,$b6,$d2,$51,$a6,$9b,$3d,$57,$74,$a7,$45,$fb,$ce,$b8,$b2,$cf,$e3,$7e,$16,$9b,$3b,$27,$81,$0e,$d2,
    $eb,$04,$2a,$ff,$f3,$d8,$0f,$8d,$2d,$27,$95,$10,$e9,$f3,$54,$2f,$06,$06,$a0,$15,$0e,$3c,$35,$b2,$9a,
    $61,$38,$a3,$25,$fb,$6e,$7d,$f0,$a2,$ea,$b3,$18,$d9,$9f,$c6,$cc,$c3,$1e,$00,$ec,$0d,$c3,$15,$f8,$87,
    $77,$92,$ab,$b0,$cb,$22,$a9,$96,$1f,$ef,$04,$cc,$3a,$29,$5b,$bc,$69,$29,$8f,$6a,$f0,$47,$81,$92,$f6,
    $f6,$57,$f1,$fe,$dd,$9f,$26,$1b,$2f,$70,$8f,$83,$e0,$a5,$90,$b6,$dd,$06,$f7,$81,$ac,$7f,$3f,$be,$ca,
    $8a,$cb,$1d,$e1,$06,$12,$86,$be,$0e,$fb,$e1,$0d,$6a,$6d,$33,$5f,$6d,$8f,$89,$fd,$5a,$38,$47,$39,$9c,
    $8c,$89,$30,$59,$a5,$a3,$9a,$11,$31,$67,$a8,$14,$57,$47,$a2,$cf,$9f,$b7,$74,$28,$ae,$1d,$75,$38,$98,
    $84,$80,$5a,$47,$8d,$15,$89,$26,$6a,$25,$f3,$18,$96,$45,$13,$8c,$b0,$f5,$f6,$d0,$be,$15,$f6,$01,$92,
    $54,$6d,$46,$c5,$2d,$5d,$fd,$2b,$ac,$6e,$00,$c4,$de,$78,$44,$5b,$cc,$c6,$03,$29,$bd,$f0,$d6,$57,$af,
    $fa,$57,$de,$e7,$f1,$b2,$1b,$23,$cb,$7c,$15,$b9,$13,$8c,$03,$eb,$d1,$b5,$a4,$6d,$4a,$c1,$2f,$38,$0b,
    $a3,$c3,$c9,$55,$a6,$6b,$a2,$7c,$04,$9e,$9b,$20,$60,$3a,$65,$14,$0a,$a0,$e6,$e5,$5f,$3f,$02,$cf,$b8,
    $12,$25,$a8,$bf,$a7,$5c,$00,$82,$17,$6c,$c0,$a1,$26,$7d,$59,$6b,$55,$5e,$89,$25,$e8,$ba,$6e,$b9,$d0,
    $fc,$37,$71,$64,$33,$ae,$ca,$54,$95,$b4,$5c,$e7,$d4,$97,$15,$f3,$a1,$f1,$84,$57,$f2,$3f,$08,$ac,$e6,
    $44,$a2,$27,$d5,$0d,$c9,$55,$d7,$31,$46,$dd,$82,$13,$40,$ad,$76,$cd,$6d,$d1,$f8,$bb,$7c,$c2,$6d,$32,
    $78,$ea,$9a,$f5,$fa,$14,$a5,$0a,$57,$ae,$21,$a6,$fc,$20,$66,$ee,$07,$b1,$99,$44,$cf,$80,$c0,$9c,$c1,
    $9d,$fa,$9a,$60,$23,$b4,$e3,$40,$79,$b5,$a8,$06,$38,$dd,$4f,$74,$67,$47,$8c,$0b,$ac,$20,$04,$0e,$c6,
    $70,$44,$80,$cb,$14,$ea,$bb,$68,$5a,$75,$11,$eb,$d3,$ec,$09,$bd,$a8,$e1,$91,$73,$d7,$fa,$34,$0f,$47,
    $23,$ee,$92,$f0,$41,$39,$ff,$80,$60,$22,$4e,$2f,$1f,$45,$79,$00,$e2,$09,$4b,$4a,$34,$f0,$6d,$cb,$4d,
    $3b,$47,$1b,$c3,$3b,$9b,$68,$68,$86,$43,$6d,$4a,$e2,$a2,$9a,$16,$4a,$8c,$ae,$b0,$9c,$d3,$3e,$6c,$56,
    $af,$1e,$58,$71,$bf,$f7,$3d,$28,$64,$c9,$0b,$84,$12,$1d,$cd,$4c,$b6,$9d,$6f,$dd,$66,$b0,$bb,$b1,$78,
    $20,$d7,$d7,$23,$ac,$64,$9a,$b8,$bf,$0c,$93,$c7,$11,$39,$a6,$16,$ad,$a2,$f3,$58,$a0,$d3,$7d,$bc,$9e,
    $6d,$ea,$9d,$81,$c7,$ae,$f9,$c9,$c3,$94,$80,$14,$6c,$9f,$b6,$6e,$ac,$38,$a8,$d6,$10,$65,$80,$b5,$96,
    $31,$04,$4d,$8c,$63,$6d,$77,$19,$cc,$93,$8a,$7f,$91,$54,$da,$08,$b4,$bb,$83,$5c,$7c,$9b,$df,$5b,$5b,
    $1a,$54,$57,$34,$b4,$75,$1a,$2b,$86,$5d,$a1,$d5,$0a,$ec,$f2,$25,$60,$c1,$06,$7a,$cd,$8d,$8d,$4b,$4e,
    $fa,$97,$26,$ac,$1f,$f8,$89,$f3,$4c,$7c,$8e,$49,$ad,$0c,$7a,$03,$d6,$4a,$2c,$ba,$48,$40,$da,$34,$17,
    $56,$4d,$22,$a4,$00,$e2,$7b,$c8,$0d,$20,$26,$76,$e5,$e1,$0a,$37,$34,$f1,$7a,$cf,$ee,$40,$52,$0c,$b9,
    $35,$93,$ce,$9a,$ec,$26,$3e,$84,$03,$e7,$95,$31,$2f,$80,$3f,$2b,$ce,$92,$34,$87,$91,$af,$a6,$e6,$49,
    $2c,$49,$3b,$ef,$9c,$41,$d8,$87,$0a,$5f,$3b,$4b,$2c,$ed,$c5,$64,$ba,$da,$31,$7c,$ae,$d6,$7c,$5a,$68,
    $4a,$61,$aa,$8b,$05,$77,$4f,$25,$46,$02,$c8,$4f,$8d,$f5,$fb,$1a,$48,$aa,$5c,$f8,$d9,$59,$15,$7e,$c6,
    $a6,$13,$dd,$3c,$3e,$6b,$52,$df,$8d,$89,$76,$9d,$29,$a5,$bd,$b7,$d8,$36,$de,$98,$69,$94,$18,$54,$eb,
    $e7,$a5,$f6,$57,$c1,$bb,$61,$6d,$2b,$23,$8f,$8d,$f2,$67,$f7,$31,$19,$90,$53,$33,$ce,$54,$9a,$45,$aa,
    $fe,$f6,$91,$62,$d1,$b0,$cb,$dd,$d6,$5b,$85,$94,$32,$31,$92,$28,$70,$e3,$f5,$91,$df,$cc,$90,$39,$39,
    $bb,$d8,$6f,$6f,$77,$43,$e4,$fd,$45,$a9,$3f,$0e,$f3,$f5,$d8,$e9,$b1,$ab,$f1,$5b,$5e,$a0,$19,$25,$d5,
    $ee,$e1,$7d,$7c,$99,$ec,$f1,$c0,$65,$d9,$b2,$c0,$e5,$f9,$5c,$f0,$2c,$71,$cb,$a8,$0e,$2a,$da,$ab,$72,
    $b0,$d3,$34,$8d,$e8,$c6,$cd,$ff,$35,$78,$53,$be,$a0,$20,$03,$0d,$5d,$06,$1f,$d4,$56,$d5,$78,$1c,$ea,
    $87,$b8,$b9,$fd,$47,$c5,$fa,$e6,$ce,$09,$1b,$eb,$d4,$a9,$11,$76,$d5,$e9,$b9,$13,$4b,$2f,$d9,$aa,$61,
    $ce,$eb,$1f,$45,$c2,$e9,$6a,$8e,$b5,$39,$9f,$55,$22,$a1,$b5,$36,$50,$f9,$c9,$74,$34,$e3,$b2,$3a,$5f,
    $3f,$9e,$9f,$bb,$ef,$5b,$ec,$de,$51,$12,$2f,$f0,$79,$02,$b5,$a1,$02,$e0,$dd,$26,$2f,$bd,$bd,$4c,$90,
    $4f,$5d,$ad,$4e,$3a,$57,$56,$8d,$fa,$df,$9e,$85,$59,$e3,$59,$bc,$b5,$46,$52,$63,$dd,$17,$db,$17,$7a,
    $f5,$0d,$bf,$af,$a3,$90,$bf,$9b,$ea,$c8,$3b,$fe,$ea,$b2,$fd,$31,$80,$78,$53,$4e,$dc,$39,$6e,$08,$cb,
    $ab,$fd,$b0,$8c,$f9,$d5,$74,$7f,$60,$9e,$08,$12,$81,$fb,$53,$51,$52,$df,$6d,$cb,$59,$3d,$3f,$64,$bc,
    $19,$7f,$7d,$79,$ad,$8f,$a5,$86,$2c,$75,$f9,$49,$13,$c3,$22,$dd,$77,$ad,$0a,$e4,$bc,$e2,$90,$ba,$ef,
    $85,$4e,$ea,$08,$fe,$a7,$d7,$56,$69,$77,$2b,$d1,$37,$e4,$4f,$b9,$9f,$7e,$b0,$97,$5c,$d5,$4e,$10,$e9,
    $94,$51,$3c,$52,$e9,$a7,$bd,$7c,$ef,$e8,$11,$9b,$d7,$99,$6c,$99,$2c,$0a,$8c,$ed,$8a,$7a,$54,$d3,$87,
    $de,$d7,$3b,$35,$66,$b4,$3a,$fa,$cd,$67,$93,$3e,$80,$57,$3c,$7a,$bb,$ab,$ce,$22,$d9,$2f,$aa,$4f,$2d,
    $e6,$a2,$3a,$6b,$99,$60,$17,$ce,$bc,$6f,$c7,$85,$84,$79,$89,$61,$6b,$e6,$c7,$f6,$ff,$0d,$eb,$ef,$c8,
    $4b,$25,$4f,$6e,$ff,$4b,$75,$e7,$9b,$c4,$87,$fb,$4e,$c9,$2c,$43,$d2,$b5,$90,$eb,$16,$6d,$68,$65,$45,
    $0b,$ab,$d1,$80,$3a,$f2,$6e,$7d,$10,$95,$98,$d7,$df,$c3,$08,$eb,$eb,$7a,$94,$56,$40,$bf,$d4,$94,$e9,
    $a5,$6b,$9a,$0d,$0c,$d3,$a2,$d8,$7c,$b8,$10,$d5,$6b,$17,$c6,$59,$8f,$77,$ec,$a8,$70,$e3,$eb,$93,$35,
    $86,$c0,$0a,$f4,$cb,$0b,$60,$75,$2a,$71,$0e,$44,$19,$46,$53,$f3,$aa,$13,$84,$4b,$4c,$82,$fe,$50,$8c,
    $17,$32,$51,$a8,$39,$e2,$ff,$fe,$23,$8e,$1a,$34,$70,$0f,$de,$8c,$d7,$f7,$f8,$fd,$db,$e5,$1f,$ed,$fd,
    $d3,$f3,$0a,$11,$38,$81,$9c,$21,$1c,$d1,$d9,$e2,$16,$34,$0a,$8b,$eb,$7a,$85,$e3,$ce,$48,$30,$83,$d1,
    $e3,$9c,$9e,$51,$85,$eb,$8a,$3a,$f4,$38,$10,$a4,$1a,$b2,$f7,$c4,$40,$94,$6f,$32,$e0,$65,$27,$90,$61,
    $2a,$da,$a6,$5e,$90,$f2,$f6,$41,$c1,$de,$95,$65,$53,$0a,$58,$d2,$8b,$b5,$8c,$f2,$48,$d7,$bd,$db,$1b,
    $af,$b1,$cf,$91,$c4,$db,$54,$bd,$e8,$d2,$78,$7e,$ae,$95,$20,$2e,$5a,$0b,$da,$1c,$1e,$e2,$26,$5f,$0f,
    $4d,$60,$6d,$c5,$15,$26,$52,$96,$ef,$fa,$62,$2b,$27,$df,$7d,$11,$35,$b4,$6d,$0b,$3b,$1d,$a4,$0a,$95,
    $5d,$7b,$1b,$36,$a4,$e7,$45,$28,$c0,$e0,$e0,$93,$39,$db,$c3,$5a,$78,$52,$40,$bd,$a4,$db,$ef,$95,$fc,
    $1f,$ae,$9c,$07,$e3,$e3,$95,$c8,$8b,$de,$a6,$81,$65,$a8,$75,$99,$fd,$23,$af,$44,$9f,$9f,$0f,$c1,$6a,
    $b4,$5a,$68,$51,$80,$5e,$b6,$60,$74,$ce,$5a,$13,$19,$3e,$04,$40,$4c,$46,$8a,$70,$91,$ae,$79,$58,$30,
    $6c,$6b,$bf,$cc,$f0,$62,$0d,$22,$ab,$60,$c4,$7e,$a4,$75,$8b,$de,$73,$74,$93,$61,$01,$35,$9f,$3e,$86,
    $aa,$6c,$5b,$b1,$7e,$97,$f8,$8f,$78,$b0,$1d,$4a,$a7,$98,$9e,$73,$47,$48,$9e,$19,$e2,$87,$b8,$72,$50,
    $1c,$2f,$3f,$a6,$b8,$2b,$66,$1a,$11,$52,$39,$eb,$60,$11,$36,$ca,$47,$65,$ca,$d7,$f6,$37,$d4,$3a,$58,
    $e1,$a6,$ec,$bd,$5c,$ae,$ef,$ad,$d3,$bb,$92,$67,$dc,$da,$97,$83,$b1,$46,$47,$aa,$b8,$d0,$0e,$53,$61,
    $6c,$77,$55,$09,$c3,$76,$a8,$22,$71,$cf,$b2,$6d,$94,$fd,$2b,$7e,$03,$9b,$e0,$87,$30,$97,$77,$42,$22,
    $1e,$6b,$c1,$8d,$ca,$e8,$b8,$a0,$40,$58,$c2,$7f,$bc,$fb,$86,$8f,$49,$9b,$ab,$1d,$85,$aa,$00,$c9,$f1,
    $70,$95,$a2,$ff,$b3,$40,$c5,$cc,$ad,$7a,$57,$c0,$8e,$44,$50,$74,$af,$34,$78,$fd,$0b,$4f,$48,$91,$8a,
    $cb,$a4,$be,$a8,$e9,$16,$34,$fa,$99,$83,$4e,$06,$af,$83,$11,$55,$5e,$27,$3a,$a2,$24,$a5,$4e,$74,$6c,
    $21,$99,$08,$b5,$98,$1f,$cb,$3f,$63,$cb,$78,$01,$5f,$b2,$b1,$66,$9e,$6b,$1d,$f7,$65,$03,$13,$6e,$d1,
    $08,$f3,$bc,$75,$25,$2f,$05,$24,$8a,$92,$c6,$a5,$02,$a1,$9f,$65,$2c,$d6,$32,$0d,$c0,$f3,$59,$0c,$e7,
    $df,$4d,$82,$48,$f4,$12,$75,$26,$77,$d3,$4e,$54,$8d,$32,$93,$25,$09,$38,$2c,$b0,$d7,$99,$6d,$cc,$9d,
    $8d,$c2,$b0,$21,$0a,$c4,$77,$1f,$1c,$d2,$ad,$6f,$a3,$e1,$3f,$1b,$5a,$71,$96,$5e,$9e,$83,$1d,$59,$eb,
    $22,$73,$7f,$69,$28,$ea,$3c,$64,$f0,$a9,$2b,$e2,$aa,$69,$ba,$6d,$a8,$94,$94,$51,$15,$0e,$b1,$35,$41,
    $1e,$c5,$bc,$77,$ec,$bf,$ce,$3d,$26,$a8,$ef,$f2,$33,$15,$88,$e9,$1d,$89,$11,$a7,$30,$5f,$15,$66,$d4,
    $ca,$c7,$c6,$7c,$ef,$f8,$6a,$86,$98,$30,$58,$eb,$b4,$70,$eb,$a0,$db,$36,$1b,$2b,$df,$73,$95,$d4,$91,
    $31,$15,$b2,$b8,$07,$03,$db,$31,$c6,$a7,$02,$ab,$98,$d5,$65,$08,$93,$9e,$03,$e1,$70,$7c,$04,$6a,$43,
    $0d,$37,$ac,$83,$16,$1d,$d2,$d2,$23,$21,$fd,$69,$ba,$55,$09,$ef,$d8,$d9,$66,$b8,$9d,$9b,$86,$a5,$80,
    $89,$36,$26,$b2,$97,$0f,$e7,$2a,$0e,$af,$cc,$cc,$ba,$56,$da,$24,$0a,$7b,$27,$69,$2b,$7f,$cc,$e3,$af,
    $cc,$c5,$77,$85,$ee,$d7,$d0,$2a,$39,$50,$79,$e0,$97,$5a,$58,$41,$9a,$3e,$3d,$f8,$f5,$34,$fb,$e4,$7c,
    $1a,$91,$a8,$9d,$ba,$c1,$8c,$f6,$f6,$37,$f8,$61,$ad,$bc,$69,$4f,$e1,$47,$84,$46,$7b,$a1,$54,$14,$2a,
    $0a,$f4,$47,$2f,$1d,$e7,$7f,$0b,$ee,$bd,$8d,$53,$9d,$ce,$ed,$3f,$7a,$f5,$3d,$f0,$38,$17,$3e,$e0,$e9,
    $8e,$f5,$5b,$bd,$5c,$8b,$9e,$8a,$ff,$7d,$a0,$5c,$8a,$fe,$ed,$2a,$34,$0f,$4d,$5a,$c9,$4f,$e5,$d5,$be,
    $3c,$ef,$4d,$8c,$1a,$77,$d1,$01,$7f,$81,$8d,$2a,$90,$8f,$3b,$96,$28,$ad,$c8,$bd,$33,$95,$a1,$ca,$25,
    $e9,$1a,$78,$c0,$1f,$21,$8f,$83,$3d,$f3,$c3,$91,$f8,$ed,$d0,$f1,$05,$e8,$7d,$a1,$df,$c4,$92,$90,$2c,
    $79,$a9,$6d,$da,$25,$24,$98,$69,$80,$94,$02,$e0,$45,$99,$2b,$a0,$18,$b2,$62,$27,$8b,$96,$44,$b0,$06,
    $c4,$5b,$bd,$61,$f3,$c9,$5f,$ec,$e6,$2b,$c4,$9c,$0d,$f7,$53,$3e,$01,$c7,$a3,$1c,$77,$f2,$17,$b9,$66,
    $17,$f1,$2c,$cc,$87,$37,$5f,$c1,$16,$ae,$6a,$e6,$2d,$a2,$59,$64,$93,$e2,$0c,$da,$ca,$18,$04,$17,$bc,
    $d6,$c3,$8e,$8c,$b4,$0a,$59,$1f,$96,$1a,$10,$d9,$7e,$de,$f3,$8c,$1e,$7d,$11,$2e,$d4,$02,$4f,$b5,$1d,
    $f1,$ee,$a9,$df,$a8,$ae,$31,$ee,$a9,$a9,$94,$eb,$a0,$47,$37,$09,$96,$9f,$9a,$f1,$90,$d9,$02,$42,$ee,
    $45,$5c,$17,$d5,$d9,$0f,$11,$8e,$a8,$7b,$2c,$63,$57,$0f,$38,$17,$ce,$d0,$1d,$1d,$13,$45,$a0,$4f,$2f,
    $fe,$3d,$71,$28,$22,$e6,$50,$c7,$7f,$8e,$62,$b9,$f5,$10,$70,$60,$39,$d3,$6b,$84,$89,$6d,$be,$08,$ac,
    $f1,$62,$cc,$7b,$10,$7c,$e7,$4d,$9b,$95,$f2,$45,$83,$b4,$2a,$fd,$a6,$61,$02,$a3,$97,$b4,$21,$d7,$7b,
    $80,$8b,$34,$a8,$cc,$87,$c2,$1b,$50,$f8,$2d,$41,$30,$f0,$42,$fb,$d4,$6e,$a6,$4d,$cb,$96,$01,$fb,$19,
    $5b,$6f,$bb,$b3,$e1,$f4,$96,$fe,$55,$c1,$88,$b2,$25,$77,$fc,$57,$a5,$6a,$03,$a7,$2e,$a9,$7e,$d3,$28,
    $4e,$84,$b9,$aa,$27,$24,$99,$41,$5a,$ca,$88,$12,$1a,$ea,$fb,$19,$30,$c7,$9b,$ba,$80,$23,$f8,$e6,$be,
    $71,$67,$d3,$10,$4f,$91,$7c,$3f,$58,$e5,$41,$f8,$b6,$7b,$9e,$9c,$30,$f2,$95,$5a,$64,$86,$50,$83,$4c,
    $51,$ea,$a4,$bb,$94,$b1,$3a,$76,$f5,$f6,$b4,$68,$7e,$6a,$c0,$d3,$20,$07,$6f,$00,$1e,$1d,$c4,$ec,$89,
    $30,$94,$4c,$eb,$53,$17,$5c,$e9,$71,$33,$71,$54,$7f,$d8,$81,$06,$5c,$30,$20,$47,$a2,$7c,$c7,$d8,$96,
    $f6,$48,$06,$40,$c6,$7f,$fb,$c7,$dd,$79,$08,$f7,$0e,$c6,$9f,$ed,$e5,$01,$7b,$eb,$b8,$d2,$40,$4d,$64,
    $b3,$d9,$82,$ae,$72,$19,$79,$9d,$af,$f0,$12,$99,$17,$ec,$49,$fe,$9c,$85,$47,$7b,$f8,$36,$62,$5d,$38,
    $4b,$0c,$60,$b9,$88,$b0,$c6,$15,$13,$9b,$a9,$07,$f7,$ec,$4a,$2c,$ea,$38,$f5,$3c,$19,$07,$9b,$f1,$3a,
    $c0,$d1,$03,$9c,$c7,$5c,$a5,$0c,$e7,$4f,$f6,$cc,$ca,$b8,$e7,$f5,$c5,$eb,$3d,$03,$70,$22,$58,$ab,$59,
    $2e,$95,$4a,$82,$e0,$03,$d6,$82,$bf,$90,$0b,$cd,$44,$f9,$47,$13,$00,$d2,$11,$69,$01,$f0,$02,$ca,$ca,
    $62,$bf,$c7,$38,$f8,$ae,$2e,$0c,$49,$0b,$8b,$8e,$fc,$90,$02,$80,$91,$97,$02,$60,$ba,$ab,$cf,$95,$c4,
    $ba,$22,$cc,$c9,$ea,$83,$7b,$fd,$4f,$9a,$9d,$2d,$6f,$66,$b7,$89,$29,$a2,$10,$cb,$8b,$f9,$80,$cb,$94,
    $f9,$63,$1a,$7b,$74,$d1,$cf,$71,$9f,$92,$dc,$22,$e4,$19,$10,$91,$c2,$4c,$b0,$07,$d9,$6e,$76,$70,$5b,
    $ab,$14,$5d,$c6,$15,$c5,$2f,$e3,$70,$32,$82,$95,$cc,$af,$da,$4e,$6b,$04,$35,$2f,$29,$20,$3f,$22,$8a,
    $c4,$3d,$74,$2a,$6d,$fe,$51,$b6,$5c,$8b,$a8,$47,$4f,$db,$86,$6a,$d7,$16,$00,$7f,$64,$48,$a5,$ad,$a9,
    $b7,$49,$ce,$e6,$42,$bb,$07,$2b,$62,$88,$87,$3d,$bd,$49,$8c,$4f,$7c,$96,$f2,$97,$34,$d0,$e5,$0c,$ff,
    $de,$e4,$d0,$4a,$25,$75,$07,$99,$f0,$7d,$36,$3f,$3e,$12,$10,$e8,$4a,$e2,$72,$43,$09,$03,$4d,$b2,$dc,
    $f4,$6e,$ad,$f8,$eb,$7f,$7e,$b1,$fc,$83,$c0,$11,$f8,$99,$98,$e8,$00,$a0,$90,$f0,$e1,$27,$41,$cd,$7a,
    $25,$47,$0d,$b7,$d3,$90,$7b,$e7,$c3,$21,$c5,$3a,$ff,$7f,$01,$bc,$a6,$a5,$3e,$36,$bb,$fc,$c9,$6e,$e6,
    $6e,$cb,$aa,$50,$63,$5e,$aa,$ad,$3b,$fb,$e7,$71,$2f,$96,$20,$da,$3d,$26,$dd,$52,$a6,$51,$a2,$4f,$b3,
    $bd,$69,$d0,$c8,$97,$2b,$49,$49,$09,$56,$e3,$d5,$25,$52,$b6,$d6,$23,$22,$9c,$cf,$c6,$21,$c2,$19,$06,
    $3c,$ed,$1e,$c0,$0e,$0c,$44,$1d,$17,$0d,$d9,$13,$bc,$08,$78,$28,$ed,$5f,$88,$ae,$11,$cd,$f6,$f4,$b6,
    $44,$db,$d4,$6e,$f7,$f5,$86,$de,$b3,$04,$75,$1b,$33,$b0,$2f,$3d,$50,$57,$fc,$07,$f0,$d9,$b7,$42,$6d,
    $9b,$0a,$f3,$19,$3f,$d6,$97,$ca,$47,$2f,$c9,$06,$ce,$7d,$9b,$ce,$c0,$71,$0f,$43,$08,$93,$8c,$df,$aa,
    $30,$d3,$bf,$3b,$54,$9b,$a4,$74,$ee,$81,$38,$f3,$f2,$25,$65,$f7,$f9,$b1,$be,$5e,$67,$c9,$6e,$0a,$c5,
    $5f,$ac,$41,$76,$83,$77,$ad,$f0,$54,$8a,$59,$41,$3e,$f4,$b6,$a6,$67,$0e,$d7,$4d,$2d,$fe,$1a,$22,$38,
    $0b,$ce,$95,$fb,$32,$fd,$a0,$3b,$bf,$4f,$4f,$20,$46,$a9,$fa,$28,$bd,$bf,$cd,$0e,$88,$65,$ad,$9f,$3c,
    $8a,$4d,$1a,$ea,$17,$75,$ee,$e0,$37,$d5,$3f,$08,$7a,$62,$c9,$58,$ed,$8b,$88,$56,$48,$45,$53,$b6,$15,
    $42,$38,$39,$fe,$b6,$88,$0d,$d0,$57,$45,$23,$30,$27,$75,$b6,$15,$11,$26,$7b,$77,$e2,$b5,$46,$3d,$0a,
    $2e,$aa,$f1,$73,$8d,$2f,$65,$8f,$4b,$5e,$04,$02,$b7,$f8,$df,$ab,$5a,$0c,$c5,$46,$83,$14,$a7,$c2,$8e,
    $d0,$6c,$5e,$75,$a8,$29,$80,$fa,$1f,$41,$0a,$d3,$b3,$c3,$6e,$37,$03,$d9,$88,$d3,$5b,$a2,$f0,$8f,$cf,
    $e3,$85,$18,$f9,$08,$8c,$61,$0c,$f5,$00,$32,$d6,$fa,$b6,$92,$59,$f0,$ea,$70,$c1,$ba,$c5,$59,$ae,$02,
    $50,$d7,$da,$ee,$ea,$8b,$c5,$0f,$4b,$ca,$c9,$f6,$f2,$73,$ba,$7d,$d3,$8a,$60,$49,$d5,$30,$08,$f1,$20,
    $e9,$d2,$30,$72,$18,$d5,$ac,$8e,$c2,$10,$db,$da,$c5,$4e,$d6,$1d,$d9,$67,$9a,$b7,$e2,$45,$e2,$77,$e6,
    $9a,$ea,$03,$3e,$4e,$c0,$f0,$0e,$ae,$1f,$00,$46,$71,$2a,$20,$bc,$3c,$a0,$68,$20,$86,$9d,$23,$ab,$21,
    $32,$c1,$41,$b4,$a8,$77,$33,$82,$eb,$70,$fa,$59,$e7,$1d,$7d,$2f,$78,$09,$fb,$8c,$38,$a5,$c3,$52,$a2,
    $e3,$c8,$d7,$3c,$50,$2e,$fe,$aa,$5f,$34,$21,$0a,$9c,$86,$18,$4e,$ab,$41,$bf,$a0,$f5,$26,$7e,$96,$9a,
    $e4,$af,$e8,$74,$d8,$14,$ea,$6d,$61,$5a,$83,$35,$5f,$18,$04,$3a,$4e,$23,$23,$25,$c8,$52,$df,$10,$e8,
    $d4,$b4,$34,$be,$5d,$ba,$d6,$7e,$9a,$d8,$93,$28,$2f,$37,$52,$fe,$20,$fe,$b9,$cf,$35,$81,$30,$09,$c1,
    $a5,$e5,$ac,$ff,$c5,$34,$57,$73,$c1,$aa,$09,$0b,$77,$32,$5c,$dd,$e9,$61,$e6,$6b,$13,$83,$4a,$ca,$f3,
    $b1,$e8,$1b,$49,$a3,$3a,$88,$79,$20,$ad,$e3,$9a,$18,$ec,$2b,$8d,$01,$ed,$2d,$76,$79,$80,$be,$a0,$1d,
    $82,$40,$90,$98,$f7,$2b,$6c,$03,$d2,$11,$0a,$a4,$1e,$b0,$61,$a1,$48,$28,$c9,$22,$ed,$61,$2b,$14,$e0,
    $77,$6b,$2c,$98,$e9,$75,$7f,$ff,$52,$37,$04,$51,$b6,$06,$52,$cb,$20,$ef,$94,$73,$f6,$90,$ea,$ca,$79,
    $6a,$84,$ab,$4d,$62,$e0,$e7,$1e,$7f,$92,$f5,$73,$ba,$11,$57,$62,$b5,$c1,$53,$2e,$c6,$76,$63,$13,$77,
    $e7,$f3,$68,$b4,$46,$02,$81,$e4,$b1,$26,$5c,$23,$34,$b2,$1f,$71,$9b,$0e,$db,$6c,$b5,$37,$f1,$66,$34,
    $bd,$4e,$14,$dd,$02,$27,$41,$fa,$b9,$5b,$de,$1d,$32,$3f,$4d,$57,$e6,$07,$8c,$a4,$96,$0c,$d8,$14,$42,
    $d5,$bd,$ca,$e5,$e1,$16,$0e,$22,$f9,$ed,$33,$3d,$28,$c8,$3f,$9c,$da,$da,$cc,$47,$c8,$3c,$cd,$99,$e1,
    $fd,$53,$2a,$a5,$fe,$78,$3a,$3e,$cc,$db,$a8,$22,$04,$c0,$88,$11,$fa,$6e,$3f,$0a,$dc,$6e,$fd,$2f,$c1,
    $24,$6f,$31,$a4,$f9,$8a,$32,$52,$0c,$49,$31,$be,$00,$67,$6c,$21,$8e,$f9,$5b,$b0,$db,$90,$88,$26,$2a,
    $d2,$86,$1c,$4d,$68,$50,$61,$9c,$58,$65,$3a,$42,$7e,$a8,$45,$4e,$e1,$9e,$ae,$5c,$14,$c6,$77,$c7,$2b,
    $e8,$04,$ee,$2b,$d4,$c0,$2c,$ed,$ed,$a0,$7d,$e4,$e1,$6c,$99,$1e,$3a,$0e,$41,$01,$ce,$74,$44,$06,$b9,
    $fb,$f8,$3c,$cc,$f0,$9f,$f2,$d1,$21,$e2,$fc,$5f,$a5,$bd,$b3,$8e,$21,$d8,$aa,$70,$f3,$22,$41,$e5,$e0,
    $bc,$03,$e5,$d5,$af,$9e,$c1,$88,$3b,$9b,$a6,$b5,$bd,$61,$94,$ee,$89,$dc,$22,$2b,$ed,$4a,$a7,$87,$cf,
    $dc,$33,$2b,$b8,$6c,$55,$60,$f2,$11,$18,$b2,$65,$8d,$5d,$f3,$7e,$78,$8e,$40,$41,$45,$40,$69,$ce,$c2,
    $c4,$3d,$98,$1a,$87,$dc,$35,$99,$a7,$e8,$d4,$66,$ff,$05,$3a,$88,$18,$7f,$80,$5a,$fe,$aa,$79,$2c,$be,
    $65,$4a,$46,$70,$a4,$ca,$fe,$d6,$89,$a6,$cb,$dd,$2e,$c7,$01,$44,$59,$79,$60,$a7,$2f,$ba,$ac,$96,$60,
    $b9,$ca,$dc,$32,$af,$14,$66,$3e,$72,$50,$8f,$99,$8c,$a6,$95,$23,$da,$97,$50,$94,$26,$02,$42,$b9,$58,
    $9e,$74,$c3,$e2,$13,$87,$a2,$7d,$a1,$4b,$06,$10,$8f,$a8,$81,$54,$76,$31,$80,$25,$16,$e5,$3a,$47,$d3,
    $9c,$eb,$07,$dd,$ef,$40,$6b,$29,$78,$e5,$3a,$d8,$15,$57,$04,$df,$95,$10,$23,$88,$e7,$a6,$15,$d3,$56,
    $85,$09,$2c,$2d,$c2,$b9,$27,$00,$b1,$a8,$2c,$5b,$f1,$6d,$ea,$dc,$56,$13,$a7,$d8,$f5,$5e,$94,$af,$6d,
    $59,$57,$bc,$e1,$b9,$3b,$a8,$64,$96,$30,$39,$64,$01,$cc,$64,$fb,$50,$33,$59,$ba,$ea,$20,$97,$bd,$7e,
    $80,$e6,$f2,$1b,$9f,$e4,$09,$6c,$39,$ac,$ca,$83,$d4,$5e,$43,$9e,$3a,$17,$80,$5b,$34,$52,$d5,$95,$ba,
    $73,$ed,$bc,$ae,$a3,$8f,$eb,$90,$35,$c2,$03,$41,$8d,$ed,$f7,$28,$1c,$e7,$c3,$71,$51,$b1,$85,$ec,$6a,
    $bb,$57,$5a,$d9,$2b,$b5,$e2,$01,$fb,$9e,$2f,$be,$8e,$a6,$24,$71,$42,$95,$f1,$e8,$38,$b4,$97,$fd,$89,
    $71,$57,$b4,$9b,$f0,$a9,$02,$fb,$e0,$1b,$08,$a8,$59,$a2,$b1,$88,$22,$fd,$0f,$6c,$4f,$5c,$09,$ed,$37,
    $45,$83,$ad,$c1,$f4,$db,$2d,$66,$56,$74,$22,$d3,$3c,$e2,$b5,$62,$99,$42,$1d,$02,$13,$60,$9a,$1e,$00,
    $db,$2e,$04,$5a,$ad,$06,$45,$35,$37,$08,$d3,$cd,$16,$44,$9e,$be,$ed,$d1,$d7,$29,$e4,$b7,$03,$f1,$59,
    $6a,$b4,$45,$13,$68,$6b,$16,$0b,$b5,$9f,$2d,$11,$4c,$41,$ad,$2a,$6b,$72,$29,$55,$56,$9c,$88,$6b,$15,
    $f0,$66,$1f,$12,$9d,$4a,$cb,$79,$3a,$76,$52,$fa,$b2,$4f,$32,$88,$06,$71,$f8,$90,$d6,$f5,$93,$31,$c1,
    $ec,$4f,$7d,$c8,$5f,$66,$38,$b0,$fe,$24,$5f,$7d,$d4,$29,$f8,$89,$ff,$58,$57,$96,$e9,$4c,$13,$81,$80,
    $22,$98,$b8,$fe,$1e,$eb,$83,$d4,$c3,$a8,$b8,$58,$9a,$52,$e5,$e4,$2f,$90,$8f,$cb,$8f,$35,$98,$27,$9f,
    $36,$b5,$59,$82,$22,$12,$cf,$b7,$48,$49,$16,$f5,$55,$dc,$e9,$68,$09,$bf,$fa,$03,$ee,$e9,$89,$3d,$3d,
    $38,$89,$33,$5b,$b4,$23,$ae,$71,$90,$af,$26,$6a,$f1,$6c,$ab,$78,$43,$05,$28,$02,$ff,$1f,$87,$6e,$c1,
    $ae,$9b,$bd,$4c,$48,$9b,$c3,$71,$26,$33,$59,$18,$c9,$90,$9e,$65,$30,$67,$70,$e8,$2f,$a3,$d4,$3b,$53,
    $9f,$70,$3a,$65,$c2,$9e,$90,$ba,$b4,$bb,$33,$60,$b8,$cc,$68,$2d,$56,$1d,$a6,$df,$62,$63,$20,$c4,$33,
    $0f,$2e,$23,$21,$9d,$d5,$0e,$64,$4e,$c8,$13,$51,$c8,$61,$4d,$47,$5d,$aa,$91,$b9,$5a,$10,$51,$88,$b5,
    $f5,$e9,$8a,$67,$9c,$25,$87,$ec,$b4,$2c,$4f,$9b,$f8,$d1,$dd,$e7,$60,$55,$72,$15,$40,$10,$c2,$af,$83,
    $e3,$20,$ca,$64,$ad,$d1,$41,$22,$2a,$08,$8e,$7e,$fe,$6d,$9e,$fc,$62,$7b,$02,$9a,$dd,$2e,$3b,$0d,$13,
    $28,$d4,$75,$49,$a3,$71,$2d,$86,$6b,$5b,$6c,$f9,$b0,$8b,$a4,$28,$ad,$ff,$b5,$97,$bb,$a6,$8a,$13,$f6,
    $cd,$c0,$1d,$10,$b9,$be,$42,$74,$92,$83,$21,$52,$71,$0b,$ed,$49,$86,$c3,$16,$1b,$52,$d9,$ac,$d9,$de,
    $c5,$9c,$81,$15,$48,$40,$cd,$e8,$cf,$c3,$c5,$d3,$97,$20,$9d,$ca,$0c,$91,$10,$73,$bc,$80,$33,$72,$26,
    $05,$85,$ec,$86,$1d,$a8,$b5,$4e,$bb,$47,$27,$74,$c1,$cc,$1e,$5e,$27,$52,$22,$fa,$86,$ec,$30,$87,$98,
    $35,$47,$0b,$4d,$d9,$ef,$b5,$e8,$69,$df,$29,$99,$45,$b0,$12,$b6,$28,$30,$5d,$5f,$5c,$0c,$c3,$dd,$51,
    $92,$b1,$47,$d8,$bb,$43,$be,$73,$3b,$2c,$98,$9f,$5b,$3c,$89,$f4,$e7,$f6,$5c,$7c,$32,$05,$c2,$48,$d8,
    $de,$1e,$e5,$f8,$56,$00,$46,$98,$95,$13,$bf,$73,$29,$88,$bc,$e6,$aa,$e6,$23,$0c,$b4,$be,$ed,$c9,$21,
    $e6,$a6,$19,$15,$97,$d4,$42,$43,$b9,$aa,$bc,$71,$58,$20,$9b,$af,$55,$64,$ad,$e1,$a0,$74,$7f,$4e,$20,
    $e1,$07,$a6,$37,$b0,$3b,$5f,$70,$da,$36,$35,$9d,$74,$57,$28,$84,$3b,$62,$9f,$63,$e8,$de,$09,$2f,$1a,
    $0f,$87,$08,$21,$fd,$a5,$ad,$eb,$c2,$ce,$5b,$cc,$ca,$3c,$1e,$2f,$c1,$a1,$ac,$4a,$60,$cb,$fe,$db,$ee,
    $1c,$2b,$5a,$e4,$12,$f7,$7e,$16,$9e,$e3,$d3,$91,$47,$5b,$19,$03,$64,$de,$e1,$cb,$77,$9c,$ae,$62,$cc,
    $83,$09,$f1,$8b,$6f,$8d,$fd,$28,$a2,$94,$c4,$47,$b6,$0f,$95,$b9,$dd,$9c,$d8,$c8,$5e,$42,$62,$4c,$e3,
    $a3,$31,$78,$ab,$c0,$d3,$de,$f2,$c6,$b1,$1a,$57,$65,$73,$2f,$4b,$48,$cf,$6a,$f7,$6e,$59,$c8,$73,$e4,
    $13,$44,$fb,$22,$ed,$04,$a9,$ac,$49,$f2,$de,$be,$c8,$7d,$1e,$8c,$c4,$29,$7c,$3a,$dd,$15,$51,$2c,$09,
    $70,$ae,$b3,$0c,$12,$44,$f8,$fc,$2a,$c2,$5c,$87,$e5,$91,$df,$7d,$9c,$f5,$ca,$fa,$5f,$99,$fb,$27,$7c,
    $08,$71,$21,$5d,$16,$b9,$3b,$eb,$3f,$8d,$e1,$d9,$c1,$92,$16,$0d,$11,$fb,$14,$c7,$d5,$4e,$e9,$f5,$ab,
    $81,$e0,$4e,$3d,$e0,$e0,$1e,$41,$71,$d7,$6a,$53,$50,$93,$64,$43,$c3,$03,$85,$7f,$ca,$4b,$88,$c0,$59,
    $d9,$c9,$2a,$8f,$20,$c0,$0f,$bc,$9b,$cc,$96,$74,$5d,$04,$fe,$42,$22,$24,$20,$8c,$65,$1d,$d2,$24,$92,
    $f0,$17,$dd,$56,$6a,$08,$14,$7e,$55,$ab,$31,$92,$4d,$de,$19,$7b,$a9,$26,$23,$26,$27,$c0,$90,$b7,$60,
    $f8,$44,$19,$0c,$2e,$b0,$0f,$59,$f5,$e1,$3e,$d8,$45,$bb,$af,$35,$6e,$5e,$61,$93,$d8,$9a,$71,$34,$cd,
    $56,$89,$5e,$f3,$e1,$45,$96,$41,$a4,$ea,$7c,$7f,$06,$20,$89,$69,$28,$3e,$81,$51,$a6,$33,$f5,$99,$46,
    $85,$92,$eb,$7e,$b1,$ad,$8c,$93,$fa,$fe,$59,$6f,$d7,$e6,$7f,$a1,$5f,$9c,$6a,$71,$a6,$00,$99,$d0,$7a,
    $ad,$be,$5b,$50,$93,$b9,$e3,$c3,$c3,$21,$df,$50,$e9,$de,$55,$5c,$fc,$62,$ce,$c2,$c1,$c0,$1f,$15,$7f,
    $67,$6e,$19,$b4,$a0,$d5,$58,$e2,$7a,$c9,$9f,$a1,$d2,$74,$bb,$d3,$bf,$8e,$11,$13,$32,$63,$12,$38,$bf,
    $81,$2e,$db,$a0,$3f,$cc,$4e,$09,$5b,$1d,$22,$1b,$07,$5f,$4a,$94,$eb,$11,$ef,$5b,$6e,$f7,$ba,$e6,$cd,
    $67,$ff,$41,$ce,$f5,$7d,$77,$b8,$09,$84,$2c,$f4,$9b,$fb,$a2,$6b,$ee,$61,$38,$68,$b1,$e2,$b9,$e2,$25,
    $fb,$04,$dc,$b4,$d9,$42,$63,$4e,$84,$df,$82,$60,$6e,$21,$94,$2a,$89,$17,$c3,$9a,$69,$30,$cf,$a1,$34,
    $43,$d8,$2c,$31,$14,$b5,$f5,$e2,$09,$50,$0c,$7a,$75,$96,$15,$9f,$e4,$d8,$59,$39,$4e,$c3,$f0,$31,$e7,
    $a3,$2e,$ec,$74,$c6,$fc,$9e,$6e,$c8,$9d,$19,$d6,$71,$c4,$59,$e2,$9f,$d3,$18,$17,$f7,$ec,$a5,$f3,$cd,
    $21,$ae,$55,$3b,$cb,$85,$0b,$7d,$9b,$cf,$54,$c2,$34,$6d,$2d,$0e,$a3,$42,$1e,$1b,$18,$27,$10,$7d,$41,
    $8b,$85,$28,$7d,$3f,$a4,$3a,$cd,$67,$6d,$26,$78,$31,$da,$12,$da,$da,$7f,$4f,$59,$78,$81,$f6,$73,$01,
    $b8,$f6,$51,$b7,$ea,$21,$d7,$6f,$fc,$fd,$ba,$0e,$41,$a8,$64,$49,$45,$eb,$78,$4a,$59,$96,$a6,$53,$c5,
    $31,$7d,$32,$3f,$4f,$4c,$fa,$af,$20,$f8,$e4,$c4,$b7,$0b,$45,$d3,$bc,$13,$4e,$2e,$fa,$f8,$7a,$4e,$f8,
    $7f,$7e,$48,$a0,$af,$44,$71,$f6,$1b,$5d,$a0,$8a,$d8,$94,$f5,$06,$57,$e1,$1a,$fc,$6c,$03,$ec,$19,$bc,
    $b2,$63,$70,$d4,$4f,$f1,$88,$0b,$3b,$0a,$c7,$40,$9f,$3a,$be,$e5,$00,$b6,$3a,$50,$50,$2f,$55,$af,$30,
    $33,$33,$9a,$af,$db,$9b,$26,$cc,$41,$40,$b9,$b8,$5e,$e3,$51,$77,$9f,$21,$e1,$6a,$79,$80,$6c,$a3,$1a,
    $59,$0a,$15,$bf,$0b,$af,$bc,$c8,$48,$56,$6c,$90,$60,$9f,$78,$a5,$2d,$95,$36,$b0,$77,$9d,$8a,$ba,$3f,
    $fd,$94,$6f,$0e,$da,$a6,$09,$dc,$1b,$01,$c5,$e0,$1a,$8f,$5d,$ec,$fc,$dc,$0d,$e9,$69,$ae,$8d,$e4,$7c,
    $87,$e6,$a9,$32,$6a,$da,$f3,$1c,$78,$c0,$2a,$0c,$80,$ae,$4f,$0a,$cd,$66,$86,$b8,$e2,$36,$be,$d8,$50,
    $7f,$8b,$d2,$3e,$cd,$18,$8c,$04,$20,$e3,$d0,$ad,$f5,$fc,$a5,$69,$f0,$f2,$77,$24,$60,$3d,$c0,$00,$68,
    $82,$9e,$57,$32,$15,$82,$b8,$91,$78,$77,$d1,$a6,$88,$ee,$66,$df,$1a,$41,$c2,$62,$4e,$41,$bf,$67,$62,
    $5b,$85,$bf,$14,$dc,$d7,$2d,$b1,$67,$d8,$e5,$7b,$15,$25,$05,$af,$2f,$9d,$1e,$0f,$f1,$24,$d7,$b8,$9e,
    $00,$6c,$c7,$fa,$4e,$70,$87,$3c,$30,$33,$f6,$71,$b8,$4d,$62,$b5,$5b,$ea,$11,$af,$c8,$96,$23,$fe,$e2,
    $7f,$a9,$3b,$7d,$fd,$33,$9c,$bf,$45,$28,$d8,$f9,$f2,$bd,$0e,$e7,$45,$2a,$91,$26,$21,$34,$96,$b6,$df,
    $ce,$b2,$a6,$07,$74,$95,$f3,$ee,$53,$7e,$be,$e4,$ce,$65,$f5,$2f,$f5,$9b,$b5,$bb,$9a,$cc,$94,$4b,$4b,
    $cb,$33,$6a,$90,$41,$a6,$b7,$ce,$db,$ea,$75,$91,$cb,$74,$e6,$66,$a3,$6a,$09,$f4,$77,$63,$3d,$05,$42,
    $66,$4b,$90,$85,$61,$7c,$5c,$a9,$f6,$d9,$1c,$37,$2d,$e6,$3d,$40,$6b,$c4,$e6,$1b,$19,$98,$ee,$85,$70,
    $57,$23,$15,$62,$43,$a5,$2c,$48,$19,$2c,$60,$16,$6b,$be,$91,$11,$5b,$37,$4a,$fd,$69,$ec,$8c,$11,$42,
    $c2,$0d,$c2,$34,$8a,$61,$1b,$5f,$f3,$95,$01,$c6,$f0,$f5,$f1,$b2,$05,$d6,$10,$f0,$05,$f8,$f2,$60,$1e,
    $fc,$23,$dd,$1d,$d2,$e1,$7e,$2f,$4e,$e8,$59,$b5,$46,$b9,$3d,$04,$f5,$2f,$64,$95,$d2,$e1,$b0,$56,$0c,
    $0e,$29,$e7,$54,$34,$86,$a1,$e5,$cf,$92,$a0,$ef,$ad,$36,$e0,$7b,$39,$7f,$43,$49,$97,$8e,$94,$fd,$8f,
    $37,$a6,$26,$53,$ea,$dd,$08,$3d,$f8,$ae,$b8,$d7,$06,$f9,$79,$5a,$ad,$ae,$88,$99,$b1,$29,$00,$6d,$a8,
    $02,$53,$e2,$07,$93,$73,$fc,$bc,$01,$43,$b9,$20,$6d,$e3,$ab,$33,$49,$0e,$c4,$52,$69,$73,$70,$05,$c2,
    $15,$6d,$49,$63,$6c,$cd,$8b,$9e,$c6,$77,$50,$19,$2b,$b3,$d0,$a6,$20,$00,$8a,$fc,$14,$cd,$ba,$f4,$fb,
    $09,$fd,$59,$2a,$77,$e3,$24,$19,$a8,$cf,$ea,$46,$0b,$b5,$51,$ff,$c9,$35,$db,$7f,$30,$98,$74,$97,$cd,
    $40,$5b,$2b,$6b,$2d,$f8,$61,$8b,$87,$9c,$d4,$7f,$2c,$6e,$82,$f3,$f7,$5a,$b6,$3c,$f1,$1a,$43,$7b,$cd,
    $5d,$4a,$02,$cf,$18,$b0,$2a,$1e,$9e,$2a,$82,$5e,$90,$af,$37,$44,$ec,$69,$ec,$66,$be,$5d,$fb,$9d,$c0,
    $61,$8c,$33,$5d,$93,$65,$8a,$f2,$3f,$de,$47,$44,$9d,$ef,$08,$08,$cd,$22,$8c,$6f,$c4,$e1,$fc,$e2,$69,
    $e0,$ea,$ea,$1f,$9b,$bd,$1c,$32,$a3,$1c,$4a,$bc,$4c,$d5,$a1,$4d,$0c,$27,$c4,$98,$40,$e4,$10,$74,$61,
    $6e,$c6,$a7,$de,$ea,$1c,$10,$10,$2e,$85,$37,$de,$56,$4e,$71,$81,$30,$ec,$69,$1e,$3f,$d3,$6c,$4d,$50,
    $a3,$0a,$d5,$56,$a3,$09,$6b,$23,$a5,$05,$f2,$f6,$61,$bb,$1c,$9d,$b3,$e3,$f4,$49,$7c,$4a,$ce,$4d,$cf,
    $89,$cd,$3a,$c4,$77,$5a,$72,$1b,$7e,$7e,$e0,$7a,$29,$a4,$5e,$df,$07,$eb,$ce,$77,$6a,$33,$3f,$a3,$bc,
    $ad,$9b,$99,$66,$ef,$7b,$dd,$52,$54,$8e,$d7,$87,$ad,$6f,$93,$8a,$c1,$c5,$58,$c3,$24,$6d,$18,$36,$ca,
    $fa,$11,$18,$04,$ae,$c6,$fc,$cc,$6c,$f8,$b8,$d8,$40,$ee,$8f,$fd,$ac,$4b,$d7,$cc,$8f,$e7,$f7,$78,$85,
    $4b,$3f,$d4,$b9,$9b,$9f,$bc,$82,$97,$4d,$ca,$55,$05,$63,$e7,$f2,$72,$66,$ac,$9e,$87,$35,$90,$bd,$a8,
    $7c,$dc,$06,$67,$59,$9a,$1a,$2c,$13,$29,$ee,$7f,$65,$bb,$26,$1e,$11,$96,$67,$e2,$be,$86,$6b,$fb,$63,
    $4f,$c3,$dd,$ac,$0b,$4a,$d0,$fc,$de,$c0,$84,$ba,$d4,$19,$24,$67,$d9,$8a,$e5,$0d,$e3,$e7,$d8,$90,$6a,
    $cd,$96,$03,$6f,$22,$4e,$75,$bd,$41,$4b,$07,$63,$d8,$d1,$e9,$d7,$09,$c2,$0b,$26,$1a,$22,$3b,$cc,$16,
    $bd,$e8,$16,$be,$bb,$70,$09,$f5,$51,$14,$4d,$03,$0e,$02,$5d,$60,$44,$d5,$60,$d1,$2c,$cb,$af,$9b,$cf,
    $21,$02,$19,$4b,$dc,$6e,$0d,$43,$6d,$7c,$95,$3c,$52,$e0,$b8,$ce,$fd,$d1,$b6,$9d,$78,$26,$3d,$b7,$86,
    $67,$1c,$19,$00,$58,$f5,$b9,$93,$2e,$ea,$0b,$3a,$31,$8e,$1e,$a1,$c4,$7c,$e2,$bb,$6f,$d6,$0d,$25,$ad,
    $11,$8b,$e9,$c9,$08,$cc,$1c,$35,$99,$74,$a1,$13,$4d,$99,$db,$e2,$c2,$91,$cf,$eb,$7c,$22,$84,$45,$2b,
    $76,$41,$34,$e6,$f9,$9d,$a8,$5c,$59,$94,$0f,$b5,$b5,$df,$a9,$c9,$c7,$13,$9c,$59,$fa,$38,$0c,$84,$0f,
    $5b,$cb,$63,$72,$0c,$c7,$76,$9f,$98,$0a,$ad,$08,$5e,$c9,$41,$37,$69,$1f,$d6,$83,$4e,$44,$e2,$d1,$99,
    $54,$73,$e4,$f0,$4c,$52,$95,$0d,$09,$73,$2f,$cf,$ac,$07,$df,$9a,$33,$d8,$03,$77,$c7,$52,$ae,$26,$8a,
    $bb,$ef,$1f,$cb,$6d,$c6,$ab,$e4,$1c,$ac,$a6,$a1,$02,$4b,$b7,$52,$6f,$db,$42,$6e,$f7,$bb,$11,$a2,$4a,
    $40,$7c,$ff,$d2,$4b,$a7,$6b,$74,$27,$c8,$f9,$14,$2d,$6b,$23,$e8,$ea,$e0,$16,$42,$81,$b9,$73,$14,$1f,
    $7c,$c8,$aa,$e4,$48,$80,$f1,$26,$b1,$ee,$d6,$ce,$55,$d8,$91,$62,$f7,$15,$b8,$d8,$f2,$d5,$71,$dc,$a0,
    $11,$ee,$86,$07,$f7,$fa,$9b,$33,$39,$b6,$2c,$12,$e2,$db,$e1,$be,$ec,$01,$8f,$d3,$0d,$65,$21,$c6,$ba,
    $24,$98,$c1,$f9,$7b,$84,$05,$e6,$1a,$22,$69,$30,$db,$7b,$8d,$b9,$83,$c0,$70,$e8,$4c,$e1,$76,$ef,$66,
    $0c,$1b,$a3,$2a,$ea,$3b,$4d,$f4,$31,$ac,$79,$94,$1b,$ff,$d9,$59,$1e,$2d,$80,$ac,$3e,$55,$73,$b3,$a7,
    $de,$aa,$eb,$3a,$ae,$ac,$87,$94,$33,$b9,$6d,$a5,$2e,$96,$20,$63,$c0,$e1,$46,$06,$ab,$da,$12,$bd,$96,
    $0c,$17,$1e,$9a,$e1,$de,$b9,$71,$99,$84,$71,$09,$2d,$4c,$73,$8e,$19,$ca,$d2,$d1,$70,$79,$56,$a0,$d2,
    $0c,$db,$57,$0c,$de,$fd,$e1,$37,$4d,$bf,$f3,$8a,$d8,$47,$60,$d9,$9b,$2a,$96,$41,$4d,$bb,$b5,$d7,$42,
    $12,$12,$22,$4d,$8a,$15,$41,$a7,$17,$80,$65,$a7,$78,$ec,$44,$1c,$28,$1c,$8b,$d2,$2e,$a0,$1c,$11,$d6,
    $99,$d3,$5c,$dc,$82,$ef,$87,$a6,$19,$4e,$f1,$20,$be,$64,$c4,$fa,$aa,$07,$30,$cc,$6a,$8a,$02,$70,$5f,
    $71,$c6,$24,$86,$6b,$c8,$af,$f3,$d8,$bc,$ca,$fa,$18,$a7,$8c,$12,$43,$33,$0b,$61,$38,$76,$dd,$7b,$63,
    $bc,$2b,$7a,$95,$5d,$0e,$9a,$7b,$67,$af,$2c,$85,$27,$79,$93,$cc,$71,$93,$e8,$ed,$94,$3c,$27,$b5,$58,
    $49,$9c,$03,$a2,$60,$75,$09,$61,$a3,$d5,$f5,$15,$cc,$62,$6e,$3f,$0e,$91,$70,$51,$b1,$c9,$7d,$48,$05,
    $13,$4f,$57,$ab,$35,$04,$15,$90,$d0,$8e,$47,$be,$cf,$b4,$37,$ce,$0b,$98,$1f,$be,$12,$d3,$5e,$67,$4b,
    $f8,$f4,$84,$f1,$18,$21,$7a,$1f,$e2,$25,$68,$1e,$31,$73,$f1,$bc,$2a,$2a,$bb,$9e,$16,$c7,$10,$fa,$86,
    $3b,$bc,$ea,$e3,$87,$01,$fb,$a8,$d3,$d2,$c7,$92,$98,$17,$db,$21,$de,$ae,$00,$b0,$49,$37,$40,$47,$af,
    $87,$15,$5a,$83,$4e,$17,$6d,$dd,$a6,$33,$72,$7c,$48,$77,$04,$f2,$b7,$9b,$41,$96,$62,$0f,$bb,$2c,$0f,
    $dc,$f2,$41,$d4,$0d,$d5,$f5,$49,$26,$72,$81,$6b,$d3,$dd,$74,$67,$91,$a5,$b1,$91,$29,$9b,$b6,$fa,$77,
    $27,$3c,$6d,$fe,$c6,$5e,$63,$b2,$90,$01,$01,$1c,$ba,$7b,$ff,$c5,$58,$f5,$79,$40,$71,$14,$4a,$39,$83,
    $93,$25,$c5,$c3,$cf,$ad,$1a,$83,$b2,$b9,$b5,$89,$38,$1a,$10,$6a,$5a,$af,$6f,$ef,$18,$46,$1f,$19,$69,
    $84,$1d,$49,$34,$02,$cd,$97,$0b,$a7,$9b,$3c,$44,$53,$a9,$ec,$9a,$06,$32,$56,$34,$03,$0d,$da,$e7,$e7,
    $ab,$13,$e1,$6a,$35,$d7,$01,$50,$d3,$11,$12,$fc,$0f,$72,$ac,$e2,$1b,$8e,$b0,$c5,$ff,$be,$86,$00,$bb,
    $c9,$d9,$67,$39,$58,$7d,$f9,$b5,$db,$1e,$05,$66,$28,$e8,$c1,$e5,$fd,$b9,$66,$1e,$4c,$f6,$97,$2a,$c0,
    $7f,$ac,$a5,$da,$dd,$61,$dc,$46,$26,$df,$36,$f1,$f8,$70,$9d,$b9,$82,$e1,$42,$c9,$5b,$87,$6f,$a1,$2b,
    $7d,$9c,$ef,$0a,$3d,$57,$f3,$3b,$ac,$60,$73,$65,$80,$ea,$af,$0a,$3d,$cb,$89,$47,$1a,$fb,$71,$29,$a6,
    $61,$b4,$66,$52,$fc,$f0,$d2,$22,$db,$e5,$2b,$f7,$fe,$1b,$9e,$70,$3c,$55,$15,$54,$65,$5d,$b0,$b6,$a7,
    $e7,$c3,$42,$56,$82,$a4,$43,$02,$ce,$b4,$16,$0d,$bb,$ab,$08,$98,$4a,$ac,$dd,$6c,$0c,$c8,$bc,$98,$37,
    $cb,$6c,$12,$0e,$1e,$a4,$a1,$40,$41,$ea,$f3,$63,$46,$48,$ed,$1a,$d6,$6c,$20,$01,$38,$9a,$b9,$7d,$05,
    $ac,$ea,$10,$28,$f3,$43,$b8,$a0,$3f,$c3,$fc,$f4,$9b,$f1,$3a,$a0,$f8,$45,$e5,$5c,$74,$ef,$89,$99,$d0,
    $9e,$51,$b3,$89,$e9,$54,$12,$82,$30,$6c,$e2,$f6,$a4,$96,$ab,$0d,$e5,$cd,$29,$cd,$ee,$cd,$90,$bc,$1f,
    $85,$04,$e3,$5d,$5b,$77,$e5,$90,$0e,$8a,$dc,$a7,$3e,$e0,$d0,$8a,$38,$33,$80,$ed,$7d,$34,$65,$ed,$ba,
    $4f,$3c,$f8,$d8,$5a,$b1,$d7,$f0,$86,$9d,$6e,$b4,$47,$c8,$fa,$a3,$2e,$a1,$c1,$1a,$a2,$06,$38,$7c,$d9,
    $a2,$3d,$cb,$ca,$92,$8c,$43,$39,$43,$17,$82,$7b,$9b,$45,$b9,$30,$e4,$85,$11,$19,$09,$5c,$37,$78,$9d,
    $8c,$6a,$46,$f9,$74,$52,$2a,$e6,$ee,$62,$8a,$5f,$5a,$df,$ef,$7f,$8e,$40,$82,$c2,$0d,$cb,$8d,$c2,$5d,
    $2a,$61,$73,$78,$d0,$36,$7a,$7e,$01,$73,$17,$93,$26,$8c,$a8,$60,$7f,$f3,$c7,$af,$02,$a0,$a4,$ca,$b8,
    $30,$07,$b8,$ea,$d2,$da,$33,$ba,$96,$03,$f5,$8d,$3d,$94,$a8,$5e,$cf,$5f,$0b,$c9,$2c,$25,$78,$82,$c5,
    $a4,$bf,$dd,$81,$bd,$47,$89,$3a,$45,$dd,$2b,$5b,$01,$37,$8f,$87,$64,$0e,$14,$92,$4c,$75,$c2,$39,$5b,
    $e2,$fc,$f3,$a0,$1a,$44,$e7,$c7,$9d,$b0,$8b,$5c,$8c,$1d,$6a,$ca,$38,$f1,$b3,$61,$08,$11,$49,$d4,$d3,
    $63,$29,$b6,$0f,$c5,$52,$84,$e6,$ca,$d4,$9e,$bd,$5b,$d1,$f5,$4b,$61,$86,$84,$fc,$fe,$d2,$bd,$c0,$76,
    $d8,$63,$b7,$4e,$22,$26,$aa,$5a,$c7,$84,$fa,$e7,$a6,$91,$e7,$3b,$2c,$09,$b8,$b2,$a6,$4a,$9c,$64,$a6,
    $ed,$b0,$db,$ef,$b5,$3c,$3b,$96,$24,$4b,$25,$f5,$ed,$39,$f2,$68,$2b,$33,$cd,$96,$d4,$b3,$f8,$66,$d2,
    $c6,$93,$ec,$b0,$f3,$53,$40,$d5,$bf,$5b,$ea,$a0,$e2,$21,$b3,$34,$02,$b3,$26,$2c,$f9,$7d,$e7,$76,$0d,
    $8a,$dc,$a5,$57,$a5,$65,$92,$77,$e5,$90,$29,$df,$61,$74,$90,$7b,$81,$e7,$5f,$73,$40,$ad,$f0,$ca,$a5,
    $87,$9a,$d9,$07,$64,$33,$f5,$86,$2d,$e1,$9a,$63,$14,$4d,$4e,$18,$50,$19,$27,$c3,$7b,$34,$e5,$7a,$cb,
    $42,$32,$53,$f2,$07,$68,$0b,$46,$2a,$7f,$82,$48,$a9,$2f,$60,$08,$99,$50,$47,$fb,$e0,$5d,$02,$3f,$a9,
    $1a,$c9,$8c,$22,$0c,$fd,$b6,$06,$a4,$67,$cd,$34,$22,$65,$e5,$e5,$50,$59,$22,$08,$1f,$79,$4a,$64,$62,
    $f6,$1b,$ef,$87,$4e,$27,$6c,$4d,$3c,$a2,$02,$c5,$89,$f1,$2a,$29,$f3,$4f,$38,$85,$f7,$6f,$85,$d4,$4b,
    $f8,$cb,$e9,$f0,$8e,$67,$b1,$0d,$66,$02,$e0,$0e,$01,$83,$67,$2d,$ad,$36,$f7,$ec,$c8,$fd,$06,$0e,$8c,
    $0f,$b3,$b3,$7e,$3b,$1a,$29,$f4,$c0,$81,$1c,$ee,$d5,$2e,$9f,$42,$d8,$f2,$63,$d9,$a3,$e0,$d0,$eb,$52,
    $d3,$c5,$6f,$06,$1a,$aa,$67,$ed,$95,$aa,$cb,$38,$6a,$64,$c1,$ed,$f3,$e4,$de,$c9,$d1,$01,$57,$48,$41,
    $84,$da,$e0,$e8,$00,$a0,$a1,$76,$ee,$d9,$93,$95,$f3,$0d,$f7,$b2,$2d,$99,$3a,$7e,$5a,$8d,$9a,$47,$9a,
    $6d,$77,$3a,$53,$5e,$0e,$be,$45,$f4,$5f,$2a,$88,$ff,$1a,$76,$a2,$e3,$d4,$a6,$22,$15,$b9,$44,$ac,$a8,
    $05,$56,$52,$c9,$5c,$c9,$8f,$ab,$ba,$69,$b3,$cf,$ae,$35,$e0,$be,$e2,$ec,$3f,$1e,$c6,$a3,$0c,$de,$7d,
    $d8,$97,$77,$31,$dd,$9a,$2c,$83,$6f,$81,$cf,$af,$fb,$cd,$40,$7d,$00,$bd,$ef,$06,$69,$cf,$52,$5f,$77,
    $df,$bb,$35,$80,$60,$b3,$26,$58,$ca,$80,$26,$b2,$4e,$61,$2f,$d0,$80,$c5,$ae,$d1,$77,$7b,$d1,$c4,$83,
    $27,$07,$05,$29,$6a,$4d,$68,$6b,$09,$38,$38,$99,$4e,$cf,$7a,$ee,$77,$e3,$9c,$a0,$a7,$4e,$27,$85,$47,
    $cc,$81,$3c,$fc,$a3,$90,$c3,$70,$25,$a8,$e3,$aa,$40,$10,$f6,$d0,$74,$d3,$f7,$e2,$e5,$37,$b5,$c6,$5a,
    $77,$05,$68,$57,$a7,$03,$14,$6e,$9a,$49,$9f,$89,$5c,$e2,$13,$58,$7d,$9b,$19,$68,$61,$93,$50,$8a,$72,
    $f7,$20,$0c,$5f,$b1,$c3,$e1,$77,$d3,$eb,$90,$95,$aa,$9c,$08,$60,$8b,$10,$5a,$6b,$83,$f3,$50,$78,$0a,
    $47,$ba,$07,$3d,$fb,$f4,$e7,$ed,$cb,$75,$10,$5d,$7c,$8d,$fc,$3d,$41,$bf,$78,$4b,$05,$d3,$ff,$ed,$cb,
    $43,$b0,$e9,$cc,$18,$d2,$32,$4e,$de,$a0,$0c,$7c,$ce,$1c,$97,$9b,$52,$01,$81,$eb,$02,$25,$d3,$37,$59,
    $ae,$d4,$e2,$3d,$28,$80,$83,$2f,$f2,$36,$71,$02,$05,$2a,$77,$09,$0d,$58,$3a,$55,$cb,$59,$4a,$41,$de,
    $ac,$30,$49,$cc,$27,$22,$89,$2f,$73,$5e,$c2,$44,$e9,$db,$92,$4a,$de,$5f,$35,$5c,$9d,$0b,$c9,$b6,$6e,
    $25,$f9,$1e,$ee,$04,$ba,$06,$9f,$89,$f6,$34,$63,$dc,$85,$d1,$02,$92,$4b,$0d,$0f,$33,$99,$b7,$d4,$35,
    $c0,$7e,$30,$72,$0d,$60,$14,$bd,$b5,$05,$d2,$35,$5d,$bc,$68,$e1,$50,$7a,$4a,$67,$0b,$05,$b5,$7e,$89,
    $03,$5e,$db,$48,$14,$01,$27,$bc,$e8,$eb,$d8,$8b,$80,$c1,$8b,$87,$ad,$85,$e9,$84,$49,$dd,$bd,$6a,$6f,
    $c9,$e1,$53,$ef,$80,$d0,$7c,$f2,$2b,$ea,$88,$db,$f0,$f2,$5b,$a9,$0e,$1b,$00,$0d,$c7,$8f,$cb,$e1,$1c,
    $ab,$4d,$5b,$34,$01,$5c,$fd,$65,$51,$d1,$43,$ef,$62,$27,$8f,$6f,$44,$10,$e4,$75,$45,$29,$d5,$6b,$91,
    $29,$dc,$6f,$82,$9e,$c1,$7d,$53,$e3,$62,$5e,$c1,$45,$dd,$83,$3d,$2f,$30,$22,$89,$44,$c2,$e4,$7d,$e3,
    $cb,$2e,$b9,$c1,$58,$68,$f9,$1d,$fa,$f6,$3b,$d5,$81,$9a,$3a,$7c,$94,$29,$8e,$73,$42,$9d,$4b,$09,$b4,
    $79,$d0,$8c,$b0,$b2,$4c,$97,$3e,$06,$0d,$a2,$ac,$67,$bf,$83,$c8,$69,$23,$7b,$24,$d4,$7a,$3e,$15,$1a,
    $dd,$d4,$c8,$3e,$35,$a3,$cc,$27,$f1,$b2,$98,$fd,$94,$80,$6b,$b0,$96,$28,$6e,$e4,$25,$26,$a6,$c1,$ea,
    $51,$5e,$7b,$b9,$21,$5e,$21,$88,$7b,$31,$95,$3e,$d8,$61,$a0,$0f,$5a,$b6,$a5,$7a,$5f,$78,$fe,$c7,$f5,
    $06,$a1,$e3,$43,$4f,$01,$71,$84,$5a,$cd,$71,$96,$f6,$b3,$b0,$7d,$d0,$9a,$c1,$34,$b2,$63,$22,$db,$9c,
    $e6,$fc,$57,$bc,$a7,$b1,$ba,$93,$66,$c6,$76,$b2,$3b,$74,$ba,$8f,$ba,$eb,$e8,$d6,$c9,$36,$01,$15,$a8,
    $af,$e3,$6c,$76,$74,$08,$6b,$bb,$56,$1d,$43,$2a,$f1,$8e,$8b,$a0,$95,$8a,$51,$21,$3c,$9f,$4a,$b7,$d6,
    $e5,$46,$e8,$5a,$19,$2d,$b0,$11,$d8,$7e,$6b,$91,$a0,$3b,$d3,$5e,$1d,$8c,$6b,$a7,$e1,$c0,$67,$9e,$59,
    $59,$a4,$da,$10,$0c,$5d,$cb,$62,$04,$c1,$99,$96,$dc,$6b,$8c,$4d,$db,$89,$1b,$02,$af,$6c,$f2,$87,$4f,
    $b3,$eb,$41,$b9,$a2,$96,$65,$69,$0c,$82,$62,$3f,$95,$c9,$29,$fa,$8f,$9c,$88,$7c,$05,$49,$62,$41,$4c,
    $d8,$a4,$53,$6c,$7f,$18,$83,$63,$04,$ae,$01,$ef,$78,$23,$ba,$6a,$36,$33,$1d,$27,$72,$50,$90,$dd,$6b,
    $62,$dd,$ac,$50,$fd,$84,$8b,$7f,$39,$39,$0e,$47,$be,$fe,$05,$0b,$a8,$a7,$27,$4d,$25,$56,$ca,$09,$9c,
    $89,$03,$cc,$ba,$8d,$1e,$ef,$c2,$42,$a8,$c3,$7a,$2e,$fe,$8f,$df,$e1,$b8,$cc,$0b,$61,$a3,$a7,$d6,$02,
    $12,$1e,$a4,$b3,$90,$5c,$ba,$80,$5e,$ac,$81,$0f,$c2,$b1,$88,$e8,$36,$20,$25,$c5,$2e,$6d,$5d,$5c,$d6,
    $c4,$10,$be,$df,$9d,$e1,$df,$1d,$be,$31,$07,$98,$18,$1e,$94,$53,$23,$ed,$c5,$f2,$1d,$eb,$b1,$7c,$d1,
    $3c,$03,$c5,$cf,$ec,$be,$2e,$7b,$43,$5a,$d6,$82,$e1,$99,$2d,$b0,$1a,$5a,$8f,$c0,$3b,$f1,$48,$5c,$62,
    $e0,$f2,$87,$0e,$21,$c7,$72,$cc,$f9,$be,$dd,$05,$bb,$6a,$75,$2b,$bb,$b8,$c3,$90,$a7,$68,$53,$9d,$84,
    $0e,$81,$e9,$a3,$93,$c3,$87,$0e,$d0,$80,$9b,$08,$77,$c0,$58,$ed,$59,$5d,$08,$da,$b4,$43,$51,$95,$f5,
    $c0,$a3,$c4,$46,$f6,$ad,$98,$6b,$4c,$c0,$32,$93,$a6,$52,$56,$f2,$e1,$31,$ff,$5a,$a2,$2f,$c7,$b6,$9a,
    $46,$2d,$8f,$4a,$e4,$d6,$b5,$c5,$03,$e9,$a1,$a3,$d2,$bf,$8c,$14,$c3,$4e,$9f,$8d,$54,$45,$ba,$d1,$56,
    $7c,$f0,$cb,$f4,$78,$81,$57,$0b,$3d,$f9,$73,$39,$5d,$13,$90,$70,$05,$d9,$7c,$cc,$cf,$54,$0f,$c7,$4d,
    $4d,$c4,$92,$d3,$68,$5a,$fc,$c9,$b6,$14,$31,$89,$e3,$eb,$8a,$ee,$7d,$3d,$b2,$48,$07,$c7,$59,$12,$35,
    $03,$f9,$3c,$c4,$37,$7e,$ac,$d3,$b3,$42,$f0,$e4,$aa,$0f,$4f,$95,$76,$01,$c0,$9f,$07,$10,$8a,$e7,$3b,
    $dd,$70,$1d,$2d,$8d,$1b,$c3,$84,$33,$d4,$c5,$07,$d0,$6f,$59,$04,$68,$92,$01,$f7,$f9,$47,$d7,$95,$10,
    $85,$90,$58,$e7,$cc,$80,$97,$0a,$17,$72,$66,$32,$83,$8b,$be,$3d,$be,$96,$db,$e0,$86,$ae,$87,$50,$06,
    $b2,$8b,$77,$98,$fa,$e3,$3b,$e7,$d2,$e6,$45,$df,$38,$cc,$5d,$71,$06,$91,$96,$47,$cd,$34,$dc,$26,$6b,
    $2f,$0b,$34,$7f,$0d,$a9,$44,$56,$43,$24,$b9,$c2,$29,$67,$81,$b1,$8f,$27,$de,$ea,$6e,$6f,$1f,$e5,$8b,
    $59,$da,$d2,$ba,$1b,$47,$f9,$18,$75,$ff,$ca,$c7,$e6,$97,$56,$bc,$26,$0a,$58,$39,$21,$df,$8c,$5d,$b7,
    $d9,$38,$34,$83,$bf,$b8,$cf,$57,$88,$6d,$e8,$92,$58,$8c,$a6,$65,$e4,$f5,$63,$77,$e1,$eb,$4e,$a3,$0f,
    $56,$52,$dd,$05,$2c,$ff,$1e,$7f,$f5,$1a,$bc,$d6,$14,$08,$0f,$7f,$8a,$98,$05,$36,$7d,$f6,$8d,$6c,$30,
    $87,$95,$f3,$a1,$fb,$45,$88,$2a,$80,$9c,$6f,$e1,$33,$ff,$b5,$17,$c3,$81,$6d,$49,$3f,$a7,$f5,$a5,$d7,
    $95,$40,$66,$d8,$3a,$9f,$48,$64,$ea,$d8,$28,$d2,$32,$f4,$a9,$9d,$b0,$36,$b4,$e6,$13,$d9,$93,$e5,$31,
    $f7,$1b,$4e,$df,$29,$cc,$d4,$f4,$e5,$d2,$f5,$cd,$48,$04,$cb,$e8,$9e,$9f,$4f,$15,$1e,$7d,$7f,$d3,$62,
    $cd,$08,$ee,$88,$b0,$4f,$b3,$48,$05,$2e,$4c,$79,$aa,$6a,$61,$db,$29,$70,$ee,$70,$33,$38,$07,$5d,$40,
    $cb,$8f,$3b,$c4,$97,$c6,$e7,$81,$7d,$d5,$79,$04,$3f,$9b,$18,$86,$ac,$a6,$e0,$a5,$52,$ef,$09,$c9,$81,
    $e2,$d4,$25,$ec,$77,$e5,$6f,$48,$20,$6e,$ac,$f7,$a2,$ee,$cb,$97,$5b,$16,$15,$62,$09,$f6,$6b,$cc,$6c,
    $a3,$42,$28,$3a,$3e,$9a,$82,$b3,$f0,$ca,$7e,$90,$dc,$7f,$80,$8b,$56,$92,$40,$82,$2a,$03,$75,$6c,$4e,
    $b7,$ad,$7b,$6a,$dc,$02,$91,$fd,$a8,$bc,$c5,$3c,$15,$a5,$62,$17,$ca,$d4,$f6,$b4,$b4,$67,$b4,$86,$f4,
    $07,$10,$f9,$6e,$58,$7f,$81,$94,$ea,$da,$a4,$d8,$7d,$1c,$b5,$2b,$a0,$96,$8b,$da,$d9,$e7,$38,$16,$f3
  {$endif}
  );

{---------------------------------------------------------------------------}
function AddrLn2Bytes: pointer;
  {-Pointer to bits of ln(2), big endian format}
begin
  AddrLn2Bytes := @L2Bytes;
end;



{$ifdef MPC_E1Ln10Tab}

{---------------------------------------------------------------------------}
{---------------------------------------------------------------------------}
const
  L10Bytes: packed array[1..MAX_TCBITS div 8] of byte = ( {calculated with t_ciln10}
    $93,$5d,$8d,$dd,$aa,$a8,$ac,$16,$ea,$56,$d6,$2b,$82,$d3,$0a,$28,$e2,$8f,$ec,$f9,$da,$5d,$f9,$0e,$83,
    $c6,$1e,$82,$01,$f0,$2d,$72,$96,$2f,$02,$d7,$b1,$a8,$10,$5c,$cc,$70,$cb,$c0,$2c,$5f,$0d,$68,$2c,$62,
    $24,$18,$41,$0b,$e2,$da,$fb,$8f,$78,$84,$02,$e5,$16,$d6,$78,$2c,$f8,$a2,$8a,$8c,$91,$1e,$76,$5a,$a6,
    $c3,$b0,$d8,$31,$fb,$ef,$66,$ce,$b0,$4a,$b3,$c6,$fa,$51,$61,$bb,$49,$d2,$19,$c7,$bb,$ca,$67,$b3,$5b,
    $23,$60,$50,$85,$8e,$93,$36,$8d,$44,$78,$9c,$4f,$5b,$08,$b0,$57,$d5,$ed,$e2,$0f,$46,$9e,$a5,$8e,$93,
    $05,$e9,$81,$e2,$47,$8f,$ca,$ad,$3a,$ee,$98,$9c,$d5,$b4,$2e,$6a,$27,$16,$19,$a4,$7e,$cb,$26,$97,$8c,
    $5d,$4f,$db,$1d,$28,$ea,$57,$d4,$fd,$c0,$e4,$0b,$f3,$cc,$1e,$14,$12,$6a,$45,$76,$5c,$de,$26,$83,$39,
    $db,$f4,$7f,$a9,$6d,$eb,$27,$10,$60,$af,$88,$48,$6e,$a9,$b7,$40,$1e,$3d,$fd,$3c,$51,$74,$8e,$6d,$6e,
    $38,$48,$c8,$d2,$5f,$af,$1b,$ca,$e8,$80,$47,$f1,$7b,$0d,$9b,$50,$a9,$49,$ea,$aa,$df,$69,$e8,$a5,$f7,
    $7e,$37,$60,$4e,$94,$39,$60,$e3,$8a,$57,$00,$ff,$de,$2d,$b1,$ad,$6b,$fb,$ff,$d8,$21,$ba,$0a,$4c,$b0,
    $46,$6d,$61,$ba,$64,$8e,$ef,$99,$c8,$e5,$f6,$97,$4f,$36,$39,$82,$a7,$8c,$a4,$5d,$df,$c8,$09,$42,$61,
    $78,$19,$12,$7a,$6e,$3b,$70,$fc,$da,$2d,$73,$2d,$47,$b5,$e4,$b1,$c8,$c0,$e5,$a1,$0a,$aa,$66,$04,$a5,
    $32,$4e,$c3,$dc,$bc,$64,$ea,$80,$6e,$19,$85,$66,$1f,$1d,$36,$6c,$20,$66,$38,$34,$4d,$5e,$84,$3f,$20,
    $64,$2b,$97,$0a,$62,$d1,$8e,$47,$8f,$7b,$d5,$8f,$cd,$08,$32,$4a,$7b,$32,$a6,$de,$f8,$5a,$05,$eb,$56,
    $32,$3a,$42,$1e,$f5,$e0,$b0,$04,$10,$a0,$a0,$d9,$c2,$60,$79,$4a,$97,$6f,$f6,$ff,$36,$3d,$b0,$0b,$6b,
    $33,$f4,$2c,$58,$de,$f8,$a3,$c5,$2d,$ed,$69,$b1,$3d,$c1,$a0,$37,$30,$b6,$52,$4d,$c1,$8c,$16,$7e,$86,
    $99,$d6,$d2,$0e,$a2,$de,$fd,$2b,$d0,$06,$f8,$b4,$be,$14,$5a,$2a,$df,$3c,$cb,$b3,$18,$9d,$a4,$9d,$bc,
    $12,$61,$c8,$b3,$e4,$da,$ad,$6a,$36,$ce,$cc,$b2,$d5,$ae,$5b,$89,$bf,$75,$2f,$b5,$df,$b3,$53,$ff,$30,
    $65,$c4,$0c,$fc,$ee,$c8,$1b,$e5,$a9,$a9,$67,$fd,$dc,$57,$c4,$b8,$33,$01,$00,$6b,$f0,$62,$4b,$40,$ed,
    $7a,$56,$c6,$cd,$cd,$a2,$d6,$fe,$91,$38,$8e,$9e,$3e,$48,$a9,$3f,$5f,$5e,$3b,$6e,$b4,$b8,$1c,$4a,$5b,
    $53,$d4,$9e,$a6,$8e,$66,$8a,$ea,$ba,$83,$c7,$f8,$fb,$5f,$06,$c3,$58,$ac,$8f,$70,$fa,$9d,$8c,$59,$8c,
    $57,$45,$02,$ba,$f5,$4c,$96,$c8,$49,$11,$f0,$04,$82,$d0,$95,$1a,$0a,$f0,$22,$ab,$ba,$b0,$80,$ec,$97,
    $ef,$d3,$67,$1e,$4e,$0e,$52,$f1,$66,$b6,$cd,$5c,$d2,$26,$0d,$c6,$77,$95,$2e,$1e,$34,$a3,$f7,$99,$67,
    $7f,$2c,$1d,$48,$f1,$29,$44,$b6,$c5,$2b,$a1,$30,$7e,$70,$4d,$67,$f9,$1c,$10,$35,$e4,$4e,$92,$7c,$63,
    $03,$cf,$12,$bf,$e2,$cd,$2e,$31,$f8,$ee,$48,$43,$34,$4d,$51,$b0,$f3,$7d,$a4,$2b,$9f,$0b,$0f,$d9,$13,
    $4f,$b2,$d9,$f8,$15,$e4,$90,$d9,$66,$28,$3f,$23,$96,$27,$66,$ec,$ea,$b1,$e4,$f3,$b5,$fc,$86,$46,$81,
    $27,$e2,$b6,$06,$d1,$0d,$3a,$45,$f4,$b6,$b7,$76,$10,$2d,$2f,$db,$b4,$20,$80,$c8,$fa,$84,$d0,$ff,$9f,
    $45,$c5,$8a,$ef,$38,$db,$24,$10,$fd,$1f,$1c,$eb,$ad,$73,$3b,$22,$81,$52,$ca,$5f,$36,$dd,$f2,$9d,$aa,
    $54,$43,$34,$b8,$de,$ea,$f6,$59,$4e,$46,$27,$13,$1e,$d4,$85,$b4,$6a,$08,$22,$e1,$28,$db,$47,$1c,$a5,
    $39,$38,$a8,$44,$c3,$be,$f7,$f3,$52,$15,$c8,$b3,$82,$bc,$4e,$3e,$4c,$6f,$15,$62,$85,$f5,$4c,$17,$ab,
    $40,$8e,$cc,$bf,$7f,$5e,$d1,$6a,$b3,$f6,$ce,$d2,$84,$6d,$f4,$57,$e1,$4f,$bb,$45,$d9,$c5,$64,$6a,$d4,
    $97,$ac,$69,$74,$94,$14,$5d,$e3,$2e,$93,$90,$71,$28,$d2,$63,$d5,$21,$79,$ef,$b4,$24,$d6,$46,$51,$d6,
    $eb,$c0,$c9,$f0,$bb,$58,$3a,$44,$c6,$41,$2c,$84,$85,$bb,$29,$a6,$4d,$31,$a2,$cd,$92,$95,$44,$69,$a3,
    $2b,$1a,$bd,$f7,$f5,$20,$2c,$a4,$aa,$6c,$93,$2e,$9b,$53,$cf,$38,$5a,$b1,$36,$27,$41,$f3,$56,$5d,$e9,
    $c0,$65,$60,$09,$90,$1c,$88,$ab,$bd,$d8,$74,$ef,$cf,$73,$3f,$76,$1a,$d4,$35,$f3,$c0,$83,$fd,$6b,$8e,
    $e0,$0b,$ef,$11,$c7,$c5,$52,$a8,$9d,$58,$ce,$4a,$21,$d7,$1e,$54,$f2,$41,$57,$f6,$c7,$d4,$62,$23,$16,
    $e9,$89,$56,$d7,$45,$00,$27,$de,$cb,$d3,$98,$d8,$4b,$98,$b3,$6a,$07,$24,$c2,$5c,$db,$23,$77,$60,$e9,
    $32,$4b,$68,$75,$23,$e5,$06,$8e,$da,$d9,$33,$92,$19,$7f,$00,$b8,$53,$a3,$26,$b3,$30,$c4,$44,$65,$12,
    $92,$96,$34,$bc,$06,$70,$e1,$77,$80,$6d,$e3,$38,$da,$c4,$55,$37,$49,$2a,$e1,$9a,$dd,$83,$cf,$45,$00,
    $0f,$5b,$42,$3b,$ce,$64,$97,$d2,$09,$e3,$0e,$18,$a1,$3c,$bf,$06,$87,$67,$97,$31,$03,$d9,$48,$53,$66,
    $81,$50,$6b,$ba,$2e,$93,$a9,$a4,$7d,$d5,$9d,$3f,$f1,$7c,$d7,$46,$8c,$20,$75,$be,$55,$2a,$43,$47,$b4,
    $a6,$38,$ef,$d4,$9a,$2e,$66,$3e,$f2,$7d,$99,$88,$92,$30,$5b,$c1,$9c,$9b,$2b,$41,$9e,$48,$b6,$3b,$76,
    $37,$51,$37,$a7,$57,$65,$d3,$40,$5d,$db,$3c,$fd,$a4,$6d,$82,$da,$f4,$7f,$79,$cc,$1e,$36,$10,$af,$77,
    $aa,$42,$d4,$47,$8d,$de,$e6,$07,$47,$37,$06,$25,$3d,$b0,$e0,$3f,$bc,$ed,$76,$c3,$c0,$68,$06,$fb,$76,
    $bf,$77,$e9,$c5,$8e,$3d,$a5,$0c,$dc,$d5,$00,$6d,$a9,$d4,$a3,$d3,$c3,$6f,$b0,$37,$ab,$ab,$f4,$d2,$ce,
    $4d,$a1,$f8,$8a,$8d,$9f,$48,$7d,$88,$d7,$ce,$99,$6e,$eb,$7e,$4a,$b9,$40,$6a,$2f,$73,$2b,$5a,$dd,$24,
    $81,$8d,$60,$93,$61,$f7,$b2,$7d,$f5,$7b,$ae,$5d,$07,$bc,$a4,$9f,$9e,$13,$cb,$41,$1a,$89,$64,$14,$d3,
    $3a,$27,$50,$3c,$00,$b4,$80,$c9,$b6,$cf,$ef,$b8,$ff,$46,$8e,$39,$d7,$d3,$8c,$2d,$1b,$66,$d6,$b4,$9b,
    $55,$fb,$da,$6e,$c6,$a2,$6c,$65,$a2,$99,$85,$60,$8b,$03,$78,$42,$18,$28,$5a,$10,$f0,$75,$88,$7c,$d5,
    $0d,$f7,$c4,$cf,$b8,$bc,$8c,$7b,$ee,$0c,$a9,$25,$ba,$83,$fc,$68,$4c,$52,$eb,$f3,$d9,$be,$03,$78,$15,
    $fc,$14,$9b,$76,$de,$ef,$0a,$32,$97,$42,$9f,$be,$3f,$28,$14,$c5,$3a,$a7,$cf,$88,$db,$48,$cb,$14,$3e,
    $82,$07,$da,$b6,$44,$b8,$36,$25,$05,$59,$3f,$e7,$3f,$79,$3b,$35,$e7,$4b,$1c,$b6,$94,$b2,$8d,$f6,$d2,
    $ed,$7e,$95,$90,$0b,$04,$a7,$e2,$af,$f7,$e9,$a8,$45,$b3,$2b,$1b,$a2,$7c,$89,$06,$1f,$d2,$be,$26,$96,
    $b4,$ca,$23,$1c,$94,$7f,$8d,$bb,$02,$91,$b0,$78,$ca,$33,$d0,$69,$18,$83,$66,$3d,$e4,$61,$e2,$97,$91,
    $6e,$d6,$0d,$64,$b8,$88,$0b,$3e,$25,$7a,$f3,$6e,$d7,$b4,$ff,$b0,$b1,$db,$b7,$ef,$be,$15,$42,$43,$35,
    $5b,$c2,$14,$0c,$89,$30,$91,$c6,$4d,$cd,$c5,$4f,$46,$05,$d9,$bd,$e0,$40,$33,$29,$ec,$4d,$a2,$0f,$dc,
    $75,$b8,$2f,$ab,$09,$6b,$6a,$b8,$8f,$20,$2c,$e6,$68,$d8,$e7,$f8,$87,$b1,$ea,$3e,$73,$8f,$30,$48,$03,
    $ee,$25,$0f,$b5,$88,$35,$c7,$ac,$c7,$a3,$3d,$89,$1d,$a4,$e3,$f5,$66,$c2,$a2,$10,$95,$d9,$cc,$a9,$44,
    $67,$a5,$24,$c8,$5e,$95,$63,$87,$54,$7d,$84,$53,$ad,$12,$c2,$71,$a1,$f3,$59,$fe,$41,$78,$da,$d0,$3a,
    $20,$f2,$a2,$e7,$dc,$3f,$1f,$c6,$ce,$27,$fe,$b6,$19,$9c,$d5,$66,$39,$16,$4f,$4d,$c8,$b3,$ed,$5b,$b8,
    $e5,$10,$3e,$61,$ef,$75,$96,$b8,$43,$94,$95,$1a,$2e,$47,$61,$c7,$b2,$c1,$5e,$33,$f0,$e2,$3e,$04,$ec,
    $7d,$b9,$db,$9a,$8f,$9d,$45,$ec,$74,$36,$5f,$e6,$56,$20,$f7,$c3,$72,$b1,$57,$fb,$1a,$bd,$72,$8d,$83,
    $81,$db,$d2,$9d,$dc,$98,$ae,$55,$cf,$3c,$81,$9f,$b4,$c9,$02,$5c,$ee,$d5,$aa,$9a,$a3,$54,$35,$a6,$44,
    $82,$43,$1d,$6b,$12,$4d,$8c,$0e,$2d,$56,$c2,$54,$d1,$6f,$a2,$67,$c7,$22,$b4,$32,$27,$1a,$ff,$f0,$9d,
    $6c,$f2,$55,$04,$c1,$75,$7a,$c8,$50,$a8,$35,$7d,$ec,$d9,$61,$55,$f0,$3c,$d7,$58,$76,$d2,$17,$92,$18,
    $26,$db,$3b,$4d,$fa,$cb,$fb,$c9,$75,$85,$ce,$84,$f2,$06,$9f,$cb,$d9,$ab,$4f,$cd,$ae,$f3,$9c,$ba,$47,
    $43,$f8,$b8,$de,$dd,$3a,$8b,$39,$74,$ec,$51,$f9,$43,$fc,$52,$ef,$22,$02,$81,$2f,$ef,$39,$a6,$fe,$85,
    $9c,$cb,$53,$79,$76,$1c,$bd,$dd,$e2,$f0,$f8,$da,$f9,$30,$e5,$a0,$42,$86,$82,$d6,$1b,$77,$7c,$81,$dd,
    $0e,$61,$a1,$f3,$18,$de,$8e,$f8,$56,$5b,$fe,$1b,$71,$90,$da,$79,$eb,$95,$dd,$14,$d2,$4e,$0f,$a2,$94,
    $8a,$5a,$f4,$0c,$75,$2e,$a8,$f0,$df,$84,$d9,$01,$ab,$f4,$90,$16,$70,$67,$df,$c1,$67,$19,$70,$2d,$e9,
    $a3,$94,$ee,$7b,$37,$b6,$c5,$e9,$cb,$d4,$42,$46,$80,$30,$11,$d1,$7e,$5c,$fa,$5a,$ab,$cb,$ec,$1e,$6f,
    $44,$b8,$5a,$cb,$f0,$c3,$ec,$8a,$0a,$6e,$a0,$70,$90,$34,$e0,$fa,$4c,$4f,$27,$0f,$ca,$5a,$ff,$ff,$d1,
    $54,$5a,$22,$fd,$64,$2e,$a8,$74,$dd,$5d,$a9,$4a,$d1,$2f,$0a,$c9,$72,$78,$0e,$85,$2b,$c0,$7d,$21,$72,
    $0f,$6a,$46,$d9,$4b,$e8,$0c,$a7,$28,$57,$0b,$c8,$98,$72,$de,$8f,$67,$f1,$70,$88,$e8,$77,$8b,$f9,$b3,
    $6d,$72,$98,$ad,$3c,$62,$88,$31,$38,$b8,$c6,$f8,$e5,$45,$fb,$cb,$78,$30,$9b,$9a,$b3,$08,$d3,$71,$19,
    $6a,$1d,$27,$d3,$03,$b3,$af,$12,$c0,$66,$db,$be,$69,$92,$5f,$e4,$68,$13,$c8,$2d,$6f,$c6,$9b,$1e,$e3,
    $0c,$2e,$d0,$f0,$14,$56,$9b,$cd,$74,$e5,$f7,$0c,$6c,$72,$06,$f0,$09,$be,$63,$d5,$54,$e5,$88,$07,$7d,
    $ac,$9c,$bd,$ef,$4e,$d7,$f8,$57,$ec,$2a,$d3,$07,$de,$71,$60,$fd,$62,$36,$17,$80,$e8,$99,$53,$9a,$86,
    $ab,$e3,$1e,$c7,$77,$5f,$d6,$50,$46,$80,$28,$7e,$1e,$40,$40,$ef,$20,$89,$22,$f3,$a5,$35,$25,$f6,$ac,
    $55,$9f,$83,$af,$a5,$28,$af,$b8,$c5,$aa,$90,$05,$fa,$68,$cf,$9d,$fc,$dc,$de,$00,$82,$79,$ff,$7c,$97,
    $92,$93,$7b,$8f,$0e,$ab,$d2,$e6,$ab,$45,$35,$d5,$8f,$ba,$35,$31,$bc,$87,$6d,$b0,$bf,$45,$54,$69,$e6,
    $5b,$0f,$4f,$15,$07,$bf,$48,$6e,$b2,$16,$7a,$82,$c9,$2c,$fe,$25,$1c,$cd,$7a,$b9,$26,$7a,$7c,$9b,$84,
    $95,$9b,$57,$eb,$24,$5b,$af,$8b,$4f,$a7,$75,$e7,$c9,$88,$bb,$af,$e3,$62,$15,$6a,$3e,$ef,$aa,$04,$ea,
    $d5,$0c,$b7,$75,$73,$bd,$f2,$b6,$32,$4a,$88,$15,$7a,$9b,$ad,$6d,$be,$e7,$24,$12,$eb,$da,$4e,$ac,$53,
    $6a,$ed,$d9,$c8,$92,$6e,$6f,$ae,$18,$90,$b3,$80,$8c,$d5,$5e,$44,$9f,$aa,$07,$54,$dd,$df,$1f,$6e,$d2,
    $1e,$a0,$67,$d6,$d0,$13,$ee,$b9,$91,$31,$f0,$bc,$f6,$2f,$16,$31,$df,$82,$7c,$03,$22,$4d,$22,$35,$79,
    $07,$68,$d2,$b8,$8d,$5a,$b9,$7b,$46,$b3,$b0,$d4,$fb,$3b,$4d,$57,$45,$bc,$be,$6b,$85,$20,$91,$56,$cb,
    $da,$64,$bb,$df,$02,$fc,$0c,$f9,$64,$f8,$3e,$13,$22,$66,$f5,$bd,$df,$80,$65,$9a,$e3,$d3,$a5,$81,$1f,
    $b9,$f4,$64,$4d,$11,$79,$84,$51,$d2,$94,$bd,$4b,$45,$1b,$94,$28,$38,$3a,$fe,$6a,$97,$75,$2f,$3c,$d7,
    $c0,$00,$d1,$c6,$68,$48,$cd,$22,$43,$4b,$20,$f6,$a9,$3d,$5a,$07,$38,$37,$27,$62,$e9,$b7,$ae,$c3,$e3,
    $40,$ea,$b8,$2b,$87,$c2,$d1,$7b,$46,$1d,$86,$3a,$c6,$3e,$30,$07,$0c,$cb,$f5,$98,$05,$a4,$90,$7c,$77,
    $26,$e5,$5a,$3a,$68,$49,$3d,$0a,$3f,$09,$77,$ed,$77,$90,$b1,$fe,$5b,$98,$30,$20,$c3,$20,$ed,$d9,$e1,
    $fd,$d7,$5d,$60,$14,$17,$67,$c5,$71,$ed,$68,$62,$20,$db,$c1,$81,$00,$53,$39,$a3,$26,$62,$46,$06,$42,
    $b0,$5e,$87,$bc,$58,$50,$ee,$74,$22,$59,$c2,$9f,$fb,$82,$f5,$31,$12,$04,$3c,$9b,$45,$89,$c1,$b5,$7b,
    $31,$34,$29,$2d,$ca,$7a,$a0,$0c,$c5,$01,$5a,$25,$43,$1e,$55,$11,$31,$ea,$55,$29,$58,$0c,$69,$4d,$ef,
    $b9,$57,$23,$3d,$76,$c4,$24,$da,$0d,$97,$ca,$b3,$c4,$cc,$4b,$5b,$47,$ee,$c0,$dc,$4d,$72,$8f,$f3,$10,
    $8c,$49,$e2,$0f,$aa,$08,$03,$5e,$d3,$cf,$41,$22,$a2,$c1,$d9,$95,$2d,$45,$0b,$26,$7e,$64,$95,$9c,$41,
    $4a,$74,$41,$4e,$c4,$01,$94,$ab,$1c,$6e,$2a,$68,$0b,$4d,$1a,$37,$52,$91,$1d,$59,$af,$9a,$8d,$3a,$d5,
    $40,$4d,$a3,$0d,$88,$ff,$4d,$d6,$78,$20,$f6,$39,$7e,$5f,$a0,$36,$c9,$a9,$41,$07,$bb,$fc,$01,$d5,$77,
    $a5,$61,$9b,$d5,$55,$4d,$28,$f7,$a5,$74,$5d,$98,$86,$54,$4b,$06,$ec,$5c,$02,$76,$0e,$dc,$a7,$35,$14,
    $71,$25,$67,$7b,$73,$c6,$b2,$a1,$3f,$d7,$a3,$46,$a8,$54,$14,$8d,$d8,$bf,$38,$21,$ff,$98,$dc,$18,$52,
    $7d,$af,$90,$63,$e9,$c7,$90,$28,$8d,$44,$84,$3a,$25,$88,$db,$ef,$c9,$42,$0c,$20,$76,$62,$90,$62,$27,
    $41,$30,$3d,$c8,$16,$66,$74,$b0,$57,$d4,$f7,$1c,$cf,$02,$1f,$85,$b5,$7d,$4e,$16,$cb,$38,$d4,$c5,$24,
    $a0,$f1,$9a,$4e,$f4,$85,$ec,$de,$39,$b2,$59,$4c,$2d,$3a,$c7,$28,$b6,$b6,$d9,$53,$94,$62,$36,$6a,$71,
    $a8,$18,$c5,$3a,$72,$f1,$fd,$94,$bc,$65,$90,$33,$ea,$11,$31,$f8,$37,$1d,$7d,$f6,$9b,$79,$f9,$72,$9b,
    $bb,$94,$8e,$97,$e9,$00,$c9,$e6,$f3,$32,$b6,$61,$9b,$2a,$73,$d0,$bd,$12,$60,$89,$b0,$ad,$93,$f1,$96,
    $a4,$f6,$3a,$b9,$3f,$ab,$4c,$64,$61,$08,$45,$b1,$6d,$ca,$88,$13,$65,$f5,$c1,$32,$d8,$75,$7b,$ad,$65,
    $39,$ec,$28,$30,$a7,$f3,$09,$e9,$91,$17,$57,$39,$e1,$4e,$fb,$d9,$8d,$7e,$89,$be,$50,$35,$90,$f1,$33,
    $0c,$0e,$58,$5c,$54,$98,$76,$87,$4a,$38,$ff,$5a,$03,$0f,$5a,$89,$42,$56,$d8,$c1,$11,$50,$89,$3c,$df,
    $3d,$7d,$a1,$8a,$91,$5b,$40,$64,$d5,$c6,$ff,$ed,$dc,$27,$d0,$a1,$d1,$0f,$11,$a3,$44,$27,$ba,$e7,$90,
    $45,$13,$df,$cc,$02,$7c,$a8,$93,$0a,$a3,$a6,$2b,$f3,$f5,$37,$5a,$0b,$fb,$b1,$22,$4f,$01,$57,$a6,$23,
    $ba,$a8,$2c,$b0,$23,$fe,$e3,$08,$5d,$04,$84,$3f,$9e,$2e,$1c,$85,$6e,$6b,$29,$ef,$5a,$fd,$bb,$71,$27,
    $01,$54,$65,$6f,$d0,$94,$16,$19,$e6,$81,$b8,$68,$75,$35,$95,$ec,$6e,$6f,$e7,$71,$96,$42,$f6,$0b,$44,
    $e9,$3b,$d3,$0e,$cf,$88,$11,$c4,$6d,$dc,$43,$b2,$5f,$7a,$ed,$84,$85,$93,$58,$88,$48,$2b,$90,$e8,$ae,
    $61,$9f,$3d,$be,$59,$60,$fa,$44,$cb,$15,$b5,$55,$ca,$55,$c5,$1c,$ee,$22,$25,$96,$a5,$60,$e0,$46,$04,
    $b1,$64,$3c,$06,$83,$51,$7c,$b9,$28,$8d,$e3,$7f,$a5,$af,$cd,$b3,$f6,$c4,$f4,$95,$d3,$a6,$ed,$00,$7a,
    $29,$c0,$cf,$86,$04,$60,$59,$60,$61,$7a,$ae,$fb,$29,$7c,$5d,$72,$e3,$11,$a4,$65,$75,$63,$0b,$d9,$b2,
    $74,$6f,$e8,$c4,$34,$36,$b0,$e8,$76,$78,$f2,$1f,$08,$0e,$c1,$64,$08,$75,$52,$5b,$76,$3c,$9f,$bb,$5c,
    $2d,$1d,$ba,$0e,$4f,$af,$50,$25,$95,$5d,$b5,$fa,$99,$49,$c9,$bb,$df,$9b,$f4,$66,$00,$81,$40,$de,$b1,
    $29,$a3,$85,$d4,$88,$37,$c2,$14,$81,$94,$c9,$70,$a1,$23,$ed,$86,$bf,$17,$74,$41,$4c,$38,$4b,$37,$55,
    $4a,$6c,$19,$2c,$e2,$ab,$77,$f8,$82,$ab,$58,$41,$c0,$c0,$d5,$d9,$91,$56,$96,$a0,$f8,$0a,$ee,$ce,$c2,
    $80,$84,$65,$0a,$3b,$49,$4a,$40,$83,$24,$5d,$2d,$30,$bd,$1b,$da,$8b,$3e,$50,$6f,$27,$c0,$76,$17,$58,
    $a3,$89,$da,$ba,$6b,$45,$d3,$73,$54,$e9,$ad,$f1,$2f,$69,$36,$c8,$9a,$9f,$b5,$8c,$27,$be,$3b,$00,$82,
    $5d,$54,$f0,$fe,$23,$39,$b8,$ff,$91,$6f,$1e,$47,$53,$8c,$55,$3f,$08,$91,$1e,$fe,$f9,$af,$e2,$0b,$b5,
    $18,$e8,$7f,$39,$a0,$f2,$3b,$bd,$f0,$ac,$e8,$ad,$d1,$8a,$2e,$bd,$c2,$99,$cb,$0d,$9b,$ba,$74,$ff,$77,
    $4d,$17,$60,$f4,$0a,$d5,$c2,$e7,$97,$16,$22,$49,$92,$b4,$ea,$dc,$38,$ec,$7a,$27,$c8,$1d,$af,$70,$cb,
    $1e,$6f,$44,$cd,$65,$60,$34,$6f,$d0,$07,$f5,$d5,$1c,$36,$8b,$6c,$c5,$c2,$dc,$5f,$e5,$55,$8e,$9b,$67,
    $70,$24,$e5,$38,$91,$e3,$fd,$a4,$89,$da,$43,$0c,$9d,$96,$15,$74,$ba,$7f,$a8,$75,$00,$2e,$a8,$44,$7c,
    $79,$41,$bc,$e5,$e7,$e2,$06,$98,$79,$85,$a3,$8c,$07,$0c,$8c,$40,$22,$cd,$62,$cc,$59,$34,$73,$23,$50,
    $a2,$2d,$ec,$29,$ac,$5e,$26,$16,$15,$56,$1d,$3a,$d5,$dc,$02,$06,$79,$03,$51,$aa,$9b,$ff,$20,$6d,$75,
    $b0,$70,$5b,$eb,$6a,$de,$ad,$6e,$d3,$96,$bc,$33,$40,$a2,$c0,$78,$93,$e5,$aa,$6a,$e8,$9a,$43,$7e,$22,
    $89,$ae,$21,$c1,$b9,$a8,$82,$a1,$21,$b6,$9e,$9f,$11,$b5,$72,$96,$dc,$35,$0b,$7e,$09,$67,$c7,$52,$52,
    $02,$40,$df,$1b,$4d,$6d,$5b,$05,$fa,$22,$78,$ce,$99,$0f,$47,$49,$cc,$38,$78,$43,$d6,$59,$32,$f1,$2c,
    $21,$11,$5f,$06,$1e,$fa,$2f,$f9,$25,$42,$f3,$f9,$55,$f0,$89,$15,$f8,$4d,$08,$a9,$04,$0b,$c3,$da,$e5,
    $a5,$4f,$fa,$5b,$70,$0c,$db,$c0,$43,$0e,$0a,$1b,$45,$f1,$7d,$19,$36,$d6,$fd,$ef,$c1,$b3,$70,$46,$22,
    $22,$cf,$6f,$50,$44,$de,$f0,$7f,$63,$1e,$ae,$3b,$f9,$65,$b6,$15,$5b,$c9,$08,$18,$0b,$bd,$47,$fa,$5a,
    $b0,$84,$2d,$4a,$0f,$05,$c5,$42,$7a,$e5,$a7,$27,$84,$1a,$ad,$37,$0e,$59,$bb,$cf,$57,$75,$21,$19,$30,
    $24,$b6,$c4,$d9,$cd,$23,$07,$30,$8f,$03,$99,$d3,$ce,$b7,$1f,$3b,$dd,$fa,$3f,$9e,$23,$62,$e7,$7c,$cf,
    $49,$13,$d2,$27,$c9,$9a,$25,$70,$ad,$64,$ab,$66,$3d,$a5,$7e,$65,$5d,$9e,$f5,$a8,$96,$c8,$d3,$82,$a3,
    $47,$6f,$c2,$0c,$63,$01,$bd,$e1,$83,$22,$67,$c9,$bf,$2b,$b3,$22,$2a,$7c,$b4,$68,$93,$da,$83,$47,$47,
    $ed,$e5,$11,$1a,$b3,$8e,$78,$ca,$dc,$b0,$3f,$01,$2d,$25,$ab,$29,$42,$0d,$18,$d3,$5c,$f6,$40,$ee,$9a,
    $d2,$10,$07,$69,$7d,$be,$a8,$93,$b6,$43,$8e,$b6,$fb,$39,$39,$2a,$21,$e3,$f4,$19,$ee,$80,$55,$80,$01,
    $cb,$a7,$ac,$f6,$9f,$26,$a5,$d6,$34,$02,$88,$cd,$d2,$37,$07,$3f,$42,$62,$ed,$c6,$e5,$9b,$80,$b7,$e1,
    $4f,$7c,$f4,$44,$22,$23,$f2,$93,$5a,$ac,$55,$c0,$b8,$93,$b9,$59,$eb,$c7,$b4,$4e,$e7,$60,$ce,$e3,$35,
    $74,$28,$37,$69,$b2,$aa,$c4,$32,$a2,$d7,$b9,$f3,$df,$cb,$1b,$12,$47,$1e,$4b,$ec,$3b,$3b,$b5,$fe,$f8,
    $50,$83,$50,$30,$11,$24,$5f,$de,$74,$3b,$5c,$1e,$06,$63,$15,$c0,$d9,$2c,$42,$f1,$50,$64,$c8,$98,$ef,
    $d0,$17,$67,$f0,$69,$dd,$09,$6d,$18,$ac,$cc,$8a,$a7,$ed,$38,$f6,$43,$97,$4e,$f0,$cb,$02,$8e,$7b,$3b,
    $51,$ce,$cb,$d5,$4c,$f7,$7c,$0e,$98,$cd,$c1,$11,$72,$02,$47,$32,$c4,$58,$d9,$6b,$cf,$7c,$3d,$8d,$78,
    $b7,$db,$13,$1e,$d6,$f1,$c4,$eb,$c4,$ef,$9e,$03,$10,$57,$5e,$0b,$5c,$a0,$8e,$81,$9f,$84,$91,$7b,$5b,
    $04,$90,$99,$bf,$76,$65,$98,$6b,$56,$88,$29,$36,$73,$14,$e2,$17,$47,$2f,$a5,$bb,$aa,$06,$7d,$28,$1e
  {$ifndef MPC_FPrec30K}
   ,$35,$fc,$9e,$f5,$65,$ce,$3a,$86,$ec,$ea,$30,$a6,$d9,$68,$95,$d4,$96,$a8,$d4,$84,$ee,$28,$9a,$17,$50,
    $ee,$2a,$31,$30,$d8,$b1,$d7,$0f,$24,$b0,$07,$cb,$86,$6c,$80,$f9,$70,$36,$d8,$52,$36,$33,$5a,$71,$93,
    $13,$11,$c8,$13,$86,$1f,$55,$bb,$5f,$1f,$9e,$10,$ee,$94,$03,$2a,$03,$8f,$f2,$be,$d9,$25,$c7,$0d,$e2,
    $58,$68,$20,$d5,$48,$67,$3d,$fa,$03,$ca,$8e,$a5,$74,$05,$ed,$70,$2a,$21,$10,$4c,$b1,$58,$04,$31,$02,
    $23,$37,$b9,$74,$f2,$e1,$85,$66,$30,$2e,$7d,$60,$31,$6c,$8e,$a2,$9f,$2a,$84,$6b,$5c,$8b,$8c,$50,$be,
    $01,$12,$d7,$5e,$ff,$7d,$54,$9a,$88,$74,$7e,$2f,$a3,$54,$73,$c0,$f6,$56,$ec,$84,$5a,$2c,$a8,$cb,$77,
    $ea,$c3,$58,$1a,$ae,$03,$ad,$ad,$d8,$70,$11,$5a,$b6,$8e,$cb,$4b,$bc,$6b,$35,$0d,$56,$13,$5d,$b4,$de,
    $4f,$24,$e8,$58,$cd,$2b,$57,$51,$b0,$ca,$ba,$cf,$4d,$2d,$3f,$76,$b4,$21,$ee,$aa,$c9,$42,$c7,$42,$00,
    $0d,$f5,$1b,$9a,$11,$aa,$f1,$ef,$1a,$4b,$62,$33,$66,$d9,$d8,$d5,$73,$a3,$c3,$d5,$6e,$9f,$8e,$48,$f5,
    $14,$9d,$d0,$93,$7c,$21,$2e,$4d,$6c,$ce,$4e,$88,$0b,$b3,$88,$8b,$71,$a1,$2b,$b5,$c8,$3d,$e0,$15,$e2,
    $a2,$e7,$48,$b7,$6e,$d1,$4b,$98,$e2,$53,$ab,$6d,$64,$88,$13,$7e,$be,$c8,$e7,$fc,$d2,$aa,$96,$e0,$bb,
    $e1,$f7,$88,$03,$7b,$7d,$a9,$06,$0b,$12,$20,$9d,$5a,$19,$7c,$08,$3a,$6e,$f5,$c7,$11,$56,$02,$f2,$c4,
    $28,$bc,$4d,$60,$dd,$ba,$ee,$c3,$95,$b4,$3d,$16,$64,$7c,$23,$21,$f1,$71,$40,$9d,$8a,$6b,$87,$0a,$ed,
    $cd,$8f,$36,$42,$65,$e9,$70,$75,$36,$dc,$d3,$12,$8a,$b2,$a0,$1b,$2b,$ff,$72,$e2,$dd,$7b,$fe,$76,$c2,
    $86,$d0,$48,$ee,$48,$32,$c4,$89,$a6,$6a,$91,$b2,$ac,$3e,$0e,$37,$c7,$d8,$3c,$d9,$63,$26,$4d,$e4,$b4,
    $4a,$a2,$1d,$b5,$85,$71,$9d,$79,$3c,$91,$45,$4c,$ed,$14,$df,$11,$81,$87,$2a,$42,$50,$4c,$b6,$07,$9b,
    $23,$a7,$f9,$ba,$76,$df,$ef,$36,$b0,$8a,$b0,$81,$71,$19,$90,$b4,$ba,$12,$4b,$e8,$99,$6e,$6e,$e7,$55,
    $66,$ee,$f7,$33,$38,$f3,$a2,$c5,$93,$d6,$15,$9c,$de,$cd,$32,$ae,$11,$7c,$3e,$0c,$ea,$8f,$fd,$88,$3c,
    $6e,$c7,$77,$a5,$11,$a4,$18,$1a,$6d,$a3,$c3,$af,$a6,$e5,$eb,$a2,$0d,$72,$da,$57,$c3,$ed,$a0,$8d,$1f,
    $e5,$1c,$13,$de,$2a,$8a,$c4,$c0,$08,$27,$fd,$3c,$dd,$a2,$f5,$99,$96,$d9,$1b,$3a,$18,$02,$e5,$8b,$ff,
    $58,$3b,$7c,$d2,$2d,$b8,$08,$22,$1f,$bc,$c4,$74,$3a,$80,$e1,$6e,$31,$f8,$82,$f2,$23,$30,$77,$e5,$64,
    $50,$0b,$cf,$54,$99,$a3,$91,$d1,$4d,$b5,$4e,$15,$33,$a6,$57,$93,$26,$b4,$b1,$95,$3d,$dc,$73,$cc,$a8,
    $b8,$f7,$9a,$9d,$8c,$ff,$87,$90,$7e,$a7,$7a,$eb,$c8,$05,$80,$ac,$a6,$33,$aa,$c1,$b3,$77,$55,$39,$00,
    $a0,$d5,$ab,$3a,$1c,$2e,$36,$f3,$e4,$c3,$74,$35,$f1,$38,$fe,$70,$d9,$dc,$e1,$3e,$6e,$ae,$f2,$d5,$e2,
    $ad,$eb,$09,$37,$11,$02,$98,$b3,$50,$18,$1d,$39,$55,$b2,$ba,$ff,$90,$3c,$05,$4f,$a0,$f5,$da,$81,$d5,
    $29,$f2,$59,$ef,$f9,$e1,$af,$93,$6b,$c2,$f7,$e0,$f6,$4a,$9c,$15,$7b,$84,$3c,$a9,$1a,$1a,$f1,$eb,$ce,
    $ce,$6e,$16,$96,$28,$5a,$54,$90,$04,$5b,$fc,$56,$ec,$5b,$11,$4a,$85,$20,$84,$a6,$80,$4f,$74,$4b,$d7,
    $0e,$bf,$fe,$07,$e6,$bf,$82,$22,$6e,$dd,$61,$9b,$3d,$4f,$f2,$49,$be,$71,$38,$fb,$7f,$e9,$09,$09,$60,
    $47,$a7,$9e,$02,$99,$a0,$5a,$91,$a9,$98,$05,$87,$04,$56,$74,$1f,$36,$ec,$f5,$7f,$04,$bb,$bf,$a6,$67,
    $b9,$86,$7d,$83,$78,$1d,$92,$5a,$62,$d9,$cc,$23,$d2,$0a,$40,$c1,$d5,$67,$52,$80,$8d,$78,$ff,$36,$9b,
    $d5,$ec,$67,$1c,$d2,$58,$08,$d6,$43,$3b,$26,$54,$cc,$17,$37,$5e,$ec,$44,$e0,$69,$ba,$5b,$e3,$68,$6b,
    $d4,$4d,$14,$d2,$64,$55,$a1,$da,$af,$0a,$be,$89,$57,$6f,$41,$04,$a2,$40,$33,$98,$60,$c9,$99,$d3,$95,
    $44,$10,$77,$b4,$01,$cd,$2b,$89,$52,$28,$9d,$c2,$80,$16,$88,$62,$7a,$32,$26,$f0,$53,$33,$34,$ea,$6d,
    $f7,$ee,$5c,$7a,$85,$a3,$54,$d5,$0f,$04,$f6,$62,$2f,$a5,$68,$75,$78,$32,$11,$1c,$d5,$26,$f8,$7f,$44,
    $b7,$fd,$3f,$d4,$b0,$aa,$0c,$fa,$6a,$2b,$dd,$4b,$e7,$3e,$53,$5c,$ab,$37,$0e,$b4,$dd,$01,$45,$57,$d4,
    $46,$f5,$b0,$69,$fd,$60,$55,$c3,$a7,$07,$e7,$a0,$30,$56,$b4,$ac,$71,$7a,$16,$6d,$ae,$bb,$2c,$e6,$3e,
    $c0,$35,$c2,$18,$d2,$21,$4b,$91,$07,$ba,$3f,$d6,$67,$9e,$96,$7f,$a8,$73,$e6,$05,$03,$d9,$87,$b7,$ea,
    $d6,$a2,$8b,$6d,$dc,$11,$57,$f0,$f3,$83,$99,$0b,$c5,$eb,$c8,$b5,$4e,$3e,$3d,$9c,$05,$1e,$7d,$48,$d4,
    $82,$b3,$21,$8c,$a2,$ee,$73,$25,$6d,$ee,$3d,$a0,$97,$b4,$23,$72,$7d,$e5,$0c,$bd,$3e,$65,$90,$60,$db,
    $bf,$46,$df,$33,$d5,$ed,$26,$69,$39,$66,$6e,$9f,$87,$a7,$c2,$45,$c2,$4e,$a6,$21,$b9,$11,$db,$46,$67,
    $d7,$61,$4c,$20,$5c,$0e,$b6,$49,$6a,$08,$24,$04,$de,$a9,$95,$4f,$ec,$64,$40,$1c,$77,$9a,$df,$58,$60,
    $7c,$05,$a2,$76,$9f,$bd,$c7,$6e,$98,$cc,$b7,$2f,$c0,$f1,$8f,$a8,$fa,$e8,$aa,$3d,$f8,$f6,$c0,$88,$7d,
    $ac,$08,$52,$06,$b1,$4f,$5d,$05,$ae,$f5,$86,$4b,$f1,$76,$9d,$88,$3e,$a7,$5e,$25,$dd,$b7,$34,$d6,$55,
    $84,$33,$5d,$05,$85,$19,$21,$40,$25,$08,$70,$51,$5a,$45,$23,$48,$a5,$94,$5d,$d4,$15,$1c,$0c,$80,$e4,
    $8a,$ac,$58,$35,$38,$b9,$2d,$ed,$46,$76,$f2,$ab,$85,$03,$74,$15,$74,$4e,$93,$9c,$3f,$43,$84,$12,$27,
    $25,$2c,$ec,$38,$da,$3c,$b0,$68,$fe,$87,$39,$6c,$73,$4c,$98,$6f,$84,$7a,$e7,$07,$6a,$18,$48,$38,$13,
    $99,$a9,$e9,$c7,$ef,$44,$18,$16,$69,$7e,$9b,$97,$59,$30,$7c,$98,$32,$b7,$8e,$dc,$6e,$79,$c2,$77,$29,
    $5e,$b4,$fc,$3b,$b5,$5e,$36,$89,$f7,$23,$05,$f4,$ed,$4f,$8f,$95,$8d,$bd,$e4,$9a,$4d,$81,$ff,$1f,$d9,
    $07,$d7,$9b,$24,$c8,$fb,$fc,$c3,$de,$9a,$0c,$6c,$f5,$2b,$15,$35,$18,$78,$8f,$ba,$45,$fd,$ec,$58,$55,
    $b7,$fa,$fe,$74,$66,$5f,$97,$02,$92,$c3,$69,$f9,$9c,$86,$35,$90,$51,$45,$e7,$72,$e2,$87,$68,$b2,$db,
    $cd,$ef,$f7,$0a,$99,$9e,$68,$51,$0b,$53,$46,$a6,$0c,$77,$7d,$8a,$81,$2d,$37,$04,$f8,$66,$d7,$c0,$ff,
    $5c,$3c,$81,$33,$b5,$9c,$63,$d9,$06,$e9,$22,$8e,$19,$c0,$9a,$f4,$a7,$b6,$59,$c5,$c4,$3e,$2e,$da,$a2,
    $75,$c6,$46,$e7,$cb,$c0,$7a,$7d,$c2,$ad,$38,$54,$05,$be,$ce,$91,$4d,$d6,$1d,$ea,$cf,$f2,$90,$ca,$31,
    $d0,$8f,$e8,$78,$b0,$98,$e2,$0b,$1e,$b4,$38,$20,$86,$cb,$1d,$ff,$23,$76,$a1,$fd,$73,$ab,$26,$c6,$57,
    $0b,$40,$04,$88,$9b,$9f,$18,$df,$89,$0e,$12,$9c,$05,$81,$79,$d7,$cb,$c1,$5d,$2e,$3c,$5d,$c5,$28,$26,
    $45,$19,$8b,$9c,$65,$dd,$2e,$dd,$dd,$a8,$66,$ef,$e7,$af,$bd,$1d,$98,$30,$7d,$20,$60,$aa,$9b,$1f,$fc,
    $10,$e5,$01,$44,$92,$be,$8e,$84,$30,$d4,$d0,$cb,$8b,$3e,$ed,$e2,$29,$f0,$d7,$8a,$b8,$8b,$6d,$eb,$30,
    $f4,$d4,$eb,$4b,$c2,$45,$3d,$85,$ae,$97,$23,$ea,$bb,$7c,$4b,$7b,$39,$18,$4c,$fc,$a8,$d4,$3c,$11,$82,
    $1c,$dc,$d8,$24,$70,$72,$e2,$fa,$d2,$0a,$b4,$2f,$4b,$ba,$99,$e3,$95,$5a,$e1,$8a,$ac,$7b,$0a,$c5,$6c,
    $05,$48,$af,$47,$88,$a3,$52,$54,$72,$8c,$00,$5f,$52,$17,$6a,$7f,$34,$ab,$b7,$f0,$cf,$5c,$14,$92,$b6,
    $f3,$7b,$fc,$f4,$cf,$c3,$9b,$8c,$41,$30,$b7,$18,$9c,$e3,$c9,$df,$b8,$ca,$3b,$19,$38,$6c,$0b,$18,$ac,
    $c1,$3f,$f5,$cd,$6e,$c4,$a7,$8b,$cc,$8b,$7c,$85,$97,$ee,$e3,$28,$08,$15,$77,$79,$b2,$49,$7c,$b3,$ed,
    $d6,$bc,$ef,$53,$31,$04,$c8,$08,$13,$10,$b6,$c1,$d1,$b1,$0d,$e4,$8c,$2a,$d2,$f6,$83,$a1,$1b,$fd,$f8,
    $6f,$e5,$e2,$94,$d4,$51,$05,$8f,$e0,$55,$41,$e5,$58,$dd,$16,$d1,$a4,$78,$3e,$9c,$16,$ed,$6d,$41,$55,
    $16,$9b,$13,$a6,$35,$d0,$f2,$8a,$1a,$29,$2e,$cb,$f4,$47,$66,$b2,$2c,$b5,$e1,$e8,$93,$70,$ea,$9b,$37,
    $24,$e9,$81,$46,$15,$0a,$04,$64,$04,$06,$92,$8a,$e2,$ca,$13,$b4,$b1,$30,$3d,$82,$c5,$e8,$1b,$91,$73,
    $a3,$f0,$b8,$ed,$5e,$b9,$47,$1d,$45,$d9,$ec,$d7,$3d,$69,$14,$75,$84,$21,$33,$d9,$a7,$d4,$83,$68,$83,
    $be,$17,$96,$10,$bb,$bb,$21,$5f,$14,$63,$54,$d8,$4d,$88,$d3,$9b,$a5,$3b,$f4,$b1,$54,$41,$2e,$47,$92,
    $34,$ea,$3b,$c5,$cc,$37,$14,$f6,$ed,$bd,$22,$e6,$30,$5b,$34,$b5,$fc,$e7,$43,$a6,$e5,$6d,$df,$a8,$b1,
    $90,$3a,$b9,$69,$a9,$79,$7f,$aa,$49,$24,$4f,$b5,$da,$bb,$fb,$12,$ee,$5b,$2d,$70,$23,$e5,$48,$86,$4c,
    $b1,$b9,$45,$34,$c9,$ad,$74,$38,$2f,$c3,$fa,$59,$71,$06,$a7,$1c,$8a,$d3,$ec,$37,$d4,$a6,$45,$8e,$84,
    $72,$e3,$09,$5c,$b5,$d6,$db,$a6,$ea,$f8,$fd,$ed,$9f,$c7,$1e,$27,$26,$bc,$39,$c5,$2c,$af,$65,$47,$b0,
    $3a,$48,$b6,$c5,$48,$92,$35,$68,$cf,$10,$fe,$00,$50,$b4,$0e,$fb,$d7,$d2,$eb,$d6,$84,$59,$e7,$13,$21,
    $9a,$47,$2b,$e3,$78,$c9,$8e,$cd,$0e,$03,$27,$fd,$94,$1a,$30,$5d,$7f,$44,$41,$af,$51,$23,$f1,$ea,$bc,
    $0f,$65,$a2,$46,$d1,$9a,$b4,$e1,$6d,$e5,$72,$30,$d0,$ed,$55,$ef,$15,$2e,$23,$e7,$da,$f0,$77,$61,$c9,
    $0f,$ac,$1e,$7b,$d1,$ba,$54,$e5,$80,$e3,$23,$ed,$e8,$76,$de,$60,$46,$6c,$7d,$d9,$16,$7e,$7c,$e2,$70,
    $62,$12,$01,$49,$58,$f8,$bf,$89,$3d,$7e,$ad,$99,$77,$3d,$0f,$51,$73,$0c,$d0,$20,$3b,$25,$fa,$e5,$ba,
    $ab,$15,$09,$75,$12,$3e,$33,$21,$50,$a3,$ac,$74,$3e,$c9,$01,$2b,$f5,$44,$5f,$45,$49,$f4,$0d,$f1,$ce,
    $fd,$46,$5d,$16,$27,$ec,$28,$d5,$64,$b9,$a5,$a6,$23,$fe,$b0,$0f,$fe,$30,$b6,$e5,$8e,$8e,$ce,$14,$22,
    $38,$7c,$69,$ec,$05,$f6,$dd,$7f,$af,$7a,$59,$47,$11,$4e,$ad,$30,$bd,$14,$56,$3f,$d3,$1d,$22,$89,$f8,
    $d9,$20,$e7,$4d,$4a,$88,$d8,$c4,$d6,$a8,$9d,$93,$f7,$2a,$55,$a8,$a4,$84,$2f,$e6,$be,$12,$67,$9e,$fa,
    $ed,$26,$6f,$f6,$77,$80,$44,$a3,$1a,$f6,$74,$1d,$63,$41,$db,$f5,$3b,$70,$c8,$1b,$52,$4f,$47,$ea,$fb,
    $30,$4f,$e8,$98,$ec,$ff,$b7,$9a,$a7,$39,$a7,$ea,$43,$de,$f0,$88,$d2,$8e,$10,$e7,$06,$5d,$97,$7b,$4f,
    $ac,$94,$34,$cf,$21,$cf,$15,$26,$7d,$92,$bc,$5a,$8e,$6f,$7d,$a6,$94,$9c,$a0,$2d,$d3,$be,$66,$e6,$c0,
    $94,$e5,$3c,$a6,$a7,$f7,$b9,$fa,$37,$08,$22,$df,$09,$c3,$5c,$06,$05,$9f,$4a,$19,$ac,$33,$3c,$82,$e5,
    $7d,$48,$22,$f0,$d3,$8d,$64,$5b,$a4,$cd,$9f,$ec,$b6,$9a,$36,$0f,$fa,$0c,$3e,$a2,$c3,$3d,$b7,$cc,$2b,
    $d4,$3b,$71,$dc,$d1,$1d,$d5,$b5,$77,$dc,$89,$e0,$ef,$5c,$f1,$d2,$ab,$bf,$e8,$4a,$70,$f8,$b2,$2f,$b5,
    $d7,$03,$7e,$83,$a1,$ad,$b3,$b6,$23,$85,$48,$8c,$9b,$ed,$b9,$8d,$99,$20,$11,$6d,$4f,$f9,$74,$b4,$69,
    $10,$08,$ac,$b1,$af,$c1,$38,$4f,$1b,$3f,$ab,$47,$7a,$04,$80,$06,$24,$73,$9f,$77,$b4,$2a,$0d,$5f,$69,
    $0f,$4c,$23,$4a,$9d,$5d,$d4,$fc,$e1,$3a,$1e,$e9,$90,$66,$bf,$5b,$b3,$7b,$58,$98,$56,$57,$b5,$c2,$04,
    $be,$5f,$39,$e6,$67,$8d,$dd,$4d,$b3,$93,$2c,$9d,$58,$67,$3d,$63,$d5,$9f,$7d,$05,$fc,$ad,$47,$ee,$f7,
    $93,$d9,$f7,$30,$41,$ad,$43,$c3,$93,$1a,$fc,$4b,$c6,$99,$93,$62,$e5,$cf,$5b,$c2,$bb,$b1,$a2,$b1,$d0,
    $94,$f4,$f4,$8b,$e3,$1d,$db,$5f,$50,$26,$51,$ab,$91,$a5,$1c,$be,$d5,$cb,$3a,$58,$a6,$d6,$c2,$5c,$70,
    $45,$f7,$55,$57,$1c,$0b,$25,$f3,$3d,$81,$19,$1e,$09,$69,$77,$a1,$c4,$d0,$7f,$55,$4a,$18,$13,$4c,$22,
    $c7,$db,$67,$e4,$7a,$38,$e5,$1d,$22,$15,$46,$6a,$3d,$6e,$aa,$35,$fa,$ac,$26,$74,$ea,$c5,$54,$36,$9f,
    $d1,$6b,$e7,$93,$09,$59,$98,$03,$32,$e5,$80,$a9,$eb,$4a,$0d,$8a,$d7,$38,$18,$9c,$e7,$2c,$18,$d3,$55,
    $10,$43,$b8,$7f,$e5,$a7,$92,$61,$15,$fd,$ec,$f3,$c0,$69,$7f,$58,$5f,$d0,$a4,$8b,$a7,$89,$67,$ec,$f1,
    $c7,$a8,$9b,$28,$30,$9b,$74,$02,$55,$8c,$a4,$3a,$d2,$6f,$28,$57,$48,$34,$e4,$17,$43,$ff,$37,$c0,$6b,
    $fc,$84,$1f,$c5,$b8,$f9,$e6,$5d,$6c,$98,$4e,$e7,$b1,$c1,$2e,$7c,$9e,$2f,$5c,$9d,$b6,$f0,$17,$9f,$93,
    $37,$e2,$da,$e7,$a5,$6b,$95,$81,$f9,$ff,$bc,$1d,$c4,$51,$e3,$e6,$65,$3a,$3d,$3b,$a3,$45,$b9,$bf,$c2,
    $94,$f5,$3e,$83,$8a,$df,$b9,$37,$9d,$5c,$38,$21,$f2,$48,$03,$d1,$3a,$94,$86,$79,$7e,$32,$78,$0b,$78,
    $07,$cc,$8f,$98,$f3,$b1,$63,$43,$7b,$aa,$25,$c9,$00,$94,$71,$cc,$2c,$97,$f4,$ce,$4a,$60,$9c,$aa,$e0,
    $60,$e4,$d2,$f1,$60,$21,$97,$b7,$1d,$d7,$9d,$da,$7e,$e9,$94,$90,$0d,$d1,$05,$43,$8f,$ad,$ff,$72,$18,
    $b9,$ad,$80,$0c,$d5,$ef,$9c,$72,$b4,$55,$e0,$42,$19,$54,$37,$6f,$28,$4c,$25,$d2,$d7,$1f,$3d,$49,$29,
    $10,$ba,$6e,$80,$54,$e3,$c2,$6f,$b0,$b8,$c8,$16,$a5,$b5,$18,$a6,$41,$8a,$b0,$a3,$98,$af,$8d,$c7,$eb,
    $4c,$28,$4b,$75,$53,$cf,$02,$32,$5b,$50,$78,$ce,$31,$e4,$ea,$c2,$29,$cb,$85,$28,$6b,$d5,$87,$1d,$26,
    $da,$98,$2d,$64,$fd,$28,$56,$8b,$8f,$01,$1f,$9e,$02,$66,$44,$4c,$fb,$1b,$fb,$81,$94,$60,$21,$84,$7a,
    $24,$66,$6d,$de,$dd,$7e,$53,$e1,$66,$2e,$3b,$05,$d0,$8e,$0c,$64,$d7,$1a,$82,$06,$86,$4e,$a4,$85,$f4,
    $09,$81,$cb,$bb,$2c,$c2,$82,$63,$3f,$8f,$6e,$e1,$21,$49,$1c,$48,$ba,$4f,$cb,$7d,$d1,$68,$aa,$76,$42,
    $79,$e3,$52,$fe,$4d,$e6,$ff,$06,$63,$a4,$64,$e3,$c5,$fa,$a6,$95,$5f,$0c,$a1,$43,$08,$7a,$60,$89,$32,
    $f7,$9a,$7d,$5b,$fc,$44,$19,$cf,$f4,$79,$fc,$fe,$64,$74,$e7,$a6,$d1,$8a,$9f,$8a,$cd,$6f,$57,$4e,$55,
    $e8,$a5,$09,$14,$4d,$d3,$04,$6a,$ca,$2a,$62,$6f,$70,$cc,$63,$bd,$0f,$82,$e2,$08,$87,$76,$39,$ba,$60,
    $d1,$5e,$e7,$59,$b1,$62,$69,$a4,$69,$82,$dc,$43,$5b,$d9,$b9,$e1,$7e,$ab,$20,$25,$80,$b5,$5f,$f8,$3b,
    $ff,$cf,$a6,$27,$2f,$ec,$6a,$cf,$10,$57,$65,$c6,$1b,$b1,$e8,$63,$2a,$0d,$7b,$ea,$07,$27,$ad,$8f,$f1,
    $de,$77,$1e,$5e,$10,$6a,$fd,$70,$07,$c8,$a3,$23,$04,$17,$c9,$75,$e0,$f4,$3d,$8a,$1c,$1a,$e2,$7f,$7d,
    $50,$ad,$94,$45,$3d,$92,$23,$1f,$50,$ad,$82,$15,$3e,$7a,$8d,$2a,$8d,$3a,$9f,$c6,$a0,$e1,$ca,$45,$30,
    $c6,$ad,$12,$24,$dd,$24,$25,$d4,$f7,$1b,$5a,$db,$de,$52,$a5,$86,$26,$72,$ba,$af,$41,$fe,$c4,$cd,$76,
    $f6,$18,$98,$47,$43,$01,$2a,$08,$6e,$79,$ab,$26,$49,$4d,$8f,$2a,$cd,$bf,$df,$fd,$14,$97,$e9,$6d,$73,
    $46,$85,$77,$af,$9d,$81,$41,$2a,$a8,$8a,$88,$ad,$13,$2c,$ad,$64,$c6,$2a,$a4,$18,$4e,$a6,$ec,$a1,$4c,
    $c9,$77,$68,$94,$96,$c8,$19,$69,$57,$2b,$e6,$1c,$84,$03,$5d,$05,$d7,$c9,$8c,$ff,$34,$fd,$73,$5d,$7f,
    $4a,$ce,$72,$8f,$ad,$98,$ba,$55,$11,$54,$dd,$40,$d2,$91,$a2,$88,$b0,$8d,$2e,$82,$ce,$e2,$cb,$5d,$0f,
    $9a,$7f,$eb,$b2,$69,$12,$88,$28,$30,$f9,$70,$64,$bc,$19,$92,$68,$9e,$69,$77,$5d,$01,$33,$60,$70,$77,
    $de,$06,$30,$ea,$0b,$a0,$23,$c8,$8a,$eb,$f3,$24,$a1,$c8,$75,$1f,$93,$01,$30,$b8,$2a,$82,$2c,$32,$90,
    $ff,$ac,$c3,$84,$41,$fe,$47,$47,$17,$e7,$ee,$31,$fe,$b0,$90,$89,$e2,$60,$ff,$93,$dd,$68,$76,$b9,$00,
    $e5,$f3,$44,$24,$a6,$25,$bc,$86,$ec,$be,$0d,$33,$57,$80,$70,$7b,$4b,$f3,$93,$a4,$5f,$65,$ef,$97,$5c,
    $39,$bf,$2d,$b6,$13,$34,$9f,$06,$af,$d7,$a0,$fa,$b4,$59,$d6,$68,$68,$49,$0b,$35,$a5,$e7,$5d,$6f,$9d,
    $1e,$d6,$a1,$de,$33,$81,$90,$45,$25,$5f,$df,$63,$17,$8a,$d5,$84,$a1,$44,$70,$04,$7e,$7d,$f0,$00,$db,
    $4c,$2b,$99,$e6,$cb,$32,$72,$14,$f0,$c7,$04,$8e,$fb,$f6,$5e,$30,$dc,$2a,$3b,$c6,$5a,$6c,$35,$de,$89,
    $16,$26,$5e,$96,$f9,$ae,$e8,$ed,$b0,$92,$a9,$4b,$76,$8f,$13,$00,$15,$2e,$96,$a2,$b5,$c3,$d8,$9b,$bd,
    $b5,$c1,$f7,$2b,$63,$24,$d2,$c6,$35,$90,$86,$2a,$6b,$9f,$63,$b0,$37,$17,$09,$08,$48,$12,$2d,$52,$c9,
    $10,$45,$0a,$9a,$9b,$88,$12,$e2,$1c,$2b,$fa,$8e,$15,$d5,$a6,$44,$7c,$90,$2f,$bc,$21,$ff,$d7,$4c,$9b,
    $42,$fb,$4a,$0d,$5e,$39,$ef,$12,$f0,$23,$05,$e1,$82,$4b,$1c,$76,$c0,$38,$77,$77,$11,$5f,$ce,$04,$fa,
    $84,$9a,$f4,$8b,$b8,$1f,$2e,$d6,$ec,$83,$43,$c2,$7c,$5a,$3b,$66,$e9,$c0,$97,$5d,$d6,$c8,$b3,$3e,$f8,
    $a3,$cb,$33,$75,$b1,$19,$73,$5c,$38,$e5,$71,$43,$be,$54,$1e,$85,$bb,$03,$28,$0d,$09,$bb,$f3,$64,$d7,
    $68,$6f,$20,$42,$cc,$7a,$41,$f9,$7f,$af,$b7,$80,$52,$49,$9d,$d7,$e6,$61,$18,$18,$a2,$de,$08,$3b,$d6,
    $c6,$27,$3a,$9d,$12,$40,$c1,$7d,$aa,$88,$c7,$a4,$50,$b7,$29,$49,$a8,$8c,$6d,$4c,$05,$dd,$06,$0f,$6f,
    $c5,$57,$6e,$67,$c4,$78,$e3,$6e,$9a,$ad,$a7,$72,$7c,$72,$a9,$f0,$4d,$4c,$fa,$b0,$38,$44,$ee,$4c,$82,
    $b1,$62,$7b,$d3,$de,$e2,$40,$13,$45,$82,$1d,$b0,$3f,$ed,$fc,$6f,$21,$0b,$d8,$fb,$38,$a7,$7d,$a9,$54,
    $e7,$3a,$f7,$98,$ee,$df,$f2,$18,$f1,$bc,$3b,$12,$a1,$7e,$48,$8a,$a7,$d3,$84,$d8,$51,$7f,$71,$2a,$4b,
    $6d,$a8,$26,$4f,$25,$c2,$6a,$38,$ed,$b3,$70,$bd,$dd,$06,$d4,$b9,$b7,$95,$95,$0a,$0d,$32,$f2,$ef,$61,
    $9d,$a0,$a8,$e9,$5e,$da,$ee,$d5,$46,$8d,$84,$f8,$06,$33,$4e,$22,$36,$cf,$ec,$2f,$2c,$e2,$b1,$71,$14,
    $87,$93,$2b,$4d,$f5,$a3,$b8,$b3,$08,$bf,$05,$c3,$ed,$a2,$1e,$07,$22,$ed,$4e,$e9,$31,$e5,$4e,$14,$ef,
    $de,$3a,$32,$49,$60,$2e,$be,$f5,$2c,$7b,$93,$76,$77,$df,$c8,$a7,$c9,$7b,$c0,$b2,$a8,$3f,$32,$8e,$25,
    $5a,$07,$eb,$2d,$f3,$a8,$17,$17,$c2,$81,$6d,$1d,$6e,$4b,$90,$48,$2e,$02,$58,$af,$a7,$e8,$11,$3b,$dc,
    $58,$82,$36,$fe,$06,$9d,$4b,$cd,$02,$38,$d2,$0a,$db,$69,$f7,$bb,$16,$58,$0e,$83,$df,$aa,$20,$3b,$e4,
    $39,$86,$e2,$f1,$b9,$ff,$99,$52,$30,$92,$10,$04,$68,$9d,$66,$a7,$40,$2a,$33,$d9,$50,$3d,$a5,$3b,$9e,
    $49,$de,$27,$61,$c9,$82,$f6,$18,$49,$e1,$8b,$b2,$38,$29,$ca,$2d,$c9,$76,$18,$e3,$f1,$e4,$e8,$7c,$19,
    $5a,$3e,$60,$73,$88,$0a,$65,$8e,$29,$31,$fc,$e0,$0f,$aa,$65,$bb,$55,$b5,$75,$36,$e1,$b3,$c7,$eb,$9f,
    $a7,$53,$67,$ad,$c8,$2a,$73,$ad,$b1,$ff,$9c,$b5,$5f,$a3,$2f,$f8,$52,$31,$f7,$3c,$58,$e3,$a5,$0b,$dd,
    $2c,$5f,$30,$2e,$fa,$a2,$e5,$88,$ac,$7e,$e4,$f7,$f8,$7f,$98,$d1,$87,$f2,$95,$9f,$f8,$17,$24,$17,$af,
    $11,$54,$7e,$ad,$4e,$ce,$b5,$3e,$50,$f6,$f6,$06,$97,$21,$ee,$7d,$eb,$4b,$75,$6b,$0c,$9c,$9e,$09,$ed,
    $83,$35,$b6,$f1,$60,$5e,$30,$0d,$d2,$0d,$24,$6e,$bd,$0a,$5b,$82,$c9,$65,$13,$fa,$33,$8a,$44,$3b,$d9,
    $74,$f3,$a7,$f2,$da,$31,$0e,$76,$f1,$cf,$f3,$3a,$8e,$16,$24,$26,$5a,$2a,$06,$0c,$03,$1b,$42,$5b,$1f,
    $bf,$eb,$9c,$7a,$1a,$c1,$39,$e1,$4a,$19,$dd,$6b,$b9,$33,$f3,$d1,$ed,$3c,$10,$30,$2b,$63,$91,$46,$e3,
    $1c,$d9,$a0,$0e,$dc,$32,$2e,$3c,$ef,$a1,$a3,$32,$3f,$ed,$be,$37,$1b,$77,$f9,$f1,$40,$cb,$9e,$0b,$a5,
    $a6,$82,$10,$43,$84,$ab,$4e,$aa,$ed,$e7,$78,$fe,$f1,$89,$b7,$f2,$b5,$9f,$cc,$f8,$ad,$64,$65,$66,$51,
    $6e,$8e,$50,$09,$a9,$7a,$14,$69,$b4,$f3,$fa,$9b,$e9,$c5,$93,$e5,$2a,$b6,$ea,$25,$fb,$92,$cc,$ef,$4f,
    $0d,$9a,$0d,$0c,$05,$72,$24,$65,$0e,$c9,$c2,$34,$79,$96,$4f,$7f,$39,$27,$ff,$30,$e9,$10,$af,$83,$3b,
    $3a,$89,$6a,$c3,$bd,$76,$ff,$4c,$1e,$1c,$a7,$ff,$a8,$eb,$2e,$59,$1b,$63,$d6,$e3,$2b,$92,$94,$44,$fc,
    $f6,$50,$86,$e4,$49,$88,$8b,$91,$3f,$22,$89,$88,$0f,$98,$e8,$ef,$72,$f6,$c0,$f9,$2c,$9d,$7c,$8b,$62,
    $05,$d9,$07,$00,$b9,$42,$56,$43,$ad,$a9,$1f,$74,$c8,$7f,$f7,$ec,$2c,$38,$e9,$06,$d8,$d1,$d5,$56,$9f,
    $97,$1f,$b2,$da,$15,$ef,$51,$f9,$23,$ce,$22,$83,$b5,$3a,$a9,$0e,$0d,$ed,$03,$e2,$5f,$0d,$ee,$94,$65,
    $74,$3a,$6d,$53,$58,$73,$5d,$77,$22,$09,$4c,$6f,$bb,$8e,$42,$95,$af,$c8,$5c,$5a,$b9,$4a,$0a,$d8,$8c,
    $16,$dc,$62,$9a,$67,$72,$f3,$be,$5c,$3f,$62,$cc,$fc,$58,$40,$c8,$ad,$49,$73,$15,$a5,$0e,$18,$93,$cd,
    $2f,$73,$aa,$ac,$8b,$61,$90,$05,$86,$21,$32,$a8,$b7,$3b,$a4,$1c,$c8,$a4,$bf,$42,$a4,$ca,$5e,$e6,$a3,
    $fc,$39,$0b,$2d,$57,$ad,$21,$37,$10,$09,$ca,$b5,$11,$3a,$0a,$93,$dd,$1f,$a3,$5c,$44,$ad,$da,$6f,$3f,
    $bc,$94,$07,$73,$32,$57,$d7,$96,$f4,$73,$87,$14,$3d,$5b,$2e,$91,$d9,$37,$40,$96,$b4,$b8,$09,$1c,$57,
    $2d,$9d,$90,$c7,$c1,$f7,$c7,$a8,$5c,$ff,$3f,$51,$50,$a0,$db,$32,$73,$e6,$37,$50,$f8,$2b,$26,$8c,$49,
    $9c,$03,$39,$6e,$c5,$89,$02,$b3,$60,$a4,$6e,$c6,$0a,$d0,$1d,$2c,$1e,$39,$c2,$a6,$9e,$83,$3c,$e1,$1a,
    $32,$34,$0a,$a9,$5e,$bf,$8c,$8a,$d7,$21,$3e,$09,$07,$14,$44,$a3,$3b,$61,$f8,$f8,$3b,$b5,$f8,$e1,$85,
    $16,$5f,$98,$b3,$65,$20,$52,$7c,$60,$f9,$b4,$d8,$16,$7e,$11,$8c,$17,$8b,$62,$be,$8d,$d5,$03,$60,$82,
    $26,$64,$3b,$d8,$03,$6d,$6a,$8e,$2b,$05,$96,$0d,$72,$75,$3d,$1e,$8a,$d0,$c3,$e7,$0a,$ea,$f4,$5f,$f5,
    $4e,$98,$0a,$30,$e8,$7b,$6d,$8f,$d7,$e1,$7f,$47,$fe,$65,$75,$90,$14,$e6,$62,$20,$87,$69,$9c,$fe,$87,
    $aa,$51,$5e,$a6,$17,$89,$84,$eb,$40,$70,$84,$39,$1e,$35,$0a,$8e,$5b,$5c,$2b,$a4,$2a,$dc,$23,$36,$a8,
    $17,$7a,$97,$55,$c1,$8a,$e6,$86,$3f,$cc,$dd,$51,$e6,$e2,$0d,$b3,$c7,$15,$46,$b8,$99,$5a,$af,$88,$98,
    $fc,$30,$6b,$e2,$20,$06,$07,$9e,$51,$b1,$02,$23,$93,$2f,$c5,$04,$ed,$b8,$7e,$80,$a4,$50,$41,$30,$3b,
    $6e,$a8,$01,$ed,$6c,$0b,$9b,$d9,$e8,$92,$57,$dd,$d9,$14,$9d,$bf,$0b,$bb,$57,$85,$7c,$a1,$13,$84,$39,
    $a9,$66,$1f,$0c,$69,$b8,$4d,$e5,$cf,$bf,$ee,$ff,$6f,$a4,$18,$19,$4c,$0f,$ea,$87,$fc,$e0,$99,$62,$6a,
    $f2,$cb,$2e,$6f,$e5,$ef,$7d,$bb,$e1,$41,$35,$d6,$0b,$cc,$03,$3b,$5e,$1e,$1d,$db,$8b,$39,$c8,$85,$3b,
    $24,$b3,$89,$9b,$48,$e1,$3e,$9b,$dd,$58,$6a,$46,$ab,$f8,$29,$15,$31,$94,$d2,$c1,$56,$45,$b6,$15,$6b,
    $a7,$30,$62,$e6,$af,$30,$c8,$bc,$79,$bf,$40,$e0,$ff,$b9,$fb,$d3,$38,$c2,$12,$39,$6a,$4e,$c8,$95,$86,
    $13,$2d,$d1,$a8,$38,$b6,$5c,$9c,$1c,$4a,$4f,$42,$3a,$16,$40,$16,$b9,$b2,$66,$6c,$b6,$00,$11,$a2,$17,
    $9d,$5f,$92,$7c,$72,$14,$87,$22,$31,$29,$62,$ca,$3d,$a3,$54,$e9,$5c,$c4,$21,$a5,$4e,$72,$06,$de,$04,
    $6d,$16,$23,$67,$35,$ed,$7e,$1c,$29,$d8,$25,$d3,$e2,$a2,$2a,$30,$f3,$96,$a2,$74,$e2,$69,$6d,$3d,$2f,
    $d6,$16,$36,$1d,$f5,$f5,$e2,$a4,$8c,$13,$7d,$d4,$17,$31,$52,$a7,$8e,$64,$fe,$76,$43,$88,$f5,$51,$5d,
    $8b,$f4,$a4,$92,$1d,$4a,$6c,$76,$1c,$a7,$7a,$9b,$26,$4a,$43,$3c,$cd,$a8,$c4,$d3,$1e,$dd,$c0,$7e,$e9,
    $be,$ba,$24,$63,$f9,$67,$34,$3a,$98,$c5,$a4,$35,$e9,$1a,$a2,$bd,$89,$ec,$aa,$b3,$1f,$67,$ff,$8a,$8e,
    $b8,$07,$3f,$f3,$4d,$50,$06,$26,$fd,$b1,$80,$d6,$07,$e5,$64,$0e,$f0,$8d,$11,$cb,$12,$2a,$8c,$83,$0a,
    $b6,$3b,$d9,$0d,$9a,$54,$15,$0b,$05,$4c,$23,$40,$b0,$82,$4b,$7a,$07,$f9,$1b,$4a,$10,$ff,$0b,$20,$51,
    $5a,$f8,$83,$88,$e5,$b2,$23,$d5,$ec,$78,$01,$43,$b6,$ff,$56,$0d,$d5,$c9,$95,$8c,$57,$f1,$85,$35,$49,
    $bc,$87,$18,$2b,$d6,$d9,$6e,$25,$11,$2f,$68,$55,$4a,$71,$13,$c4,$48,$b7,$93,$46,$c9,$88,$e2,$99,$23,
    $2b,$34,$d7,$32,$f5,$f6,$08,$f2,$5a,$32,$19,$cd,$f9,$c8,$5f,$25,$43,$00,$54,$31,$9e,$5f,$a7,$fd,$79,
    $8c,$12,$8e,$46,$57,$7a,$19,$91,$47,$38,$76,$80,$d8,$b1,$64,$99,$78,$5a,$6c,$54,$79,$c3,$b3,$0c,$51,
    $3d,$9f,$c9,$c7,$0e,$e1,$05,$2b,$3c,$c7,$44,$90,$cb,$94,$8e,$0c,$8a,$df,$75,$c8,$e5,$1d,$66,$e2,$dd,
    $e2,$2b,$15,$f6,$4a,$4c,$28,$7f,$9a,$92,$5c,$a8,$1f,$5b,$33,$ef,$18,$da,$78,$f9,$2e,$a8,$7b,$b0,$b5,
    $e0,$12,$5a,$17,$c5,$6b,$6f,$5b,$57,$96,$75,$3d,$34,$6a,$4f,$5d,$fd,$c9,$1b,$32,$61,$dd,$17,$1c,$c9,
    $f9,$ab,$1e,$f4,$e1,$2e,$0f,$3c,$46,$f4,$7e,$33,$65,$78,$47,$2b,$d3,$18,$91,$14,$41,$bd,$6a,$da,$64,
    $7e,$7b,$35,$c4,$4b,$3a,$1b,$f3,$ed,$6d,$7a,$e7,$95,$f0,$6a,$ed,$44,$1b,$3d,$59,$35,$49,$80,$a1,$10,
    $a7,$11,$67,$9c,$cd,$6d,$1e,$29,$9a,$48,$55,$38,$a2,$86,$29,$33,$d8,$bf,$1f,$b4,$c4,$17,$c5,$ce,$fa,
    $bd,$61,$49,$b1,$9d,$4a,$3a,$a9,$ce,$41,$13,$5f,$bf,$fb,$25,$45,$3d,$d5,$d4,$51,$05,$b7,$51,$85,$d4,
    $2b,$e1,$72,$cd,$de,$83,$8e,$bd,$a2,$75,$78,$fa,$1f,$44,$2e,$2d,$6e,$55,$50,$99,$96,$92,$bd,$df,$7b,
    $ae,$50,$12,$d3,$e6,$16,$06,$fe,$22,$88,$ff,$e6,$60,$f1,$56,$3a,$90,$e7,$3f,$1a,$b2,$52,$a1,$62,$f2,
    $66,$37,$95,$a3,$46,$51,$c9,$89,$85,$71,$f4,$89,$df,$b2,$aa,$6c,$be,$79,$b7,$29,$a0,$5f,$f5,$23,$47,
    $51,$29,$01,$fd,$9f,$82,$6f,$82,$75,$1a,$d4,$aa,$0f,$75,$94,$87,$ba,$bd,$0a,$80,$f5,$69,$b4,$63,$d1,
    $be,$15,$bc,$f6,$58,$e3,$e9,$c7,$08,$ad,$87,$a5,$cd,$93,$e2,$b9,$0c,$90,$bd,$e6,$11,$b3,$bb,$9b,$a8,
    $3f,$d2,$34,$5e,$8f,$9e,$b4,$42,$bc,$fb,$01,$c5,$4c,$57,$2e,$3d,$96,$ed,$38,$71,$0d,$1f,$0e,$c2,$3a,
    $e9,$a6,$de,$de,$b5,$1d,$3a,$e2,$d8,$98,$86,$6f,$ea,$f9,$a7,$12,$69,$f8,$74,$dc,$11,$cf,$94,$31,$03,
    $bd,$36,$9e,$92,$51,$09,$e0,$ec,$fe,$ff,$ae,$e9,$a8,$b2,$d1,$c3,$be,$78,$88,$3d,$9f,$89,$e8,$6b,$85,
    $3b,$8b,$06,$2d,$25,$8a,$fb,$cf,$a0,$65,$cd,$2d,$5e,$28,$30,$48,$d9,$18,$28,$2b,$79,$bc,$cd,$be,$26,
    $98,$0e,$50,$6d,$fc,$73,$d7,$d1,$aa,$bf,$45,$9f,$2f,$be,$e3,$7a,$97,$49,$3a,$7f,$85,$aa,$d6,$a7,$0d,
    $de,$f7,$b3,$42,$23,$bb,$5f,$b1,$70,$03,$01,$82,$4b,$27,$78,$67,$1d,$8a,$5b,$36,$97,$21,$76,$da,$c1,
    $e9,$fe,$d7,$bf,$0a,$51,$aa,$7a,$57,$e0,$b5,$0a,$d1,$ba,$4f,$66,$9d,$c8,$c9,$45,$a4,$bd,$e5,$75,$f9,
    $7b,$f9,$00,$a9,$7a,$b8,$4d,$ba,$68,$d6,$1b,$e1,$38,$05,$08,$c9,$9e,$ee,$7f,$47,$67,$e9,$1e,$00,$b3,
    $77,$6b,$50,$84,$c3,$20,$1b,$da,$52,$0f,$3e,$c7,$be,$10,$55,$a4,$29,$de,$6c,$35,$7f,$a3,$42,$2b,$68,
    $2d,$b5,$f7,$da,$90,$32,$62,$06,$31,$68,$8f,$4a,$5b,$92,$bb,$fd,$23,$6c,$d9,$41,$b1,$11,$6d,$61,$c9,
    $87,$c9,$c0,$71,$28,$3a,$1c,$00,$52,$78,$82,$ed,$ea,$aa,$e5,$9d,$1c,$c3,$9b,$c5,$60,$ae,$d6,$a6,$b4,
    $7f,$c8,$9d,$68,$0d,$1e,$a8,$e8,$a4,$92,$63,$b6,$e0,$24,$aa,$a4,$69,$d3,$e4,$e0,$b1,$b3,$64,$5d,$13,
    $36,$a4,$7a,$a3,$b2,$2a,$8b,$22,$e5,$20,$25,$92,$7c,$6e,$c0,$16,$42,$b2,$26,$b6,$ba,$b9,$d5,$a0,$b3,
    $9c,$86,$89,$92,$74,$69,$f9,$fd,$f7,$77,$56,$46,$65,$d9,$f5,$8c,$06,$e2,$46,$42,$cb,$36,$5a,$d1,$02,
    $4b,$4f,$e7,$56,$6b,$50,$6e,$13,$f0,$dc,$26,$d6,$ea,$b3,$44,$14,$68,$cd,$fa,$20,$35,$d6,$07,$36,$84,
    $61,$64,$33,$4f,$5c,$c4,$4b,$3e,$cd,$ca,$19,$da,$83,$41,$e3,$2e,$9f,$26,$6c,$3f,$eb,$ff,$6c,$02,$38,
    $d0,$91,$67,$18,$4b,$26,$24,$86,$c8,$8a,$7c,$22,$4d,$ed,$8b,$0b,$b2,$3e,$76,$4e,$83,$20,$86,$16,$d6,
    $cf,$c3,$e2,$a3,$7f,$fa,$dd,$8e,$9c,$fc,$60,$9f,$ab,$62,$ac,$c8,$35,$28,$a0,$0d,$0b,$ea,$1f,$69,$9d,
    $91,$c3,$19,$6c,$65,$0b,$04,$64,$f6,$3e,$bb,$e0,$2c,$f1,$58,$22,$4b,$23,$56,$a8,$f6,$df,$71,$ff,$fa,
    $a1,$ca,$3c,$d9,$c6,$69,$f2,$47,$58,$4e,$e8,$c7,$5c,$72,$13,$1e,$f1,$7a,$4b,$3e,$43,$71,$ac,$58,$08,
    $93,$39,$76,$a4,$94,$42,$2b,$ec,$6f,$72,$b1,$fb,$af,$10,$43,$19,$2c,$3b,$6d,$8c,$f2,$11,$7d,$68,$9d,
    $69,$8d,$f3,$ef,$15,$6e,$33,$cd,$0a,$f0,$6e,$d8,$09,$e5,$6f,$eb,$c4,$b6,$3c,$b3,$26,$78,$9d,$fc,$4c,
    $b9,$a7,$28,$54,$e3,$51,$42,$c2,$7d,$c8,$11,$e8,$a0,$ef,$8a,$57,$a5,$aa,$f4,$08,$73,$b5,$39,$90,$87,
    $0f,$ae,$40,$a1,$02,$d6,$9a,$1e,$e9,$b2,$f0,$92,$0b,$53,$9c,$bc,$4f,$be,$1d,$b7,$de,$87,$31,$c7,$ec,
    $6b,$9a,$9e,$79,$2a,$b7,$10,$e1,$82,$ed,$f0,$50,$67,$34,$85,$ed,$76,$23,$a9,$7a,$17,$5c,$93,$7d,$f8,
    $f9,$93,$ab,$96,$e3,$07,$31,$eb,$be,$b1,$01,$e1,$90,$8c,$8f,$2a,$3d,$88,$94,$dc,$56,$72,$9b,$f3,$93,
    $7d,$c8,$29,$07,$0b,$31,$fd,$73,$9a,$03,$a3,$8f,$8f,$6d,$b9,$e7,$91,$e0,$89,$b1,$ba,$30,$58,$ce,$a5,
    $29,$59,$35,$b8,$c5,$40,$8a,$ac,$b3,$ea,$5e,$4f,$d7,$1c,$1f,$49,$83,$67,$e4,$99,$a8,$ee,$22,$91,$45,
    $3b,$52,$c6,$70,$05,$de,$eb,$3f,$82,$75,$f2,$9e,$84,$43,$af,$00,$48,$36,$60,$b7,$60,$6e,$9b,$d0,$17,
    $9f,$6c,$b2,$55,$8e,$08,$bc,$7e,$7b,$86,$95,$a5,$f1,$1d,$db,$fc,$d0,$95,$38,$58,$77,$94,$70,$d2,$d7,
    $8d,$52,$73,$5d,$39,$83,$f6,$e3,$be,$d4,$cd,$4b,$95,$88,$f8,$7a,$8a,$74,$c7,$43,$bf,$e8,$8d,$f7,$b3,
    $2d,$2c,$17,$fb,$34,$69,$f6,$fc,$e7,$6c,$d9,$b4,$a3,$c1,$df,$a4,$d3,$15,$67,$22,$48,$65,$cf,$f4,$b2,
    $7b,$9e,$0d,$03,$06,$df,$b7,$5b,$0a,$42,$e4,$27,$ca,$a4,$3c,$e2,$7c,$24,$2d,$4d,$83,$52,$03,$fd,$1d,
    $c6,$81,$4d,$52,$b3,$e1,$c5,$ab,$e2,$c4,$fc,$90,$96,$bd,$b9,$b2,$3d,$22,$b1,$be,$16,$9d,$a5,$95,$91,
    $d1,$5b,$85,$3e,$96,$86,$56,$86,$27,$16,$d4,$8d,$80,$c2,$90,$1c,$99,$19,$dc,$be,$50,$fb,$b2,$ee,$ad,
    $90,$b5,$9c,$d0,$33,$4f,$97,$9b,$b6,$04,$88,$da,$1f,$ab,$4a,$77,$86,$c8,$3a,$ec,$c1,$ad,$15,$e5,$97,
    $09,$ef,$fb,$b9,$28,$c2,$78,$1c,$99,$dd,$40,$48,$a3,$d6,$fa,$b2,$78,$db,$2b,$74,$36,$07,$dd,$3d,$83,
    $42,$a1,$e9,$60,$0d,$b7,$f6,$d6,$fc,$90,$0a,$90,$61,$d2,$57,$8e,$63,$37,$21,$3c,$c9,$c3,$81,$0c,$43,
    $fd,$37,$d8,$1f,$89,$46,$1d,$0a,$17,$39,$07,$51,$ed,$81,$cb,$03,$31,$1b,$f4,$e9,$ee,$25,$ce,$3f,$af,
    $5b,$9a,$13,$d9,$56,$be,$be,$cb,$52,$bc,$e1,$9c,$0a,$07,$f5,$01,$1a,$8a,$db,$00,$79,$d2,$4a,$0d,$32,
    $0b,$17,$7d,$8a,$96,$ac,$21,$9e,$ed,$84,$4b,$dc,$52,$f5,$3f,$7d,$39,$58,$c7,$c9,$43,$13,$3b,$c6,$9d,
    $33,$83,$1a,$ae,$28,$a5,$b2,$26,$21,$4f,$7f,$59,$9d,$bb,$7c,$6d,$0c,$43,$d3,$11,$3d,$b4,$a6,$f9,$dc,
    $1f,$c0,$78,$13,$d7,$5a,$0d,$d6,$e7,$72,$34,$cb,$db,$cb,$6b,$ab,$a6,$f0,$df,$be,$bb,$02,$f9,$68,$d3,
    $b9,$dd,$b8,$ea,$cb,$ce,$67,$9c,$ed,$57,$e5,$a3,$28,$33,$9a,$3d,$8d,$2e,$0a,$aa,$14,$45,$4c,$46,$f3,
    $e0,$87,$eb,$90,$e4,$8c,$7c,$bb,$c0,$d3,$a4,$a9,$fb,$8f,$b6,$17,$66,$85,$77,$c7,$ae,$79,$b4,$71,$af,
    $e8,$7b,$01,$00,$e8,$68,$2e,$9c,$5f,$2f,$b7,$f8,$8d,$73,$2f,$ac,$33,$d3,$9b,$e9,$67,$3f,$2b,$c9,$8a,
    $37,$4f,$5a,$15,$79,$95,$ba,$62,$fe,$e6,$cf,$6f,$11,$81,$0b,$64,$00,$9c,$66,$db,$64,$c7,$90,$1d,$82,
    $24,$da,$2d,$07,$a9,$5f,$b0,$4b,$64,$a6,$fd,$57,$ad,$18,$f3,$26,$3a,$42,$bc,$e4,$04,$07,$fe,$e4,$d3,
    $d8,$54,$cc,$07,$e7,$d0,$38,$aa,$fd,$8c,$70,$32,$a3,$19,$f0,$5b,$f9,$e9,$03,$29,$b9,$4b,$3c,$67,$aa,
    $6e,$a2,$f9,$2e,$31,$b0,$87,$3d,$54,$3c,$79,$de,$21,$9c,$de,$0c,$82,$2d,$d2,$86,$55,$87,$24,$32,$43,
    $40,$42,$1d,$7a,$7b,$a4,$66,$26,$ae,$b5,$e3,$a1,$69,$59,$a6,$ad,$2d,$96,$ea,$d4,$e3,$41,$e7,$d8,$a2,
    $0d,$9e,$79,$2e,$31,$2a,$8e,$c9,$4f,$4e,$26,$c4,$f1,$28,$a4,$4d,$57,$94,$27,$51,$a4,$b1,$de,$5a,$e5,
    $79,$0d,$28,$08,$33,$c1,$44,$71,$49,$b1,$7b,$8f,$96,$2b,$dc,$77,$6e,$2d,$e7,$7a,$d7,$31,$32,$27,$19,
    $27,$28,$ae,$98,$4b,$37,$b5,$32,$1a,$57,$bd,$c8,$2d,$f5,$c4,$e4,$37,$65,$00,$55,$81,$bf,$c8,$35,$51,
    $43,$3a,$ff,$85,$c0,$f6,$51,$32,$2a,$1b,$a4,$fd,$72,$88,$05,$4c,$e3,$db,$91,$94,$34,$ea,$0f,$9f,$23,
    $bc,$da,$0d,$32,$21,$64,$50,$8c,$78,$ec,$d4,$a8,$49,$11,$25,$20,$57,$ed,$23,$6a,$e3,$40,$38,$f9,$15,
    $0b,$6c,$47,$ac,$fd,$99,$de,$99,$1a,$06,$15,$3d,$f0,$17,$39,$b7,$61,$6e,$d2,$80,$16,$3d,$8a,$18,$3d,
    $b5,$80,$0e,$7b,$3f,$da,$7c,$df,$c9,$30,$70,$39,$a6,$e5,$e9,$03,$af,$81,$4d,$99,$16,$47,$e4,$dd,$0e,
    $c4,$0f,$8f,$8b,$ed,$69,$94,$72,$f5,$bc,$51,$89,$c8,$ad,$ce,$06,$5c,$13,$4d,$19,$91,$62,$a7,$84,$31,
    $18,$91,$8f,$d9,$65,$c3,$71,$b6,$d9,$e3,$42,$0d,$11,$11,$cc,$59,$90,$10,$59,$52,$e2,$df,$32,$ce,$dc,
    $48,$24,$8e,$19,$cc,$80,$ad,$43,$e0,$f7,$c7,$41,$60,$e5,$77,$6a,$4b,$0a,$1a,$2d,$e4,$1c,$53,$1c,$09,
    $31,$5f,$9c,$1c,$49,$fb,$2b,$3b,$47,$b1,$55,$3a,$40,$fa,$9a,$43,$ea,$63,$52,$af,$da,$08,$ea,$bc,$0a,
    $5e,$d3,$ab,$72,$41,$f0,$7f,$f8,$85,$28,$bb,$c1,$41,$24,$cc,$4d,$20,$a0,$81,$a3,$7c,$5f,$0b,$c5,$dd,
    $22,$91,$52,$f7,$38,$c7,$5c,$50,$91,$69,$35,$8b,$a8,$b2,$0d,$c7,$d1,$7a,$87,$b1,$d3,$72,$e4,$96,$21,
    $2a,$fb,$84,$8e,$65,$8c,$c9,$2b,$3f,$7e,$87,$76,$1f,$9e,$6d,$fb,$4a,$46,$d8,$e7,$5d,$3e,$3e,$f2,$90,
    $81,$0d,$37,$cb,$bd,$d4,$75,$12,$44,$53,$96,$4e,$32,$08,$53,$2c,$2a,$82,$d3,$9d,$e1,$e8,$a8,$6d,$f5,
    $51,$29,$96,$25,$a1,$07,$44,$bc,$56,$07,$e1,$9b,$59,$ef,$03,$25,$8a,$12,$ab,$4f,$a3,$b1,$60,$61,$61,
    $22,$e1,$1d,$f9,$9b,$32,$45,$70,$55,$85,$85,$d7,$26,$e4,$a8,$87,$6e,$72,$89,$3c,$0a,$be,$24,$d7,$67,
    $04,$39,$a5,$ab,$13,$bd,$47,$5b,$33,$6e,$fe,$b4,$0b,$64,$a5,$17,$3d,$b5,$4a,$2d,$8f,$0b,$64,$36,$24,
    $7b,$5f,$bb,$17,$08,$d4,$f6,$5a,$c6,$7b,$d1,$9b,$56,$b4,$77,$ae,$3b,$a1,$e9,$f3,$f7,$5c,$71,$c4,$df,
    $de,$ac,$8f,$a9,$f8,$69,$6c,$a3,$3d,$4b,$97,$4c,$72,$b1,$cb,$2a,$e3,$aa,$a0,$6c,$cd,$cc,$cd,$72,$c9,
    $96,$9a,$0c,$94,$0a,$a2,$b0,$c1,$2f,$b9,$b6,$25,$19,$d4,$ec,$d2,$74,$7a,$59,$ce,$2d,$c4,$08,$03,$78,
    $16,$91,$eb,$27,$bc,$a4,$a4,$48,$52,$af,$c5,$4b,$73,$d4,$96,$c1,$5f,$7d,$da,$fa,$cd,$69,$29,$49,$77,
    $bb,$1f,$2b,$ca,$0b,$58,$d1,$c7,$68,$60,$b7,$a1,$09,$02,$df,$07,$d3,$9f,$20,$44,$0b,$5f,$19,$ac,$08,
    $1c,$6d,$7d,$c3,$9b,$b2,$33,$28,$19,$07,$9f,$15,$2e,$4d,$2e,$37,$c8,$73,$03,$82,$0d,$68,$9b,$84,$de,
    $20,$55,$e2,$8d,$97,$e7,$01,$cd,$a6,$2d,$44,$a5,$a7,$70,$08,$cf,$90,$a7,$75,$ee,$f7,$65,$72,$5d,$e9,
    $03,$46,$7b,$3c,$89,$e9,$fd,$05,$51,$5e,$aa,$07,$11,$80,$1b,$98,$1b,$29,$f0,$78,$1a,$84,$e2,$c0,$8f,
    $4f,$a0,$68,$9e,$69,$b5,$77,$c1,$58,$b1,$d9,$3f,$d0,$a7,$7c,$c8,$90,$19,$c1,$d4,$c8,$5c,$00,$1f,$41,
    $78,$bf,$7d,$9b,$a1,$97,$7d,$4b,$3c,$d9,$52,$25,$84,$73,$01,$6b,$a3,$b1,$cd,$da,$93,$e5,$e7,$e0,$8d,
    $28,$ef,$b2,$e3,$2e,$39,$64,$31,$34,$b2,$ae,$2e,$b2,$a5,$84,$ee,$2b,$fc,$7f,$16,$9c,$00,$3c,$c6,$93,
    $c7,$5b,$b2,$4c,$25,$91,$01,$87,$2a,$dc,$f7,$07,$5c,$c5,$87,$88,$b1,$f1,$6d,$bf,$7e,$3c,$f8,$71,$ec,
    $dc,$dc,$54,$c5,$05,$91,$16,$61,$41,$cd,$76,$be,$18,$39,$a3,$ac,$fd,$38,$64,$3c,$df,$40,$87,$3b,$1b,
    $74,$08,$5e,$f4,$7f,$0f,$a1,$87,$90,$43,$25,$64,$b5,$87,$8d,$79,$ac,$7b,$04,$db,$4c,$64,$9b,$2f,$dd,
    $28,$19,$c5,$19,$e4,$95,$39,$9c,$87,$5e,$73,$c4,$2f,$18,$9b,$e5,$38,$b2,$a7,$a3,$8f,$a7,$5d,$c0,$c3,
    $c7,$49,$bd,$d9,$c6,$b8,$2b,$c3,$dd,$c9,$f8,$79,$5c,$08,$74,$ad,$41,$95,$fa,$aa,$97,$52,$df,$78,$f4,
    $69,$6a,$03,$1a,$35,$99,$72,$b0,$cb,$15,$35,$c2,$c3,$b6,$c4,$0d,$10,$86,$9b,$02,$56,$57,$7a,$12,$1d,
    $2e,$e5,$c2,$f0,$32,$ba,$fa,$57,$c6,$3d,$35,$33,$49,$01,$fc,$63,$cb,$50,$2a,$fc,$89,$3a,$bc,$35,$5e,
    $69,$fe,$d2,$ef,$3e,$dc,$e7,$26,$73,$53,$df,$df,$88,$a5,$85,$0e,$e7,$dc,$34,$34,$59,$ed,$f4,$e6,$95,
    $03,$43,$26,$7a,$fe,$a6,$12,$5a,$4a,$37,$37,$fc,$1a,$c3,$27,$d9,$f4,$31,$55,$88,$18,$43,$20,$b6,$9c,
    $0f,$57,$fb,$27,$44,$d5,$32,$52,$45,$b2,$bc,$60,$b1,$84,$b0,$fe,$a1,$51,$7c,$10,$ba,$e4,$d1,$85,$a0,
    $92,$0a,$e6,$4e,$e4,$0d,$d9,$ce,$1e,$e2,$a3,$a9,$2b,$98,$41,$34,$32,$58,$7c,$6f,$0c,$af,$f7,$2b,$a5,
    $f8,$ff,$8f,$d2,$a8,$5f,$24,$95,$04,$18,$2d,$98,$62,$46,$ea,$9d,$45,$d0,$e8,$c1,$0e,$e6,$38,$91,$5d,
    $69,$2b,$7a,$bd,$b8,$ef,$50,$bf,$4b,$0a,$cd,$0f,$c9,$9c,$be,$02,$54,$66,$19,$c8,$1c,$e2,$91,$c4,$b4,
    $b9,$f6,$57,$4b,$c7,$1d,$ad,$63,$30,$16,$fe,$b6,$87,$9a,$64,$d9,$0d,$33,$50,$a9,$1a,$0d,$4e,$48,$d8,
    $07,$9b,$19,$8d,$1c,$ac,$4d,$98,$a5,$a2,$e7,$77,$02,$0f,$be,$7a,$1b,$b9,$c5,$28,$dd,$dd,$25,$e8,$c1,
    $11,$71,$71,$11,$18,$e9,$1c,$3e,$70,$53,$50,$36,$ec,$12,$a5,$37,$f1,$7a,$f8,$b4,$48,$91,$82,$a7,$c9,
    $7d,$56,$b1,$92,$44,$4c,$98,$43,$36,$3f,$41,$35,$0b,$66,$90,$96,$cb,$08,$a9,$44,$96,$9c,$c0,$27,$ad,
    $5e,$ae,$60,$1e,$72,$92,$26,$dc,$41,$76,$a4,$37,$46,$cf,$13,$e2,$41,$e8,$8f,$19,$64,$20,$cf,$bd,$55,
    $47,$bf,$d8,$0c,$aa,$a8,$c7,$f6,$82,$a7,$6b,$9b,$91,$ac,$e5,$14,$f9,$d4,$08,$45,$06,$7d,$bc,$c9,$8d,
    $a8,$f1,$bf,$21,$9f,$e3,$ca,$f8,$52,$ec,$d6,$1a,$c8,$5c,$49,$f0,$55,$78,$ae,$cb,$9b,$62,$6e,$f6,$7e,
    $e8,$16,$f7,$80,$a9,$7d,$38,$e1,$c7,$59,$a7,$19,$5d,$e2,$e3,$76,$55,$c7,$fc,$bf,$39,$7a,$0c,$f3,$c7,
    $4d,$11,$6c,$c9,$63,$0b,$26,$87,$8e,$89,$9a,$37,$c6,$cb,$1a,$f2,$eb,$3e,$89,$e1,$6f,$24,$51,$fb,$d7,
    $33,$17,$1c,$0d,$55,$2f,$aa,$a9,$8d,$25,$91,$cf,$a6,$1f,$2f,$45,$df,$66,$ba,$94,$c4,$b4,$ce,$b5,$51,
    $5e,$24,$2b,$05,$96,$8c,$a5,$e4,$6b,$9f,$4b,$d8,$8b,$b8,$f9,$0f,$3f,$18,$37,$50,$12,$d9,$fa,$fc,$8c,
    $13,$8f,$bf,$04,$98,$78,$95,$cc,$18,$7c,$30,$d9,$f0,$19,$7c,$2a,$fc,$78,$3e,$27,$b4,$46,$20,$bf,$02,
    $28,$e8,$d7,$25,$d1,$59,$1a,$ba,$db,$1a,$ab,$5f,$96,$81,$7f,$f6,$0a,$b8,$ba,$05,$2e,$e9,$89,$2e,$0b,
    $a7,$c3,$52,$79,$ae,$e3,$9e,$e4,$82,$3d,$57,$70,$54,$9d,$fd,$16,$8e,$21,$fb,$5e,$61,$26,$a9,$b5,$12,
    $75,$28,$c6,$a9,$0a,$ba,$0b,$da,$3e,$3e,$90,$7e,$ac,$4b,$3f,$98,$d7,$7e,$13,$ec,$cd,$45,$6e,$43,$1d,
    $df,$37,$fa,$fa,$de,$0e,$b9,$d9,$13,$a2,$fd,$8f,$bc,$c6,$8b,$92,$32,$9e,$b8,$bb,$6a,$c8,$86,$73,$5d,
    $e1,$1e,$2a,$7b,$5b,$fa,$82,$a8,$0d,$96,$d4,$cb,$22,$b0,$c8,$97,$f6,$0d,$04,$9f,$c8,$98,$af,$1b,$24,
    $09,$f2,$99,$9c,$24,$53,$de,$ab,$24,$2d,$b4,$05,$e6,$ee,$a6,$97,$7f,$1f,$cd,$01,$43,$81,$20,$37,$0b,
    $98,$f7,$db,$b7,$e8,$19,$30,$ac,$fe,$f3,$9f,$df,$4b,$4c,$51,$ad,$fd,$e0,$eb,$a6,$05,$3f,$36,$32,$ea,
    $b3,$96,$59,$ff,$46,$21,$ff,$e3,$61,$0a,$5b,$8b,$c4,$e6,$e4,$fc,$f0,$a4,$22,$1b,$97,$4d,$2a,$ef,$97,
    $7f,$77,$79,$80,$ec,$f1,$bd,$56,$01,$c0,$9a,$70,$7b,$9c,$64,$a2,$0d,$ef,$93,$3a,$6d,$f4,$ff,$9b,$84,
    $f7,$25,$ff,$7f,$21,$63,$bd,$f5,$b7,$fa,$c2,$8b,$47,$4f,$fd,$a7,$75,$51,$a9,$48,$b3,$92,$9a,$72,$38,
    $67,$0d,$f8,$78,$ef,$32,$dc,$5a,$b0,$11,$4e,$92,$09,$4c,$85,$7f,$cb,$8d,$b7,$93,$a1,$ec,$f1,$a9,$f9,
    $96,$31,$03,$f8,$60,$9e,$7e,$e4,$43,$bb,$92,$fe,$0d,$49,$25,$ee,$5c,$6b,$be,$1a,$f2,$d2,$d0,$e1,$04,
    $bd,$43,$23,$f8,$39,$cf,$4c,$b9,$cc,$45,$96,$9c,$5c,$90,$d6,$2f,$8d,$c6,$1c,$fe,$0d,$ca,$81,$59,$4b,
    $51,$51,$61,$9e,$14,$3f,$d9,$cc,$2a,$12,$9d,$9a,$83,$2f,$58,$56,$28,$b3,$e4,$f4,$90,$75,$8d,$23,$10,
    $78,$88,$73,$01,$59,$51,$f4,$9c,$73,$b7,$6a,$90,$29,$49,$dd,$6e,$14,$b1,$4c,$ad,$cf,$ff,$e0,$a8,$6a,
    $51,$11,$4b,$a5,$6c,$6a,$0e,$a9,$0d,$5e,$a1,$38,$5b,$09,$3d,$78,$77,$f6,$fd,$0a,$dd,$b6,$3a,$4c,$d7,
    $2c,$9b,$c3,$5f,$39,$30,$89,$84,$bd,$92,$ea,$f2,$e2,$34,$f0,$c2,$01,$76,$8c,$74,$06,$19,$e2,$74,$f4,
    $0f,$5f,$54,$05,$23,$6e,$09,$c7,$47,$09,$68,$91,$62,$78,$63,$22,$9d,$3b,$4d,$1c,$19,$6d,$ee,$cb,$d4,
    $8d,$20,$67,$9b,$9b,$07,$76,$92,$89,$d8,$76,$e8,$6a,$46,$a7,$2f,$3d,$44,$47,$57,$50,$76,$3a,$3a,$5f,
    $24,$59,$37,$ec,$55,$02,$6f,$a1,$4d,$0f,$9d,$b4,$2e,$fb,$3e,$7f,$85,$9c,$e7,$db,$bb,$ed,$ac,$86,$84,
    $39,$85,$f0,$d7,$16,$0c,$92,$69,$49,$67,$7f,$b0,$65,$79,$8a,$48,$1a,$13,$ff,$4b,$30,$f9,$1f,$fd,$17,
    $9c,$11,$ea,$f5,$c7,$43,$64,$39,$c1,$eb,$a7,$b1,$15,$27,$40,$d9,$22,$b5,$95,$fb,$8c,$e6,$b4,$a7,$d2,
    $d9,$c0,$67,$9e,$13,$3d,$c6,$d2,$5c,$50,$2e,$16,$8a,$97,$6d,$c8,$d1,$4f,$1d,$6b,$16,$ca,$1d,$7d,$27,
    $83,$f0,$5d,$8c,$45,$72,$f9,$68,$fd,$c9,$00,$f3,$95,$23,$42,$ee,$f3,$9d,$96,$5d,$43,$6b,$91,$8b,$d9,
    $b4,$5f,$4e,$de,$ae,$7e,$1f,$83,$98,$19,$70,$2b,$20,$15,$ab,$a6,$6b,$d2,$b6,$02,$96,$6f,$b5,$f3,$5d,
    $ee,$5e,$33,$82,$04,$33,$60,$d8,$ec,$80,$dd,$88,$1c,$2d,$82,$41,$f6,$fd,$2b,$dc,$0c,$ad,$4a,$1e,$e3,
    $df,$b8,$f1,$4b,$84,$09,$14,$88,$36,$b9,$2a,$ce,$0a,$23,$6f,$30,$da,$9d,$86,$0a,$9f,$00,$f7,$3d,$d8,
    $e3,$e2,$07,$40,$6a,$e8,$fb,$fa,$fe,$9b,$48,$bb,$81,$1b,$b0,$f5,$96,$ce,$8f,$8f,$9b,$44,$c4,$24,$5e,
    $cd,$53,$73,$19,$cc,$26,$0e,$bd,$4c,$48,$db,$a5,$3b,$67,$ef,$0c,$e7,$22,$09,$dd,$4a,$17,$ea,$e5,$80,
    $cd,$dc,$31,$4b,$8c,$c8,$d4,$26,$f8,$b8,$cb,$73,$f2,$23,$de,$1a,$33,$a1,$10,$ba,$9d,$71,$e7,$d9,$79,
    $f9,$ec,$1a,$81,$0c,$9d,$53,$6f,$f5,$3b,$09,$c0,$08,$aa,$e6,$99,$c1,$99,$28,$47,$9f,$3b,$8e,$d9,$24,
    $a8,$b1,$bd,$7e,$4d,$07,$c0,$c3,$27,$aa,$9e,$e3,$52,$e7,$6a,$b6,$61,$14,$bc,$0c,$31,$aa,$8b,$23,$6d,
    $ae,$c4,$35,$3a,$65,$01,$ca,$f5,$60,$56,$9a,$71,$64,$af,$a9,$56,$90,$57,$6e,$c5,$59,$6d,$50,$22,$16,
    $f8,$05,$d1,$33,$1c,$11,$19,$8d,$4d,$58,$e2,$90,$f7,$fb,$d0,$14,$c8,$13,$61,$5a,$41,$c0,$40,$f1,$bb,
    $97,$fd,$ad,$38,$2a,$06,$aa,$ad,$28,$37,$0e,$42,$67,$db,$8e,$96,$45,$0f,$73,$01,$d8,$f1,$80,$a4,$9c,
    $5d,$74,$fd,$03,$14,$44,$e6,$14,$9a,$14,$e9,$66,$0f,$84,$fc,$e1,$d3,$2c,$8d,$e1,$6c,$1b,$80,$e5,$96,
    $da,$3e,$d4,$8b,$5a,$8b,$67,$44,$2e,$3b,$96,$67,$58,$d3,$43,$88,$d5,$14,$68,$f9,$9a,$2b,$b2,$81,$08,
    $53,$b9,$53,$27,$df,$c1,$24,$22,$5b,$bf,$44,$81,$53,$67,$9f,$5a,$88,$b5,$b7,$36,$f0,$da,$fe,$4e,$db,
    $52,$39,$b1,$b5,$84,$c3,$0d,$f8,$20,$12,$86,$ab,$2f,$66,$20,$1c,$11,$7e,$d3,$66,$80,$b4,$d4,$a9,$40,
    $d8,$ee,$c5,$6d,$23,$99,$62,$b0,$36,$17,$52,$47,$7f,$22,$0c,$49,$ae,$49,$be,$bd,$7c,$ec,$9a,$19,$c1,
    $12,$e0,$72,$c2,$a0,$f1,$9e,$73,$87,$eb,$9b,$75,$0f,$f0,$7b,$9e,$18,$45,$ab,$ed,$e0,$2a,$1c,$eb,$d8,
    $29,$8c,$86,$88,$24,$70,$b9,$b7,$58,$41,$95,$21,$c9,$ec,$f4,$e2,$1e,$37,$92,$c8,$44,$7d,$cd,$b5,$95,
    $70,$c8,$51,$11,$83,$a7,$14,$ea,$be,$ca,$5f,$ea,$c3,$49,$06,$9d,$29,$15,$00,$a1,$b4,$88,$2c,$38,$2d,
    $57,$55,$c5,$57,$39,$63,$ca,$5d,$66,$a0,$82,$c7,$e7,$e5,$4d,$f8,$ef,$3b,$98,$d6,$77,$f7,$a2,$1b,$ce,
    $e3,$c0,$45,$88,$13,$0b,$1b,$88,$68,$81,$b5,$15,$d5,$ec,$ec,$09,$db,$05,$3f,$bd,$2a,$32,$a2,$db,$10,
    $47,$5d,$29,$20,$65,$25,$33,$0f,$23,$50,$e2,$31,$a4,$7f,$fd,$d9,$99,$db,$6a,$c1,$52,$98,$2f,$dd,$bb,
    $50,$5f,$a3,$05,$fc,$35,$6b,$18,$40,$f9,$8c,$e7,$8f,$42,$19,$0b,$1c,$ec,$90,$54,$81,$b9,$34,$9a,$2a,
    $e8,$a0,$5f,$4a,$fd,$39,$08,$57,$0e,$15,$98,$69,$f7,$9b,$96,$76,$6e,$6a,$bf,$cd,$86,$87,$ba,$eb,$00,
    $e6,$a9,$a7,$80,$bd,$2d,$e8,$76,$02,$ea,$41,$42,$a4,$10,$cd,$10,$f5,$40,$10,$7a,$8d,$9f,$eb,$f9,$75,
    $74,$fa,$f3,$ec,$4c,$74,$77,$15,$4e,$37,$4b,$53,$6c,$56,$be,$55,$11,$f2,$78,$26,$61,$5f,$f2,$02,$27,
    $0b,$76,$64,$79,$fa,$7b,$3c,$39,$b4,$a0,$ac,$a4,$6d,$ba,$b4,$c8,$76,$a0,$c2,$e5,$2f,$cd,$c6,$81,$35,
    $e6,$20,$50,$63,$87,$48,$fc,$bf,$87,$f5,$59,$ed,$89,$d7,$fd,$05,$16,$8d,$18,$72,$8e,$f2,$19,$62,$56,
    $37,$47,$7b,$74,$ff,$f0,$07,$e0,$f2,$da,$be,$97,$a8,$90,$b2,$19,$d1,$ca,$0e,$7f,$18,$99,$76,$82,$02,
    $27,$c2,$2b,$a6,$ba,$6b,$52,$5b,$21,$c0,$58,$38,$67,$b8,$4b,$80,$3a,$12,$d5,$df,$99,$fd,$51,$10,$60,
    $ee,$f1,$a5,$0e,$12,$25,$70,$0d,$e5,$44,$97,$fb,$cf,$b5,$ef,$55,$39,$3f,$31,$00,$71,$ce,$58,$76,$a7,
    $ee,$74,$62,$ba,$39,$91,$8d,$33,$36,$da,$2e,$10,$7d,$12,$4b,$31,$69,$ed,$1a,$56,$42,$b3,$97,$45,$92,
    $65,$97,$da,$a5,$8d,$1a,$fb,$7e,$b1,$c4,$18,$12,$1e,$a1,$b5,$e2,$b4,$93,$7c,$4b,$1d,$e6,$64,$58,$0a,
    $07,$fd,$3d,$9b,$3c,$e6,$1c,$73,$77,$fa,$16,$fb,$08,$92,$b2,$8b,$94,$0e,$cc,$69,$fc,$f5,$f1,$00,$67,
    $d9,$68,$1d,$f2,$ea,$e4,$75,$a7,$4f,$4d,$5f,$8b,$75,$27,$ef,$98,$86,$45,$ca,$90,$b9,$56,$75,$81,$29,
    $87,$b7,$19,$eb,$f1,$8d,$41,$ee,$e9,$cd,$2b,$d1,$90,$17,$7f,$c0,$38,$15,$63,$6a,$64,$28,$b1,$d1,$83,
    $36,$2b,$e1,$91,$ec,$f6,$7f,$f4,$07,$6f,$c7,$bb,$b2,$57,$b9,$1b,$84,$52,$d8,$f6,$24,$d0,$d0,$97,$af,
    $9e,$6e,$68,$f6,$43,$d9,$e5,$62,$6f,$2e,$3c,$e7,$0e,$bf,$77,$a9,$10,$6c,$4e,$bb,$4f,$44,$a7,$4b,$6f,
    $8c,$ea,$7f,$bf,$5c,$9d,$e7,$60,$77,$1c,$99,$3e,$c8,$85,$10,$97,$71,$ba,$77,$70,$9d,$28,$d8,$ca,$44,
    $5d,$02,$10,$b0,$df,$1d,$2c,$0c,$ce,$83,$a0,$5d,$5c,$dc,$9a,$46,$62,$bc,$7a,$b3,$41,$22,$f3,$bf,$31,
    $88,$91,$cd,$ea,$d8,$23,$ac,$cb,$9f,$d5,$0e,$3e,$e3,$42,$9e,$7e,$1d,$ea,$6a,$c9,$88,$4d,$4a,$23,$5c,
    $2c,$fd,$37,$2a,$f1,$f6,$b1,$0d,$a8,$ec,$62,$f6,$9b,$43,$88,$1f,$2e,$e8,$b3,$d0,$df,$ac,$dd,$3e,$51,
    $63,$5d,$cc,$66,$3d,$e1,$44,$6d,$c1,$09,$c1,$6e,$a9,$40,$ed,$9d,$44,$fe,$1e,$b7,$a3,$04,$46,$a4,$3b,
    $71,$ab,$23,$1d,$19,$87,$41,$46,$75,$da,$3d,$7c,$c4,$1e,$89,$bf,$7a,$1e,$71,$c8,$f7,$0c,$9a,$b4,$8b,
    $c9,$e8,$97,$27,$54,$a6,$5d,$f7,$c3,$73,$ee,$f0,$4f,$57,$40,$78,$83,$11,$09,$65,$22,$06,$b6,$01,$ea,
    $b4,$0b,$18,$93,$6c,$d9,$df,$59,$af,$47,$ac,$66,$a2,$44,$65,$c0,$94,$15,$bd,$c2,$26,$c4,$53,$ad,$25,
    $90,$ed,$a4,$c4,$3f,$4f,$c3,$3e,$f0,$72,$36,$a8,$5b,$d3,$ef,$60,$97,$17,$e0,$44,$a7,$b7,$46,$b0,$08,
    $c1,$91,$1c,$7d,$0c,$1c,$f7,$68,$92,$88,$82,$d5,$cc,$e5,$9b,$e8,$ed,$44,$ce,$98,$20,$2e,$75,$2b,$26,
    $f2,$2e,$66,$56,$0e,$4e,$0c,$aa,$f6,$27,$33,$a9,$61,$6c,$18,$1e,$78,$b1,$87,$ee,$e5,$3b,$8f,$1e,$ec,
    $52,$47,$42,$a5,$42,$71,$0f,$18,$4f,$57,$dd,$59,$85,$90,$77,$d9,$d3,$62,$0a,$06,$f0,$39,$54,$df,$c5,
    $02,$c8,$39,$61,$50,$2d,$8e,$bb,$3a,$4b,$cf,$37,$9c,$50,$c7,$bf,$5d,$35,$2b,$98,$f1,$0f,$ce,$b1,$38,
    $18,$9d,$0c,$1b,$48,$a1,$ce,$82,$5b,$22,$3c,$09,$3f,$67,$4b,$94,$d0,$6d,$14,$43,$65,$48,$e3,$cf,$69,
    $62,$64,$cd,$37,$86,$8f,$7d,$c4,$1d,$fc,$9c,$f4,$bc,$d6,$4e,$26,$33,$6f,$b9,$47,$03,$42,$0e,$ff,$53,
    $ee,$42,$58,$c5,$0c,$c5,$c2,$6d,$6e,$37,$28,$98,$c3,$55,$2d,$22,$37,$37,$52,$08,$45,$25,$22,$5d,$15,
    $14,$d4,$ec,$dd,$68,$0f,$b3,$bf,$a5,$8f,$29,$2f,$d0,$a0,$fd,$0a,$1a,$cf,$9a,$3a,$13,$a4,$83,$d4,$7b,
    $ce,$04,$5d,$c4,$4e,$c5,$93,$24,$f8,$57,$32,$8c,$f7,$5e,$df,$d7,$c1,$d1,$65,$27,$44,$45,$d8,$1e,$70,
    $bb,$db,$ce,$97,$15,$7a,$01,$bf,$1d,$91,$20,$38,$78,$7f,$c7,$ba,$72,$ac,$9a,$a9,$dd,$d1,$a9,$7f,$0a,
    $60,$f9,$ba,$24,$8e,$01,$2d,$f4,$3f,$1f,$ce,$1f,$8a,$5a,$da,$73,$7d,$99,$32,$89,$ca,$d1,$05,$c2,$a2,
    $dd,$56,$3f,$f2,$11,$bf,$32,$f2,$b5,$8a,$4b,$2a,$5f,$44,$c2,$9b,$c4,$ca,$be,$6d,$07,$26,$b4,$c7,$44,
    $33,$9f,$49,$38,$33,$a4,$14,$07,$19,$49,$7c,$5b,$73,$57,$57,$12,$e7,$7c,$18,$f7,$c9,$ca,$78,$92,$ba,
    $fe,$0a,$34,$e7,$ff,$ed,$81,$8a,$2e,$27,$86,$91,$89,$d3,$e8,$66,$20,$0c,$db,$f0,$9f,$01,$77,$79,$d4,
    $19,$bc,$bc,$17,$c2,$0b,$1e,$8f,$be,$2e,$e1,$a5,$9f,$bb,$d1,$c3,$31,$d7,$8b,$83,$68,$28,$eb,$e8,$12,
    $4c,$30,$b1,$48,$8a,$51,$07,$a2,$80,$1a,$a2,$ac,$0a,$48,$a0,$71,$8e,$13,$7a,$66,$02,$27,$c3,$9c,$40,
    $f0,$ac,$c3,$87,$f1,$e5,$77,$6f,$cd,$a1,$60,$6d,$71,$0c,$51,$b3,$00,$99,$60,$c5,$ac,$b5,$ea,$19,$bb,
    $67,$5d,$84,$54,$30,$f8,$62,$b8,$bf,$48,$df,$ce,$6a,$79,$38,$f1,$b9,$7c,$56,$b6,$a9,$cb,$45,$fe,$d5,
    $49,$64,$89,$1c,$2b,$a8,$77,$c4,$d6,$47,$0c,$79,$b5,$67,$15,$4c,$92,$c0,$da,$cb,$82,$68,$f5,$99,$dd,
    $cd,$28,$bd,$ee,$11,$80,$63,$51,$c6,$a3,$99,$e6,$0f,$07,$2b,$52,$2a,$a4,$e8,$7f,$dc,$71,$1e,$48,$8f,
    $8e,$ee,$5e,$19,$a3,$cf,$9d,$bb,$0f,$00,$02,$df,$ec,$a2,$da,$63,$85,$f8,$49,$8e,$ee,$29,$56,$0b,$73,
    $62,$e3,$95,$3c,$93,$2f,$96,$3e,$0d,$96,$88,$ac,$4c,$d8,$e8,$a5,$26,$fd,$2b,$7b,$3d,$a1,$cf,$02,$e4,
    $f8,$80,$eb,$b7,$be,$ba,$70,$d1,$be,$9f,$a0,$1e,$4f,$cc,$2c,$ee,$3b,$86,$f7,$94,$72,$8e,$98,$2d,$b4,
    $39,$e2,$65,$3b,$78,$45,$a0,$6c,$10,$10,$49,$aa,$b2,$e9,$4c,$ba,$96,$5a,$51,$6a,$eb,$1b,$94,$c6,$d1,
    $ff,$af,$53,$6a,$de,$29,$b2,$11,$6b,$b8,$64,$00,$52,$aa,$1c,$fe,$92,$c8,$4a,$30,$e3,$a4,$b7,$db,$d5,
    $a4,$d0,$0f,$2e,$b3,$dd,$bb,$59,$43,$46,$70,$44,$3d,$ba,$8d,$85,$ce,$04,$22,$c9,$ba,$a0,$06,$d1,$db,
    $eb,$69,$15,$77,$d0,$13,$dc,$cf,$be,$92,$9e,$9a,$e5,$8f,$08,$c3,$c6,$ce,$5b,$d0,$33,$cb,$47,$be,$3a,
    $d3,$c1,$80,$42,$60,$28,$ec,$70,$f0,$6d,$49,$10,$9b,$2d,$ae,$02,$cb,$7a,$07,$f5,$3f,$aa,$65,$22,$4f,
    $7b,$c6,$ae,$fe,$1d,$1d,$82,$55,$be,$c1,$19,$c6,$13,$dd,$a1,$65,$39,$a2,$69,$8f,$46,$0c,$1b,$0c,$6e,
    $b2,$23,$b9,$c5,$2a,$8c,$76,$7a,$3d,$e1,$74,$34,$36,$01,$31,$81,$36,$3e,$01,$42,$2c,$d6,$11,$d1,$59,
    $59,$41,$83,$5f,$7d,$94,$f0,$a8,$5c,$3a,$ae,$05,$1d,$d9,$f5,$60,$de,$65,$f2,$f6,$a0,$bb,$8c,$e3,$5d,
    $63,$8d,$d9,$8e,$32,$c6,$b3,$cb,$c1,$e6,$94,$60,$1d,$7b,$43,$80,$7f,$08,$11,$29,$af,$e6,$54,$54,$5b,
    $2c,$ff,$ba,$0d,$a2,$37,$0c,$10,$63,$91,$3b,$e8,$c0,$96,$60,$ef,$47,$2c,$79,$89,$8b,$de,$10,$7f,$d8,
    $22,$c5,$31,$62,$0c,$b3,$5b,$31,$c6,$9d,$64,$58,$de,$ab,$23,$cc,$2f,$85,$e0,$90,$b5,$9a,$5a,$89,$43,
    $07,$2a,$be,$ad,$c8,$33,$55,$c2,$e5,$8f,$00,$8a,$04,$89,$91,$94,$09,$fb,$f5,$58,$eb,$ad,$0e,$de,$15,
    $29,$97,$74,$33,$7d,$12,$bd,$59,$e9,$34,$bf,$be,$99,$b2,$fc,$af,$e7,$cc,$63,$f4,$8d,$51,$ca,$98,$bb,
    $dd,$34,$50,$6e,$23,$42,$0c,$f1,$9f,$04,$f4,$06,$ff,$90,$84,$ef,$c4,$9f,$8c,$80,$53,$bf,$f7,$ff,$e7,
    $85,$35,$36,$a1,$2c,$25,$80,$c8,$d3,$76,$f9,$e9,$64,$02,$92,$92,$0d,$16,$f2,$44,$0e,$ea,$24,$fd,$4e,
    $65,$04,$ad,$54,$8c,$67,$25,$53,$1c,$11,$2c,$d6,$be,$79,$c4,$ca,$5c,$57,$1a,$7a,$dd,$a6,$d0,$52,$34,
    $fe,$92,$83,$8d,$90,$8f,$aa,$39,$e4,$37,$63,$08,$10,$e1,$5d,$bd,$f1,$f4,$f8,$e4,$f5,$29,$07,$63,$ca,
    $92,$59,$44,$99,$d7,$a2,$e3,$b7,$81,$ec,$7c,$b6,$a0,$d4,$c2,$d8,$39,$73,$4f,$f2,$af,$80,$56,$3b,$26,
    $cc,$76,$07,$31,$af,$84,$e9,$a7,$f2,$5b,$cf,$0a,$dc,$92,$9c,$2b,$d3,$16,$c1,$e1,$c6,$f8,$95,$6c,$1f,
    $3a,$97,$d8,$76,$a1,$35,$12,$bc,$8e,$af,$22,$f4,$2e,$3a,$db,$0f,$87,$a9,$68,$4b,$44,$1b,$93,$91,$e8,
    $e9,$65,$9c,$0c,$4e,$4f,$8c,$87,$11,$e1,$45,$ed,$61,$d8,$26,$53,$17,$d5,$c7,$7f,$ad,$e3,$d3,$2c,$70,
    $5f,$0c,$75,$0a,$be,$8b,$bd,$72,$ef,$b7,$a7,$31,$b7,$90,$e3,$02,$d6,$b3,$52,$c4,$26,$66,$fa,$16,$43,
    $1d,$05,$be,$88,$af,$c9,$0d,$0f,$ec,$11,$77,$ef,$5b,$8a,$3c,$f7,$b5,$0c,$c8,$ee,$7b,$fd,$8f,$19,$6c,
    $80,$47,$19,$7d,$5a,$30,$6e,$2d,$e5,$56,$53,$2a,$21,$61,$89,$21,$0f,$0c,$39,$d4,$bd,$42,$11,$ee,$76,
    $4a,$04,$0e,$95,$02,$f0,$2e,$38,$21,$86,$75,$34,$b8,$03,$25,$5d,$ad,$d3,$6e,$e7,$63,$45,$7f,$23,$ea,
    $90,$f1,$99,$5d,$a7,$30,$05,$f2,$a7,$60,$d2,$63,$31,$90,$06,$8d,$2d,$42,$0d,$4d,$05,$e5,$1f,$2e,$1c,
    $60,$43,$e9,$90,$15,$2a,$ea,$d6,$44,$59,$42,$25,$77,$68,$50,$b0,$76,$cf,$3b,$3c,$09,$dc,$ff,$71,$8f,
    $26,$9a,$02,$3d,$77,$0e,$f8,$b1,$20,$1e,$30,$74,$ef,$f4,$bd,$92,$fd,$70,$f6,$e8,$8d,$38,$d0,$9c,$7d,
    $7e,$0d,$9f,$36,$f6,$52,$9f,$57,$0d,$dc,$a5,$db,$5d,$e3,$96,$e1,$17,$2d,$b3,$3c,$b9,$68,$64,$09,$35,
    $5c,$9f,$65,$38,$17,$f7,$c9,$ff,$11,$8e,$f3,$d4,$56,$0b,$00,$e3,$a3,$83,$5d,$aa,$60,$bc,$ba,$34,$d6,
    $29,$49,$b8,$2f,$5d,$ee,$3b,$25,$cd,$f5,$39,$b7,$00,$ed,$c4,$9b,$11,$f0,$88,$5a,$9d,$42,$16,$48,$dc,
    $81,$48,$71,$36,$03,$3f,$44,$0e,$d9,$c9,$90,$15,$22,$38,$53,$ff,$d4,$fb,$c1,$95,$1d,$86,$bb,$5c,$81,
    $4c,$c4,$f9,$97,$e1,$10,$f1,$ac,$c4,$ab,$d9,$69,$d6,$ad,$d4,$43,$34,$8f,$3b,$2f,$a8,$a1,$41,$f6,$0b,
    $17,$8c,$8c,$32,$e3,$b0,$80,$47,$dd,$f1,$7c,$98,$21,$97,$81,$98,$8f,$98,$19,$2d,$a7,$fa,$db,$07,$72,
    $42,$a3,$0c,$1f,$c7,$7e,$56,$24,$43,$09,$55,$54,$b5,$45,$12,$ab,$94,$61,$1c,$1f,$e8,$9e,$82,$15,$2b,
    $11,$e6,$ac,$43,$c9,$77,$72,$b8,$9c,$04,$b2,$e3,$a5,$51,$cb,$cf,$62,$38,$68,$6b,$72,$c5,$1d,$5c,$06,
    $a7,$6a,$92,$5e,$a0,$9e,$3e,$bc,$1f,$52,$67,$96,$05,$6f,$ca,$59,$2f,$23,$b8,$59,$68,$b3,$75,$2d,$c2,
    $2d,$39,$58,$75,$04,$e9,$e2,$04,$65,$ea,$0c,$f5,$2b,$13,$64,$bb,$96,$41,$47,$99,$98,$3f,$cf,$72,$8d,
    $93,$a7,$f2,$0d,$8e,$49,$9b,$9d,$12,$8a,$79,$9c,$5b,$52,$fb,$fc,$14,$78,$94,$60,$4e,$b3,$50,$48,$a9,
    $c0,$90,$0f,$0c,$cb,$fb,$c0,$cc,$fc,$75,$64,$91,$2b,$fb,$b0,$54,$96,$fe,$c0,$5f,$d2,$f2,$88,$bc,$66,
    $7d,$43,$ca,$75,$ed,$25,$a6,$da,$6d,$aa,$8d,$29,$a5,$6c,$34,$0d,$d6,$94,$f6,$81,$74,$29,$b6,$ef,$03,
    $6b,$94,$66,$85,$9a,$a2,$20,$b2,$2b,$64,$76,$71,$2a,$92,$ff,$43,$59,$20,$87,$04,$e3,$b0,$4a,$76,$84,
    $e1,$eb,$62,$78,$0b,$69,$7f,$ef,$a3,$fc,$68,$bc,$5b,$fd,$78,$1b,$18,$35,$7a,$29,$7e,$d3,$9b,$7e,$dc,
    $48,$21,$2d,$ed,$fa,$c4,$2c,$ed,$6e,$b9,$ba,$d0,$9b,$e8,$05,$b4,$a0,$4e,$fb,$2f,$84,$ba,$3a,$da,$9f,
    $26,$37,$33,$c3,$f6,$2d,$03,$dc,$52,$27,$9f,$6c,$d7,$bd,$dd,$da,$24,$b7,$af,$8d,$38,$3b,$ca,$96,$a4,
    $59,$d6,$a9,$0b,$1d,$f5,$17,$e7,$51,$61,$24,$83,$6c,$92,$1f,$5b,$5c,$c3,$82,$01,$c6,$34,$42,$5b,$b2,
    $9f,$1b,$a9,$68,$e0,$be,$2e,$91,$41,$3f,$4c,$ca,$bb,$2f,$45,$5a,$9d,$24,$22,$27,$05,$fa,$d7,$92,$e6,
    $6a,$85,$6b,$28,$a9,$0a,$d1,$07,$41,$ca,$c9,$25,$f0,$b6,$d5,$d1,$89,$41,$66,$bf,$39,$29,$6d,$66,$56,
    $a2,$f4,$6c,$ea,$45,$4f,$75,$ba,$07,$15,$54,$53,$49,$e3,$94,$96,$8f,$05,$48,$54,$89,$ac,$a1,$62,$6a,
    $13,$bc,$13,$37,$e8,$14,$8f,$ae,$99,$4e,$4d,$54,$a8,$e1,$7a,$0d,$81,$1e,$c3,$25,$5e,$b7,$ae,$eb,$9c,
    $5a,$d1,$3e,$7c,$9e,$89,$d3,$ec,$ce,$29,$4b,$4c,$b0,$75,$75,$fa,$5a,$84,$34,$ff,$08,$36,$34,$c9,$91,
    $f3,$ed,$4f,$cb,$40,$50,$2a,$b9,$c3,$4b,$92,$9a,$30,$fc,$41,$f9,$56,$6e,$52,$f2,$68,$03,$cf,$92,$71,
    $a1,$10,$72,$e8,$17,$23,$3a,$d3,$f9,$65,$3c,$c3,$bd,$66,$e2,$fe,$42,$5e,$88,$b4,$6a,$82,$0f,$fe,$11,
    $c5,$ed,$5d,$37,$ca,$8a,$9d,$69,$28,$11,$56,$53,$23,$09,$44,$cd,$84,$46,$00,$1c,$51,$64,$02,$40,$72,
    $f7,$36,$5d,$08,$cb,$35,$63,$58,$a4,$03,$69,$87,$5a,$cf,$80,$8f,$a7,$a3,$51,$0c,$3f,$0b,$bf,$23,$2d,
    $ed,$2c,$d9,$58,$b3,$fd,$95,$31,$68,$c1,$df,$29,$58,$72,$0e,$98,$99,$dc,$ae,$4b,$91,$ed,$cd,$ea,$18,
    $e6,$50,$c0,$90,$5d,$75,$8d,$63,$1b,$6a,$08,$82,$8d,$04,$d6,$00,$f9,$44,$86,$67,$42,$84,$a9,$5a,$80,
    $6c,$10,$1d,$6a,$eb,$03,$3e,$e1,$93,$a4,$ae,$73,$d1,$d5,$0f,$4c,$8a,$83,$6f,$34,$db,$c1,$40,$9b,$8d,
    $1e,$f5,$6f,$16,$c4,$dc,$f3,$72,$3c,$45,$fd,$15,$ba,$93,$75,$f7,$f6,$a6,$47,$a0,$58,$f9,$eb,$eb,$65,
    $95,$77,$61,$3b,$41,$1b,$7f,$2f,$ec,$5a,$bb,$ed,$71,$42,$50,$58,$8b,$5e,$d2,$ce,$b2,$ca,$eb,$de,$65,
    $13,$b3,$f3,$9c,$e1,$3a,$df,$f9,$34,$24,$74,$41,$de,$0f,$e0,$74,$53,$36,$f2,$95,$24,$75,$56,$2a,$7b,
    $bf,$37,$92,$6d,$bb,$3b,$3a,$f3,$1b,$b1,$5f,$78,$f2,$af,$b3,$cb,$4f,$12,$52,$19,$c0,$a9,$bf,$e0,$49,
    $74,$13,$a5,$d1,$e3,$5c,$e4,$7e,$b0,$8f,$a8,$c4,$90,$79,$8a,$38,$68,$80,$06,$58,$75,$4d,$85,$8a,$d5,
    $5f,$8b,$96,$9d,$00,$22,$2d,$17,$3c,$44,$8f,$56,$6f,$3e,$52,$5d,$5f,$b3,$4c,$4b,$39,$fc,$0f,$81,$8c,
    $56,$83,$b7,$7a,$7e,$00,$ac,$1e,$c0,$8c,$03,$30,$bb,$cf,$ce,$cf,$ae,$55,$ee,$58,$4d,$30,$99,$16,$6c,
    $61,$74,$d6,$75,$a1,$18,$78,$06,$8c,$e0,$47,$21,$8a,$8e,$48,$28,$8e,$2b,$93,$7e,$25,$06,$78,$a4,$78
  {$endif}
  );

{---------------------------------------------------------------------------}
function AddrLn10Bytes: pointer;
  {-Pointer to bits of ln(10), big endian format}
begin
  AddrLn10Bytes := @L10Bytes;
end;


{---------------------------------------------------------------------------}
{---------------------------------------------------------------------------}
const
  EBytes: packed array[1..MAX_TCBITS div 8] of byte = ( {calculated with t_ciexp1}
    $ad,$f8,$54,$58,$a2,$bb,$4a,$9a,$af,$dc,$56,$20,$27,$3d,$3c,$f1,$d8,$b9,$c5,$83,$ce,$2d,$36,$95,$a9,
    $e1,$36,$41,$14,$64,$33,$fb,$cc,$93,$9d,$ce,$24,$9b,$3e,$f9,$7d,$2f,$e3,$63,$63,$0c,$75,$d8,$f6,$81,
    $b2,$02,$ae,$c4,$61,$7a,$d3,$df,$1e,$d5,$d5,$fd,$65,$61,$24,$33,$f5,$1f,$5f,$06,$6e,$d0,$85,$63,$65,
    $55,$3d,$ed,$1a,$f3,$b5,$57,$13,$5e,$7f,$57,$c9,$35,$98,$4f,$0c,$70,$e0,$e6,$8b,$77,$e2,$a6,$89,$da,
    $f3,$ef,$e8,$72,$1d,$f1,$58,$a1,$36,$ad,$e7,$35,$30,$ac,$ca,$4f,$48,$3a,$79,$7a,$bc,$0a,$b1,$82,$b3,
    $24,$fb,$61,$d1,$08,$a9,$4b,$b2,$c8,$e3,$fb,$b9,$6a,$da,$b7,$60,$d7,$f4,$68,$1d,$4f,$42,$a3,$de,$39,
    $4d,$f4,$ae,$56,$ed,$e7,$63,$72,$bb,$19,$0b,$07,$a7,$c8,$ee,$0a,$6d,$70,$9e,$02,$fc,$e1,$cd,$f7,$e2,
    $ec,$c0,$34,$04,$cd,$28,$34,$2f,$61,$91,$72,$fe,$9c,$e9,$85,$83,$ff,$8e,$4f,$12,$32,$ee,$f2,$81,$83,
    $c3,$fe,$3b,$1b,$4c,$6f,$ad,$73,$3b,$b5,$fc,$bc,$2e,$c2,$20,$05,$c5,$8e,$f1,$83,$7d,$16,$83,$b2,$c6,
    $f3,$4a,$26,$c1,$b2,$ef,$fa,$88,$6b,$42,$38,$61,$1f,$cf,$dc,$de,$35,$5b,$3b,$65,$19,$03,$5b,$bc,$34,
    $f4,$de,$f9,$9c,$02,$38,$61,$b4,$6f,$c9,$d6,$e6,$c9,$07,$7a,$d9,$1d,$26,$91,$f7,$f7,$ee,$59,$8c,$b0,
    $fa,$c1,$86,$d9,$1c,$ae,$fe,$13,$09,$85,$13,$92,$70,$b4,$13,$0c,$93,$bc,$43,$79,$44,$f4,$fd,$44,$52,
    $e2,$d7,$4d,$d3,$64,$f2,$e2,$1e,$71,$f5,$4b,$ff,$5c,$ae,$82,$ab,$9c,$9d,$f6,$9e,$e8,$6d,$2b,$c5,$22,
    $36,$3a,$0d,$ab,$c5,$21,$97,$9b,$0d,$ea,$da,$1d,$bf,$9a,$42,$d5,$c4,$48,$4e,$0a,$bc,$d0,$6b,$fa,$53,
    $dd,$ef,$3c,$1b,$20,$ee,$3f,$d5,$9d,$7c,$25,$e4,$1d,$2b,$66,$9e,$1e,$f1,$6e,$6f,$52,$c3,$16,$4d,$f4,
    $fb,$79,$30,$e9,$e4,$e5,$88,$57,$b6,$ac,$7d,$5f,$42,$d6,$9f,$6d,$18,$77,$63,$cf,$1d,$55,$03,$40,$04,
    $87,$f5,$5b,$a5,$7e,$31,$cc,$7a,$71,$35,$c8,$86,$ef,$b4,$31,$8a,$ed,$6a,$1e,$01,$2d,$9e,$68,$32,$a9,
    $07,$60,$0a,$91,$81,$30,$c4,$6d,$c7,$78,$f9,$71,$ad,$00,$38,$09,$29,$99,$a3,$33,$cb,$8b,$7a,$1a,$1d,
    $b9,$3d,$71,$40,$00,$3c,$2a,$4e,$ce,$a9,$f9,$8d,$0a,$cc,$0a,$82,$91,$cd,$ce,$c9,$7d,$cf,$8e,$c9,$b5,
    $5a,$7f,$88,$a4,$6b,$4d,$b5,$a8,$51,$f4,$41,$82,$e1,$c6,$8a,$00,$7e,$5e,$0d,$d9,$02,$0b,$fd,$64,$b6,
    $45,$03,$6c,$7a,$4e,$67,$7d,$2c,$38,$53,$2a,$3a,$23,$ba,$44,$42,$ca,$f5,$3e,$a6,$3b,$b4,$54,$32,$9b,
    $76,$24,$c8,$91,$7b,$dd,$64,$b1,$c0,$fd,$4c,$b3,$8e,$8c,$33,$4c,$70,$1c,$3a,$cd,$ad,$06,$57,$fc,$cf,
    $ec,$71,$9b,$1f,$5c,$3e,$4e,$46,$04,$1f,$38,$81,$47,$fb,$4c,$fd,$b4,$77,$a5,$24,$71,$f7,$a9,$a9,$69,
    $10,$b8,$55,$32,$2e,$db,$63,$40,$d8,$a0,$0e,$f0,$92,$35,$05,$11,$e3,$0a,$be,$c1,$ff,$f9,$e3,$a2,$6e,
    $7f,$b2,$9f,$8c,$18,$30,$23,$c3,$58,$7e,$38,$da,$00,$77,$d9,$b4,$76,$3e,$4e,$4b,$94,$b2,$bb,$c1,$94,
    $c6,$65,$1e,$77,$ca,$f9,$92,$ee,$aa,$c0,$23,$2a,$28,$1b,$f6,$b3,$a7,$39,$c1,$22,$61,$16,$82,$0a,$e8,
    $db,$58,$47,$a6,$7c,$be,$f9,$c9,$09,$1b,$46,$2d,$53,$8c,$d7,$2b,$03,$74,$6a,$e7,$7f,$5e,$62,$29,$2c,
    $31,$15,$62,$a8,$46,$50,$5d,$c8,$2d,$b8,$54,$33,$8a,$e4,$9f,$52,$35,$c9,$5b,$91,$17,$8c,$cf,$2d,$d5,
    $ca,$ce,$f4,$03,$ec,$9d,$18,$10,$c6,$27,$2b,$04,$5b,$3b,$71,$f9,$dc,$6b,$80,$d6,$3f,$dd,$4a,$8e,$9a,
    $db,$1e,$69,$62,$a6,$95,$26,$d4,$31,$61,$c1,$a4,$1d,$57,$0d,$79,$38,$da,$d4,$a4,$0e,$32,$9c,$cf,$f4,
    $6a,$aa,$36,$ad,$00,$4c,$f6,$00,$c8,$38,$1e,$42,$5a,$31,$d9,$51,$ae,$64,$fd,$b2,$3f,$ce,$c9,$50,$9d,
    $43,$68,$7f,$eb,$69,$ed,$d1,$cc,$5e,$0b,$8c,$c3,$bd,$f6,$4b,$10,$ef,$86,$b6,$31,$42,$a3,$ab,$88,$29,
    $55,$5b,$2f,$74,$7c,$93,$26,$65,$cb,$2c,$0f,$1c,$c0,$1b,$d7,$02,$29,$38,$88,$39,$d2,$af,$05,$e4,$54,
    $50,$4a,$c7,$8b,$75,$82,$82,$28,$46,$c0,$ba,$35,$c3,$5f,$5c,$59,$16,$0c,$c0,$46,$fd,$82,$51,$54,$1f,
    $c6,$8c,$9c,$86,$b0,$22,$bb,$70,$99,$87,$6a,$46,$0e,$74,$51,$a8,$a9,$31,$09,$70,$3f,$ee,$1c,$21,$7e,
    $6c,$38,$26,$e5,$2c,$51,$aa,$69,$1e,$0e,$42,$3c,$fc,$99,$e9,$e3,$16,$50,$c1,$21,$7b,$62,$48,$16,$cd,
    $ad,$9a,$95,$f9,$d5,$b8,$01,$94,$88,$d9,$c0,$a0,$a1,$fe,$30,$75,$a5,$77,$e2,$31,$83,$f8,$1d,$4a,$3f,
    $2f,$a4,$57,$1e,$fc,$8c,$e0,$ba,$8a,$4f,$e8,$b6,$85,$5d,$fe,$72,$b0,$a6,$6e,$de,$d2,$fb,$ab,$fb,$e5,
    $8a,$30,$fa,$fa,$be,$1c,$5d,$71,$a8,$7e,$2f,$74,$1e,$f8,$c1,$fe,$86,$fe,$a6,$bb,$fd,$e5,$30,$67,$7f,
    $0d,$97,$d1,$1d,$49,$f7,$a8,$44,$3d,$08,$22,$e5,$06,$a9,$f4,$61,$4e,$01,$1e,$2a,$94,$83,$8f,$f8,$8c,
    $d6,$8c,$8b,$b7,$c5,$1e,$ef,$6d,$49,$ea,$8a,$b4,$f2,$c3,$df,$5b,$b4,$e0,$73,$5a,$b0,$d6,$87,$49,$2f,
    $e2,$6d,$d4,$06,$58,$16,$bb,$a7,$7e,$ae,$97,$3f,$3b,$40,$ce,$e8,$84,$0a,$82,$f6,$f8,$ed,$27,$5c,$9c,
    $be,$27,$82,$74,$02,$39,$75,$6f,$66,$48,$e4,$d8,$a1,$87,$ad,$09,$8a,$5f,$16,$10,$4e,$5d,$45,$51,$cf,
    $3c,$a8,$f5,$b7,$96,$31,$2e,$c6,$e4,$6b,$65,$ea,$fc,$0a,$3a,$59,$97,$69,$3b,$3a,$03,$77,$04,$f8,$37,
    $d0,$c8,$bb,$68,$3f,$4e,$26,$c5,$d7,$a3,$44,$34,$23,$14,$8c,$29,$ad,$5d,$b2,$53,$f1,$4a,$d3,$9d,$2c,
    $b8,$d0,$83,$a4,$0b,$20,$bb,$9e,$ba,$e0,$17,$0b,$ad,$d9,$45,$fa,$af,$fe,$e1,$dd,$cd,$e3,$0e,$01,$b2,
    $47,$41,$fc,$42,$75,$9d,$49,$f0,$61,$5a,$0d,$12,$05,$53,$5a,$48,$58,$41,$d5,$6f,$bc,$c2,$e0,$09,$f9,
    $7a,$2c,$13,$96,$4b,$a2,$cb,$a8,$d5,$28,$98,$7a,$ae,$d2,$14,$59,$1c,$b5,$1a,$69,$9a,$09,$3c,$93,$51,
    $2c,$57,$38,$50,$bf,$2d,$94,$c1,$fc,$87,$36,$ae,$f5,$a1,$63,$31,$eb,$8f,$3b,$81,$ab,$99,$2f,$b6,$46,
    $81,$04,$b9,$bc,$ce,$e8,$13,$ea,$d5,$ee,$39,$48,$59,$72,$a0,$74,$31,$f4,$46,$0e,$17,$5b,$48,$c2,$91,
    $f8,$7a,$2d,$4e,$bc,$36,$69,$ea,$b0,$94,$9e,$d1,$b1,$00,$9e,$10,$6a,$8e,$16,$90,$31,$b1,$45,$eb,$e3,
    $b4,$9a,$3d,$1a,$66,$43,$5b,$6b,$8a,$da,$3d,$8b,$f7,$6d,$bc,$c0,$e9,$d6,$62,$fa,$35,$6d,$c6,$8e,$a0,
    $0d,$c4,$b1,$f9,$ae,$c6,$f2,$f0,$32,$1e,$fa,$96,$a3,$b8,$fa,$38,$7a,$39,$ba,$ea,$67,$01,$1f,$6d,$7d,
    $b8,$4d,$c9,$e5,$b7,$09,$8d,$39,$59,$54,$b6,$03,$17,$ee,$be,$09,$aa,$70,$f7,$0b,$06,$7a,$65,$a7,$89,
    $d9,$6f,$e2,$07,$ba,$a2,$fd,$9c,$14,$07,$da,$8e,$73,$25,$79,$4d,$64,$45,$fe,$7f,$b0,$2b,$7e,$de,$e5,
    $65,$4a,$6f,$4b,$1e,$f9,$76,$d9,$5c,$1f,$13,$29,$37,$a1,$a4,$24,$bc,$00,$a7,$67,$8d,$49,$1b,$d2,$54,
    $e9,$f8,$1d,$79,$46,$02,$1a,$3e,$96,$f5,$82,$8c,$a2,$cf,$fa,$94,$c2,$13,$45,$d3,$27,$7a,$32,$3d,$a8,
    $65,$f5,$98,$d6,$6d,$a6,$88,$a4,$b4,$97,$c4,$bc,$6c,$c8,$8e,$4a,$c4,$0d,$6a,$05,$62,$84,$3e,$55,$83,
    $fc,$64,$9f,$89,$a6,$fc,$a9,$b8,$59,$12,$95,$90,$f2,$9c,$2c,$bd,$98,$78,$a8,$f2,$84,$d3,$94,$57,$a0,
    $71,$8b,$44,$a6,$e8,$ed,$b5,$9b,$38,$ee,$b6,$55,$0d,$b0,$57,$f3,$9b,$b3,$35,$4a,$ab,$49,$c4,$11,$f7,
    $bd,$56,$c2,$10,$cd,$52,$47,$e1,$96,$64,$31,$1c,$59,$98,$46,$bb,$77,$37,$12,$e0,$1e,$84,$6f,$82,$13,
    $61,$6c,$3b,$ea,$02,$f4,$c7,$c2,$80,$87,$91,$60,$09,$60,$b4,$7d,$81,$6c,$6b,$8f,$8c,$9d,$2b,$90,$94,
    $d7,$68,$9d,$3d,$ff,$b5,$f1,$68,$e1,$a7,$33,$6f,$29,$63,$15,$37,$49,$26,$1a,$96,$4f,$c6,$2e,$e3,$10,
    $8a,$47,$9e,$59,$32,$64,$f4,$53,$2e,$b0,$d8,$ea,$00,$68,$81,$23,$53,$8f,$1b,$e3,$b3,$e9,$d7,$f3,$34,
    $02,$69,$92,$6b,$d7,$b0,$83,$13,$5f,$b5,$e1,$0d,$ed,$1a,$da,$ab,$d6,$e7,$6d,$2b,$7d,$00,$b3,$d9,$01,
    $4c,$04,$63,$a6,$14,$af,$90,$80,$fc,$8b,$2e,$da,$37,$fc,$af,$ce,$29,$ba,$b8,$2d,$b4,$39,$ee,$ff,$93,
    $74,$e5,$f5,$09,$00,$7d,$85,$06,$6c,$7c,$e3,$00,$e2,$f4,$92,$eb,$4b,$cc,$1c,$70,$b8,$51,$1c,$87,$fd,
    $ca,$19,$94,$c3,$86,$f1,$fd,$b7,$27,$d8,$85,$f8,$96,$16,$b0,$ff,$df,$23,$ec,$7a,$1a,$08,$cf,$58,$b7,
    $a1,$79,$b7,$dc,$ef,$f4,$72,$07,$91,$59,$b3,$a5,$8d,$65,$40,$8b,$0e,$e4,$41,$59,$0b,$c1,$ca,$50,$e5,
    $10,$6f,$36,$60,$fa,$97,$c6,$38,$6f,$33,$53,$b3,$15,$b2,$ad,$0e,$43,$7d,$85,$56,$c8,$6f,$62,$ad,$f1,
    $cb,$c0,$0d,$88,$6d,$34,$60,$0d,$eb,$39,$11,$bb,$04,$83,$94,$85,$34,$61,$8b,$ac,$17,$1a,$ed,$5f,$7b,
    $5e,$26,$c7,$66,$06,$8c,$6e,$5a,$8c,$ec,$d4,$83,$9d,$99,$17,$73,$9b,$15,$07,$82,$65,$6d,$ec,$56,$b6,
    $00,$23,$1f,$5a,$e7,$24,$25,$7c,$15,$8f,$bb,$fe,$c0,$77,$9b,$13,$ff,$6a,$36,$f5,$90,$ff,$40,$ff,$d7,
    $17,$17,$42,$be,$d1,$f8,$ce,$d0,$73,$26,$c0,$53,$6d,$49,$b2,$b9,$3f,$55,$05,$7d,$5c,$b1,$3a,$46,$04,
    $c9,$d2,$92,$a1,$43,$87,$d8,$cd,$0e,$30,$54,$19,$68,$c4,$53,$fb,$73,$73,$ca,$cd,$93,$a0,$70,$56,$2b,
    $58,$06,$30,$89,$85,$94,$81,$df,$88,$31,$2c,$5a,$97,$81,$eb,$1e,$44,$a5,$f6,$87,$5f,$c8,$ae,$4e,$de,
    $ef,$57,$c1,$2f,$24,$a9,$d6,$17,$6b,$49,$cf,$13,$37,$9e,$f1,$ed,$9e,$cc,$a8,$e0,$82,$9e,$71,$e1,$d7,
    $45,$15,$17,$1b,$44,$7d,$2c,$2c,$d1,$be,$b7,$a3,$ba,$7b,$34,$27,$a3,$83,$78,$6f,$dc,$27,$86,$13,$2a,
    $42,$4d,$c0,$62,$13,$dd,$9f,$1c,$85,$9f,$fa,$43,$f1,$c1,$e1,$0e,$70,$6f,$95,$e5,$2e,$f3,$10,$93,$6e,
    $ab,$07,$d3,$40,$fb,$06,$cd,$f1,$f4,$4d,$b1,$26,$9d,$b0,$16,$bb,$bb,$5c,$ef,$91,$ec,$21,$c3,$96,$94,
    $44,$57,$4d,$31,$77,$2d,$d2,$a0,$40,$77,$ca,$d6,$ab,$fe,$67,$83,$c1,$cf,$af,$d8,$c7,$68,$a7,$96,$c7,
    $dd,$22,$15,$f0,$69,$f0,$7f,$ab,$98,$85,$3e,$7a,$de,$65,$48,$3a,$3c,$50,$2d,$b1,$27,$08,$c4,$19,$d0,
    $49,$ed,$a2,$47,$79,$53,$f4,$39,$17,$9a,$50,$58,$96,$00,$f4,$49,$b0,$e5,$43,$03,$6f,$dc,$ad,$8a,$6a,
    $74,$a9,$bb,$b3,$d3,$01,$3d,$37,$ae,$b2,$13,$fe,$e1,$4e,$e0,$60,$06,$cf,$67,$6f,$46,$47,$00,$12,$3c,
    $f0,$43,$06,$df,$00,$d7,$de,$79,$35,$89,$c4,$aa,$66,$71,$71,$a9,$b3,$d1,$f5,$70,$e0,$2e,$c7,$7d,$49,
    $36,$79,$df,$f1,$ee,$f7,$fb,$8f,$8f,$9d,$ea,$d7,$91,$7d,$8c,$b1,$7e,$e8,$4e,$f5,$5e,$cf,$31,$9c,$3f,
    $7e,$82,$c2,$fd,$92,$a5,$81,$24,$ab,$aa,$85,$2e,$c5,$5f,$f5,$22,$64,$8d,$a3,$27,$f4,$28,$ba,$fe,$51,
    $38,$2c,$e5,$85,$37,$2f,$d1,$4e,$be,$5d,$73,$ac,$f6,$94,$1f,$2e,$d0,$20,$c2,$08,$1d,$72,$40,$19,$cb,
    $bc,$85,$2c,$17,$19,$d1,$c3,$8e,$3e,$cc,$5d,$ab,$86,$69,$70,$57,$ee,$9b,$ce,$15,$a4,$ea,$81,$21,$36,
    $5c,$c7,$93,$49,$2d,$d1,$81,$a4,$b0,$3a,$10,$60,$e8,$d1,$c8,$6c,$5f,$16,$3c,$e0,$67,$37,$ec,$78,$02,
    $3a,$bf,$3b,$57,$e7,$54,$ed,$de,$36,$9c,$58,$50,$e2,$97,$76,$12,$cd,$83,$c1,$bc,$30,$50,$aa,$74,$2b,
    $9f,$c5,$46,$a2,$77,$5d,$c1,$89,$46,$17,$d6,$72,$4d,$d6,$36,$1c,$a8,$0e,$b5,$a5,$b8,$23,$4b,$f8,$c3,
    $40,$62,$18,$74,$6b,$27,$32,$29,$9c,$26,$5c,$c9,$24,$12,$f2,$21,$cc,$8e,$ea,$57,$23,$74,$12,$e6,$f8,
    $c4,$9d,$a6,$8c,$79,$c7,$a6,$22,$8c,$ef,$47,$23,$bc,$09,$28,$e6,$0f,$a1,$17,$7b,$f2,$bc,$00,$e9,$a2,
    $23,$77,$89,$88,$e1,$17,$d6,$1a,$fd,$37,$22,$3d,$04,$5d,$5f,$a1,$ca,$03,$15,$07,$60,$34,$40,$3a,$d1,
    $1b,$0d,$be,$f5,$32,$a8,$37,$7b,$da,$32,$bb,$aa,$ec,$c8,$a1,$96,$15,$f6,$76,$97,$36,$20,$33,$29,$12,
    $1e,$29,$0b,$2e,$79,$47,$c6,$7f,$55,$42,$ea,$8a,$59,$8c,$40,$7c,$a7,$44,$91,$32,$fe,$db,$32,$00,$7c,
    $fb,$d5,$9b,$90,$fe,$31,$ad,$bf,$d7,$d9,$30,$fa,$53,$0e,$dd,$67,$c0,$94,$58,$97,$bc,$1b,$e2,$bf,$c0,
    $12,$0d,$de,$f3,$18,$e0,$cb,$3e,$33,$3a,$9d,$05,$60,$24,$15,$ce,$e2,$25,$52,$67,$2c,$29,$b5,$33,$1a,
    $11,$d6,$dc,$44,$1d,$29,$57,$f4,$d8,$92,$81,$ad,$0e,$aa,$b7,$31,$3c,$da,$ad,$09,$0b,$33,$7b,$cd,$bf,
    $91,$f3,$15,$a0,$93,$a7,$2c,$a0,$c7,$b8,$78,$f2,$58,$e2,$e3,$7f,$ec,$de,$83,$f4,$a6,$13,$4b,$8b,$0c,
    $f4,$de,$2a,$67,$03,$f4,$3e,$fc,$ed,$91,$0c,$6b,$75,$55,$6f,$9b,$8a,$47,$e6,$fd,$d4,$e8,$40,$d5,$58,
    $1d,$bd,$9b,$2e,$87,$0e,$f3,$71,$59,$8b,$d8,$e7,$08,$9f,$52,$b4,$f9,$74,$47,$15,$58,$6c,$2a,$c0,$99,
    $12,$06,$66,$42,$3f,$19,$80,$a0,$a7,$66,$2e,$09,$61,$ac,$11,$e4,$0f,$d9,$25,$66,$df,$ab,$4d,$26,$55,
    $02,$d2,$d9,$c7,$31,$b7,$53,$be,$bb,$92,$3a,$97,$58,$ad,$ac,$28,$6d,$65,$54,$50,$71,$d5,$7c,$68,$11,
    $81,$35,$0c,$ac,$9e,$66,$d3,$bd,$1b,$a1,$14,$b6,$22,$11,$a4,$89,$11,$05,$be,$db,$4c,$6a,$10,$eb,$37,
    $d3,$3d,$e8,$7b,$70,$3d,$4b,$72,$e3,$86,$7f,$cb,$23,$e4,$b9,$00,$e5,$d9,$7a,$4a,$7e,$15,$63,$f5,$53,
    $ba,$34,$54,$4f,$57,$a9,$fb,$07,$df,$a0,$4a,$df,$dc,$ce,$1e,$81,$dd,$31,$42,$f4,$f3,$b7,$71,$44,$4d,
    $c2,$86,$0e,$06,$ba,$19,$fe,$12,$db,$ce,$5b,$ba,$94,$59,$15,$2b,$af,$b9,$2e,$c0,$34,$bf,$04,$43,$11,
    $26,$e2,$64,$75,$8e,$b6,$b7,$a5,$52,$01,$7c,$22,$7c,$4b,$ac,$51,$df,$54,$96,$e0,$dc,$ec,$d0,$d3,$0f,
    $ad,$5a,$de,$8b,$c4,$63,$f7,$33,$00,$52,$10,$36,$fb,$90,$ef,$b2,$6a,$28,$d1,$d1,$f7,$ae,$80,$27,$7a,
    $4b,$55,$a7,$00,$b3,$5c,$b1,$8b,$9c,$4d,$4f,$92,$a4,$26,$3d,$1c,$eb,$89,$0c,$a5,$2e,$fc,$66,$d4,$a1,
    $65,$3e,$e7,$31,$ea,$ef,$92,$6b,$34,$ef,$a0,$8f,$e0,$cb,$ab,$e3,$cf,$99,$04,$75,$fb,$60,$e6,$3f,$39,
    $e3,$98,$07,$a1,$b3,$cc,$a2,$62,$9e,$57,$51,$ef,$c4,$c5,$20,$87,$e2,$81,$d5,$88,$42,$5e,$26,$bf,$08,
    $c5,$f8,$f7,$8a,$4b,$3a,$e3,$e9,$ee,$2a,$13,$59,$9c,$f6,$3d,$42,$fd,$cc,$90,$1a,$9e,$f1,$4e,$e7,$65,
    $46,$3d,$0e,$d1,$fb,$a1,$22,$8f,$73,$2b,$63,$c8,$20,$f2,$98,$58,$64,$35,$96,$93,$ca,$98,$3d,$4f,$03,
    $8e,$a5,$bb,$98,$d2,$cd,$78,$7c,$b3,$e9,$96,$13,$b1,$15,$af,$a8,$06,$d0,$3b,$cc,$c4,$22,$a3,$7e,$23,
    $66,$79,$21,$d3,$12,$26,$0a,$ba,$0e,$96,$51,$98,$c6,$82,$20,$a0,$84,$8c,$9e,$aa,$92,$b0,$60,$6d,$e5,
    $77,$a4,$7f,$99,$fc,$be,$fb,$7b,$51,$45,$76,$7a,$b4,$2f,$cf,$7b,$0a,$33,$07,$9b,$ab,$18,$3c,$9b,$33,
    $03,$05,$41,$b1,$84,$d9,$7e,$3c,$c0,$c9,$68,$d8,$b7,$74,$ad,$15,$40,$1c,$2d,$e8,$ee,$c9,$cb,$2d,$6d,
    $1a,$73,$f8,$ea,$08,$a7,$0c,$d3,$5a,$9b,$35,$c0,$8d,$0a,$cd,$eb,$b5,$f8,$1c,$3d,$48,$4e,$1b,$d1,$bb,
    $01,$3e,$99,$69,$0d,$da,$db,$36,$49,$db,$af,$37,$5d,$25,$0b,$e2,$dc,$ea,$1c,$7f,$40,$44,$d9,$52,$8c,
    $7e,$e3,$0d,$9b,$ca,$df,$1d,$d0,$20,$51,$f7,$a1,$0e,$e5,$d8,$41,$43,$40,$9c,$88,$bd,$eb,$00,$a8,$5d,
    $b4,$5b,$1f,$e0,$fb,$40,$1a,$b4,$eb,$6e,$ea,$21,$ee,$6c,$1b,$dd,$8b,$6c,$80,$65,$be,$c9,$d8,$e5,$f8,
    $81,$ff,$e8,$de,$d1,$96,$fb,$a9,$10,$e9,$a8,$43,$58,$a2,$a4,$33,$a3,$3b,$62,$b1,$97,$6e,$f5,$d4,$2b,
    $19,$02,$7d,$3c,$31,$4f,$85,$10,$e3,$a8,$e7,$9c,$47,$52,$32,$cc,$c1,$c8,$72,$03,$c0,$f4,$57,$5f,$51,
    $18,$dc,$f6,$b5,$33,$cf,$e4,$85,$c4,$63,$55,$26,$96,$17,$91,$46,$ce,$71,$39,$06,$13,$77,$a1,$7f,$1b,
    $af,$81,$7a,$b2,$03,$9f,$8d,$23,$95,$02,$a0,$51,$1a,$39,$1c,$3c,$50,$f8,$b3,$17,$eb,$7c,$9f,$3c,$b7,
    $7e,$7a,$06,$09,$20,$3b,$94,$27,$d5,$a0,$4e,$e0,$75,$16,$46,$04,$34,$d1,$37,$98,$43,$54,$c1,$2e,$12,
    $3b,$c9,$d6,$ce,$e3,$85,$87,$85,$04,$e2,$ce,$8b,$dc,$96,$45,$0c,$eb,$13,$d0,$5d,$58,$51,$d3,$4f,$fd,
    $ca,$11,$8e,$dd,$0b,$6e,$8d,$c6,$b2,$ec,$a4,$ac,$09,$d6,$b5,$6f,$e2,$08,$30,$64,$a1,$01,$19,$9d,$f6,
    $6a,$9a,$7b,$e9,$e6,$d2,$40,$93,$99,$2f,$60,$fa,$24,$1f,$3f,$0f,$91,$2c,$34,$47,$dd,$32,$fa,$67,$e6,
    $92,$49,$fc,$fb,$bd,$e2,$24,$af,$cb,$dd,$5d,$2c,$2b,$00,$be,$e8,$b9,$f8,$ab,$62,$63,$85,$67,$f8,$65,
    $42,$3f,$3f,$c7,$7a,$2c,$27,$cf,$39,$57,$36,$b9,$19,$2c,$cf,$aa,$a8,$7c,$3a,$4d,$7b,$43,$14,$8c,$6f,
    $8d,$d8,$0c,$0f,$60,$72,$c1,$6a,$7c,$8b,$a0,$f7,$58,$71,$05,$c1,$18,$bd,$88,$dd,$3f,$27,$af,$2b,$b8,
    $ef,$34,$52,$4b,$8a,$d7,$57,$99,$1c,$ff,$05,$20,$41,$21,$82,$e1,$db,$5a,$14,$09,$b5,$4f,$51,$c5,$06,
    $31,$b2,$50,$b9,$23,$2a,$23,$8a,$eb,$6f,$14,$be,$8a,$ff,$cb,$9c,$2d,$4f,$ea,$ef,$39,$26,$6e,$b1,$f6,
    $26,$31,$d0,$e6,$03,$af,$5b,$4f,$15,$39,$8a,$09,$36,$d2,$62,$92,$95,$6e,$f0,$f8,$55,$a3,$ac,$c6,$3a,
    $b3,$4f,$e5,$ee,$37,$f3,$cc,$51,$ad,$04,$d7,$8d,$54,$40,$7b,$25,$0b,$c8,$79,$eb,$ec,$45,$c9,$80,$7a,
    $b2,$f5,$d6,$04,$b6,$90,$8a,$85,$e4,$a8,$e9,$13,$60,$9b,$3a,$a7,$38,$80,$b1,$aa,$6f,$c7,$4b,$67,$60,
    $e2,$b5,$38,$ab,$ae,$18,$8d,$26,$5d,$c7,$26,$a8,$48,$38,$d9,$9f,$c9,$a4,$5c,$84,$b0,$6e,$13,$c2,$1d,
    $96,$cb,$2a,$f2,$aa,$26,$3d,$78,$cd,$8d,$22,$bd,$75,$28,$d3,$f5,$57,$dc,$6a,$26,$86,$ca,$c2,$54,$ae,
    $f2,$2c,$74,$7b,$e1,$a8,$58,$d2,$af,$ca,$8f,$0e,$a4,$9e,$e0,$c7,$79,$37,$bc,$31,$42,$2d,$e2,$55,$28,
    $f1,$1b,$06,$d5,$29,$11,$8f,$0c,$04,$9e,$74,$ac,$39,$92,$c5,$e3,$aa,$6a,$73,$e9,$55,$f3,$0e,$72,$a4,
    $f8,$ff,$bf,$3f,$81,$96,$fb,$f5,$43,$29,$28,$af,$4e,$6d,$46,$ca,$b0,$45,$98,$49,$28,$1c,$63,$c3,$26,
    $33,$7d,$78,$9e,$27,$10,$6e,$88,$fb,$85,$7e,$63,$04,$55,$ff,$b4,$58,$9f,$a6,$84,$23,$10,$e9,$6b,$a2,
    $fe,$d2,$f4,$77,$8f,$fd,$5f,$8d,$c5,$0e,$51,$ab,$b6,$2e,$e5,$a4,$6c,$43,$9d,$5e,$e9,$d4,$0b,$e3,$97,
    $ac,$dc,$2e,$3b,$a6,$4b,$ab,$07,$5a,$55,$55,$04,$73,$42,$ee,$18,$ad,$64,$1a,$a3,$c2,$bb,$14,$8d,$8e,
    $a0,$37,$c0,$62,$0c,$1d,$0b,$f1,$69,$c6,$e7,$a5,$49,$c6,$ce,$ca,$d4,$ca,$16,$f6,$26,$da,$5c,$1b,$72,
    $d2,$86,$22,$f6,$11,$13,$69,$98,$d6,$0f,$f6,$e1,$38,$65,$e3,$d1,$7c,$bd,$81,$9c,$1d,$ab,$79,$e3,$ba
  {$ifndef MPC_FPrec30K}
   ,$a0,$26,$5b,$fc,$42,$52,$aa,$70,$fd,$81,$98,$57,$b3,$8a,$99,$29,$93,$47,$e4,$1d,$f5,$f3,$cd,$d5,$56,
    $65,$02,$c0,$f6,$0a,$2c,$0c,$df,$ee,$73,$a0,$b2,$f2,$33,$2f,$45,$59,$67,$29,$66,$51,$6c,$10,$ce,$b3,
    $27,$0c,$73,$06,$81,$89,$38,$d6,$c2,$da,$ed,$dd,$5f,$20,$12,$e7,$e9,$4a,$4b,$45,$21,$8a,$76,$2e,$2e,
    $aa,$31,$16,$6a,$bf,$1a,$21,$4f,$c4,$a2,$ee,$b8,$46,$44,$78,$23,$19,$41,$0d,$fa,$fa,$98,$3b,$34,$9d,
    $aa,$9d,$39,$96,$57,$4f,$28,$d8,$7f,$90,$09,$45,$90,$3f,$1f,$a0,$d3,$2a,$17,$cb,$47,$e5,$18,$3a,$b7,
    $e3,$9f,$a4,$ad,$c3,$96,$ea,$3b,$83,$d2,$6e,$f9,$59,$ab,$ee,$08,$0a,$5c,$3f,$a1,$24,$b4,$aa,$46,$6a,
    $e2,$d5,$d8,$a5,$9a,$cc,$20,$10,$69,$72,$93,$3c,$f6,$e3,$d8,$ee,$fc,$b6,$00,$f8,$e2,$20,$a2,$92,$09,
    $4e,$78,$73,$0c,$af,$c9,$86,$53,$32,$38,$8f,$98,$a4,$2f,$36,$b1,$55,$cc,$c9,$1a,$70,$85,$c5,$a7,$ad,
    $f8,$f5,$71,$1c,$1e,$68,$4b,$67,$8f,$1d,$07,$ff,$fe,$99,$3c,$8b,$fd,$82,$6d,$f2,$50,$8a,$9f,$fb,$a3,
    $54,$37,$53,$98,$28,$c0,$49,$9e,$f3,$1e,$06,$08,$63,$51,$cd,$7a,$16,$3f,$f5,$18,$e4,$a9,$3c,$13,$9b,
    $79,$5b,$61,$e6,$88,$f1,$49,$37,$bc,$3e,$e2,$ff,$b4,$e1,$9b,$7f,$72,$cd,$9c,$eb,$fe,$15,$89,$ec,$0d,
    $61,$5b,$7e,$3c,$70,$35,$13,$92,$1f,$03,$8f,$2a,$98,$fa,$62,$45,$65,$72,$bb,$18,$31,$7e,$44,$d5,$3a,
    $9d,$89,$c3,$2a,$f1,$0f,$fb,$db,$f5,$17,$69,$2d,$b1,$da,$c6,$88,$77,$b3,$e6,$47,$5b,$a3,$44,$c1,$62,
    $2f,$8a,$f2,$99,$9e,$29,$f0,$ea,$52,$ce,$17,$07,$84,$b6,$47,$b9,$03,$be,$37,$7b,$66,$98,$c4,$b9,$21,
    $94,$00,$b8,$c8,$f6,$70,$77,$41,$b2,$0b,$1d,$67,$a0,$61,$d5,$48,$6b,$06,$b9,$60,$31,$4f,$06,$a1,$7b,
    $91,$25,$42,$05,$e1,$60,$f5,$31,$ae,$f5,$8f,$7e,$a8,$57,$a6,$b7,$d4,$ab,$99,$ab,$e9,$b7,$bd,$76,$6b,
    $3d,$03,$68,$61,$03,$09,$1f,$5b,$0e,$fa,$de,$e2,$90,$17,$5c,$d9,$7b,$4a,$57,$75,$5d,$82,$b3,$1e,$e7,
    $46,$2c,$b5,$a8,$42,$79,$ae,$15,$61,$87,$ac,$36,$7e,$4d,$ba,$0a,$42,$ab,$28,$98,$8b,$cd,$4c,$3a,$ff,
    $9b,$9c,$b1,$ed,$cd,$e9,$58,$05,$df,$dc,$1a,$83,$89,$c1,$90,$a7,$9e,$9d,$1c,$10,$e7,$08,$6a,$9e,$06,
    $f9,$2f,$35,$d4,$62,$f2,$0d,$c6,$db,$a6,$34,$e4,$17,$6d,$0d,$f1,$10,$ae,$26,$99,$b9,$bb,$d7,$96,$fe,
    $aa,$d0,$d3,$a5,$a0,$73,$25,$16,$e8,$af,$40,$02,$b5,$0d,$f0,$15,$1f,$93,$d0,$f0,$52,$19,$13,$a7,$b1,
    $6a,$9e,$ea,$2d,$b3,$4a,$0f,$cc,$0e,$23,$94,$60,$b7,$8a,$3b,$10,$84,$26,$65,$96,$fc,$13,$8a,$90,$85,
    $f6,$92,$eb,$1b,$e4,$ad,$b2,$d5,$98,$a3,$c8,$d0,$18,$d4,$81,$c9,$63,$95,$ed,$f7,$f9,$86,$be,$5a,$06,
    $3e,$75,$4b,$a9,$2b,$b6,$ca,$7a,$79,$c4,$1d,$7f,$f3,$09,$fd,$0b,$49,$c1,$4a,$40,$70,$05,$b9,$7f,$e2,
    $27,$47,$5d,$41,$76,$49,$39,$34,$9f,$9e,$2c,$c1,$f7,$6d,$18,$fc,$c2,$0d,$51,$f6,$89,$d9,$96,$ea,$e8,
    $0b,$e2,$b1,$c1,$ea,$31,$e6,$a5,$33,$a9,$2e,$f8,$90,$b1,$25,$ff,$a8,$44,$4b,$cc,$9c,$2a,$2f,$2d,$ce,
    $11,$14,$26,$ed,$70,$2b,$82,$45,$21,$a7,$5c,$67,$f7,$bf,$f7,$08,$ff,$50,$1e,$61,$cb,$2e,$96,$82,$c6,
    $dd,$b6,$4e,$aa,$3c,$fd,$c9,$fb,$6e,$66,$eb,$bf,$a2,$ab,$9f,$f4,$54,$7d,$3d,$5b,$33,$78,$ea,$9d,$45,
    $1e,$e5,$7b,$f1,$ad,$ca,$a1,$de,$2b,$74,$bf,$f3,$fc,$22,$6e,$d1,$c1,$40,$88,$eb,$a0,$1a,$71,$1e,$49,
    $37,$24,$e9,$89,$0b,$94,$99,$69,$63,$ca,$16,$ff,$77,$6c,$13,$48,$3e,$40,$09,$78,$5f,$ab,$70,$dd,$46,
    $a0,$a5,$4b,$6b,$c8,$f9,$9b,$0d,$64,$d8,$df,$1b,$e6,$0c,$97,$88,$6f,$aa,$e4,$23,$37,$4e,$63,$ee,$9f,
    $93,$ff,$7b,$9d,$e3,$00,$ad,$18,$10,$a5,$39,$d8,$87,$83,$f9,$0b,$98,$f7,$91,$e2,$e9,$3a,$a5,$fa,$84,
    $72,$6d,$6b,$35,$83,$6c,$34,$5c,$cb,$5f,$8c,$df,$b0,$3d,$31,$52,$36,$87,$44,$e9,$26,$4a,$b9,$2b,$ce,
    $4c,$11,$ff,$56,$31,$e8,$53,$5f,$a3,$a5,$2c,$bc,$72,$1b,$bf,$b0,$d7,$a9,$d1,$c3,$58,$81,$c4,$06,$fe,
    $3b,$38,$2a,$55,$f8,$a2,$85,$31,$e7,$fe,$69,$a0,$cd,$05,$7a,$94,$4e,$62,$03,$43,$92,$47,$60,$25,$16,
    $cb,$53,$a5,$1a,$7e,$8f,$a5,$d9,$fe,$55,$0c,$d3,$bb,$e3,$c9,$56,$ff,$be,$de,$8e,$d1,$c0,$00,$52,$fd,
    $ea,$84,$2c,$11,$4a,$c5,$51,$58,$86,$73,$00,$9c,$5a,$79,$b2,$69,$01,$4f,$d6,$d4,$5c,$ea,$06,$4e,$de,
    $eb,$a7,$c4,$96,$6a,$73,$dd,$dc,$5f,$5d,$07,$21,$3b,$40,$15,$97,$1a,$b6,$ac,$67,$a6,$ec,$43,$62,$66,
    $70,$f2,$e0,$b7,$29,$47,$a3,$04,$f3,$9c,$85,$fc,$03,$cc,$0a,$43,$e0,$ac,$39,$c4,$ab,$77,$5b,$f5,$c0,
    $99,$9c,$83,$09,$f8,$61,$9d,$1b,$6e,$a8,$6e,$2b,$05,$61,$6a,$26,$4b,$8c,$c8,$80,$96,$b3,$ac,$9e,$8a,
    $b7,$b5,$72,$0a,$b5,$45,$b1,$63,$82,$a8,$e7,$f3,$75,$16,$8a,$8c,$d3,$ac,$13,$4a,$d3,$91,$6d,$ab,$c8,
    $ee,$07,$53,$2e,$de,$77,$7e,$38,$84,$94,$ef,$31,$93,$f9,$f1,$ba,$69,$ef,$83,$62,$25,$01,$fa,$8c,$d2,
    $d5,$05,$ff,$f4,$ec,$70,$ad,$b1,$c1,$75,$da,$d1,$65,$e5,$09,$0f,$f8,$a8,$cd,$ea,$1b,$1f,$75,$7f,$28,
    $5a,$da,$87,$62,$78,$57,$41,$36,$c9,$f9,$d0,$20,$28,$55,$c1,$a4,$3b,$65,$79,$8f,$3b,$18,$0d,$60,$3b,
    $5c,$b3,$95,$be,$8a,$92,$d8,$20,$e8,$97,$5d,$5f,$74,$da,$4f,$cd,$60,$ad,$a8,$4f,$70,$f7,$50,$cc,$43,
    $43,$79,$7f,$52,$90,$67,$29,$38,$09,$d7,$17,$db,$98,$f4,$a9,$6d,$d6,$4c,$44,$3f,$62,$53,$b7,$e3,$50,
    $35,$28,$fe,$15,$24,$dc,$0b,$9f,$c0,$f7,$73,$8c,$11,$c5,$c1,$28,$bf,$90,$64,$75,$27,$75,$12,$21,$1e,
    $53,$f0,$7a,$07,$92,$20,$e2,$80,$dd,$fb,$33,$63,$1b,$67,$43,$93,$5f,$44,$54,$b1,$8f,$c1,$02,$34,$4c,
    $ed,$a6,$44,$de,$ac,$4f,$ff,$d2,$f6,$76,$b6,$61,$44,$17,$d8,$c6,$1b,$fc,$8f,$5f,$98,$29,$e6,$60,$1f,
    $27,$40,$96,$00,$2c,$39,$ea,$07,$22,$8c,$07,$2a,$8d,$bc,$26,$bb,$e6,$a7,$90,$39,$df,$05,$3a,$3b,$7d,
    $a2,$cc,$d5,$6e,$c7,$e7,$3c,$bf,$8d,$69,$62,$08,$b1,$5b,$77,$5c,$48,$9f,$3d,$77,$3b,$ba,$15,$2a,$1a,
    $46,$55,$d9,$e1,$55,$ce,$58,$9e,$25,$a5,$05,$9a,$48,$c0,$b3,$6a,$01,$15,$0c,$a1,$7e,$e1,$3f,$5e,$53,
    $92,$72,$92,$ea,$c1,$af,$e4,$1b,$f3,$c0,$a6,$26,$97,$5b,$86,$bf,$df,$da,$0e,$86,$58,$cc,$28,$82,$9c,
    $3d,$a9,$c8,$d2,$4e,$f0,$0a,$13,$c6,$4f,$c6,$4f,$16,$cb,$e9,$3d,$41,$34,$36,$a5,$73,$1a,$4e,$fb,$6d,
    $b4,$b2,$f6,$cb,$c9,$18,$e6,$40,$49,$fd,$ff,$3c,$f4,$01,$6a,$e8,$f2,$9e,$49,$fe,$50,$6c,$14,$0c,$e1,
    $4a,$e3,$f1,$3a,$38,$93,$0c,$86,$c9,$ab,$8d,$1d,$71,$a1,$9f,$b8,$5b,$15,$b6,$67,$b4,$83,$99,$e8,$22,
    $04,$38,$22,$71,$05,$a3,$0f,$2c,$bd,$fa,$05,$aa,$ec,$49,$f3,$19,$04,$4d,$35,$93,$1d,$12,$00,$29,$a3,
    $5e,$bb,$72,$22,$8a,$37,$7c,$f0,$73,$4f,$59,$4d,$75,$6a,$bc,$1e,$24,$d3,$ae,$2b,$17,$e8,$ab,$db,$89,
    $98,$93,$e4,$ad,$e4,$10,$d0,$b2,$d8,$75,$02,$ad,$70,$71,$ca,$59,$c7,$96,$fb,$be,$6b,$5f,$9d,$61,$a2,
    $d5,$53,$50,$62,$b4,$10,$f7,$2c,$88,$b3,$47,$8d,$8a,$89,$f5,$8e,$ca,$fd,$16,$78,$4d,$dd,$8f,$56,$f7,
    $bf,$17,$9e,$bb,$49,$9e,$35,$8f,$2e,$b3,$48,$69,$1b,$a8,$a7,$d4,$3a,$57,$78,$80,$76,$25,$a8,$8a,$54,
    $74,$1f,$42,$4d,$45,$de,$10,$a2,$1d,$d0,$84,$b1,$a1,$dd,$67,$ad,$e5,$76,$af,$e0,$b3,$8f,$5f,$15,$7b,
    $4e,$2c,$f0,$cd,$bb,$aa,$7d,$c6,$2d,$85,$9a,$82,$c8,$d3,$2c,$0e,$ca,$b7,$67,$8b,$5a,$4f,$fe,$5d,$f6,
    $f7,$4c,$76,$0f,$ea,$5e,$eb,$71,$c2,$f3,$c4,$35,$3b,$d2,$c5,$a1,$d3,$91,$10,$45,$43,$35,$8c,$0a,$b5,
    $65,$78,$09,$80,$ea,$7a,$c9,$ee,$a1,$60,$bd,$b2,$fd,$e9,$b4,$18,$ce,$5e,$06,$cf,$23,$62,$49,$13,$c3,
    $bb,$e3,$bd,$df,$bc,$9f,$e7,$32,$c1,$46,$27,$7d,$44,$84,$df,$a8,$f4,$fc,$81,$28,$08,$8f,$19,$46,$70,
    $1f,$a1,$5d,$53,$fb,$57,$51,$e4,$2e,$67,$f0,$4d,$9f,$78,$02,$96,$18,$a0,$5d,$77,$f7,$5e,$14,$fa,$0c,
    $91,$df,$8c,$4e,$77,$31,$71,$fc,$aa,$74,$15,$91,$21,$ad,$8f,$df,$6e,$fd,$88,$09,$2b,$1c,$c1,$f9,$19,
    $42,$a8,$2c,$00,$43,$bc,$e0,$27,$82,$d5,$4e,$9c,$ec,$71,$9f,$0f,$25,$f5,$50,$31,$4c,$15,$a2,$aa,$47,
    $9f,$4d,$bd,$fe,$67,$cb,$d7,$5e,$1f,$01,$eb,$7b,$d8,$b4,$e3,$9c,$5a,$47,$1f,$a1,$9f,$ae,$98,$1f,$31,
    $4d,$59,$0b,$9c,$76,$bf,$78,$ce,$b9,$60,$07,$a4,$43,$02,$72,$50,$c3,$34,$54,$52,$b1,$be,$2b,$2c,$7a,
    $5e,$59,$bb,$97,$df,$13,$db,$1d,$e9,$d4,$fc,$bd,$92,$c5,$54,$40,$50,$38,$7e,$c7,$fd,$0a,$0e,$bd,$1a,
    $e1,$cc,$be,$7d,$15,$d0,$ed,$d9,$51,$b0,$3c,$1e,$bb,$45,$83,$6a,$ea,$48,$fa,$8c,$50,$f5,$b3,$be,$49,
    $f1,$03,$15,$3a,$fe,$64,$5f,$11,$49,$59,$58,$9c,$da,$65,$f3,$68,$e5,$bc,$ed,$7d,$26,$6e,$fe,$51,$bf,
    $bd,$d5,$bf,$67,$cc,$d5,$a5,$ff,$2e,$00,$ad,$9e,$6c,$b2,$00,$32,$00,$60,$6b,$55,$1d,$43,$d3,$70,$5a,
    $66,$2a,$fb,$0b,$d9,$e5,$8b,$2c,$31,$f0,$95,$d0,$33,$35,$85,$c6,$da,$66,$c3,$35,$29,$4a,$89,$d6,$b8,
    $33,$c6,$65,$f6,$24,$ad,$c5,$68,$5c,$e9,$ee,$32,$ae,$5a,$de,$79,$24,$6c,$29,$62,$83,$98,$c8,$8b,$00,
    $b4,$08,$fd,$3a,$63,$8a,$ef,$0e,$f6,$b2,$69,$96,$f9,$1a,$6c,$9f,$e9,$d5,$cd,$e9,$dc,$d1,$3c,$11,$0b,
    $88,$57,$b1,$d7,$ec,$82,$1d,$47,$ef,$ba,$0c,$a6,$9f,$eb,$c6,$2c,$c1,$60,$83,$c6,$7f,$60,$f3,$3f,$f8,
    $6d,$3e,$69,$5c,$63,$b5,$96,$9e,$25,$cf,$10,$2e,$da,$8b,$c4,$9b,$7a,$b8,$3d,$98,$26,$e5,$f7,$13,$ec,
    $20,$54,$58,$d5,$a9,$b8,$59,$40,$d2,$7c,$87,$19,$56,$d7,$6f,$38,$4d,$22,$29,$36,$6b,$c0,$b4,$16,$88,
    $b4,$c3,$29,$78,$de,$ca,$6e,$58,$8c,$d0,$fc,$7d,$0a,$68,$2f,$fa,$63,$06,$4c,$ba,$24,$ae,$76,$bd,$ef,
    $b8,$be,$60,$19,$20,$79,$6f,$60,$95,$7e,$20,$35,$01,$13,$58,$30,$25,$85,$3b,$79,$b0,$3b,$e4,$75,$75,
    $a4,$97,$62,$57,$c6,$0a,$26,$02,$51,$ff,$a6,$6e,$02,$65,$1e,$7d,$bd,$c2,$cb,$a1,$1a,$bc,$f3,$5a,$8c,
    $f1,$ef,$6c,$02,$d7,$db,$96,$1d,$b9,$42,$18,$cf,$7a,$3b,$45,$4e,$40,$ee,$c8,$0c,$ce,$7b,$8c,$9e,$e5,
    $68,$f1,$f8,$07,$a7,$50,$d4,$86,$57,$93,$1c,$85,$4d,$3d,$6c,$c1,$d6,$01,$18,$26,$84,$97,$db,$7b,$25,
    $eb,$22,$95,$77,$1c,$88,$ab,$c6,$56,$56,$a3,$08,$2e,$86,$85,$6e,$57,$c7,$dd,$46,$0b,$ee,$93,$00,$88,
    $fe,$37,$20,$06,$1f,$7c,$32,$c2,$fe,$03,$ab,$11,$fe,$fe,$9b,$a6,$73,$5a,$b1,$b8,$4a,$2c,$5b,$bd,$fd,
    $1c,$55,$7d,$49,$20,$58,$d5,$d7,$45,$03,$c4,$eb,$33,$de,$79,$52,$1b,$9d,$d3,$61,$37,$1a,$5a,$34,$ee,
    $fb,$5e,$df,$68,$b4,$65,$0b,$d3,$dd,$92,$7b,$a5,$ab,$c3,$0e,$9a,$e3,$1d,$28,$d8,$17,$25,$34,$2c,$90,
    $0b,$23,$56,$4c,$3f,$61,$ef,$df,$2c,$6b,$85,$8b,$5f,$74,$aa,$91,$e4,$f0,$30,$c4,$76,$97,$58,$50,$8d,
    $87,$d4,$e7,$c4,$22,$55,$86,$fe,$d1,$cf,$c3,$3c,$1a,$d3,$52,$cb,$78,$5d,$10,$51,$16,$77,$31,$c3,$f4,
    $90,$ce,$2f,$3d,$6f,$14,$4a,$60,$8e,$ab,$97,$8c,$c9,$01,$b4,$5c,$f3,$7b,$e0,$e2,$de,$ce,$23,$d0,$fc,
    $85,$ee,$57,$ac,$e6,$7a,$f5,$d7,$01,$04,$01,$91,$43,$93,$1b,$f3,$cb,$e0,$0c,$e8,$68,$55,$dd,$10,$4f,
    $f1,$78,$51,$2c,$1c,$d6,$3b,$8d,$c9,$ca,$8e,$c8,$0b,$77,$50,$0c,$66,$af,$d4,$22,$ee,$47,$ef,$51,$ef,
    $f1,$b8,$9e,$c9,$2f,$93,$c6,$d6,$78,$6f,$56,$8a,$a3,$40,$23,$ee,$7f,$74,$1a,$31,$59,$0f,$15,$ff,$91,
    $e3,$d5,$d1,$cf,$d3,$c5,$99,$33,$f6,$18,$55,$22,$3f,$d7,$36,$67,$8f,$42,$99,$a7,$a8,$fc,$86,$6a,$3f,
    $9a,$38,$06,$49,$02,$5d,$4f,$6d,$7e,$72,$cd,$a5,$09,$1c,$34,$d6,$26,$66,$d4,$cc,$6f,$29,$a6,$66,$18,
    $14,$fd,$22,$e0,$54,$2f,$d3,$31,$21,$d5,$9f,$f8,$80,$90,$8a,$f9,$56,$2c,$b5,$5a,$d7,$73,$ee,$82,$07,
    $ef,$a2,$67,$96,$b9,$ea,$c9,$00,$76,$73,$89,$9d,$54,$a4,$d5,$0a,$05,$57,$d9,$4b,$56,$21,$87,$4c,$8e,
    $67,$cf,$0b,$2e,$a7,$e5,$0b,$48,$1c,$bf,$b8,$ec,$d6,$c2,$9e,$9b,$4b,$a0,$8a,$a9,$90,$74,$fc,$ba,$41,
    $07,$55,$e5,$d9,$67,$f6,$6e,$7e,$af,$99,$8b,$99,$b8,$d5,$dc,$b0,$f6,$5f,$36,$3c,$14,$f8,$a6,$29,$4f,
    $cc,$ec,$ab,$8e,$d3,$f5,$91,$b0,$c0,$de,$13,$60,$23,$c3,$ca,$ea,$16,$79,$97,$c0,$1d,$69,$e2,$3e,$fc,
    $7f,$39,$67,$eb,$ae,$90,$71,$46,$e0,$b0,$1d,$f5,$83,$d6,$87,$74,$07,$6e,$a3,$84,$2f,$ee,$38,$aa,$63,
    $c0,$5b,$93,$b9,$4a,$8a,$55,$7a,$73,$6c,$ea,$b3,$03,$68,$cc,$37,$6e,$00,$db,$21,$34,$df,$cf,$05,$29,
    $c9,$9f,$f0,$a8,$2f,$c4,$a0,$4b,$bb,$bf,$3f,$0b,$8a,$3e,$c5,$fa,$65,$5d,$b7,$8c,$29,$dc,$a6,$a8,$ad,
    $64,$0d,$3d,$19,$62,$d3,$ec,$52,$cd,$e8,$16,$a9,$e5,$f4,$b3,$c6,$11,$21,$94,$ed,$17,$64,$c0,$90,$71,
    $7d,$b3,$59,$4c,$1e,$d5,$97,$24,$6b,$cc,$1a,$a6,$cf,$fb,$b4,$42,$6f,$a1,$5f,$ac,$a3,$26,$25,$f4,$f7,
    $3b,$99,$ed,$f2,$e2,$6e,$d5,$f7,$0a,$4b,$fb,$4f,$75,$d3,$06,$48,$c9,$35,$d7,$fb,$2f,$78,$9a,$57,$03,
    $42,$82,$f0,$21,$dc,$a2,$3c,$eb,$04,$c0,$50,$d3,$53,$77,$cd,$bb,$67,$6c,$4b,$4d,$37,$e2,$c6,$8f,$6b,
    $6d,$a3,$6a,$35,$a1,$cb,$ff,$e1,$34,$ba,$58,$17,$9b,$d9,$de,$e0,$d4,$df,$f7,$b7,$80,$d8,$7e,$5d,$07,
    $43,$fa,$73,$34,$35,$3d,$e9,$95,$88,$6c,$a9,$07,$d1,$25,$53,$11,$52,$f9,$2d,$1e,$75,$5e,$0d,$26,$b8,
    $8c,$7a,$21,$80,$a4,$77,$03,$ef,$a4,$ed,$32,$50,$43,$97,$f4,$9e,$3b,$06,$a2,$d7,$c1,$58,$d1,$7a,$73,
    $43,$f2,$82,$e1,$49,$59,$d3,$46,$30,$ce,$37,$01,$a2,$6f,$18,$09,$d8,$2f,$cd,$23,$95,$77,$ff,$8b,$95,
    $83,$e1,$b7,$8e,$f8,$c0,$f9,$df,$3c,$dd,$9e,$bc,$d8,$72,$d7,$1d,$43,$e6,$5f,$34,$31,$b7,$0e,$50,$8b,
    $cd,$b0,$da,$d2,$18,$b1,$f8,$b4,$77,$52,$1e,$74,$c1,$66,$8c,$32,$37,$b8,$fa,$92,$c0,$aa,$d0,$ce,$4f,
    $ac,$fb,$0d,$8b,$24,$ba,$69,$a3,$03,$c8,$e7,$56,$78,$c3,$22,$7a,$92,$6d,$e0,$b0,$b3,$1b,$6a,$a0,$f1,
    $78,$e9,$18,$f4,$79,$93,$47,$0d,$cf,$22,$d9,$83,$c3,$71,$9a,$b6,$c3,$52,$1b,$4b,$da,$e1,$55,$70,$24,
    $e4,$a8,$72,$d2,$90,$c9,$a4,$c1,$e2,$a0,$71,$1f,$5c,$46,$d5,$3f,$e7,$1b,$bc,$f2,$b6,$4d,$86,$99,$a8,
    $a9,$a7,$67,$b9,$4f,$31,$b0,$9c,$cf,$bf,$1f,$1f,$99,$f0,$00,$97,$67,$ad,$8d,$a0,$f1,$f8,$60,$fd,$5b,
    $10,$7d,$b5,$07,$25,$ed,$e1,$f6,$8c,$99,$d4,$f7,$2d,$53,$c6,$f0,$d3,$9c,$99,$3e,$0c,$ea,$72,$4f,$7c,
    $6f,$8e,$c1,$54,$37,$8a,$3b,$f4,$4a,$15,$62,$0b,$70,$42,$4b,$75,$66,$2c,$d7,$34,$3c,$d4,$92,$6e,$79,
    $a9,$26,$1c,$c6,$53,$85,$d9,$d2,$61,$b1,$a6,$ac,$35,$af,$1b,$71,$15,$29,$c9,$2e,$60,$e3,$03,$cb,$6e,
    $e3,$e1,$23,$7d,$ff,$99,$4f,$76,$c6,$fa,$e2,$b8,$fc,$cf,$71,$f4,$c2,$6b,$96,$69,$c0,$2f,$6f,$96,$e0,
    $20,$34,$af,$b6,$4e,$db,$c1,$0f,$cf,$ec,$5f,$33,$02,$06,$5f,$4a,$1d,$df,$c5,$e1,$f4,$19,$0c,$51,$d4,
    $e1,$49,$ac,$26,$0b,$52,$c2,$70,$a1,$d2,$f5,$c1,$81,$f1,$5f,$c8,$1c,$7e,$33,$ed,$0d,$ff,$f5,$2f,$e6,
    $27,$ca,$dc,$f8,$ea,$86,$70,$04,$d1,$5f,$2c,$82,$52,$33,$e7,$24,$17,$48,$59,$1f,$2e,$34,$8d,$11,$f4,
    $24,$4d,$02,$f2,$6e,$b2,$78,$9e,$21,$49,$dd,$4f,$dc,$98,$19,$51,$43,$32,$97,$eb,$bf,$8e,$46,$23,$ca,
    $5b,$e3,$27,$ab,$1d,$67,$c3,$ba,$87,$0e,$9d,$90,$4a,$7e,$e7,$86,$a0,$31,$71,$53,$7c,$33,$21,$2c,$68,
    $91,$8b,$42,$ea,$4f,$2f,$ed,$5d,$93,$47,$79,$74,$84,$74,$67,$21,$f5,$1a,$62,$a6,$55,$6c,$45,$aa,$c9,
    $e4,$35,$f8,$87,$39,$b0,$de,$3f,$ae,$d3,$be,$08,$bf,$67,$e7,$15,$08,$92,$eb,$7e,$28,$6d,$bc,$55,$3a,
    $99,$d1,$81,$b8,$92,$60,$a3,$0b,$65,$c7,$ed,$f1,$5e,$df,$5b,$51,$fc,$f8,$a8,$44,$b2,$a8,$fa,$39,$e6,
    $b0,$47,$d2,$24,$b9,$6b,$32,$d3,$82,$f5,$0f,$d2,$02,$97,$91,$4f,$a4,$0d,$6c,$a4,$68,$b9,$70,$c2,$21,
    $13,$e0,$ac,$cd,$83,$f7,$63,$e7,$6b,$62,$0d,$41,$cc,$bb,$0e,$15,$d4,$b4,$e2,$1d,$d6,$53,$50,$a1,$1c,
    $40,$24,$87,$43,$a0,$77,$f1,$7f,$db,$72,$cb,$1c,$20,$9f,$39,$47,$1b,$bb,$cd,$f6,$09,$5b,$a5,$6c,$93,
    $31,$4a,$d4,$75,$46,$e5,$1a,$8f,$32,$1c,$89,$44,$aa,$65,$2f,$e2,$1e,$ed,$27,$6b,$21,$f9,$01,$df,$21,
    $27,$07,$04,$93,$0c,$c3,$74,$1e,$92,$ec,$63,$10,$d1,$50,$bd,$28,$39,$8d,$f9,$94,$3c,$9b,$9f,$4b,$5b,
    $21,$de,$d3,$83,$e0,$dd,$6e,$f6,$a1,$59,$77,$ac,$cd,$e7,$7c,$21,$6a,$2c,$4a,$5f,$9b,$f5,$96,$de,$98,
    $eb,$ce,$84,$30,$62,$d4,$1e,$8a,$ec,$f8,$a3,$97,$3f,$e5,$1a,$8c,$23,$29,$c5,$06,$75,$6a,$9c,$56,$a5,
    $f4,$40,$e6,$01,$14,$4b,$4e,$bf,$6f,$d5,$dc,$9b,$a1,$03,$0d,$de,$c8,$59,$da,$89,$26,$4d,$d5,$fa,$39,
    $00,$86,$e0,$18,$f4,$39,$3a,$a5,$4b,$77,$7d,$5d,$dc,$48,$4c,$89,$d4,$42,$3b,$11,$0e,$7a,$ce,$04,$ce,
    $93,$e1,$b8,$aa,$4d,$be,$cf,$02,$02,$bf,$ca,$f5,$2c,$4d,$3f,$ca,$a3,$7a,$97,$0c,$5b,$d7,$e5,$80,$20,
    $83,$b8,$d1,$89,$65,$20,$ff,$ef,$d3,$5a,$ec,$e9,$1d,$84,$72,$45,$3d,$b8,$e0,$c0,$9e,$5b,$c8,$1c,$1c,
    $a5,$a7,$ae,$1f,$4d,$29,$57,$dd,$e0,$2a,$94,$2c,$0c,$07,$f0,$b4,$a8,$0c,$e0,$72,$3f,$e6,$e0,$cd,$5a,
    $00,$e0,$83,$8b,$a8,$be,$b3,$18,$a2,$86,$1c,$8b,$3b,$ac,$4c,$e8,$79,$e3,$ca,$47,$ad,$4a,$a9,$88,$f9,
    $94,$e7,$2e,$e7,$5a,$ab,$27,$95,$b8,$52,$e9,$09,$38,$64,$97,$6e,$2b,$ca,$1f,$12,$92,$43,$19,$59,$59,
    $78,$b0,$0a,$9a,$a0,$0c,$21,$41,$9d,$05,$2f,$0a,$a9,$07,$b4,$d5,$59,$9a,$d1,$46,$26,$30,$d0,$5a,$88,
    $14,$65,$29,$97,$1c,$11,$d6,$d2,$4c,$8a,$fe,$72,$36,$4a,$aa,$08,$49,$ac,$31,$d8,$76,$b3,$78,$20,$d9,
    $a5,$7a,$8c,$2a,$aa,$01,$7c,$17,$d7,$aa,$54,$ae,$7c,$f2,$29,$a1,$04,$b5,$9a,$ab,$15,$95,$f1,$7f,$91,
    $eb,$b4,$b9,$3c,$72,$cd,$ad,$12,$db,$2b,$da,$b8,$93,$e7,$cc,$31,$e4,$38,$42,$42,$69,$5e,$1a,$51,$eb,
    $4b,$0b,$f0,$b4,$88,$4e,$d4,$14,$14,$9e,$d2,$29,$ec,$c7,$6f,$89,$2c,$d4,$15,$19,$0f,$67,$43,$fd,$65,
    $54,$0e,$d7,$81,$4c,$cb,$57,$4c,$15,$79,$be,$14,$1b,$96,$e9,$f5,$af,$f9,$1b,$2a,$c2,$51,$55,$9b,$bb,
    $0e,$ed,$1b,$61,$7f,$34,$f3,$bd,$e6,$ac,$a5,$4a,$d5,$7e,$0f,$15,$b8,$65,$28,$e8,$e8,$44,$d7,$26,$70,
    $3e,$69,$de,$03,$c8,$05,$02,$eb,$1f,$c8,$bc,$97,$13,$6f,$c9,$03,$87,$f2,$e2,$0f,$f7,$b0,$07,$90,$8b,
    $e8,$8d,$89,$78,$d0,$39,$62,$a6,$4e,$f6,$fb,$e3,$90,$66,$b7,$e1,$97,$ca,$21,$cc,$05,$97,$bc,$a5,$70,
    $56,$e5,$46,$bb,$56,$0c,$c2,$12,$68,$3c,$fe,$20,$1f,$88,$b9,$be,$ed,$1a,$2b,$84,$49,$1a,$ca,$10,$2d,
    $4b,$5d,$ce,$cb,$55,$c7,$84,$d1,$e4,$a9,$62,$74,$41,$e6,$48,$dc,$e4,$6c,$d4,$70,$d6,$12,$64,$8b,$62,
    $57,$7e,$a9,$35,$ed,$9a,$15,$98,$4a,$f6,$94,$c6,$b9,$c2,$76,$bb,$4b,$e5,$07,$f3,$52,$fb,$9b,$92,$01,
    $71,$8b,$0d,$9b,$97,$5f,$4e,$3a,$e2,$23,$92,$3e,$90,$05,$11,$b6,$36,$5d,$9c,$6c,$01,$e6,$4a,$ce,$dc,
    $4b,$c8,$ef,$4a,$b5,$35,$eb,$54,$04,$c9,$e4,$7c,$be,$51,$1f,$d4,$c4,$ce,$b0,$32,$64,$71,$e5,$29,$39,
    $bc,$a4,$a7,$55,$5c,$be,$fd,$1b,$3d,$9c,$b3,$5c,$fa,$d5,$2c,$11,$ec,$35,$0b,$11,$f8,$c5,$73,$28,$25,
    $65,$fc,$86,$2f,$77,$70,$e2,$29,$0b,$7e,$dd,$a7,$4a,$41,$f4,$14,$3a,$57,$95,$ce,$aa,$57,$44,$60,$a9,
    $24,$3b,$c6,$40,$bb,$19,$07,$1f,$33,$f0,$33,$d6,$7f,$a6,$f1,$a9,$23,$a3,$9d,$3d,$c2,$26,$47,$87,$1f,
    $d7,$3b,$f2,$66,$9c,$e7,$af,$37,$65,$63,$ad,$d8,$fa,$03,$47,$6d,$bf,$38,$a2,$5b,$d3,$1e,$59,$45,$5b,
    $89,$8e,$32,$d8,$a3,$bc,$66,$0f,$6a,$65,$4d,$3b,$a6,$9f,$0e,$f6,$6d,$6c,$0d,$54,$82,$4d,$eb,$c8,$73,
    $39,$b1,$ad,$1e,$0b,$64,$d7,$98,$0d,$a8,$dd,$3a,$8a,$5a,$c2,$27,$71,$29,$cf,$fc,$36,$a2,$ae,$e7,$b4,
    $64,$49,$85,$6f,$17,$83,$1e,$cc,$14,$e8,$44,$4a,$87,$cd,$e8,$81,$e1,$f8,$cc,$0e,$57,$68,$c0,$28,$3b,
    $12,$fa,$97,$5e,$7d,$48,$c2,$e8,$e9,$e1,$bd,$49,$50,$90,$b1,$32,$da,$fc,$97,$e2,$f6,$28,$e5,$25,$85,
    $ee,$d2,$1e,$dd,$85,$d1,$44,$4f,$42,$ec,$61,$cd,$2f,$bb,$7e,$4e,$57,$59,$bd,$a1,$a1,$63,$be,$0f,$d3,
    $dd,$e6,$13,$94,$2f,$53,$79,$1d,$1a,$b5,$cf,$e6,$00,$42,$cd,$b8,$99,$7c,$0a,$80,$0e,$35,$17,$bb,$14,
    $57,$f5,$0d,$6c,$46,$64,$d6,$67,$d4,$01,$b6,$c1,$65,$d6,$e3,$8f,$68,$cb,$83,$05,$3d,$30,$8c,$9b,$62,
    $00,$90,$f9,$01,$3b,$82,$77,$52,$c2,$5e,$dd,$1f,$f4,$6a,$bb,$9b,$21,$7d,$2a,$ba,$50,$2e,$64,$25,$47,
    $c6,$1a,$4d,$c0,$e8,$ba,$c1,$c8,$e2,$d8,$4f,$22,$06,$dd,$f3,$0e,$14,$5c,$a3,$f8,$cd,$48,$02,$4d,$99,
    $79,$cf,$97,$08,$7f,$fe,$d6,$22,$b9,$d2,$92,$da,$74,$4c,$d5,$44,$e4,$26,$03,$0f,$db,$ae,$8d,$8f,$a0,
    $60,$95,$95,$0c,$9e,$9c,$f5,$7e,$d5,$fa,$56,$a9,$05,$50,$ea,$5e,$5f,$69,$ea,$cb,$49,$cd,$da,$bd,$40,
    $87,$9a,$b6,$64,$ce,$4b,$ae,$43,$55,$77,$8a,$8b,$24,$7f,$f3,$f1,$1a,$ae,$20,$0a,$f2,$02,$83,$3a,$f7,
    $99,$37,$29,$ae,$9f,$fe,$43,$2a,$2a,$b4,$8b,$29,$f5,$71,$75,$40,$75,$9b,$1f,$0e,$67,$dc,$ae,$33,$2d,
    $1e,$b8,$22,$e0,$54,$52,$b6,$c3,$20,$24,$45,$12,$79,$14,$a2,$4a,$6b,$d7,$60,$f2,$44,$08,$22,$9a,$94,
    $20,$4a,$e3,$df,$4d,$94,$3f,$f5,$23,$ad,$f8,$36,$b4,$65,$c8,$bf,$3c,$ee,$36,$a0,$3d,$d4,$ce,$90,$15,
    $f3,$2d,$df,$63,$9d,$eb,$57,$d3,$a8,$08,$99,$00,$20,$71,$a0,$87,$8f,$c5,$8f,$1b,$9e,$10,$6b,$6c,$38,
    $94,$11,$99,$a8,$1a,$60,$6a,$28,$ee,$5c,$43,$c6,$3f,$9a,$e4,$ea,$95,$92,$3b,$b5,$a3,$e7,$39,$7e,$08,
    $b5,$fb,$ac,$1a,$12,$83,$de,$3d,$fc,$87,$fd,$43,$b9,$d0,$1b,$8d,$a0,$37,$fb,$3a,$0d,$54,$70,$db,$b3,
    $57,$bd,$55,$37,$12,$3a,$97,$de,$8b,$b6,$6b,$f8,$bd,$69,$d5,$b2,$21,$32,$c3,$89,$06,$1f,$99,$f2,$b6,
    $25,$95,$20,$55,$f1,$0d,$f7,$6a,$d6,$f0,$59,$8e,$40,$ab,$32,$75,$2e,$94,$cb,$f1,$32,$f9,$2a,$57,$b8,
    $cf,$e2,$39,$b4,$4f,$9e,$ef,$bb,$84,$49,$fb,$6c,$67,$fc,$2d,$f7,$f0,$49,$1c,$e4,$e8,$da,$7c,$15,$0f,
    $5f,$7d,$84,$77,$ed,$68,$98,$9d,$7d,$52,$de,$a0,$de,$9c,$e6,$43,$64,$d6,$dd,$e8,$06,$88,$fb,$ab,$7c,
    $0a,$8b,$18,$71,$9d,$1a,$64,$73,$19,$d2,$57,$d3,$fe,$50,$2c,$18,$5b,$dc,$6c,$ae,$9f,$f2,$d2,$43,$b0,
    $01,$78,$91,$24,$ba,$31,$d6,$42,$56,$8e,$1f,$62,$0c,$75,$40,$80,$e4,$2a,$b9,$75,$5c,$5e,$f5,$e8,$7b,
    $74,$0f,$4e,$70,$e7,$85,$10,$f7,$d3,$77,$31,$a6,$d0,$3b,$e5,$cd,$50,$29,$86,$6b,$f2,$ef,$20,$c0,$73,
    $fc,$84,$4b,$8a,$a5,$d2,$d6,$79,$76,$4f,$2b,$42,$56,$b3,$a2,$bc,$90,$d7,$56,$8f,$82,$65,$de,$5f,$e9,
    $ac,$35,$a1,$55,$5e,$62,$c6,$54,$56,$57,$34,$bf,$6f,$46,$36,$ce,$de,$5e,$1d,$36,$a8,$75,$0e,$53,$0d,
    $7c,$14,$22,$1b,$d3,$b7,$81,$83,$97,$8c,$07,$72,$12,$16,$2e,$0e,$3b,$51,$b9,$7a,$56,$13,$94,$8e,$c0,
    $53,$f9,$c2,$7f,$1a,$3e,$37,$38,$9e,$e4,$22,$e1,$2d,$23,$0b,$89,$f6,$03,$0b,$e7,$7e,$04,$5f,$40,$de,
    $2b,$18,$1d,$88,$1c,$a5,$a4,$a7,$28,$f0,$ff,$e2,$13,$16,$2d,$2a,$c9,$b9,$c2,$a8,$8c,$9b,$4d,$eb,$41,
    $c4,$b8,$51,$3e,$c9,$db,$e9,$f2,$67,$85,$b3,$78,$e6,$7b,$d5,$2b,$33,$8b,$43,$16,$25,$25,$61,$6f,$d2,
    $9c,$bc,$8c,$e4,$b9,$8b,$5b,$3a,$10,$26,$f4,$f1,$7a,$07,$d8,$2b,$83,$b4,$ba,$f7,$56,$75,$59,$e0,$41,
    $ad,$66,$5c,$d8,$5a,$e5,$48,$26,$47,$e9,$b6,$6f,$67,$9b,$c8,$34,$32,$28,$a8,$47,$b2,$89,$98,$fa,$f9,
    $e6,$2f,$bc,$32,$c8,$d2,$6c,$a2,$94,$61,$86,$0c,$95,$2b,$4b,$c2,$98,$e3,$b8,$28,$2f,$07,$ac,$9a,$90,
    $f6,$ae,$a2,$0e,$51,$34,$4a,$de,$44,$08,$76,$23,$8b,$57,$0b,$ae,$d9,$b9,$72,$78,$4a,$3f,$6c,$53,$5f,
    $f9,$3b,$3b,$a3,$18,$7c,$e0,$27,$4e,$98,$d4,$4f,$3a,$99,$a4,$e3,$7f,$df,$bb,$24,$2d,$fe,$d8,$95,$91,
    $5c,$87,$51,$a9,$83,$74,$a5,$aa,$c8,$e0,$45,$58,$6a,$af,$4e,$31,$d3,$e8,$e5,$f8,$74,$f1,$d9,$26,$33,
    $82,$7c,$c2,$a3,$95,$19,$57,$54,$d6,$7b,$59,$3e,$f4,$d3,$5c,$39,$f8,$7d,$59,$41,$56,$fd,$a1,$b4,$e7,
    $24,$75,$92,$34,$38,$29,$ac,$ba,$b1,$65,$95,$f8,$0c,$6e,$18,$f9,$de,$8a,$47,$41,$2e,$a1,$99,$20,$1e,
    $61,$4a,$bf,$f9,$05,$ec,$2b,$fc,$53,$ce,$17,$cd,$d2,$21,$bb,$8e,$84,$a3,$6e,$f3,$60,$6f,$19,$7f,$b5,
    $68,$ff,$cb,$21,$b4,$60,$ad,$60,$f7,$ab,$22,$dc,$f0,$1f,$d7,$aa,$b5,$d6,$7c,$cf,$0d,$82,$92,$fb,$7c,
    $7f,$7f,$fa,$8d,$2f,$25,$5d,$6f,$2a,$22,$9c,$ce,$a5,$d3,$74,$4f,$96,$9b,$8c,$19,$eb,$87,$11,$1c,$6e,
    $2c,$f5,$b3,$80,$58,$10,$94,$93,$d6,$b2,$29,$f3,$f6,$10,$9d,$69,$c3,$9a,$24,$74,$dd,$a2,$32,$5a,$8c,
    $61,$a8,$05,$97,$e5,$dd,$a1,$5d,$fb,$58,$d0,$c6,$8a,$b0,$f7,$9b,$5a,$6e,$76,$a2,$3a,$35,$50,$ef,$bc,
    $fc,$84,$35,$eb,$e8,$fc,$ae,$d9,$ce,$d6,$eb,$2c,$ce,$cb,$53,$90,$eb,$8a,$6a,$7f,$93,$27,$f7,$2d,$7a,
    $ce,$14,$31,$e4,$a6,$45,$d8,$29,$39,$5a,$12,$b6,$01,$ba,$1b,$52,$25,$49,$f9,$0a,$f2,$9c,$d8,$a8,$87,
    $e4,$3f,$15,$8c,$ef,$b9,$74,$8a,$59,$9d,$26,$c1,$d9,$f5,$fc,$ec,$1d,$a7,$1f,$55,$9d,$87,$3b,$79,$d7,
    $8b,$b5,$b9,$5a,$71,$a3,$52,$72,$b4,$0c,$4f,$97,$5e,$96,$2c,$5c,$16,$0a,$5f,$b8,$5d,$08,$6b,$37,$ff,
    $35,$8d,$fa,$d3,$53,$10,$4d,$9b,$84,$b9,$7c,$4e,$ec,$d4,$87,$ce,$f3,$28,$e2,$0e,$85,$ec,$8a,$ab,$6d,
    $36,$6a,$43,$7f,$37,$6b,$61,$d2,$5c,$e1,$e8,$cf,$10,$09,$71,$7e,$e4,$b6,$03,$39,$81,$c3,$5c,$31,$f6,
    $a7,$87,$1c,$25,$1b,$77,$87,$0c,$00,$bd,$e8,$25,$00,$2d,$dc,$3f,$2d,$9e,$8c,$7a,$35,$26,$78,$c2,$40,
    $fe,$9e,$9c,$36,$8b,$37,$f4,$96,$3d,$d3,$74,$dc,$cd,$6c,$d0,$3c,$d0,$e4,$d8,$f1,$f5,$f5,$fa,$db,$2f,
    $97,$57,$a1,$03,$b6,$9e,$5c,$8e,$9d,$b1,$a2,$0e,$53,$43,$94,$96,$01,$14,$a7,$80,$e1,$1d,$e4,$c5,$11,
    $15,$92,$86,$34,$5e,$b1,$5c,$b7,$4b,$48,$05,$c6,$32,$28,$b8,$2e,$58,$88,$5b,$2c,$c7,$6b,$0b,$67,$a1,
    $4f,$c9,$6b,$82,$7d,$19,$2e,$06,$8a,$76,$96,$5c,$fc,$74,$8f,$1f,$8a,$f6,$21,$e8,$01,$53,$22,$55,$5c,
    $86,$eb,$98,$b8,$28,$a0,$27,$59,$fc,$c1,$3f,$10,$b9,$79,$86,$26,$14,$16,$d2,$92,$f1,$7e,$8f,$38,$7d,
    $06,$1f,$6b,$8c,$45,$28,$03,$70,$87,$20,$49,$55,$33,$49,$01,$02,$2a,$35,$93,$73,$f7,$9b,$4e,$35,$65,
    $5f,$92,$50,$0d,$d1,$8a,$3a,$37,$d6,$37,$d3,$da,$b9,$28,$98,$4c,$9d,$d7,$cb,$df,$99,$af,$d3,$3d,$e7,
    $8f,$47,$85,$61,$49,$25,$5d,$0a,$10,$f7,$9c,$dc,$31,$97,$f9,$9e,$db,$22,$88,$1d,$e3,$39,$ae,$58,$b3,
    $bb,$dd,$e4,$2a,$49,$e0,$6e,$b2,$92,$b5,$ea,$6a,$76,$92,$f7,$68,$2a,$29,$eb,$f1,$46,$84,$56,$1c,$42,
    $1e,$e4,$be,$a3,$5a,$78,$3b,$53,$df,$7d,$18,$0c,$a6,$92,$fc,$5b,$af,$ac,$c8,$27,$fb,$e2,$12,$51,$b8,
    $d8,$07,$5e,$93,$0a,$02,$e2,$aa,$53,$8f,$62,$f0,$5b,$1a,$ae,$b3,$f0,$15,$91,$bd,$9a,$04,$4b,$a6,$ff,
    $49,$43,$99,$1e,$9a,$12,$13,$8b,$2b,$0c,$91,$31,$4c,$97,$1f,$13,$2d,$44,$2f,$20,$11,$fd,$37,$cb,$69,
    $2f,$da,$15,$c9,$1b,$f4,$2d,$18,$8d,$3e,$e7,$98,$5e,$2f,$7f,$16,$3f,$ac,$a6,$a0,$11,$2a,$61,$d8,$a8,
    $e3,$71,$b0,$b5,$c0,$e6,$c6,$cc,$b7,$5a,$b9,$ae,$1e,$d4,$d6,$67,$46,$f7,$6f,$42,$55,$be,$11,$47,$d7,
    $5c,$30,$c3,$ec,$87,$d5,$54,$a6,$b1,$2e,$99,$a7,$97,$b5,$48,$14,$6b,$1c,$5d,$60,$d4,$72,$15,$85,$60,
    $9e,$51,$38,$1c,$c5,$25,$0a,$05,$74,$1f,$47,$27,$88,$03,$f0,$66,$dd,$f5,$e4,$73,$54,$7e,$0a,$50,$11,
    $6b,$07,$b0,$bc,$2c,$bb,$7b,$af,$80,$af,$ad,$2e,$4e,$4f,$91,$5c,$f7,$ec,$da,$70,$8a,$b5,$8e,$fb,$d1,
    $67,$7b,$f2,$a7,$bf,$17,$69,$9b,$60,$67,$6a,$65,$d8,$73,$e0,$7d,$4f,$95,$c3,$83,$5c,$fb,$c3,$b2,$17,
    $2e,$59,$65,$a8,$05,$1e,$ba,$41,$c1,$cc,$ce,$9b,$4d,$65,$4a,$d2,$76,$6c,$fc,$9c,$65,$ef,$e3,$c7,$ca,
    $2b,$24,$b8,$4b,$c7,$b2,$3b,$0e,$fc,$36,$52,$d4,$1e,$e3,$32,$93,$c0,$74,$d6,$36,$c8,$80,$98,$a3,$e1,
    $34,$b8,$c9,$5a,$fe,$c3,$c5,$b4,$f1,$f8,$c8,$d0,$9f,$d2,$e9,$04,$77,$66,$93,$5b,$cf,$18,$22,$c8,$30,
    $86,$07,$80,$e0,$2c,$46,$b6,$f4,$ae,$d4,$c2,$70,$8d,$5b,$79,$93,$78,$ff,$a0,$b6,$87,$fd,$67,$41,$37,
    $c1,$e1,$c0,$c4,$43,$22,$81,$c0,$ea,$0e,$b2,$e1,$ad,$14,$a3,$ff,$ef,$13,$6d,$07,$49,$8d,$a0,$2f,$41,
    $46,$5a,$a1,$60,$94,$20,$83,$23,$ef,$27,$5d,$58,$4b,$24,$a6,$a0,$10,$6f,$50,$3f,$78,$80,$de,$9a,$6c,
    $58,$3f,$58,$54,$18,$84,$ca,$34,$a1,$42,$b9,$91,$e0,$7c,$25,$70,$28,$e8,$f8,$5b,$b8,$22,$b3,$1e,$e2,
    $84,$68,$ca,$a5,$6a,$7a,$50,$f3,$52,$46,$24,$4f,$dc,$03,$82,$eb,$00,$49,$cb,$a5,$80,$c6,$6d,$c5,$34,
    $ed,$24,$01,$e2,$c6,$86,$50,$69,$33,$b3,$c2,$88,$2f,$d2,$11,$0d,$7a,$5b,$4f,$b1,$7b,$27,$bb,$82,$bb,
    $4d,$0f,$e3,$fd,$df,$ab,$0f,$05,$5d,$69,$00,$48,$58,$f1,$1e,$d9,$7f,$4e,$a2,$44,$3b,$10,$80,$8d,$db,
    $e1,$1d,$c1,$be,$91,$d2,$9f,$05,$55,$01,$02,$fd,$09,$e1,$2c,$fc,$f4,$df,$26,$a4,$b8,$e3,$a4,$fd,$a1,
    $a5,$37,$b4,$fd,$a9,$dd,$1e,$bf,$af,$af,$44,$e1,$09,$f6,$dc,$6a,$c8,$12,$fb,$08,$fc,$f0,$42,$e9,$b7,
    $1d,$c9,$3b,$6a,$9d,$79,$ac,$19,$29,$00,$4d,$5d,$b7,$d4,$a2,$75,$6e,$a4,$19,$55,$af,$31,$a5,$c0,$e3,
    $dc,$12,$89,$79,$d8,$9d,$1d,$8c,$36,$8e,$09,$9e,$8a,$62,$9f,$e3,$ea,$46,$b4,$1a,$c6,$c9,$ff,$d3,$aa,
    $98,$aa,$cf,$b7,$0e,$96,$84,$9c,$20,$a4,$3d,$a7,$bf,$11,$ba,$11,$b9,$48,$e7,$bd,$45,$6a,$c3,$7d,$4e,
    $fc,$fd,$a8,$97,$4b,$ee,$f5,$8d,$d9,$4b,$45,$e4,$63,$d2,$f1,$24,$52,$4d,$e1,$16,$8d,$29,$77,$db,$58,
    $fa,$44,$fb,$f2,$66,$d5,$f1,$4a,$d0,$5e,$42,$88,$15,$4f,$01,$f2,$ab,$33,$2f,$d5,$91,$49,$80,$49,$a4,
    $f2,$2b,$42,$85,$7b,$5f,$e3,$de,$45,$98,$12,$5c,$57,$1f,$70,$29,$ee,$f5,$3e,$c6,$aa,$d1,$fc,$58,$97,
    $0d,$af,$09,$98,$aa,$3c,$8e,$f8,$48,$31,$2d,$33,$44,$06,$bc,$ef,$20,$f0,$26,$af,$c4,$22,$3b,$39,$29,
    $20,$4f,$c3,$fc,$d8,$a9,$47,$a7,$61,$35,$f9,$68,$4a,$9e,$81,$ac,$89,$59,$85,$68,$81,$0b,$08,$b1,$f5,
    $85,$1f,$74,$b0,$7f,$07,$f5,$9c,$ef,$4e,$47,$20,$0b,$f5,$67,$a2,$7c,$c3,$1c,$3f,$42,$8c,$57,$72,$33,
    $b3,$d5,$e2,$0d,$01,$aa,$d2,$33,$e9,$42,$65,$10,$32,$d8,$ce,$7e,$77,$ef,$cf,$07,$39,$53,$a1,$21,$94,
    $64,$45,$67,$ed,$ce,$30,$97,$af,$11,$32,$36,$6f,$fc,$4a,$3d,$c6,$a4,$59,$a8,$85,$63,$da,$f6,$26,$30,
    $e6,$5e,$6d,$63,$1a,$fc,$e3,$9e,$f0,$15,$98,$3b,$9f,$bc,$5a,$1e,$92,$27,$49,$16,$bd,$85,$be,$95,$28,
    $c9,$6a,$7c,$94,$f5,$67,$96,$be,$51,$9a,$55,$0b,$b3,$f0,$e4,$17,$f2,$5d,$23,$e6,$2b,$da,$b7,$57,$96,
    $55,$31,$81,$5f,$f5,$30,$e3,$7d,$8d,$68,$f6,$72,$fe,$ae,$5c,$1c,$d0,$26,$c1,$5d,$37,$fd,$03,$40,$e4,
    $a8,$cd,$53,$6b,$92,$24,$bf,$b2,$c9,$bd,$40,$51,$2d,$d1,$a4,$86,$4c,$7a,$22,$1e,$8f,$4c,$fd,$be,$92,
    $dd,$ed,$1f,$50,$54,$b5,$3d,$38,$d1,$f3,$82,$5c,$10,$6a,$bf,$a8,$d8,$d5,$60,$9a,$be,$02,$6f,$34,$04,
    $9b,$bb,$e3,$3c,$84,$20,$f8,$20,$4d,$68,$b4,$1e,$41,$53,$9a,$54,$90,$1a,$59,$25,$b3,$fe,$ea,$87,$eb,
    $64,$56,$58,$ea,$a1,$1f,$2d,$02,$b0,$77,$44,$91,$11,$b7,$95,$bc,$c0,$84,$08,$19,$56,$cd,$bc,$f8,$04,
    $a5,$ed,$d5,$10,$ef,$dd,$85,$9c,$95,$86,$9e,$10,$81,$ac,$67,$08,$bf,$55,$bb,$ea,$25,$5b,$b2,$23,$72,
    $1f,$4d,$bb,$8b,$d6,$f4,$af,$7f,$1d,$e5,$3b,$6e,$9e,$81,$29,$18,$d4,$b9,$01,$f5,$03,$6a,$c2,$6b,$e1,
    $8f,$65,$bc,$e4,$87,$06,$47,$6e,$b7,$07,$36,$66,$74,$f8,$b0,$4c,$98,$bb,$7f,$af,$58,$ca,$30,$3b,$28,
    $23,$74,$27,$fe,$5f,$49,$b0,$2a,$b9,$fc,$7c,$af,$54,$7d,$e9,$e8,$f9,$7c,$3b,$73,$3e,$ca,$28,$89,$a8,
    $9e,$56,$6b,$fe,$02,$ed,$e7,$e9,$a9,$9e,$71,$84,$cc,$9d,$d9,$5f,$f5,$8c,$8d,$cb,$3f,$c4,$49,$7c,$e3,
    $70,$85,$54,$8c,$d9,$e2,$d9,$71,$cd,$f7,$24,$12,$98,$0f,$c9,$ff,$e2,$65,$43,$44,$95,$83,$65,$dd,$9c,
    $9a,$19,$f6,$5c,$5b,$42,$78,$fc,$7f,$a7,$53,$8d,$80,$f1,$8a,$cc,$6f,$70,$0f,$e6,$db,$8d,$ed,$5e,$cc,
    $b6,$ab,$2b,$53,$60,$44,$85,$7b,$70,$06,$99,$ed,$fd,$89,$94,$cf,$49,$c5,$53,$ed,$27,$10,$f9,$08,$be,
    $f0,$8a,$37,$78,$be,$24,$04,$8e,$29,$51,$28,$f8,$98,$89,$e4,$81,$02,$d2,$08,$c2,$29,$60,$fc,$a3,$dd,
    $5b,$37,$7a,$e5,$a6,$ba,$0d,$80,$c6,$6c,$71,$24,$39,$3b,$49,$b7,$1b,$e7,$3c,$9e,$cf,$c7,$5a,$af,$32,
    $5e,$e9,$41,$cc,$15,$fb,$55,$3c,$c5,$7f,$91,$a0,$0b,$e5,$2d,$c0,$1d,$02,$ac,$a7,$25,$58,$e0,$da,$b5,
    $58,$9d,$32,$a7,$1b,$4c,$5e,$ae,$93,$f6,$51,$d3,$b1,$79,$af,$0c,$5b,$28,$04,$d4,$43,$8c,$e4,$76,$a2,
    $b1,$86,$cb,$af,$26,$bf,$c3,$5f,$16,$b8,$6e,$95,$b1,$ed,$dd,$fb,$65,$96,$e9,$4f,$e2,$29,$59,$f8,$e0,
    $49,$32,$b9,$18,$e9,$f5,$49,$31,$63,$ba,$78,$0f,$da,$8b,$3b,$28,$ab,$e3,$ca,$38,$d7,$08,$e8,$14,$e0,
    $da,$55,$de,$e8,$4e,$82,$d8,$63,$b5,$0b,$1e,$e6,$cb,$c7,$ab,$20,$99,$7b,$ed,$0c,$c1,$25,$33,$b0,$94,
    $50,$b9,$f2,$d9,$ae,$c7,$b3,$4b,$ff,$3a,$36,$9d,$03,$9f,$60,$ef,$55,$ef,$ab,$92,$13,$d2,$45,$f1,$07,
    $3b,$b0,$39,$1c,$fa,$9f,$ac,$4a,$c9,$30,$52,$42,$8e,$68,$92,$6a,$27,$0a,$f4,$6e,$f8,$90,$ab,$8e,$dd,
    $f2,$1a,$3b,$59,$c3,$6a,$03,$58,$f0,$74,$61,$76,$e4,$e1,$bc,$a7,$17,$e4,$42,$d1,$0a,$88,$ad,$49,$0f,
    $95,$79,$33,$f8,$8c,$44,$4c,$eb,$20,$8c,$e2,$e6,$46,$da,$59,$47,$eb,$66,$53,$28,$ed,$d6,$ef,$76,$f7,
    $0e,$ad,$a8,$31,$7c,$3e,$b2,$fb,$be,$d4,$ef,$04,$5f,$df,$9e,$91,$24,$f2,$93,$be,$e1,$dc,$a9,$45,$fe,
    $27,$cf,$1d,$95,$d4,$d2,$e7,$72,$9f,$c9,$d1,$df,$54,$8d,$8f,$71,$68,$4d,$3a,$be,$7d,$6e,$d9,$ed,$cd,
    $f7,$6d,$0b,$6f,$41,$17,$e7,$02,$17,$58,$5b,$de,$3d,$51,$28,$87,$01,$9d,$9e,$a5,$55,$c3,$42,$a4,$8f,
    $eb,$dc,$ee,$3b,$12,$09,$e5,$2e,$dc,$d5,$35,$e4,$83,$39,$68,$10,$14,$24,$75,$ed,$0d,$6b,$72,$42,$83,
    $3a,$54,$af,$c1,$bf,$87,$3a,$ee,$60,$2e,$01,$dd,$82,$c1,$ae,$22,$f4,$1c,$fb,$55,$ae,$3a,$d5,$0b,$26,
    $2a,$b6,$1d,$41,$12,$36,$a9,$04,$2b,$fb,$4c,$fb,$b4,$77,$97,$77,$4b,$4c,$df,$f2,$58,$61,$a9,$bf,$6c,
    $45,$73,$dc,$69,$08,$9c,$89,$de,$8f,$37,$c1,$97,$46,$51,$5a,$07,$1a,$4a,$cf,$16,$2d,$5a,$1c,$ea,$c0,
    $65,$2f,$c7,$13,$f1,$6a,$25,$c1,$54,$41,$71,$12,$d2,$07,$f0,$58,$cf,$4c,$2e,$74,$53,$72,$97,$40,$8c,
    $0e,$c9,$eb,$7c,$e5,$9e,$3f,$19,$53,$83,$50,$20,$99,$03,$49,$ec,$f4,$ee,$6b,$bc,$1b,$c0,$bf,$0b,$fc,
    $1f,$37,$6a,$98,$ea,$e8,$27,$1b,$fd,$d9,$d4,$d6,$80,$d4,$c5,$a1,$8a,$ef,$2c,$5a,$9c,$2a,$1f,$46,$c2,
    $97,$9e,$14,$6b,$ac,$cc,$a8,$4a,$5e,$8b,$53,$7a,$24,$c0,$19,$aa,$ca,$59,$cb,$12,$0c,$b7,$cb,$a6,$83,
    $d6,$d6,$f8,$b8,$6c,$72,$ea,$ec,$95,$c7,$a9,$bf,$fd,$d8,$7b,$81,$f8,$30,$ea,$b3,$69,$2f,$d4,$34,$49,
    $b3,$a7,$a5,$af,$b5,$9d,$d6,$e2,$1b,$26,$bb,$ad,$0b,$34,$29,$15,$90,$50,$2d,$b7,$71,$74,$30,$ab,$88,
    $05,$93,$fe,$0a,$2b,$49,$ce,$27,$2b,$1d,$f6,$e8,$a0,$d8,$33,$27,$93,$75,$1e,$90,$00,$54,$bb,$62,$b3,
    $df,$6d,$4a,$6a,$5f,$8c,$78,$0b,$60,$3c,$23,$46,$6a,$99,$72,$b8,$88,$5a,$76,$11,$2d,$a7,$f6,$5f,$3d,
    $7d,$2f,$ca,$8b,$c3,$86,$1e,$ec,$90,$04,$2f,$47,$00,$46,$db,$d7,$05,$52,$e5,$03,$2c,$74,$8a,$47,$d5,
    $71,$70,$64,$32,$21,$54,$65,$a2,$f0,$88,$e6,$30,$ee,$c3,$27,$ca,$1e,$b4,$38,$1b,$f3,$51,$2f,$b7,$12,
    $b0,$20,$4c,$9a,$4a,$c9,$13,$26,$99,$08,$e9,$4c,$a2,$ad,$2b,$a6,$ed,$5a,$5e,$81,$1f,$99,$88,$9a,$8f,
    $ea,$6b,$36,$71,$45,$aa,$87,$8f,$9e,$da,$d4,$21,$99,$4f,$69,$4f,$d0,$f1,$54,$48,$cf,$64,$a4,$2e,$d8,
    $26,$7f,$d6,$d3,$3e,$68,$68,$8c,$00,$72,$28,$38,$88,$29,$d5,$f1,$2e,$f7,$78,$4b,$40,$83,$ac,$08,$5b,
    $42,$f2,$08,$87,$4d,$ee,$68,$1c,$df,$ad,$96,$6c,$bf,$11,$e8,$f8,$6b,$f2,$51,$e4,$d6,$3b,$3c,$3c,$3b,
    $51,$fc,$cf,$cf,$1e,$bd,$f6,$f4,$5e,$73,$5e,$8b,$9f,$fe,$25,$e2,$89,$3c,$fb,$cf,$43,$4b,$76,$ee,$99,
    $1d,$e1,$8c,$19,$a3,$3d,$88,$23,$38,$c2,$eb,$15,$db,$00,$4b,$58,$78,$4d,$95,$00,$7c,$4f,$1d,$5c,$63,
    $61,$f5,$10,$94,$fe,$38,$3d,$b6,$cb,$a0,$8c,$27,$97,$1a,$c8,$78,$54,$0b,$43,$23,$55,$4a,$f6,$30,$96,
    $bd,$15,$a6,$7b,$ad,$82,$8e,$55,$5f,$5e,$3c,$91,$9c,$4e,$b2,$b4,$8a,$1f,$a4,$65,$d5,$9e,$c4,$ee,$c3,
    $9d,$2f,$3a,$e8,$46,$72,$b8,$77,$b9,$6e,$51,$e7,$bd,$ca,$ee,$f4,$21,$0c,$78,$9c,$73,$32,$dc,$7e,$bc,
    $5b,$5a,$a0,$a0,$7b,$47,$ee,$44,$da,$f7,$b2,$2b,$c5,$5c,$cf,$35,$e7,$99,$b3,$50,$63,$04,$7a,$11,$69,
    $cb,$50,$02,$94,$db,$85,$08,$d4,$b7,$84,$1e,$65,$42,$22,$43,$60,$69,$ba,$a3,$d2,$3d,$5d,$2b,$2e,$46,
    $11,$4c,$ba,$6e,$c0,$21,$f3,$df,$4d,$09,$54,$d1,$45,$fe,$cd,$d6,$d1,$9a,$96,$c6,$88,$14,$d0,$e0,$59,
    $24,$34,$5d,$57,$4c,$c5,$af,$ed,$36,$ae,$a1,$db,$fb,$88,$c7,$8f,$65,$8f,$8c,$3c,$74,$c1,$d9,$5e,$30,
    $af,$e3,$7b,$09,$ed,$99,$78,$2b,$3e,$d6,$06,$66,$8b,$e2,$b8,$c6,$99,$7e,$39,$e9,$0c,$bb,$22,$f1,$b1,
    $ab,$22,$02,$26,$89,$32,$8b,$95,$d3,$1c,$e6,$2e,$de,$a4,$14,$99,$a2,$d5,$3f,$1a,$c0,$2a,$9f,$70,$0d,
    $a5,$18,$1a,$60,$cf,$ab,$ea,$7c,$c9,$ef,$ab,$9b,$80,$9e,$c5,$99,$1f,$23,$23,$8c,$b3,$55,$dc,$14,$cb,
    $0d,$18,$9e,$63,$d2,$53,$59,$58,$a8,$07,$04,$dc,$3a,$18,$8a,$f5,$dd,$51,$ed,$d1,$59,$39,$b8,$4b,$4a,
    $f2,$72,$f1,$89,$b5,$81,$07,$c3,$3b,$25,$9a,$15,$34,$9e,$a8,$8d,$40,$7b,$28,$e7,$a2,$ec,$65,$23,$8c,
    $c8,$b2,$de,$7e,$65,$f3,$f9,$d5,$5d,$cd,$42,$7d,$75,$4a,$ca,$fe,$db,$67,$25,$35,$73,$1a,$5d,$6e,$a5,
    $33,$ab,$77,$13,$99,$6a,$c9,$20,$40,$86,$87,$a4,$f7,$3c,$e2,$c6,$6c,$d5,$1b,$ea,$9e,$dd,$a4,$6b,$cd,
    $58,$13,$28,$34,$ac,$39,$ca,$80,$94,$10,$84,$9f,$c1,$79,$ec,$18,$d4,$13,$db,$03,$be,$bc,$3f,$f4,$e2,
    $33,$6e,$4a,$e5,$c5,$1c,$76,$68,$be,$22,$15,$53,$ef,$d2,$09,$8b,$64,$85,$d4,$f0,$06,$70,$7d,$e7,$8c,
    $0a,$39,$40,$2f,$06,$0b,$5b,$95,$9f,$b8,$e5,$d6,$96,$aa,$b7,$50,$c7,$d0,$e4,$89,$9e,$74,$27,$f9,$9e,
    $89,$f4,$74,$39,$0d,$37,$f2,$d4,$11,$ce,$6e,$05,$a0,$17,$e1,$ca,$46,$e1,$22,$1f,$2f,$11,$4e,$21,$73,
    $6c,$db,$10,$f5,$26,$3b,$33,$d1,$0f,$84,$be,$e7,$c3,$da,$de,$98,$35,$9f,$60,$c4,$21,$7b,$a7,$fa,$cd,
    $7c,$1b,$70,$da,$82,$df,$ba,$4d,$31,$ca,$a9,$b1,$d5,$47,$cd,$47,$c4,$87,$88,$b4,$d8,$f8,$52,$66,$c3,
    $dd,$a7,$ef,$d4,$31,$5d,$ea,$0a,$98,$7f,$e9,$c4,$6f,$08,$ed,$50,$e0,$6f,$f1,$49,$ec,$0d,$a1,$b9,$41,
    $9f,$64,$a3,$ce,$36,$5c,$d9,$11,$1b,$e9,$c9,$0b,$9e,$1f,$1a,$07,$1b,$7a,$b1,$4f,$38,$9f,$a0,$94,$82,
    $8d,$5e,$63,$98,$72,$17,$aa,$7f,$58,$fb,$6c,$e0,$bf,$10,$e9,$5e,$16,$2d,$da,$e6,$cc,$f0,$59,$2e,$04,
    $f1,$0e,$8b,$60,$41,$56,$15,$71,$5b,$cf,$c4,$45,$51,$93,$2a,$5d,$f1,$7a,$ab,$8a,$e9,$5e,$3e,$66,$76,
    $70,$2e,$07,$5c,$9d,$3e,$f6,$b5,$77,$f1,$5f,$d6,$35,$4f,$a0,$02,$0a,$b3,$0f,$a8,$2f,$0d,$7f,$65,$6d,
    $7b,$2c,$36,$34,$ea,$47,$18,$02,$c4,$f9,$27,$21,$17,$0a,$8a,$a5,$0f,$e3,$22,$bf,$c5,$54,$f1,$51,$13,
    $36,$5c,$ad,$c1,$63,$25,$e5,$11,$62,$2c,$e9,$12,$df,$80,$5b,$c9,$26,$ab,$3b,$fb,$dd,$ba,$b3,$6f,$e7,
    $9f,$3f,$a4,$97,$66,$a8,$16,$fc,$94,$3c,$e1,$82,$17,$fb,$67,$ca,$c1,$5a,$16,$c6,$bb,$30,$62,$aa,$f2,
    $44,$1f,$04,$2a,$8c,$d2,$3d,$36,$0f,$4b,$d2,$d1,$ac,$03,$e2,$9b,$74,$d2,$de,$f4,$ce,$64,$34,$c6,$b1,
    $40,$93,$de,$bc,$f8,$d1,$e5,$48,$76,$ba,$99,$5b,$a1,$a6,$e3,$2c,$4e,$66,$ac,$27,$8f,$3e,$6f,$3b,$3b,
    $0e,$3f,$1b,$14,$19,$80,$46,$b2,$b7,$8e,$b1,$8a,$11,$ee,$75,$aa,$5f,$d9,$27,$4e,$9e,$09,$a1,$a9,$6a,
    $e3,$b4,$99,$ca,$47,$48,$3f,$10,$43,$3c,$37,$06,$4f,$d4,$2f,$ca,$41,$db,$21,$23,$2b,$6a,$0c,$27,$6e,
    $60,$45,$b7,$60,$ef,$53,$cd,$c0,$3f,$d1,$38,$74,$c4,$ca,$ab,$f3,$a8,$1c,$fa,$2c,$e1,$99,$26,$0c,$b5,
    $80,$4a,$7f,$7d,$95,$b0,$51,$ba,$71,$b7,$87,$a5,$05,$ce,$0b,$92,$45,$0b,$55,$c7,$cf,$80,$e0,$b8,$3a,
    $06,$b6,$1e,$81,$fb,$08,$3b,$fa,$4f,$22,$f6,$98,$92,$d0,$46,$3a,$3a,$85,$e4,$82,$4b,$a2,$7f,$4e,$10,
    $e0,$46,$ef,$a4,$9d,$ae,$83,$9b,$73,$60,$89,$06,$30,$db,$91,$27,$0d,$04,$a0,$46,$89,$64,$d6,$b9,$f4,
    $37,$8c,$3b,$e1,$43,$f1,$d7,$57,$54,$c7,$33,$2d,$e8,$a5,$d4,$80,$02,$4c,$4a,$2c,$aa,$04,$62,$66,$30,
    $dd,$29,$7d,$91,$b3,$1e,$0b,$85,$3e,$ba,$13,$88,$e9,$a4,$1f,$2d,$32,$46,$9b,$84,$a6,$b3,$53,$8f,$dc,
    $89,$d8,$74,$72,$a5,$fd,$db,$71,$a6,$af,$ba,$1f,$bb,$25,$9a,$14,$bf,$a7,$47,$ad,$fe,$8b,$cd,$c6,$28,
    $1a,$e9,$7b,$d6,$2c,$87,$d4,$fe,$e0,$f9,$fa,$aa,$07,$3d,$b7,$05,$2e,$a6,$37,$0f,$00,$61,$4e,$a1,$49,
    $8e,$29,$49,$59,$95,$16,$29,$36,$81,$f0,$30,$db,$0c,$a7,$d5,$61,$7e,$73,$c5,$9d,$9c,$dc,$56,$f1,$11,
    $02,$0a,$f4,$15,$4e,$3c,$fe,$6a,$8b,$e4,$cf,$96,$ff,$10,$2b,$f8,$5d,$7d,$52,$49,$b1,$a7,$ae,$ea,$46,
    $47,$eb,$a5,$5a,$67,$20,$36,$a4,$7a,$c7,$cc,$51,$ab,$f8,$73,$c9,$a8,$c5,$ad,$09,$02,$57,$8c,$e7,$01,
    $28,$27,$e1,$ae,$e2,$27,$e1,$d5,$4f,$50,$e6,$a3,$56,$c0,$53,$67,$56,$30,$03,$40,$e9,$99,$2b,$16,$e4,
    $c4,$5b,$6b,$66,$0c,$75,$77,$aa,$13,$a9,$9d,$44,$54,$83,$eb,$9a,$f3,$2d,$1b,$36,$b0,$1a,$50,$de,$eb,
    $7d,$2f,$3b,$3c,$73,$c0,$ce,$bf,$fc,$f5,$90,$62,$b5,$c7,$cc,$1e,$4b,$f2,$da,$a7,$de,$d6,$f0,$c3,$9f,
    $53,$81,$ff,$a4,$5e,$50,$ba,$cd,$1d,$58,$b2,$f7,$c7,$3e,$22,$db,$19,$a3,$c2,$e6,$9d,$b7,$9b,$6e,$6a,
    $3c,$77,$b2,$26,$8c,$c0,$26,$fd,$5f,$ad,$13,$ed,$5f,$43,$04,$ef,$57,$0f,$c7,$42,$74,$f0,$dd,$56,$44,
    $47,$f9,$97,$b7,$ee,$3f,$15,$71,$f7,$69,$02,$cb,$50,$c3,$82,$7a,$f0,$d1,$77,$ae,$b2,$fe,$f6,$bf,$af,
    $a6,$f8,$0d,$cf,$9f,$b3,$b0,$57,$25,$0f,$d6,$50,$26,$81,$af,$60,$97,$bf,$2e,$51,$d3,$0b,$c4,$5f,$ef,
    $e3,$3e,$34,$30,$3a,$0c,$31,$61,$9a,$f1,$cf,$a6,$1a,$1a,$8c,$5b,$79,$40,$7b,$8f,$d8,$f5,$ca,$1d,$7b,
    $b0,$be,$bf,$64,$7b,$45,$16,$4e,$e5,$28,$52,$ac,$73,$ba,$5d,$f4,$0d,$2d,$d7,$f2,$c2,$7c,$eb,$1a,$1f,
    $70,$fb,$8e,$96,$68,$1a,$c4,$40,$2c,$51,$bc,$4f,$71,$25,$d5,$70,$0b,$7f,$d8,$99,$4c,$56,$9a,$62,$9d,
    $66,$a8,$7f,$29,$11,$c9,$9c,$1a,$c4,$54,$29,$c1,$aa,$6c,$8f,$20,$7c,$39,$49,$fb,$de,$42,$f7,$4b,$b6,
    $e7,$6b,$51,$70,$54,$c6,$e1,$55,$7e,$a4,$00,$1f,$41,$85,$cf,$d6,$1a,$42,$65,$bf,$f5,$37,$4d,$c8,$6a,
    $97,$4f,$ca,$79,$d1,$65,$70,$82,$31,$c5,$79,$a5,$ce,$18,$2d,$6f,$91,$95,$0b,$4a,$9c,$c9,$90,$0f,$0e,
    $a5,$05,$06,$6f,$da,$e5,$6e,$2f,$27,$9b,$fc,$07,$0d,$ef,$2e,$96,$26,$43,$37,$d5,$45,$9f,$31,$df,$5e,
    $ff,$60,$5d,$2b,$dc,$1a,$0a,$5c,$3b,$35,$1b,$b4,$c3,$08,$25,$f3,$fb,$f3,$84,$43,$ab,$05,$38,$65,$1b,
    $e4,$ab,$9e,$c1,$e7,$00,$a9,$86,$e2,$ce,$3f,$69,$c5,$3f,$1d,$d3,$e7,$09,$59,$ae,$a9,$52,$f8,$d2,$c2,
    $0c,$10,$f0,$9b,$2d,$ac,$0c,$58,$8a,$eb,$18,$5b,$1f,$43,$de,$f2,$f3,$e6,$55,$1a,$9d,$6d,$06,$64,$9c,
    $df,$89,$0b,$bf,$18,$9c,$e7,$3d,$6c,$a2,$35,$cc,$77,$dd,$b6,$1a,$fd,$9c,$2c,$d7,$ec,$ac,$a3,$0b,$9c,
    $ce,$ac,$47,$db,$d9,$3f,$b0,$e0,$d0,$87,$12,$3c,$0f,$43,$50,$98,$f9,$bd,$e6,$12,$eb,$99,$ed,$e4,$f1,
    $b1,$71,$cb,$8c,$17,$5f,$76,$46,$a7,$6f,$38,$49,$78,$89,$b4,$50,$fa,$6e,$c1,$d1,$06,$7e,$d1,$dd,$c4,
    $12,$dc,$97,$9a,$2b,$a4,$a2,$11,$53,$f5,$61,$d0,$ae,$56,$7d,$5f,$5b,$53,$87,$c1,$e5,$cc,$ba,$2a,$7b,
    $8e,$2b,$5c,$d0,$26,$37,$c1,$1b,$99,$6a,$94,$16,$76,$f3,$a1,$2c,$d9,$60,$d2,$10,$e0,$3c,$4f,$85,$94,
    $ce,$cb,$3b,$2e,$ab,$5d,$dd,$33,$ad,$6b,$35,$f0,$48,$94,$5e,$f5,$a7,$56,$7b,$7d,$61,$b1,$f0,$67,$14,
    $96,$f8,$34,$9b,$26,$1f,$a8,$ea,$c7,$2c,$bc,$84,$3a,$39,$5b,$ed,$52,$76,$da,$66,$3e,$61,$87,$ce,$bf,
    $ba,$1d,$a3,$65,$c0,$77,$92,$95,$f1,$37,$a6,$40,$33,$fe,$1e,$c6,$d5,$7a,$47,$7a,$98,$7f,$75,$6a,$d9,
    $29,$f9,$51,$54,$18,$d7,$d8,$da,$dd,$fd,$4d,$6a,$5e,$1e,$a0,$21,$a9,$78,$ee,$58,$3c,$f8,$10,$d7,$72,
    $b9,$9f,$4a,$5d,$65,$0d,$76,$18,$c5,$67,$91,$e4,$3d,$1d,$42,$f1,$9a,$a7,$09,$4a,$1d,$d9,$46,$68,$11,
    $90,$e6,$24,$00,$6d,$b4,$e8,$37,$14,$98,$b8,$9d,$b1,$9d,$8f,$5a,$9f,$bf,$82,$80,$84,$b9,$26,$7e,$e1,
    $5a,$48,$5a,$7b,$11,$82,$5a,$f3,$69,$cf,$18,$91,$ab,$5e,$ce,$e8,$ea,$ab,$dd,$86,$02,$86,$8c,$eb,$e2,
    $7f,$7d,$ad,$b8,$12,$be,$3d,$09,$dd,$7c,$fd,$33,$a2,$eb,$99,$0d,$9b,$c2,$a2,$82,$14,$dd,$6f,$9e,$3d,
    $ce,$5b,$9e,$89,$6e,$f1,$e4,$7f,$a5,$a2,$d6,$3d,$81,$c5,$38,$4c,$dc,$dc,$e4,$be,$d9,$ab,$85,$cb,$9f,
    $52,$91,$32,$24,$17,$b6,$0d,$90,$5a,$19,$9f,$bc,$b3,$f2,$dc,$09,$a1,$ad,$b9,$8a,$66,$6e,$68,$0e,$53,
    $ee,$2e,$90,$f4,$01,$28,$16,$a5,$be,$94,$9b,$39,$d0,$45,$d9,$90,$ba,$ba,$ce,$51,$e6,$76,$dc,$13,$fb,
    $cc,$58,$98,$27,$54,$89,$65,$fd,$0e,$f7,$81,$38,$93,$97,$7f,$cd,$21,$e1,$6e,$45,$d6,$68,$e9,$a9,$31,
    $8a,$1e,$70,$1e,$e6,$d7,$8c,$51,$03,$26,$8e,$21,$cc,$c2,$2c,$82,$3f,$a6,$b6,$f0,$78,$6d,$6f,$52,$63,
    $87,$e2,$45,$8c,$09,$41,$f3,$91,$28,$fb,$4a,$4e,$e7,$22,$20,$b2,$29,$9d,$db,$41,$49,$25,$51,$b8,$b4,
    $9f,$e7,$c4,$54,$4a,$f4,$f0,$0a,$d8,$6a,$2e,$d7,$19,$0f,$bd,$da,$55,$d6,$05,$ba,$33,$fb,$61,$2f,$2e,
    $40,$4a,$87,$e2,$fe,$b9,$45,$62,$3b,$d5,$14,$c0,$66,$5b,$fc,$43,$22,$ca,$db,$1b,$85,$74,$b4,$e2,$56,
    $ea,$ee,$9e,$9f,$85,$dd,$09,$94,$02,$05,$11,$3d,$94,$c3,$c9,$e3,$04,$fc,$e8,$ca,$07,$b8,$00,$fa,$24,
    $13,$5e,$6a,$7e,$33,$6f,$02,$59,$cf,$4e,$14,$82,$10,$e9,$ad,$e9,$44,$9d,$5b,$e8,$45,$d2,$1a,$f2,$58,
    $f6,$99,$38,$3e,$e3,$73,$5c,$8a,$af,$2b,$40,$4c,$43,$fb,$91,$d8,$af,$5b,$b0,$34,$83,$68,$9a,$04,$20,
    $b5,$cf,$75,$37,$6b,$6d,$50,$fa,$25,$b8,$40,$52,$24,$cc,$7f,$99,$f7,$fe,$44,$5d,$5e,$53,$47,$c6,$22,
    $18,$ca,$df,$89,$55,$42,$47,$3d,$ff,$1a,$ef,$71,$6c,$03,$df,$b8,$c7,$05,$ad,$da,$c3,$e3,$c2,$a1,$d6,
    $3f,$49,$c6,$90,$3f,$1b,$fb,$b2,$27,$5c,$52,$09,$8a,$93,$03,$ff,$70,$4f,$33,$7c,$ca,$5c,$85,$97,$64,
    $de,$a2,$31,$09,$9f,$8c,$44,$59,$bb,$19,$a7,$82,$ae,$ee,$f9,$c3,$60,$1f,$8b,$61,$24,$88,$e4,$8f,$70,
    $0b,$e0,$b9,$d2,$93,$cb,$19,$e6,$ed,$f8,$87,$04,$3b,$2c,$22,$69,$f1,$11,$c4,$19,$ab,$7d,$28,$9a,$c0,
    $20,$2f,$7b,$5e,$5c,$98,$e2,$6b,$b6,$9e,$8b,$a4,$f3,$ee,$f3,$42,$8e,$39,$44,$2b,$99,$90,$cf,$f8,$19,
    $07,$9e,$e7,$d5,$ad,$b3,$fd,$42,$3f,$c7,$16,$d6,$53,$04,$27,$66,$3b,$f7,$23,$54,$98,$46,$98,$58,$7d,
    $98,$74,$08,$85,$28,$12,$74,$bf,$44,$29,$74,$fc,$e8,$c1,$61,$d1,$66,$71,$69,$ba,$55,$1d,$a7,$66,$12,
    $de,$97,$6b,$15,$c6,$54,$55,$45,$bf,$2d,$7f,$5c,$0e,$44,$d8,$4f,$a0,$5f,$d3,$2c,$0a,$19,$18,$32,$4a,
    $f8,$ee,$81,$c2,$57,$f1,$53,$10,$d3,$66,$95,$c6,$00,$0a,$5d,$22,$c3,$28,$01,$c4,$be,$dd,$c8,$7d,$37,
    $5a,$08,$ac,$91,$de,$5a,$1c,$d8,$69,$8b,$54,$15,$c4,$47,$a3,$f4,$f3,$83,$ea,$d0,$05,$24,$92,$26,$f4,
    $b1,$8d,$9e,$15,$33,$fd,$75,$28,$46,$d6,$b8,$76,$42,$01,$38,$c0,$91,$03,$2b,$e2,$b9,$8b,$8e,$6b,$24,
    $96,$c0,$ac,$1c,$6f,$00,$0b,$c9,$1e,$a3,$67,$b9,$31,$08,$c0,$60,$36,$57,$77,$c0,$71,$20,$1e,$df,$d0,
    $c4,$11,$56,$8c,$b3,$3c,$3d,$1c,$62,$1f,$f9,$a3,$c0,$2e,$e4,$21,$27,$eb,$30,$5d,$1d,$d7,$37,$fc,$77,
    $35,$f1,$49,$95,$7d,$80,$b7,$d6,$35,$c9,$ff,$81,$76,$a2,$0f,$02,$6e,$31,$99,$d8,$89,$ef,$ac,$9a,$fb,
    $45,$c3,$97,$26,$37,$b2,$c2,$ee,$04,$27,$1b,$f0,$6c,$b1,$99,$34,$27,$01,$20,$ae,$44,$b4,$ea,$89,$a4,
    $cc,$bd,$b8,$a5,$cf,$7f,$04,$10,$24,$c1,$60,$c2,$4d,$a9,$3a,$4d,$dd,$5d,$eb,$2a,$a2,$c6,$6c,$00,$c3,
    $63,$1d,$2e,$de,$db,$61,$49,$e2,$4e,$a3,$32,$16,$1f,$e5,$59,$0d,$59,$43,$1d,$0c,$da,$79,$4f,$a0,$12,
    $74,$1f,$fa,$4a,$93,$11,$7a,$95,$8a,$fb,$f3,$cc,$b9,$f6,$f7,$89,$59,$56,$c8,$06,$a3,$ae,$ca,$62,$3b,
    $3d,$11,$36,$48,$36,$65,$cd,$85,$92,$67,$58,$25,$dd,$bd,$5a,$f3,$69,$6a,$a9,$07,$60,$cf,$1e,$6f,$02,
    $39,$8a,$71,$dc,$1f,$a3,$7e,$92,$4a,$48,$fd,$8e,$41,$8d,$cd,$ea,$40,$8d,$03,$e6,$a2,$14,$a0,$71,$bf,
    $e9,$96,$84,$73,$e1,$02,$de,$f5,$f1,$43,$5e,$28,$ae,$4a,$98,$2a,$ab,$dd,$35,$60,$62,$41,$a7,$4a,$e6,
    $f6,$34,$af,$68,$11,$1e,$da,$4f,$77,$a7,$57,$42,$bb,$a7,$f3,$e5,$f7,$f3,$0f,$42,$30,$54,$10,$1c,$0c,
    $b0,$d5,$f0,$58,$a6,$7f,$41,$cf,$8a,$90,$01,$74,$66,$03,$ea,$1f,$56,$47,$01,$f6,$e0,$1d,$63,$a4,$fe,
    $6a,$76,$75,$39,$1d,$5f,$c1,$79,$b8,$b2,$09,$90,$69,$f4,$fc,$fe,$c3,$fd,$4d,$93,$0c,$b5,$2e,$5c,$c6,
    $84,$7e,$1c,$c2,$94,$c1,$a4,$9f,$fa,$64,$f4,$70,$20,$29,$a1,$e9,$a6,$67,$bb,$b4,$2b,$6d,$bf,$3a,$10,
    $90,$bb,$32,$d8,$c8,$af,$b1,$a3,$27,$8b,$72,$97,$11,$77,$56,$f1,$29,$e5,$9e,$7b,$14,$91,$83,$f1,$c6,
    $3f,$ab,$f4,$6d,$fa,$f0,$38,$3b,$2e,$a7,$0c,$52,$fa,$6e,$d6,$c5,$9d,$f1,$0e,$d7,$39,$ab,$c0,$0e,$19,
    $35,$7a,$94,$b6,$1f,$a0,$bd,$c8,$11,$ee,$ad,$97,$12,$a4,$a5,$5d,$6e,$cc,$9c,$37,$ee,$0d,$40,$09,$3c,
    $6a,$73,$5f,$66,$53,$b9,$dc,$67,$85,$06,$33,$17,$74,$c2,$e4,$c4,$89,$5b,$29,$aa,$00,$7e,$35,$c6,$c5,
    $06,$99,$a7,$93,$18,$f6,$ac,$69,$82,$34,$48,$2b,$c6,$1d,$00,$74,$32,$cc,$5a,$a4,$0c,$71,$62,$60,$04,
    $95,$ed,$4f,$01,$b0,$b1,$90,$a4,$8e,$4e,$7d,$74,$5f,$c4,$ce,$82,$f6,$19,$89,$be,$c6,$a1,$6d,$49,$42,
    $cb,$46,$15,$2d,$39,$3c,$25,$e1,$e0,$5b,$c2,$80,$84,$1d,$f5,$0b,$55,$c2,$2e,$8f,$eb,$af,$85,$41,$3c,
    $42,$a3,$42,$cf,$f2,$41,$05,$30,$0f,$c6,$88,$7e,$c7,$1d,$34,$bd,$4f,$43,$6a,$27,$88,$d4,$71,$0a,$80,
    $76,$58,$d3,$d2,$18,$8a,$7b,$93,$43,$de,$a5,$a8,$71,$17,$57,$fa,$43,$e1,$a7,$50,$b2,$5d,$c6,$08,$a8,
    $d2,$02,$78,$7c,$37,$b9,$1c,$49,$1e,$1e,$2b,$b6,$6c,$b9,$6c,$f6,$98,$a4,$c8,$7d,$b6,$97,$62,$c1,$a1,
    $c2,$c5,$4e,$49,$88,$bb,$b5,$69,$ba,$c7,$f6,$99,$18,$cd,$79,$76,$77,$55,$60,$33,$76,$25,$bf,$be,$2d,
    $3e,$ea,$57,$89,$da,$11,$ea,$69,$f5,$8d,$3e,$5f,$0f,$47,$dd,$b4,$34,$0a,$30,$39,$02,$71,$81,$84,$30,
    $0e,$28,$89,$4f,$ab,$48,$dc,$b5,$1a,$d1,$b1,$f7,$4a,$4e,$5d,$eb,$2d,$a0,$ff,$4e,$bd,$af,$b3,$45,$43,
    $71,$29,$c8,$8e,$37,$50,$bf,$e0,$b5,$2b,$6c,$17,$17,$3f,$b0,$95,$ab,$a3,$f7,$0a,$5d,$5d,$bd,$5d,$0d,
    $0d,$a3,$2c,$9f,$e7,$3f,$05,$0a,$bf,$74,$32,$6d,$27,$2c,$7c,$c8,$e0,$16,$e7,$c6,$f5,$d7,$5b,$42,$a7,
    $c7,$e0,$26,$63,$c1,$54,$95,$8e,$17,$be,$d0,$f0,$2b,$6c,$7b,$7f,$13,$75,$c4,$c9,$48,$3b,$01,$82,$59,
    $bf,$53,$39,$4c,$32,$f1,$ca,$b0,$4f,$f3,$a0,$47,$d2,$71,$da,$b7,$06,$2d,$f2,$3d,$61,$27,$df,$eb,$2b,
    $cc,$f7,$34,$37,$c5,$df,$f2,$96,$96,$84,$73,$7a,$ea,$9a,$b9,$5b,$a8,$19,$e6,$89,$a8,$48,$28,$0f,$e5,
    $3f,$1d,$ef,$d4,$85,$82,$d8,$0d,$10,$41,$7a,$e6,$ac,$72,$6a,$98,$91,$da,$01,$7b,$0c,$39,$4d,$24,$68,
    $36,$fb,$26,$6f,$ef,$46,$4a,$53,$3e,$00,$95,$b1,$19,$c2,$24,$8c,$33,$32,$cd,$7f,$f3,$bd,$ab,$db,$1c,
    $5e,$b7,$78,$db,$68,$6e,$1d,$72,$4d,$fc,$20,$63,$f3,$e7,$c2,$9f,$50,$83,$88,$5b,$6f,$3c,$ac,$3e,$c7,
    $fc,$2a,$f8,$97,$73,$b6,$da,$b3,$8a,$55,$23,$36,$a2,$1a,$76,$68,$8a,$be,$0b,$a6,$0b,$52,$0c,$34,$dd,
    $ae,$49,$fc,$e7,$23,$b8,$99,$6a,$eb,$1e,$fd,$42,$e4,$c3,$fa,$95,$59,$35,$7e,$e4,$dd,$cf,$68,$bb,$8d,
    $d1,$ca,$a9,$26,$e3,$4a,$1a,$01,$24,$ef,$39,$3e,$1e,$36,$01,$b9,$06,$cd,$04,$fe,$6e,$79,$7a,$b3,$f1,
    $0b,$4c,$b3,$87,$0c,$b2,$58,$91,$fd,$a3,$43,$82,$e8,$f7,$77,$6c,$71,$98,$99,$a8,$77,$f0,$52,$c3,$ff,
    $1a,$c3,$a4,$6a,$13,$0b,$75,$dc,$6c,$c8,$fe,$e7,$e0,$c8,$40,$f9,$5c,$39,$06,$76,$6d,$44,$3f,$72,$90,
    $ee,$95,$83,$51,$ea,$ed,$2a,$18,$7e,$3f,$d6,$84,$0d,$31,$c7,$fe,$a8,$e3,$4b,$1a,$35,$ce,$b0,$d6,$46,
    $2c,$9f,$59,$8f,$97,$85,$62,$f6,$d2,$cf,$26,$75,$98,$16,$d6,$e2,$79,$b5,$7a,$33,$ce,$54,$c9,$e1,$42,
    $5b,$dc,$1b,$d4,$1d,$72,$ed,$95,$f5,$25,$80,$c2,$3c,$b4,$8d,$d7,$31,$d7,$03,$c0,$10,$33,$f9,$50,$b4,
    $81,$55,$85,$55,$9b,$e6,$cc,$ec,$f1,$64,$9e,$64,$31,$bd,$dd,$12,$fb,$b5,$17,$97,$14,$07,$19,$32,$98,
    $dd,$43,$2f,$c1,$ca,$f1,$10,$7a,$14,$a2,$14,$de,$45,$aa,$5c,$38,$b4,$b4,$f2,$cd,$57,$ec,$f6,$0a,$fc,
    $c7,$d1,$12,$4b,$28,$70,$92,$34,$d2,$3a,$1b,$e8,$97,$7f,$41,$6d,$aa,$e3,$d2,$12,$ab,$7a,$59,$0a,$ae,
    $79,$a2,$e3,$23,$0b,$6f,$1f,$c0,$a2,$1f,$9f,$e0,$9c,$a0,$26,$b6,$b8,$57,$6d,$d3,$9a,$88,$a0,$66,$c6,
    $02,$8d,$35,$17,$72,$e8,$dc,$f0,$89,$71,$33,$d8,$3e,$d0,$a8,$2b,$ed,$b2,$2b,$2f,$ef,$f1,$4e,$38,$cb,
    $6a,$9b,$40,$91,$cb,$7c,$89,$5c,$a3,$04,$d0,$b0,$6b,$6c,$00,$39,$ad,$04,$8f,$7d,$f2,$6a,$fc,$77,$77,
    $93,$da,$e8,$b6,$5e,$bc,$26,$1b,$92,$ac,$cf,$12,$04,$12,$57,$89,$23,$11,$0d,$bc,$ee,$47,$03,$97,$d2,
    $18,$36,$68,$2a,$61,$01,$12,$30,$5b,$4e,$d7,$40,$ec,$65,$0d,$5e,$23,$fa,$40,$be,$32,$a3,$7f,$2c,$21,
    $28,$c5,$06,$7d,$02,$a9,$51,$c4,$63,$dd,$47,$5f,$12,$ea,$38,$89,$a4,$27,$bf,$c2,$fa,$ca,$b9,$19,$d1,
    $3b,$89,$9f,$45,$8a,$e8,$ca,$f6,$c9,$38,$d5,$d2,$e8,$b0,$0b,$ba,$cd,$79,$0f,$ec,$dd,$27,$a9,$af,$54,
    $55,$40,$08,$20,$df,$b2,$a2,$4f,$3a,$c9,$48,$e1,$c4,$3d,$27,$5c,$87,$05,$90,$b4,$d4,$a5,$3e,$6d,$6a,
    $b5,$a5,$aa,$ba,$a9,$c6,$4b,$1d,$84,$f4,$09,$5d,$b0,$a6,$66,$e0,$51,$84,$6f,$d3,$9e,$ab,$6c,$bc,$f1,
    $29,$71,$11,$3e,$3a,$40,$c3,$e3,$b4,$38,$ad,$5b,$37,$7f,$3d,$3c,$0e,$4d,$bc,$ef,$ce,$77,$c4,$4f,$b6,
    $49,$06,$27,$f1,$98,$b3,$be,$63,$a6,$0c,$bd,$74,$0b,$82,$12,$ff,$cb,$70,$b4,$a8,$5e,$0a,$04,$75,$3f,
    $1e,$9e,$d8,$1d,$59,$c6,$3a,$9a,$13,$70,$64,$a0,$b9,$04,$fd,$c8,$fc,$da,$95,$80,$9c,$90,$4e,$1e,$fc,
    $55,$0d,$e0,$c0,$ed,$6c,$4c,$fe,$87,$a9,$37,$f8,$b7,$ee,$1b,$82,$f3,$56,$d1,$a4,$96,$b6,$79,$0e,$25,
    $3b,$a7,$9a,$f0,$20,$c2,$a3,$f4,$14,$bf,$57,$4d,$1d,$4c,$61,$49,$78,$91,$b4,$95,$d2,$89,$da,$4a,$64,
    $66,$70,$65,$2b,$bb,$b8,$5e,$55,$6f,$5c,$d3,$f4,$03,$0d,$e5,$1d,$7f,$ef,$cf,$83,$41,$2d,$71,$38,$68,
    $ff,$79,$2a,$5f,$f3,$29,$e7,$b0,$9a,$37,$ab,$a7,$52,$22,$74,$87,$4a,$27,$37,$da,$1c,$53,$f5,$a4,$4b,
    $78,$fa,$96,$dd,$23,$b8,$25,$da,$11,$33,$44,$16,$cc,$49,$53,$db,$91,$ff,$61,$08,$bd,$75,$e1,$99,$29,
    $18,$20,$78,$e5,$7a,$a3,$1a,$24,$2f,$7b,$ab,$ed,$71,$ae,$23,$4d,$c1,$2c,$9d,$e8,$7d,$53,$9d,$f6,$ed,
    $43,$c0,$4b,$b5,$ee,$40,$5a,$90,$72,$8a,$07,$52,$04,$f6,$61,$33,$79,$8d,$31,$10,$2f,$d6,$9b,$09,$32,
    $47,$ac,$ce,$ed,$7f,$83,$84,$9c,$7b,$44,$66,$7f,$73,$af,$48,$4a,$8b,$e6,$a2,$45,$61,$2e,$ad,$70,$12,
    $3e,$b8,$5e,$87,$2d,$5b,$07,$de,$42,$5f,$95,$64,$ba,$ec,$f2,$02,$ea,$f7,$37,$8c,$08,$00,$4d,$9c,$af,
    $7f,$41,$4a,$10,$a6,$25,$5a,$44,$9a,$6b,$4d,$1b,$c5,$95,$fd,$7f,$56,$1e,$9c,$dd,$6a,$5b,$a5,$a5,$81,
    $62,$df,$34,$2a,$6d,$fc,$02,$9b,$6f,$b5,$25,$35,$6f,$ec,$a7,$fc,$53,$bc,$92,$68,$28,$ac,$ad,$b5,$ac
  {$endif}
  );

{---------------------------------------------------------------------------}
function AddrEBytes: pointer;
  {-Pointer to bits of exp(1), big endian format}
begin
  AddrEBytes := @EBytes;
end;

{$endif}


end.

